
#include "TreeBuilder.h"
#include "ptg_gen.h"
#include "vlAccess.h"


TreeBuilder::TreeBuilder() 
{
  code = PTGNULL;
  nodeSize = PTGNULL;
  nodeCount = 0;
  convertion = 72.0;
}


VLValue vlGetClassName(TreeContext x)
{
  VLList l;
  l.append("::c::getClassName");
  l.append(x);
  return l.eval();
}


void TreeBuilder::addNode(TreeContext tree, VLPoint size) 
{
  nodeCount++;
  VLString objId = VLString (tree);
  code = PTGSeq(code, PTGDotNode(PTGAsIs(objId), 
    PTGAsIs(VLString(VLDouble(size.getX()/convertion))), 
    PTGAsIs(VLString(VLDouble(size.getY()/convertion)))
  ));
}


void TreeBuilder::addEdge(TreeContext toTree, TreeContext fromTree ) 
{
  code = PTGSeq(code, PTGDotEdge(PTGAsIs(VLString(fromTree)), PTGAsIs(VLString(toTree))));
}


void TreeBuilder::solve(VLString ratio, VLPoint size) 
{

  code = PTGDotFrame(code, ratio, size.getX(), size.getY());
  VLString dotInput = PTGOutString(code);
  VLList l;
  l.append("TreeBuilder::runDot");
  l.append(dotInput);
  VLString dotOutput = VLString(l.eval());
  analyse(dotOutput);
}


void TreeBuilder::analyse(char *input)
{
  float dummy1,x,y,w,h;
  int ix, iy, ih, iw;
  char node [20];
  char name[20];
  char dummy[20];
  char edge[20];
  char head[20];
  char tail[20];
  sscanf(input,"%s%f%f%f",node,&dummy1, &x, &y);
  ix = int(convertion*x);
  iy = int(convertion*y);
  graphSize=VLPoint(ix,iy);
  
  for (int j=0; j<nodeCount;j++){
    while(input[0] != '\n') input++; input++;
    sscanf(input,"%s%s%f%f%f%f%s%s%s%s%s",node,name,&x,&y,&w,&h,dummy,dummy,dummy,dummy,dummy);
    VLRegion region;
    ix = int(convertion * x);
    iy = int(convertion * y);
    iw = int(convertion * w);
    ih = int(convertion * h);
    region.setPos(VLPoint(ix-iw/2,graphSize.getY()-iy-ih/2));
    region.setSize(VLPoint(iw,ih));
    nodes.set(name,region);
  }
  
  for (int j=0; j<nodeCount - 1;j++)
  {
     while(input[0] != '\n') input++; input++;
     int count;
     char countl[20];
     VLPointArray pointList;
     VLList keyList;
     sscanf(input,"%s%s%s%i",edge,tail,head,&count);
     sscanf(input,"%s%s%s%s",dummy,dummy,dummy,countl);
     input += strlen(edge);
     input += strlen(tail);
     input += strlen(head);
     input += strlen(countl);
     input += 4;
     for(int k=0;k < count;k++)
     {
          char xl[20];
          char yl[20];
          
         sscanf(input,"%f%f",&x,&y);
         sscanf(input,"%s%s",xl,yl);
         input += strlen(xl) + 1;
         input += strlen(yl) + 1;
         ix = int(convertion * x);
         iy = int(convertion * y);
         pointList.append(ix,graphSize.getY() - iy);
     }
     keyList.append(tail);
     keyList.append(head);
     edges.set(keyList,pointList);
  }
}


VLPoint TreeBuilder::getTreeSize() 
{
  return graphSize;
}



VLRegion TreeBuilder::getPosition(VLString objId) 
{
  VLString obj ("_",objId);
  VLRegion temp = (VLRegion)nodes.get(obj);
  return ((VLRegion)nodes.get(obj));
}

VLPointArray TreeBuilder::getEdgeList(VLString tailObjId,VLString headObjId)
{
  VLString tailObj ("_",tailObjId);
  VLString headObj ("_",headObjId);
  //printf("tail: %s, head: %s\n",tailObj.getCharPtr(),headObj.getCharPtr());
  return ((VLPointArray)edges.get(vlList(headObj,tailObj)));

}
