
#ifndef VLHASHTABLE_H
#define VLHASHTABLE_H

#include <tcl.h>
#include "BasicTypes.h"

class VLHashtable
{

private:
    Tcl_HashTable table;

public:
  VLHashtable() { Tcl_InitHashTable(&table, TCL_STRING_KEYS); }
  ~VLHashtable() { Tcl_DeleteHashTable(&table); }

  void set(char *key, VLValue value)
    {
      int newValue;
      Tcl_HashEntry *e=Tcl_CreateHashEntry(&table, key, &newValue);
      Tcl_SetHashValue(e,value.getObj());
      Tcl_IncrRefCount(value.getObj());
    }

  void set(VLValue key, VLValue value)
  {
      set(Tcl_GetStringFromObj(key.getObj(),NULL), value);
  }

  VLValue get(char *key)
  {
      Tcl_HashEntry *e=Tcl_FindHashEntry(&table, key);
      if(e) return VLValue((Tcl_Obj*)Tcl_GetHashValue(e));
      else return VLValue();
  }

  VLValue get(VLValue key)
  {
      char *str=Tcl_GetStringFromObj(key.getObj(),NULL);
      return get(str);
  }

  bool keyExists (VLValue key)
  {
	  char *str=Tcl_GetStringFromObj(key.getObj(),NULL);
      return keyExists(str);
  }

  bool keyExists (char *key)
  {
	  Tcl_HashEntry *e=Tcl_FindHashEntry(&table, key);
	  return e;
  }

  void setMax(VLValue key, int i)
  {
      int curr = VLInt(get(key));
      set(key, VLInt(i>curr? i: curr));
  }

  void incr(VLValue key)
  {
    int curr = VLInt(get(key));
    set(key, VLInt(curr+1));
  }
};


typedef VLHashtable *VLHashtablePtr;

#endif
