
#ifndef _VLObjectPointerPtrLIST_H
#define _VLObjectPointerPtrLIST_H

/* include header file defining VLObjectPointerPtr if VLObjectPointerPtr is set: */
#define EMPTYVLObjectPointerPtrHDR
#ifndef EMPTYHDR
#include "VLObjectPointerPtr.h"
#endif
#undef EMPTYVLObjectPointerPtrHDR

#include "VLObjectPointerPtrFunctionTypes.h"

typedef struct _VLObjectPointerPtrLE *VLObjectPointerPtrList;
typedef VLObjectPointerPtrList    *VLObjectPointerPtrListPtr;

struct _VLObjectPointerPtrLE {
VLObjectPointerPtr              head;
VLObjectPointerPtrList  tail;
};

#define NULLVLObjectPointerPtrList              ((VLObjectPointerPtrList)0)
#define NullVLObjectPointerPtrList()    ((VLObjectPointerPtrList)0)
#define SingleVLObjectPointerPtrList(e) (ConsVLObjectPointerPtrList((e),NULLVLObjectPointerPtrList))

#if defined(__STDC__) || defined(__cplusplus)

extern void FinlVLObjectPointerPtrList(void);
   /* Deallocates all VLObjectPointerPtrLists. */
   /* Dynamic storage is allocated on first need */

extern VLObjectPointerPtrList ConsVLObjectPointerPtrList (VLObjectPointerPtr e, VLObjectPointerPtrList l);
   /* Constructs a VLObjectPointerPtrList of an element e and a given tail l.
    * e is the first element of the list. */

extern VLObjectPointerPtr HeadVLObjectPointerPtrList (VLObjectPointerPtrList l);
   /* Returns the first element of the list l.
    * The list l must not be empty. */

extern VLObjectPointerPtrList TailVLObjectPointerPtrList (VLObjectPointerPtrList l);
   /* Returns the tail of the list l.
    * If l is empty, an empty list is returned. */

extern int LengthVLObjectPointerPtrList (VLObjectPointerPtrList l);
   /* Returns the number of elements in the list l. */

extern VLObjectPointerPtr IthElemVLObjectPointerPtrList (VLObjectPointerPtrList l, int i);
   /* Returns the i-th element of the List l. The head of l
    * is referred to as 1. If the value of
    * i is greater than the length of the list, an error
    * is reported and the program exits.
    */

extern VLObjectPointerPtrList CopyVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtrMapFct cp);
   /* Copies the list l. Elements are copied by calls of cp. */

extern VLObjectPointerPtrList AppVLObjectPointerPtrList (VLObjectPointerPtrList l1, VLObjectPointerPtrList l2);
   /* Concatenates two lists l1 and l2. The resulting list contains l2 at
    * the end of a copy of list l1. */

extern VLObjectPointerPtrList AppElVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtr e);
   /* Appends an element e to the list l.
    * The list l is not copied, it is modified as a side-effect
    * of this function. */

extern void InsertAfterVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtr e);
   /* This function requires a non-empty list l. The element e is inserted
    * just after the first element of l. 
    * The list l is modified as a side-effect of this function. */

extern VLObjectPointerPtrList OrderedInsertVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtr e, VLObjectPointerPtrCmpFctType fcmp);
   /* Inserts the element e into the list l maintaining l in
    * ascending order with respect to the compare fcmp. 
    * The list l is modified as a side-effect of this function. */

extern VLObjectPointerPtrListPtr RefEndConsVLObjectPointerPtrList (VLObjectPointerPtrListPtr addr, VLObjectPointerPtr e);
   /* Appends an element e to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern VLObjectPointerPtrListPtr RefEndAppVLObjectPointerPtrList (VLObjectPointerPtrListPtr addr, VLObjectPointerPtrList l);
   /* Appends a list l to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern int ElemInVLObjectPointerPtrList (VLObjectPointerPtr e, VLObjectPointerPtrList l, VLObjectPointerPtrCmpFctType cmpfct);
   /* This function returns true (1) iff the element e is in the List l.
    * List elements are compared by the function cmpfct. */

extern VLObjectPointerPtrList AddToSetVLObjectPointerPtrList (VLObjectPointerPtr e, VLObjectPointerPtrList l, VLObjectPointerPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. */

extern VLObjectPointerPtrList AddToOrderedSetVLObjectPointerPtrList 
       (VLObjectPointerPtr e, VLObjectPointerPtrList l, VLObjectPointerPtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. l is assumed to be ordered increasingly in the sense of
    * cmpfct. */

extern VLObjectPointerPtrList MapVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtrMapFct f);
   /* Returns a new VLObjectPointerPtrList obtained by applying f to each element of l. */

extern int CompVLObjectPointerPtrList (VLObjectPointerPtrList l1, VLObjectPointerPtrList l2, VLObjectPointerPtrCmpFctType f);
   /* Compares the lists l1 and l2 lexicographically by applying f
    * to the corresponding elements. */

extern VLObjectPointerPtr SumVLObjectPointerPtrList (VLObjectPointerPtrList l, VLObjectPointerPtrSumFct f, VLObjectPointerPtr a);
   /* Applies the binary function f to the elements of the List:
    *   f( f(... f(a, e1), e2, ...), en)
    * If l is empty a is returned. */

#else

extern void FinlVLObjectPointerPtrList ();
extern VLObjectPointerPtrList ConsVLObjectPointerPtrList ();
extern VLObjectPointerPtr HeadVLObjectPointerPtrList ();
extern VLObjectPointerPtrList TailVLObjectPointerPtrList ();
extern int LengthVLObjectPointerPtrList ();
extern VLObjectPointerPtr IthElemVLObjectPointerPtrList ();
extern VLObjectPointerPtrList CopyVLObjectPointerPtrList ();
extern VLObjectPointerPtrList AppVLObjectPointerPtrList ();
extern VLObjectPointerPtrList AppElVLObjectPointerPtrList ();
extern void InsertAfterVLObjectPointerPtrList ();
extern VLObjectPointerPtrList OrderedInsertVLObjectPointerPtrList ();
extern VLObjectPointerPtrListPtr RefEndConsVLObjectPointerPtrList ();
extern VLObjectPointerPtrListPtr RefEndAppVLObjectPointerPtrList ();
extern int ElemInVLObjectPointerPtrList ();
extern VLObjectPointerPtrList AddToSetVLObjectPointerPtrList ();
extern VLObjectPointerPtrList MapVLObjectPointerPtrList ();
extern int CompVLObjectPointerPtrList ();
extern VLObjectPointerPtr SumVLObjectPointerPtrList ();

#endif

#ifdef MONITOR
#define DAPTO_RESULTVLObjectPointerPtrList(l) DAPTO_RESULT_PTR(l)
#define DAPTO_ARGVLObjectPointerPtrList(l)    DAPTO_ARG_PTR (l, VLObjectPointerPtrList)
#endif

#endif
