
#ifndef VISAUTOPATTERNCODE_H
#define VISAUTOPATTERNCODE_H
#include <limits.h>
#include "VisBasicCode.h"
#include "DynamicalArray.h"
#include "VisContainerPosition2PtrList.h"
#include "VisContainerInterface2PtrList.h"
#include "vlDraw.h"
#include "vlLog.h"

#define CREATEVisContainerPosition2PtrList(k) \
   ConsVisContainerPosition2PtrList(k,NULLVisContainerPosition2PtrList)

#define CREATEVisContainerInterface2PtrList(k) \
   ConsVisContainerInterface2PtrList(k,NULLVisContainerInterface2PtrList)




enum { VisDerivpos_term=-1, VisDerivpos_frame=-2 };

typedef class VisDrawingTrafo
{
public:

  TreeContext nodePtr;
  TreeContext listRootNodePtr;

  virtual ~VisDrawingTrafo() {}
  VisDrawingTrafo() {}

  void setNodePtr(TreeContext x) { nodePtr=x; }
  virtual void draw(int derivpos=0, TreeContext primIdNodePtr=VLObjectPointer())=0;
  virtual VisContainerInterface2 *expand(void)=0;
  virtual void setSubElement (VisContainerInterface2 *, VLConstCharPtr contName)=0;
  virtual int containerExists(VLConstCharPtr name)=0;
  void setSubElements(VisContainerPosition2PtrList list)
  {
    while(list!=NULLVisContainerPosition2PtrList)
    {
      VisContainerPosition2 *p=HeadVisContainerPosition2PtrList(list);
      list=TailVisContainerPosition2PtrList(list);
      setSubElement(p->element, p->containerName);
    }
  }
} *VisDrawingTrafoPtr;


typedef class VisDrawing
{
public:
  virtual ~VisDrawing() {}


  virtual VisDrawingTrafo *getDrawingTrafo(void)=0;
  DynamicalArray<VLConstCharPtr > containerNames;
  DynamicalArray<VLConstCharPtr > dummyContainerNames;

  int hasDummyContainer(VLConstCharPtr name)
  {
    for(int i=0; i<dummyContainerNames.getSize(); i++) if(!strcmp(dummyContainerNames[i],name)) return 1;
    return 0;
  }

  int getContainerId(VLConstCharPtr name) 
  { 
    for(int i=0; i<containerNames.getSize(); i++) if(!strcmp(containerNames[i],name)) return i;
    return -1;
  }
} *VisDrawingPtr;



struct VisTileDrawingContainer
{
  VisContainerInterface2 *interface;
  VisFloatVector p1,p2,buoy;
  VisIntVector align;
  VisTileDrawingContainer(void) {}
  VisTileDrawingContainer(VisFloatVector aP1, VisFloatVector aP2,
    VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
    { p1=aP1; p2=aP2; align.x=alignX.type; align.y=alignY.type; 
      buoy.x=alignX.value; buoy.y=alignY.value;
      interface=NULL;
    }
};

struct VisTileDrawingDragLine
{
  int pos;
  int repetitions;
  int size;
  int priority;
  VisTileDrawingDragLine(int aPos, int aPriority, int aRepetitions) 
  { pos=aPos; priority=aPriority; repetitions=aRepetitions; }
  VisTileDrawingDragLine(void) {}
  int getRepSize(void) { return repetitions*size; }
};

struct VisTileDrawingInsertionLine
// Position einer Einfuegesetelle (eine Linie)
{
  VisFloatVector p1,p2;
  VLConstCharPtr after;
  VisTileDrawingInsertionLine(void) {}
  VisTileDrawingInsertionLine(VisFloatVector aP1, VisFloatVector aP2, VLConstCharPtr aAfter) :
    p1(aP1), p2(aP2), after(aAfter) {}
};

class VisTileDrawing: public VisDrawing, public VisDrawingTrafo
{
public:
  //Optionen:
  int tileCountX, tileCountY;
  VLConstCharPtr *tileName; //Array der Groesse tileCountX*tileCountY
  DynamicalArray<int> sumX; // Breite des Bereichs 0..i
  DynamicalArray<int> sumY; // Hoehe des Bereichs 0..i
  DynamicalArray<int> sizeX; // Breite der Spalte i
  DynamicalArray<int> sizeY; // Hoehen der Zeile i
  VisContainerInterface2 containerInterface;
  DynamicalArray<VisTileDrawingContainer> container;
  DynamicalArray<VisTileDrawingDragLine> dragLineX; // speichert unter anderem die Repetitions
  DynamicalArray<VisTileDrawingDragLine> dragLineY;
  DynamicalArray<VisTileDrawingInsertionLine> insertionLine;
  VisFloatVector alignmentPosition;
  VisTileDrawing(void) { alignmentPosition=VisFloatVector(0.0,0.0); }

  int expandX(float fleft, float fright, int minSize);
  int expandY(float fleft, float fright, int minSize);
  int getPosX(float i) { return getPosX((int)i); }
  int getPosY(float i) { return getPosY((int)i); }
  int getPosX(int i) { return expandX(0,i,0)+containerInterface.position.x; }
  int getPosY(int i) { return expandY(0,i,0)+containerInterface.position.y; }
  VisDrawingTrafo *getDrawingTrafo(void);
  VisContainerInterface2 *expand(void);
  void draw(int derivpos, TreeContext primIdNodePtr=VLObjectPointer());
  void setSubElement (VisContainerInterface2 *, VLConstCharPtr name);
  int containerExists(VLConstCharPtr name) { return getContainerId(name)!=-1; }
  virtual void initDrawing(void) { assert(0); }

  //Zeichenfunktionen
  void addContainer(VLConstCharPtr name, float x1, float y1, float x2, float y2, 
                         VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
    { container.add(
       VisTileDrawingContainer(VisFloatVector(x1,y1),VisFloatVector(x2,y2),alignX,alignY)); 
      containerNames.add(name);
    }
  void addDragLineX(int x,int priority=0, int mincnt=0) 
   { dragLineX.add(VisTileDrawingDragLine(x,priority,mincnt)); }
  void addDragLineY(int y,int priority=0, int mincnt=0) 
  { dragLineY.add(VisTileDrawingDragLine(y,priority,mincnt)); }
  void addInsertionLine(float x1, float y1, float x2, float y2, VLConstCharPtr after)
    { insertionLine.add(VisTileDrawingInsertionLine(
        VisFloatVector(x1,y1),VisFloatVector(x2,y2),after)); }
};



class VisVectorDrawingCoord: public DynamicalArray<int>
{
public: 
  void mult(int factor);
  void scale(int left, int right, double factor, int quantum);
  void stretch(int left, int right, int delta, int quantum);
  int min(void);
  int max(void);
};



struct VisVectorDrawingContainerProp
{
  int idX; // posX[2*idX],... und contXPreferedSize[idX],... gehoeren zu diesem Kontainer
  int idY;
  VisAlignmentAdvice alignX;
  VisAlignmentAdvice alignY;
  VisVectorDrawingContainerProp(void) {}
  VisVectorDrawingContainerProp(int aIdX, int aIdY, 
     VisAlignmentAdvice aAlignX, VisAlignmentAdvice aAlignY) 
  { idX=aIdX; idY=aIdY; alignX=aAlignX; alignY=aAlignY; }
};



class VisVectorDrawingFigureProp
{
public:
  int posId;
  int points;
  VLConstCharPtr figure;
  VLConstCharPtr figureStyle;
  VisVectorDrawingFigureProp(void) {}
  VisVectorDrawingFigureProp(int aPosId, int aPoints, 
               VLConstCharPtr aFigure, VLConstCharPtr aFigureStyle)
  { posId=aPosId; points=aPoints; figure=aFigure; figureStyle=aFigureStyle; }
};



class VisVectorDrawing: public VisDrawing
{
public:
  VisVectorDrawing(void) { boundingBox=0; inited=false; stretchIsometrical=VisIntVector(-1,-1); }
  VisVectorDrawingCoord contPosX; // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord contPosY; //           eine bestimmte Groesse benoetigt
  VisVectorDrawingCoord boundPosX;
  VisVectorDrawingCoord boundPosY;
  DynamicalArray<int> startintId2X;
  DynamicalArray<int> startintId2Y;
  DynamicalArray<int> endintId2X;
  DynamicalArray<int> endintId2Y;
  DynamicalArray<VisVectorDrawingContainerProp> contProp;
  VisVectorDrawingCoord intPosX;   // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord intPosY;   //                        gedehnt werden kann
  DynamicalArray<int> intQuantumX; // intQuantumX[i] is Vergroesserungsquantum 
  DynamicalArray<int> intQuantumY; // des Intervalls intPosX[2*i]...intPosX[2*i+1]
  DynamicalArray<int> intPriorityX; // intPriorityX[i] is Vergroesserungsprioritaet 
  DynamicalArray<int> intPriorityY; // des Intervalls intPosX[2*i]...intPosX[2*i+1]
  VisVectorDrawingCoord figurePosX;
  VisVectorDrawingCoord figurePosY;
  VisVectorDrawingCoord insertionLineX;     // Koordinaten einer Einfuegesetelle.
  VisVectorDrawingCoord insertionLineY;     // insertionLineAfter: Hinter dem entspr.
  DynamicalArray<VLConstCharPtr> insertionLineAfter; // Containerelement wird eingefuegt.
  VisIntVector stretchIsometrical;
  DynamicalArray<VisVectorDrawingFigureProp> figureProp;
  int boundingBox;
  bool inited;

  void flipX()
  {
    figurePosX.mult(-1);

    for(int i=0; i<contPosX.getSize(); i+=2) {
        int dummy = contPosX.value(i);
        contPosX.value(i)   = -contPosX.value(i+1);
        contPosX.value(i+1) = -dummy;
    }
    for(int i=0; i<intPosX.getSize()/2; i++) {
        int j = intPosX.getSize()-1-i;
        int dummy = intPosX.value(i);
        intPosX.value(i) = -intPosX.value(j);
        intPosX.value(j) = -dummy;
    }
  }

  void flipY() {
    figurePosY.mult(-1);

    for(int i=0; i<contPosY.getSize(); i+=2) {
        int dummy = contPosY.value(i);
        contPosY.value(i)   = -contPosY.value(i+1);
        contPosY.value(i+1) = -dummy;
    }
    for(int i=0; i<intPosY.getSize()/2; i++) {
        int j = intPosY.getSize()-1-i;
        int dummy = intPosY.value(i);
        intPosY.value(i) = -intPosY.value(j);
        intPosY.value(j) = -dummy;
    }
  }

  void flipXY()
  {
    VisVectorDrawingCoord dummy;
    DynamicalArray<int> dummy2;

    dummy = figurePosX;
    figurePosX = figurePosY;
    figurePosY = dummy;

    dummy = contPosX;
    contPosX = contPosY;
    contPosY = dummy;

    dummy = intPosX;
    intPosX = intPosY;
    intPosY = dummy;

    dummy2 = intQuantumX;
    intQuantumX = intQuantumY;
    intQuantumY = dummy2;

    dummy2 = intPriorityX;
    intPriorityX = intPriorityY;
    intPriorityY = dummy2;
  }


  // Zeichenfunktionen  
  void figure(int points, VLConstCharPtr aFigure, VLConstCharPtr aStyle)
  { figureProp.add(VisVectorDrawingFigureProp(figurePosX.getSize(),points,aFigure,aStyle)); }
  void point(int x, int y)
  { figurePosX.add(x); figurePosY.add(y); }
  void container(VLConstCharPtr name,int x1, int y1, int x2, int y2, 
                         VisAlignmentDefinition alignX, VisAlignmentDefinition alignY)
  { contProp.add(VisVectorDrawingContainerProp(contPosX.getSize()/2,contPosY.getSize()/2,
                                                            alignX.type, alignY.type));
    contPosX.add(x1); 
    if(alignX.type==VisAlignBuoy) { contPosX.add(alignX.value); contPosX.add(alignX.value); }
    contPosX.add(x2);
    contPosY.add(y1); 
    if(alignY.type==VisAlignBuoy) { contPosY.add(alignY.value); contPosY.add(alignY.value); }
    contPosY.add(y2);
    containerNames.add(name);
  }

  void dummyContainer(VLConstCharPtr name) {
     dummyContainerNames.add(name);
  }

  void insertionLine(int x1, int y1, int x2, int y2, VLConstCharPtr after)
  { 
    insertionLineX.add(x1); insertionLineX.add(x2);
    insertionLineY.add(y1); insertionLineY.add(y2);
    insertionLineAfter.add(after);
  }
  void stretchX(int left, int right, int priority=0, int quantum=1) 
  { intPosX.add(left); intPosX.add(right); 
    intQuantumX.add(quantum); intPriorityX.add(priority); }
  void stretchY(int left, int right, int priority=0, int quantum=1) 
  { intPosY.add(left); intPosY.add(right); 
    intQuantumY.add(quantum); intPriorityY.add(priority); }

  // andere Methoden
  VisDrawingTrafo *getDrawingTrafo(void);
  VisVectorDrawing *rotate(bool flip, int rot);
  virtual void draw(void) {}
};



class VisVectorDrawingTrafo: public VisDrawingTrafo
{
public:
  VisContainerInterface2 containerInterface;
  VisVectorDrawingCoord contPosX; // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord contPosY; //           eine bestimmte Groesse benoetigt
  VisVectorDrawingCoord intPosX;  // linke, rechte Seite eines Intervalls, das
  VisVectorDrawingCoord intPosY;  //                        gedehnt werden kann
  VisVectorDrawingCoord figurePosX;
  VisVectorDrawingCoord figurePosY;
  VisVectorDrawingCoord boundPosX;
  VisVectorDrawingCoord boundPosY;
  VisVectorDrawingCoord insertionLineX;
  VisVectorDrawingCoord insertionLineY;
  DynamicalArray<int> contXPreferedSize;
  DynamicalArray<int> contYPreferedSize;
  DynamicalArray<VisContainerInterface2*> interface;
  VisVectorDrawing *drawing;

  VisVectorDrawingTrafo(void) { drawing=NULL; }
  void init(VisVectorDrawing &d);
  void scaleX(int intId2, double factor);
  void scaleY(int intId2, double factor);
  void stretchX(int intId2, int delta);

  void stretchY(int intId2, int delta);

  void isometricalStretchX(int intId2, int delta)
  {
    double factor=1+delta/double(intPosX[intId2+1]-intPosX[intId2]);
    scaleX(intId2,factor);
    if(2*drawing->stretchIsometrical.x==intId2) 
    { assert(drawing->stretchIsometrical.y>=0); scaleY(drawing->stretchIsometrical.y,factor); }
  }
  void isometricalStretchY(int intId2, int delta)
  {
    double factor=1+delta/double(intPosY[intId2+1]-intPosY[intId2]);
    scaleY(intId2,factor);
    if(2*drawing->stretchIsometrical.y==intId2) 
    { assert(drawing->stretchIsometrical.x>=0); scaleX(drawing->stretchIsometrical.x,factor); }
  }


  void evaluateInterfaces(void);
  void fit(void);
  VisIntVector preferedSize(void) 
  { return VisIntVector(boundPosX[1]-boundPosX[0], boundPosY[1]-boundPosY[0]); }

  void setSubElement(VisContainerInterface2 *elem,VLConstCharPtr name);
  int containerExists(VLConstCharPtr name) { return drawing->getContainerId(name)!=-1; }
  virtual void draw(int derivpos, TreeContext primIdNodePtr=VLObjectPointer());
  virtual VisContainerInterface2 *expand(void);
};



#define DECL_TILEDRAWING(x) class T##x: public VisTileDrawing { void initDrawing(void); }; \
                            extern T##x x;
#define IMPL_TILEDRAWING(x) T##x x; \
                            void T##x::initDrawing(void)

#define DECL_VECTORDRAWING(x) class T##x: public VisVectorDrawing { void draw(void); }; \
                              extern T##x x;
#define IMPL_VECTORDRAWING(x) T##x x; \
                              void T##x::draw(void)


#endif
