#ifndef _PTGGEN_H
#define _PTGGEN_H

#include <stdio.h>
#include "eliproto.h"

/* Include-files specified in .ptg.phi-files */

#include "ptg.h"
/* Definition of Output-Macros */

#if !defined(PTG_OUTPUT_FILE)
#    define PTG_OUTPUT_FILE FILE *
#    define PTG_OUTPUT_DEFAULT 1
#else
#    define PTG_OUTPUT_DEFAULT 0
#endif
#if !defined(PTG_OUTPUT_STRING)
#    define PTG_OUTPUT_STRING(file,param) fputs(param, file)
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    fprintf(file,"%d",param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   fprintf(file,"%ld",param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  fprintf(file,"%d",(int) param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   fputc(param, file)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  fprintf(file,"%g",(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) fprintf(file,"%g",param)
#    endif

#else
     /* PTG_OUTPUT_STRING defined */
     extern void _PTGPrintInt ELI_ARG((PTG_OUTPUT_FILE,int));
     extern void _PTGPrintLong ELI_ARG((PTG_OUTPUT_FILE,long));
     extern void _PTGPrintChar ELI_ARG((PTG_OUTPUT_FILE,char));
     extern void _PTGPrintDouble ELI_ARG((PTG_OUTPUT_FILE,double));
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    _PTGPrintInt(file,param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   _PTGPrintLong(file,param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  _PTGPrintInt(file,(int)param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   _PTGPrintChar(file,param)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  _PTGPrintDouble(file,(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) _PTGPrintDouble(file,param)
#    endif
#endif

/* Define PTGNode Type */
#if defined(__cplusplus)
struct _SPTG0;
typedef void (* _PTGProc)(struct _SPTG0 *);
#else
typedef void (* _PTGProc)();
#endif

typedef struct _SPTG0
{
  _PTGProc _print;
} * _PPTG0;
typedef _PPTG0 PTGNode;                      /* the only exported type */

/* predefined static PTGNULL-Node */

extern struct _SPTG0 _PTGNULL;
#define PTGNULL (& _PTGNULL)
#define PTGNull() (& _PTGNULL)

/* output functions */

#if PTG_OUTPUT_DEFAULT
extern PTGNode PTGOut ELI_ARG((PTGNode root));
extern PTGNode PTGOutFile ELI_ARG((char *filename, PTGNode root));
extern PTGNode PTGOutFPtr ELI_ARG((FILE *output, PTGNode root));
#endif
extern PTGNode PTGProcess ELI_ARG((PTG_OUTPUT_FILE file, PTGNode root));

/* Memory Management functions. */

extern void PTGFree ELI_ARG((void));

/* functions for making PTG nodes */

PTGNode PTGAddNL ELI_ARG((PTGNode p1));
PTGNode PTGId ELI_ARG((int p1));
PTGNode PTGAsIs ELI_ARG((char * p1));
PTGNode PTGNumb ELI_ARG((int p1));
PTGNode PTGCString ELI_ARG((char * p1));
PTGNode PTGCChar ELI_ARG((int p1));
PTGNode PTGPString ELI_ARG((char * p1));
PTGNode PTGSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGCommaSeq ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGEol PTGAddNL
PTGNode PTGDotFrame ELI_ARG((PTGNode p1, char * p2, int p3, int p4));
PTGNode PTGDotEdge ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGDotNode ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGDatabasePropertiesFile ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, char * p6, char * p7));
PTGNode PTGBuildPropertiesFile ELI_ARG((char * p1, char * p2, char * p3, char * p4));
PTGNode PTGMailPropertiesFile ELI_ARG((char * p1, char * p2));
PTGNode PTGMessages ELI_ARG((void));
PTGNode PTGMessages_de ELI_ARG((void));
PTGNode PTGMasterTemplate ELI_ARG((PTGNode p1, char * p2, PTGNode p3, PTGNode p4, char * p5));
PTGNode PTGNavOnLeftBody ELI_ARG((void));
PTGNode PTGNavOnRightBody ELI_ARG((void));
PTGNode PTGNavOnTopBody ELI_ARG((void));
PTGNode PTGNavOnBottomBody ELI_ARG((void));
PTGNode PTGIndexJSP ELI_ARG((PTGNode p1, char * p2));
PTGNode PTGTilesDefXMLFile ELI_ARG((char * p1, char * p2, PTGNode p3));
PTGNode PTGTileDef ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGFacesConfigFile ELI_ARG((PTGNode p1));
PTGNode PTGFacesNavigationRulesContent ELI_ARG((PTGNode p1));
PTGNode PTGFacesNavigationCase ELI_ARG((char * p1));
PTGNode PTGFacesManagedBeansContent ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGFacesManagedBean ELI_ARG((char * p1, char * p2, PTGNode p3));
PTGNode PTGManagedParamProperty ELI_ARG((char * p1));
PTGNode PTGJSPFirstLine ELI_ARG((char * p1));
PTGNode PTGJSPHeader ELI_ARG((PTGNode p1));
PTGNode PTGJSPTile ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGJSPContentTile ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGServletDeclaration ELI_ARG((char * p1, char * p2));
PTGNode PTGJavaBean ELI_ARG((char * p1, char * p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8));
PTGNode PTGActionServlet ELI_ARG((char * p1, char * p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8));
PTGNode PTGActionMethod ELI_ARG((PTGNode p1, char * p2));
PTGNode PTGJavaPropertyMethods ELI_ARG((char * p1, char * p2, PTGNode p3));
PTGNode PTGJavaAttribute ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGPageAccess ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGPageAccessPropertiesFile ELI_ARG((PTGNode p1));
PTGNode PTGActionMethodBody ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGJSFTryCatchEnv ELI_ARG((PTGNode p1, char * p2, char * p3, PTGNode p4, PTGNode p5));
PTGNode PTGServletTryCatchEnv ELI_ARG((PTGNode p1, char * p2, char * p3, PTGNode p4));
PTGNode PTGTryCatch ELI_ARG((PTGNode p1));
PTGNode PTGServletForward ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGMailingActionCode ELI_ARG((char * p1, char * p2, char * p3, char * p4, PTGNode p5));
PTGNode PTGMailingContextParam ELI_ARG((char * p1, char * p2));
PTGNode PTGJSFLogoutActionCode ELI_ARG((void));
PTGNode PTGServletLogoutActionCode ELI_ARG((void));
PTGNode PTGLoginActionCode ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6));
PTGNode PTGJSFSetLoginSessionAttrs ELI_ARG((char * p1));
PTGNode PTGServletSetLoginSessionAttrs ELI_ARG((char * p1));
PTGNode PTGFrame ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGFrameRef ELI_ARG((PTGNode p1));
PTGNode PTGWebAction ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGPHPOperation ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGDataBaseOperation ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGLengthValidator ELI_ARG((char * p1, int p2, int p3));
PTGNode PTGEmailValidator ELI_ARG((void));
PTGNode PTGCreditCardValidator ELI_ARG((void));
PTGNode PTGRegExprValidator ELI_ARG((char * p1));
PTGNode PTGIntegerNumberConverter ELI_ARG((int p1, int p2));
PTGNode PTGDoubleNumberConverter ELI_ARG((int p1, int p2, int p3, int p4));
PTGNode PTGPatternConverter ELI_ARG((char * p1));
PTGNode PTGConverterById ELI_ARG((char * p1));
PTGNode PTGJSFForm ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4, char * p5));
PTGNode PTGOutputMessages ELI_ARG((PTGNode p1, PTGNode p2, char * p3));
PTGNode PTGInputHidden ELI_ARG((char * p1, char * p2));
PTGNode PTGFormularDateElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5));
PTGNode PTGFormularFileUploadElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, char * p6));
PTGNode PTGFormularTextElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7));
PTGNode PTGFormularTextAreaElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, int p7, PTGNode p8));
PTGNode PTGFormularPasswordElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7, char * p8));
PTGNode PTGFormularCheckboxElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5));
PTGNode PTGFormularRadioGroupElement ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, char * p7));
PTGNode PTGSelectOneMenu ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7));
PTGNode PTGSelectListbox ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, char * p7, int p8, PTGNode p9));
PTGNode PTGSelectItem ELI_ARG((char * p1, char * p2));
PTGNode PTGSelectItems ELI_ARG((char * p1, char * p2));
PTGNode PTGSelectItemsMethod ELI_ARG((char * p1, char * p2));
PTGNode PTGFormularDataSourceSelectOption ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGAttr ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGTextFragment ELI_ARG((char * p1));
PTGNode PTGStrongTextFragment ELI_ARG((PTGNode p1));
PTGNode PTGEmphTextFragment ELI_ARG((PTGNode p1));
PTGNode PTGImageFragment ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4, char * p5, char * p6, int p7));
PTGNode PTGParagraph ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGHeading ELI_ARG((PTGNode p1, int p2, PTGNode p3));
PTGNode PTGImageMap ELI_ARG((char * p1, char * p2, char * p3, PTGNode p4));
PTGNode PTGMapArea ELI_ARG((PTGNode p1, char * p2, char * p3, char * p4));
PTGNode PTGRef ELI_ARG((PTGNode p1, PTGNode p2, char * p3));
PTGNode PTGTable ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGTableRow ELI_ARG((PTGNode p1));
PTGNode PTGTableCell ELI_ARG((PTGNode p1));
PTGNode PTGOutputFragment ELI_ARG((char * p1, char * p2));
PTGNode PTGMetaInformation ELI_ARG((PTGNode p1));
PTGNode PTGMetaInformationElement ELI_ARG((char * p1, char * p2));
#define PTGStyleSheet PTGMetaInformation
PTGNode PTGStyleSheetBlock ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGStyleSheetPair ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGBatchSeparator ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGLFSep ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGNewLine ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGConcat ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGConcat3 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGConcat4 ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGActionLink ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGQueryString ELI_ARG((PTGNode p1));
PTGNode PTGListOutputActionRequestParamMapping ELI_ARG((char * p1, char * p2));
PTGNode PTGRequestParamMapping ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGRequestParamMappingBean ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGAmpersand ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGAmpersandPlus ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGApostroph ELI_ARG((PTGNode p1));
PTGNode PTGQuote ELI_ARG((char * p1));
PTGNode PTGDataSourceActualParam ELI_ARG((PTGNode p1));
PTGNode PTGWebNodeActualParam ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGThisPrefix ELI_ARG((PTGNode p1));
PTGNode PTGModifiedExpression ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGNavigationBar ELI_ARG((PTGNode p1, char * p2, char * p3));
PTGNode PTGNavigationBarElement ELI_ARG((char * p1, char * p2, char * p3, char * p4));
PTGNode PTGDataSourceModifyOperation ELI_ARG((PTGNode p1, PTGNode p2, char * p3));
PTGNode PTGDBInsertBody ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGDBDeleteBody ELI_ARG((char * p1, char * p2, PTGNode p3));
PTGNode PTGDBUpdateBody ELI_ARG((PTGNode p1, char * p2, PTGNode p3));
PTGNode PTGBindingValuesMap ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGUpdateBlobCode ELI_ARG((char * p1));
PTGNode PTGDataSourceTableSelect ELI_ARG((char * p1));
PTGNode PTGDataSourceProjectionSelect ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGDataSourceProjectionMethod ELI_ARG((char * p1, char * p2, PTGNode p3, PTGNode p4));
PTGNode PTGDataSourceProjectionMethodWithoutParams ELI_ARG((char * p1, char * p2));
PTGNode PTGLimitDataSourceMethod ELI_ARG((char * p1, char * p2, PTGNode p3, PTGNode p4, char * p5, char * p6));
PTGNode PTGLimitDataSourceMethodWithoutParams ELI_ARG((char * p1, char * p2, char * p3, char * p4));
PTGNode PTGDataSourceVariableExpression ELI_ARG((PTGNode p1));
PTGNode PTGConstantsClass ELI_ARG((char * p1));
PTGNode PTGServiceLocatorInterface ELI_ARG((char * p1, PTGNode p2, PTGNode p3));
PTGNode PTGServiceLocatorInterfaceDaoMethod ELI_ARG((char * p1));
PTGNode PTGServiceLocatorBean ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5));
PTGNode PTGServiceLocatorAttributeDaoDef ELI_ARG((char * p1, char * p2));
PTGNode PTGServiceLocatorInitDao ELI_ARG((char * p1, char * p2));
PTGNode PTGServiceLocatorMethodDao ELI_ARG((char * p1, char * p2));
PTGNode PTGServiceLocatorServletSupport ELI_ARG((char * p1));
PTGNode PTGDaoImport ELI_ARG((char * p1, char * p2));
PTGNode PTGModelImport ELI_ARG((char * p1, char * p2));
PTGNode PTGBeanModelImports ELI_ARG((char * p1));
PTGNode PTGDatabaseService ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGAbstractBean ELI_ARG((char * p1));
PTGNode PTGApplicationBean ELI_ARG((char * p1));
PTGNode PTGSessionBean ELI_ARG((char * p1));
PTGNode PTGDaoInterface ELI_ARG((char * p1, char * p2, char * p3, char * p4));
PTGNode PTGDaoHibernateImpl ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5));
PTGNode PTGSpringBeansFile ELI_ARG((PTGNode p1));
PTGNode PTGDatabaseSpringBeans ELI_ARG((PTGNode p1, char * p2));
PTGNode PTGSpringDaoBean ELI_ARG((char * p1, char * p2));
PTGNode PTGHibernateMapping ELI_ARG((char * p1, char * p2, PTGNode p3, char * p4));
PTGNode PTGHibernateNativeIdMapping ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGHibernateSequenceIdMapping ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGHibernateColumnMapping ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGHibernateBlobColumnMapping ELI_ARG((char * p1, char * p2));
PTGNode PTGHibernateHasOneMapping ELI_ARG((char * p1, char * p2));
PTGNode PTGHibernateHasManyMapping ELI_ARG((char * p1, char * p2, char * p3, char * p4, char * p5, char * p6));
PTGNode PTGWebXML ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGBeanPropertyInitCode ELI_ARG((char * p1, char * p2, char * p3));
PTGNode PTGFormularDataSourceSelectStmt ELI_ARG((char * p1));
PTGNode PTGListOutputElementInitCode ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGDataTableColumn ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGDataTableColumnText ELI_ARG((char * p1));
PTGNode PTGDataTableColumnDate ELI_ARG((char * p1));
PTGNode PTGDataTableColumnDatetime ELI_ARG((char * p1));
PTGNode PTGDataTableColumnImage ELI_ARG((char * p1));
PTGNode PTGDataTableColumnNumeration ELI_ARG((void));
PTGNode PTGDataTable ELI_ARG((char * p1, char * p2, int p3, PTGNode p4, char * p5, PTGNode p6, char * p7, char * p8, char * p9));
PTGNode PTGPanelGrid ELI_ARG((int p1, PTGNode p2));
PTGNode PTGVisibleTag ELI_ARG((char * p1, PTGNode p2));

/* prototypes for used function call insertions */

void PtgOutId ELI_ARG((PTG_OUTPUT_FILE, int));
void CPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void CPtgOutchar ELI_ARG((PTG_OUTPUT_FILE, int));
void PPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void IndentNewLine ELI_ARG((PTG_OUTPUT_FILE));
void IndentIncr ELI_ARG((PTG_OUTPUT_FILE));
void IndentDecr ELI_ARG((PTG_OUTPUT_FILE));

#ifdef MONITOR
/* Monitoring support for structured values */
#define DAPTO_RESULTPTGNode(n) DAPTO_RESULT_PTR(n)
#define DAPTO_ARGPTGNode(n) DAPTO_ARG_PTR(n, PTGNode)
#endif

#endif

