# contains common procedures
# used by code generation as well as check functions
# and maybe somewhere else...

namespace eval common {}

set common::converterTypeMapping(IntegerNumberConverter) "DBLong" 
set common::converterTypeMapping(IntegerConverter) "DBInteger"
set common::converterTypeMapping(DoubleNumberConverter) "DBDouble"
set common::converterTypeMapping(DoubleConverter) "DBDouble"
set common::converterTypeMapping(LongConverter) "DBLong" 
set common::converterTypeMapping(BigIntegerConverter) "DBBigInt"
set common::converterTypeMapping(FloatConverter) "DBFloat"
set common::converterTypeMapping(BooleanConverter) "DBBoolean"
set common::converterTypeMapping(DateTimeConverter) "DBDate DBTimeStamp"
set common::converterTypeMapping(PatternConverter) "DBDouble"

set common::allTypes "DBString DBText DBInteger DBLong DBFloat DBDouble DBBigInt DBBoolean DBDate DBTimeStamp DBBlob"
set common::javaKeywords "true false null abstract continue for new switch assert default goto \
package synchronized boolean do if private this break double implements protected throw \
byte else import public throws case enum instanceof return transient catch extends int short \
try char final interface static void class finally long strictfp volatile const float native super while"

proc common::isJavaKeyword {identifier} {
  variable javaKeywords
  set index [lsearch -exact $javaKeywords $identifier]
  if [equal $index -1] {
    return false
  }
  return true
}

proc common::getTypeClassName {column} {
   set columnType [c::getList "$column.type.CHILDREN"]
   if [equal [llength $columnType] 0] {
     return "**TypeNotSet***"
   }
   return [c::getClassName $columnType]
}

proc common::getAllTypes {} {
  variable allTypes
  return $allTypes
}

proc common::getPossibleTypeClassNamesOfConstantExpression {constExpr} {
  variable allTypes
  if [equal [llength $constExpr] 0] {
    return $allTypes
  }
  
  set value [c::get "$constExpr.value.VALUE"]
  
  if [string equal $value "null"] { return $allTypes }
  # Attention! type-safety cannont be garanteed for a custom object initiation...
  if [string equal -length 4 $value "new "] { return $allTypes } 
  if [string is integer $value] { return "DBInteger" }
  if [string is double $value]  { return "DBFloat DBDouble" }
  if [string is boolean $value] { return "DBBoolean" }
  
  return "DBString DBText"
}

proc common::getTypeClassNamesByFormElement {formElement} {
  set formElemClassName [c::getClassName $formElement]
  if [equal $formElemClassName "FormularFileUploadElement"] {
    return "DBBlob"
  } elseif [equal $formElemClassName "FormularDateElement"] {
    return "DBDate DBTimeStamp"
  } elseif [equal $formElemClassName "FormularTextAreaElement"] {
    return "DBText"
  } elseif [equal $formElemClassName "FormularCheckboxElement"] {
    return "DBBoolean"
  }
  
  # Typ der restlichen FormularElement anhand des Converters bestimmen
  set converter [c::getList $formElement.converter.CHILDREN]
  return [common::getTypeClassNamesByConverter $converter]
}

proc common::getTypeClassNamesByConverter {converter} {
  if [equal [llength $converter] 0] { return "DBString" }
  
  set converterClassName [c::getClassName $converter]
  variable converterTypeMapping
  return $converterTypeMapping($converterClassName)
}
