
namespace eval MDIFrame {
    set reuseViews 1
}


########################################################################## commands

proc ::MDIFrame::getReuseViews {} {
    return $::MDIFrame::reuseViews
}

proc ::MDIFrame::showAboutDialog {} {
    lappend message "$::cfg::appName"
    if ![equal $::cfg::appWebsite unknown] {
        lappend message $::cfg::appWebsite
    }
    lappend message ""
    lappend message [i18n::text "This environment was specified by"]
    lappend message "$cfg::authorName <$cfg::authorEmail>"
    lappend message ""
    lappend message [i18n::text "It was generated by DEViL."]
    lappend message "(http://ag-kastens.upb.de/forschung/devil)"
    lappend message ""
    lappend message "Editor Version: $::cfg::editorVersion"
    lappend message "Grammar Version: $::cfg::grammarVersion"
    lappend message "DEViL Version: $::main::devilVersion"
    lappend message "Tcl/Tk Version: $::tcl_patchLevel"

    CDlg::Information [join $message "\n"] [i18n::text "About this application"]
}


proc MDIFrame::newFile {} {
    set file [file::new]
    file::openMainView $file
}

proc MDIFrame::openFile {} {
    set file [file::interactiveOpen]
}

proc ::MDIFrame::exit {} {
    variable master
    ::MDI::closemaster ::${master}::
}


proc ::MDIFrame::Arrange_Icons { } {
    variable master

    ::MDI::arrall ::${master}::
}

proc ::MDIFrame::Minimize_all { } {
    variable master

    ::MDI::minall ::${master}::
}

proc ::MDIFrame::Minimize { client } {
    variable master
    
    ::MDI::minclient $client ::${master}::
}

proc ::MDIFrame::Restore_all { } {
    variable master

    ::MDI::resall ::${master}::
}

proc ::MDIFrame::NextWindow { } {
    variable master

    ::MDI::next ::${master}::
}

proc ::MDIFrame::PrevWindow { } {
    variable master

    ::MDI::prev ::${master}::
}

proc ::MDIFrame::clientcommand {cmdname args} {
    variable master
    set client [lindex [GetWindowList] 0]
    if { "$client" == "" } {
        ::CDlg::Information [i18n::text "No current window"]
    return
    }
    set abstractView [getAbstractView $client]
    eval [list $abstractView wincmd $cmdname] $args
}


proc ::MDIFrame::getCurrentFile {} {
    return [clientcommand getFile]
}

########################################################################## ueberarbeitet


proc ::MDIFrame::New_Window { title abstractView args} {

    variable master
    variable nextslave

    set clientname "slave[incr nextslave]"
    set newslave [eval {::MDI::slave $clientname $master \
        -title [MakeTitleUnique $title] -userargs [list $abstractView ""] \
        -closecmd [list ::MDIFrame::on_close $clientname]} $args]
    UpdateWindowMenu
    return $clientname
}


proc ::MDIFrame::getAbstractView { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct

    set result [lindex $cstruct(userargs) 0]
    #if [empty $result] {
    #    error "Window ist not a view."
    #}
    return $result
}


proc ::MDIFrame::on_close { client } {
    if ![empty [getAbstractView $client]] {
        [getAbstractView $client] destroy
    }
    after idle {
        ::MDIFrame::UpdateWindowMenu
    }
    return 1
}


proc ::MDIFrame::Setup {} {
    
    # First, make a Menu
    
    set mid [Menu::Create .menu]
    variable mainMenu $mid
    
    Menu::Menu $mid [i18n::text "&File"]
    set File [i18n::text "File"]
    Menu::Command $mid $File [i18n::text "&New"] {::MDIFrame::newFile}
    Menu::Command $mid $File [i18n::text "&Open"] {::MDIFrame::openFile}
    Menu::Separator $mid $File
    Menu::Command $mid $File [i18n::text "&Save"] {::MDIFrame::clientcommand wincmd_save}
    Menu::Command $mid $File [i18n::text "Save &as"] {::MDIFrame::clientcommand wincmd_saveAs}
    Menu::Command $mid $File [i18n::text "E&xport Representation"] { ::MDIFrame::clientcommand wincmd_exportRepresentation}
    Menu::Command $mid $File [i18n::text "Information"] { ::MDIFrame::clientcommand wincmd_fileInfo}
    Menu::Separator $mid $File
    Menu::Command $mid $File [i18n::text "&Main view"] {::MDIFrame::clientcommand wincmd_showMainView}
    Menu::Command $mid $File [i18n::text "&Tree view"] {::MDIFrame::clientcommand wincmd_showAbstree}
    Menu::Command $mid $File [i18n::text "E&rror view"] {::MDIFrame::clientcommand wincmd_showErrWindow}
    Menu::Separator $mid $File
    Menu::Command $mid $File [i18n::text "&Close file"] {::MDIFrame::clientcommand wincmd_closeFile}
    Menu::Command $mid $File [i18n::text "&Exit"] {::MDIFrame::exit}
    
    Menu::Menu $mid [i18n::text "&Edit"]
    set Edit [i18n::text "Edit"]
    Menu::Command $mid $Edit [i18n::text "&Undo"] {::MDIFrame::clientcommand wincmd_undo}
    Menu::Command $mid $Edit [i18n::text "Cu&t"] {::MDIFrame::clientcommand wincmd_cut}
    Menu::Command $mid $Edit [i18n::text "&Copy"] {::MDIFrame::clientcommand wincmd_copy}
    Menu::Command $mid $Edit [i18n::text "&Paste"] {::MDIFrame::clientcommand wincmd_paste}
    Menu::Command $mid $Edit [i18n::text "&Delete"] {::MDIFrame::clientcommand wincmd_delete}
    Menu::Separator $mid $Edit
    Menu::Command $mid $Edit [i18n::text "Find value..."] {::MDIFrame::clientcommand wincmd_findRegexp}
    Menu::Command $mid $Edit [i18n::text "Find node by internal ID..."] {file::findNodeByInternalId}
    Menu::Separator $mid $Edit
    Menu::Command $mid $Edit [i18n::text "&Edit properties"] {::MDIFrame::clientcommand wincmd_editProperties}
    Menu::Command $mid $Edit [i18n::text "Select next"]    {::MDIFrame::clientcommand wincmd_selectNext}
    Menu::Command $mid $Edit [i18n::text "Select prev"]    {::MDIFrame::clientcommand wincmd_selectPrev}
    Menu::Command $mid $Edit [i18n::text "Select parent"] {::MDIFrame::clientcommand wincmd_selectParent}
    Menu::Command $mid $Edit [i18n::text "Select first child"] {::MDIFrame::clientcommand wincmd_selectFirstChild}
    Menu::Command $mid $Edit [i18n::text "Select next child"]  {::MDIFrame::clientcommand wincmd_selectNextChild}
    Menu::Command $mid $Edit [i18n::text "Select all"]  {::MDIFrame::clientcommand wincmd_selectAll}
    
    Menu::Menu $mid [i18n::text "&View"]
    set View [i18n::text "View"]   
    Menu::Command $mid $View [i18n::text "Zoom &in"] {::MDIFrame::clientcommand wincmd_zoomIn}
    Menu::Command $mid $View [i18n::text "Zoom &out"] {::MDIFrame::clientcommand wincmd_zoomOut}
    Menu::Command $mid $View [i18n::text "E&xport"] {::MDIFrame::clientcommand wincmd_exportViewInteractive}

    Menu::Menu $mid [i18n::text "&Process"]
    set Process [i18n::text "Process"]
    foreach processor [file::getProcessorNames] {
        Menu::Command $mid $Process [file::getProcessorLabel $processor] [list ::MDIFrame::clientcommand wincmd_startProcessor $processor]
    }

    Menu::Menu $mid [i18n::text "&Window"]
    set Window [i18n::text "Window"]    
    Menu::Check $mid $Window [i18n::text "Reuse views"] {::MDIFrame::reuseViews}
    Menu::Separator $mid $Window
    Menu::Command $mid $Window [i18n::text "&Minimize all"] {::MDIFrame::Minimize_all}
    Menu::Command $mid $Window [i18n::text "&Arrange Icons"] {::MDIFrame::Arrange_Icons}
    Menu::Command $mid $Window [i18n::text "&Restore all"] {::MDIFrame::Restore_all}
    Menu::Command $mid $Window [i18n::text "&Next"] {::MDIFrame::NextWindow}
    Menu::Command $mid $Window [i18n::text "&Previous"] {::MDIFrame::PrevWindow}
    Menu::Command $mid $Window [i18n::text "Open console"] {::console::open}
    Menu::Separator $mid $Window

    # NOTE: If the number of items is changed in "Window" menu,
    # the constant in UpdateWindowMenu has to be changed, too!!!

    Menu::Menu $mid [i18n::text "&Help"]
    set Help [i18n::text "Help"]
    Menu::Command $mid $Help [i18n::text "&About"] {::MDIFrame::showAboutDialog}


    # Shortcuts
    Menu::Bind $mid . <Control-n> $File [i18n::text "New"]
    Menu::Bind $mid . <Control-o> $File [i18n::text "Open"]
    Menu::Bind $mid . <Control-s> $File [i18n::text "Save"]

    Menu::Bind $mid . <Control-p> $View [i18n::text "Export"]
    Menu::Bind $mid . <Control-plus> $View [i18n::text "Zoom in"]
    Menu::Bind $mid . <Control-minus> $View [i18n::text "Zoom out"]

    Menu::Bind $mid . <Control-z> $Edit [i18n::text "Undo"]
    Menu::Bind $mid . <Control-c> $Edit [i18n::text "Copy"]
    Menu::Bind $mid . <Control-x> $Edit [i18n::text "Cut"]
    Menu::Bind $mid . <Control-v> $Edit [i18n::text "Paste"]
    Menu::Bind $mid . <Delete>    $Edit [i18n::text "Delete"]

    Menu::Bind $mid . <Control-f> $Edit [i18n::text "Find value..."]
    Menu::Bind $mid . <Control-i> $Edit [i18n::text "Find node by internal ID..."]

    Menu::Bind $mid . <space>     $Edit [i18n::text "Edit properties"]
    Menu::Bind $mid . <Down>      $Edit [i18n::text "Select next"]
    Menu::Bind $mid . <Up>        $Edit [i18n::text "Select prev"]
    Menu::Bind $mid . <Left>      $Edit [i18n::text "Select parent"]
    Menu::Bind $mid . <Right>     $Edit [i18n::text "Select first child"]
    Menu::Bind $mid . <n>         $Edit [i18n::text "Select next child"]
    Menu::Bind $mid . <Control-a> $Edit [i18n::text "Select all"]
    # bei <Tab> gibt es leider einen Konflikt
    # Eigentlich ist aber <space> das Problem!

    # Make a Status Bar

    frame .mainmessagebar -relief sunken -borderwidth 1
    pack .mainmessagebar -side bottom -fill x

    frame .mainmessagebar.progress -width 30
    pack .mainmessagebar.progress -side left -fill y
    label .mainmessagebar.progress.text -text " " -foreground black -relief flat
    pack .mainmessagebar.progress.text -fill both

    frame .mainmessagebar.indicator -width 30
    pack .mainmessagebar.indicator -side right -fill y
    label .mainmessagebar.indicator.text -text " " -foreground black -width 5
    pack .mainmessagebar.indicator.text -fill both
    bind .mainmessagebar.indicator.text <Button-1> {::MDIFrame::SetError 0}

    label .mainmessagebar.text -text " " -foreground black
    pack .mainmessagebar.text -side left -fill both



    frame .mdi 
    pack .mdi -side top -fill both -expand yes

    # Initialize MDI Module

    variable master [::MDI::master mdi .mdi -menupath .menu -onMasterClose ::file::tryCloseAll]
    variable nextslave 0

    # Set width / height / position of Frame

    wm geometry . 909x763+1+1

    frame .toolbar -relief raised -borderwidth 1

    button .toolbar.newfile -image img::button_newfile \
        -command {::MDIFrame::newFile}
    pack .toolbar.newfile -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.newfile [i18n::text "Creates new file"]

    button .toolbar.open -image img::button_open \
        -command {::MDIFrame::openFile}
    pack .toolbar.open -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.open [i18n::text "Loads file from disc"]

    button .toolbar.save -image img::button_save \
        -command {::MDIFrame::clientcommand wincmd_save}
    pack .toolbar.save -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.save [i18n::text "Saves current file to disc"]

    button .toolbar.print -image img::button_print \
        -command {::MDIFrame::clientcommand wincmd_exportViewInteractive}
    pack .toolbar.print -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.print \
        [i18n::text "Saves content of current view as graphics file"]

    button .toolbar.cut -image img::button_cut \
        -command {::MDIFrame::clientcommand wincmd_cut}
    pack .toolbar.cut -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.cut \
        [i18n::text "Cuts the selected item to the clipboard"]

    button .toolbar.copy -image img::button_copy \
        -command {::MDIFrame::clientcommand wincmd_copy}
    pack .toolbar.copy -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.copy \
        [i18n::text "Copies the selected item to the clipboard"]

    button .toolbar.paste -image img::button_paste \
        -command {::MDIFrame::clientcommand wincmd_paste}
    pack .toolbar.paste -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.paste \
        [i18n::text "Pastes the content of the clipboard"]

    button .toolbar.showMainView -image img::button_mainView \
        -command {::MDIFrame::clientcommand wincmd_showMainView}
    pack .toolbar.showMainView -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.showMainView \
        [i18n::text "Activates the main view of current file"]

    button .toolbar.showAbstree -image img::button_tree \
        -command {::MDIFrame::clientcommand wincmd_showAbstree}
    pack .toolbar.showAbstree -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.showAbstree \
        [i18n::text "Activates the `abstract structure tree' window of current file"]

    button .toolbar.showErrWindow -image img::button_errors \
        -command {::MDIFrame::clientcommand wincmd_showErrWindow}
    pack .toolbar.showErrWindow -side left -padx 0 -pady 0
    balloon::defineMessage .toolbar.showErrWindow \
        [i18n::text "Activates error window of current file"]
  
    pack .toolbar -after .menu -side top -fill x
    SetMainTitle
    SetStatus ""
    SetIndicator ""
    SetProgress ""
    SetError 0
}


####################################################################### alte Funktionen


proc ::MDIFrame::SetMainTitle {} {
    tk appname $::cfg::appName
    wm title . $::cfg::appName
}

proc ::MDIFrame::SetStatus { text } {
    .mainmessagebar.text configure -text $text
}

proc ::MDIFrame::SetProgress {text } {
    if [empty $text] {
        set text [i18n::text "Ready"]
    }
    .mainmessagebar.progress.text configure -text $text
}


proc ::MDIFrame::UpdateBlink {on} {
    variable errorBlink
    if { ! $errorBlink } {
        return
    }
    if $on {
        SetIndicator "\#\#\#\#\#"
        after 300 ::MDIFrame::UpdateBlink 0
    } else {
        SetIndicator ""
        after 300 ::MDIFrame::UpdateBlink 1
    }
}

proc ::MDIFrame::SetError {err } {
    SetIndicator ""
    variable errorBlink
    set errorBlink $err
    UpdateBlink 1
}

proc ::MDIFrame::SetIndicator {text } {
    .mainmessagebar.indicator.text configure -text $text
}


proc ::MDIFrame::Totalsize {} {
    variable master

    upvar #0 ::${master}:: mstruct 
    return [list [winfo width $mstruct(path)] [winfo height $mstruct(path)]]
}



proc ::MDIFrame::GetWindowList { } {
    # Reihenfolge entspricht der Reihenfolge der Fenster.
    variable master
    
    return [::MDI::clients ::${master}::]
}

proc ::MDIFrame::GetSortedWindowList { } {
    variable master
    
    set wlist [::MDI::clients ::${master}::]
    set slist ""
    foreach client $wlist {
    lappend slist [list $client [GetTitle $client]]
    }
    set slist [lsort -index 1 -dictionary $slist]
    set wlist ""
    foreach clienttitle $slist {
    lappend wlist [lindex $clienttitle 0]
    }
    return $wlist
}


proc ::MDIFrame::GetTitle { client } {
    variable master

    ::MDI::gettitle ::${master}:: $client
}

proc ::MDIFrame::SetTitle { client title } {
    variable master
    upvar #0 ::${master}::${client} cstruct
    
    ::MDI::settitle ::${master}:: $client $title
    UpdateWindowMenu
}

proc ::MDIFrame::GetCoords { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct

    return [list $cstruct(rootx) $cstruct(rooty) [expr $cstruct(rootx) + $cstruct(width)] [expr $cstruct(rooty) + $cstruct(height)]]
}

proc ::MDIFrame::SetClientpath { client path } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    set cstruct(clientpath) $path
}

proc ::MDIFrame::SetSelectcommand { client command } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    if {$cstruct(userargs) != ""} {
    set cstruct(userargs) [lreplace $cstruct(userargs) 1 1 $command]
    }
}

proc ::MDIFrame::GetSelectcommand { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    return [lindex $cstruct(userargs) 1]
}


proc ::MDIFrame::SetDropcommand { client command } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    set cstruct(dropcommand) $command
}

proc ::MDIFrame::GetDropcommand { client } {
    variable master
    upvar #0 ::${master}::${client} cstruct 

    if {[info exists cstruct(dropcommand)]} {
    return $cstruct(dropcommand)
    } else {
    return ""
    }
}

proc ::MDIFrame::FindDropTarget { X Y } {
    variable master

    set client [GetClient [winfo containing $X $Y]]
    set cmd [GetDropcommand $client]
    if {$cmd != "" && [IsInside $client $X $Y]} {
    return $cmd
    }
   
}

proc ::MDIFrame::IsInside { client X Y } {
    variable master
    upvar #0 ::${master}:: mstruct ::${master}::${client} cstruct 

    set clx [winfo rootx $cstruct(clientpath)]
    set cly [winfo rooty $cstruct(clientpath)]
    set clw [winfo width $cstruct(clientpath)]
    set clh [winfo height $cstruct(clientpath)]

    return [expr $X >= $clx && $X <= ($clx + $clw) \
        &&   $Y >= $cly && $Y <= ($cly + $clh)]
}
    
proc ::MDIFrame::GetWindowPath { client } {
    variable master
    upvar #0 ::${master}:: mstruct ::${master}::${client} cstruct 

    return $cstruct(path)
}


proc ::MDIFrame::GetUserpath { client } {
    variable master

    ::MDI::getuserpath ::${master}:: $client
}


proc ::MDIFrame::CloseWindow { client } {
    # scheint nicht zu funktionieren!! -> CloseClient
    variable master

    ::MDI::closeclient ::${master}:: $client
}

proc ::MDIFrame::MakeVisible { client } {
    variable master

    if {[::MDI::validclient ::${master}:: $client]} {
    ::MDI::mclient $client ::${master}:: norm
    ::MDI::setcurrent $client ::${master}::
    }
}

proc ::MDIFrame::UpdateWindowMenu { } {
    variable master
    variable mainMenu
    
    set Window [i18n::text "Window"]
    ::Menu::Delete $mainMenu $Window 8

    ::Menu::Separator $mainMenu $Window
    foreach file [file::getAllFiles] {
        ::Menu::Command $mainMenu $Window [file::getFilename $file] [list file::openMainView $file]
    }

    ::Menu::Separator $mainMenu $Window
    foreach client [GetSortedWindowList] {
    set title [GetTitle $client]
    ::Menu::Command $mainMenu $Window $title "::MDIFrame::MakeVisible $client"
    }
}

proc ::MDIFrame::FindWindow { title } {
    
    foreach client [GetWindowList] {
    set window_title [GetTitle $client]
    if {"$title" == "$window_title"} {
        return $client
    }
    }

    return ""
}

proc ::MDIFrame::GetClient { win } {
    if {[regexp {\.([^.]*)\.([^.]*)\.([^.]*)\..*} $win match mdi1 mdi2 client]} {
    if {$mdi1 == "mdi" && $mdi2 == "mdi"} {
        return $client
    } else {
        return ""
    }
    } else {
    return ""
    }
}

proc ::MDIFrame::FindClient { win } {
    
    foreach client [GetWindowList] {
    set userpath [GetUserpath $client]
    if {"$userpath" == "$win"} {
        return $client
    }
    }

    return ""
}

proc ::MDIFrame::CloseClient { client { force 0}} {
    variable master

    if {$force} {
    ::MDI::destroyclient $client ::${master}::
    } else {
    ::MDI::closeclient $client ::${master}::
    }
}

proc ::MDIFrame::MakeTitleUnique { title } {
    if {[FindWindow $title] == ""} {
    return $title
    }
    
    set no 1
    set new_title "$title:$no"
    while {[FindWindow $new_title] != ""} {
    incr no
    set new_title "$title:$no"
    }

    return $new_title
}

proc ::MDIFrame::GetWidth {} {
    winfo width .mdi.mdi
}

proc ::MDIFrame::GetHeight {} {
    winfo height .mdi.mdi
}

