

# Modul: dc
#
# Bibliothek wiederverwendbarer Dilogkomponenten.
#


# --- dc --------------------------------------------------------------------------------


namespace eval dc {}

proc dc::create {self} {
    oop::new $self
    oop::addClass $self dc
}


proc dc::renew {self argList} {
    oop::data $self
    handleParams $argList arg [list -setCmd "" -getCmd "" -onInput ""]
    set data(setCmd) $arg(setCmd)
    set data(getCmd) $arg(getCmd)
    set data(onInput) $arg(onInput)
    set data(shouldExpand) 0
}


proc dc::readValue {self} {
    oop::data $self
    if ![empty $data(getCmd)] {
        oop::call setValue $self [eval $data(getCmd)]
    }
}


proc dc::writeValue {self} {
    oop::data $self
    if ![empty $data(setCmd)] {
        eval $data(setCmd) [list [oop::call getValue $self]]
    }
}


proc dc::changed {self} {
    oop::data $self
    if ![empty $data(onInput)] {
        evalPercentSubst $data(onInput) [list ::dc::percentSubst $self]
    }
}


proc dc::shouldExpand {self} {
    oop::data $self
    return $data(shouldExpand)
}


proc dc::percentSubst {self percent} {
    switch $percent {
        %s {
            return $self
        }
        %v {
            return [oop::call getValue $self]
        }
        default {
            return ""
        }
    }
}


proc dc::bindDestroy {self} {
    bind $self <Destroy> [list dc::delete $self]
}


proc dc::delete {self} {
    # Don't call directly! Call "destroy $self" instead!
    oop::delete $self
}


# --- dcWithLabel -----------------------------------------------------------------------


namespace eval dcWithLabel {}


proc dcWithLabel::create {name type args} {
    # does not inherit from dc!!!
    oop::new $name
    oop::addClass $name dcWithLabel
    frame $name
    dc::bindDestroy $name

    label $name.label -anchor w -pady 5
    pack $name.label -side left -anchor n
    ${type}::create $name.dc
    pack $name.dc -padx 10 -pady 5 -side left -fill both -expand 1
    eval renew $name $args
}


proc dcWithLabel::getValue {name} {
    return [oop::call getValue $name.dc]
}


proc dcWithLabel::setValue {name value} {
    oop::call setValue $name.dc $value
}


proc dcWithLabel::writeValue {name} {
    oop::call writeValue $name.dc
}


proc dcWithLabel::readValue {name} {
    oop::call readValue $name.dc
}


proc dcWithLabel::shouldExpand {name} {
    return [oop::call shouldExpand $name.dc]
}


proc dcWithLabel::renew {name args} {
    set remainder [partialHandleArgs [list  -label unnamed -labelWidth 0]]
    $name.label configure -text $arg(label) -width $arg(labelWidth)
    eval oop::call renew $name.dc $remainder
}


# --- dcCheckbox --------------------------------------------------------------------------


namespace eval dcCheckbox {}


proc dcCheckbox::create {name args} {
    dc::create $name
    oop::addClass $name dcCheckbox

    checkbutton $name \
        -command [list dc::changed $name] \
        -variable ::oop::${name}(value)
    dc::bindDestroy $name
    eval renew [list $name] $args
}


proc dcCheckbox::getValue {name} {
    oop::data $name
    return $data(value)
}


proc dcCheckbox::setValue {name value} {
    oop::data $name
    set data(value) $value
}


proc dcCheckbox::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -anchor w -enabled 1 -onvalue 1 -offvalue 0]]
    dc::renew $name $remainder
    $name configure -anchor $arg(anchor) -state [lindex "disabled normal" $arg(enabled)]  -onvalue $arg(onvalue) -offvalue $arg(offvalue)
}


# --- dcListbox ------------------------------------------------------------------------

namespace eval dcListbox {}

proc dcListbox::create {name args} {
    dc::create $name
    oop::addClass $name dcListbox
    oop::data $name
    
    set data(elements) ""

    frame $name
    dc::bindDestroy $name

    listbox $name.list -listvariable ::oop::${name}(elements) -exportselection 0 -selectmode browse
    CDlg::mkScrollableY $name list
    bind $name.list <<ListboxSelect>> [list dc::changed $name]
    eval renew [list $name] $args
}


proc dcListbox::getValue {name} {
    oop::data $name
    set index [$name.list curselection]
    if [llength $index]==1 {
        set value [$name.list get $index]
    } else {
        set value ""
    }
    foreach {label id} $data(values) {
        if [equal $value $label] {
            return $id
        }
    }
    return ""
}


proc dcListbox::setValue {name value} {
    oop::data $name
    $name.list selection clear 0 end
    set index 0
    foreach {label id} $data(values) {
        if [equal $id $value] {
            $name.list selection set $index
        }
        incr index
    }
}


proc dcListbox::renew {name args} {
    oop::data $name

    set remainder [partialHandleArgs [list -values "" -width 20 -height 5 -enabled 1]]
    dc::renew $name $remainder
    set data(shouldExpand) 1
    $name.list configure -width $arg(width) -height $arg(height)
    set data(values) $arg(values)

    set elements ""
    foreach {label id} $arg(values) {
        lappend elements $label
    }
    set data(elements) $elements
}


# --- dcEntry ---------------------------------------------------------------------------

namespace eval dcEntry {}

proc dcEntry::create {name args} {
    dc::create $name
    oop::addClass $name dcEntry

    entry $name
    dc::bindDestroy $name
    bind $name <FocusOut> [list ::dc::changed $name]

    eval renew [list $name] $args
}


proc dcEntry::getValue {name} {
    return [$name get]
}


proc dcEntry::setValue {name value} {
    $name delete 0 end
    $name insert 0 $value
}


proc dcEntry::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -width 30 -enabled 1 -show ""]]
    dc::renew $name $remainder
    
    if {$arg(show)  != ""} {
        $name configure -width $arg(width) -state [lindex "disabled normal" $arg(enabled)] -show $arg(show)
    } else {
        $name configure -width $arg(width) -state [lindex "disabled normal" $arg(enabled)]
    }
    
}


# --- dcText ---------------------------------------------------------------------------

namespace eval dcText {}


proc dcText::create {name args} {
    dc::create $name
    oop::addClass $name dcText
    oop::data $name

    frame $name
    dc::bindDestroy $name

    text $name.text
    CDlg::mkScrollableXY $name text
    bind $name.text <FocusOut> [list ::dc::changed $name]

    bindtags $name.text [linsert [bindtags $name.text] 2 BreakReturn]
    bind BreakReturn <Return> "break"
    bind $name.text <Tab> "dcText::focusNext $name; break"

    eval renew [list $name] $args
}


proc dcText::focusNext {name} {
    focus [tk_focusNext $name.text]
}


proc dcText::getValue {name} {
    return [string range [$name.text get 1.0 end] 0 end-1]
}


proc dcText::setValue {name value} {
    $name.text delete 1.0 end
    $name.text insert 1.0 $value
}


proc dcText::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -width 30 -height 5 -wrap none -enabled 1]]
    dc::renew $name $remainder
    set data(shouldExpand) 1
    $name.text configure -width $arg(width) -height $arg(height) -wrap $arg(wrap) -state [lindex "disabled normal" $arg(enabled)]
}


# --- dcScale ---------------------------------------------------------------------------

namespace eval dcScale {}


proc dcScale::create {name args} {
    dc::create $name
    oop::addClass $name dcScale
    oop::data $name

    scale $name -orient horizontal -relief raised
    dc::bindDestroy $name

    eval renew [list $name] $args
}


proc dcScale::getValue {name} {
    return [$name get]
}


proc dcScale::setValue {name value} {
    if [empty $value] {
        set value 0
    }
    $name set $value
}


proc dcScale::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -length 50 -from 0 -to 10 -enabled 1]]
    dc::renew $name $remainder
    $name configure -length $arg(length) -from $arg(from) -to $arg(to) -state [lindex "disabled normal" $arg(enabled)]
}



# --- dcRadio ---------------------------------------------------------------------------

namespace eval dcRadio {}


proc dcRadio::create {name args} {
    dc::create $name
    oop::addClass $name dcRadio
    oop::data $name

    frame $name
    dc::bindDestroy $name

    eval renew [list $name] $args
}


proc dcRadio::getValue {name} {
    oop::data $name
    return $data(value)
}


proc dcRadio::setValue {name value} {
    oop::data $name
    set data(value) $value
}


proc dcRadio::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -values "" -enabled 1]]
    set data(values) $arg(values)
    dc::renew $name $remainder
    foreach old [pack slaves $name] {
        destroy $old
    }
    set index 0
    foreach {label id} $data(values) {
        radiobutton $name.$index -anchor w -text $label -value $id \
            -variable ::oop::${name}(value) -command [list dc::changed $name] \
            -state [lindex "disabled normal" $arg(enabled)]
        pack $name.$index -anchor w
        incr index
    }
}


# --- dcOption ---------------------------------------------------------------------------

namespace eval dcOption {}


proc dcOption::create {name args} {
    dc::create $name
    oop::addClass $name dcOption
    oop::data $name

    menubutton $name -indicatoron 1 -menu $name.menu -relief raised -direction flush -takefocus 1 -highlightthickness 1
    dc::bindDestroy $name

    menu $name.menu -tearoff 0
    eval renew [list $name] $args
    setValue $name ""
}


proc dcOption::getValue {name} {
    oop::data $name
    return $data(value)
}


proc dcOption::setValue {name value} {
    oop::data $name
    set data(value) $value
    $name configure -text "(no value)"
    foreach {label id} $data(values) {
        if [equal $id $value] {
            $name configure -text $label
        }
    }
}


proc dcOption::changeCmd {name newValue} {
    setValue $name $newValue
    dc::changed $name
}



proc dcOption::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -values "" -enabled 1]]
    set data(values) $arg(values)
    dc::renew $name $remainder
    $name configure -state [lindex "disabled normal" $arg(enabled)]
    $name.menu delete 0 end
    foreach {label id} $data(values) {
        $name.menu add command -label $label -command [list dcOption::changeCmd $name $id]
    }
}



# --- dcColor ---------------------------------------------------------------------------


namespace eval dcColor {}


proc dcColor::create {name args} {
    dc::create $name
    oop::addClass $name dcColor
    oop::data $name

    menubutton $name \
	    -text "  " \
	    -menu $name.menu \
	    -indicatoron 1 \
	    -direction flush \
            -relief raised \
            -takefocus 1 \
            -highlightthickness 1
    dc::bindDestroy $name

    menu $name.menu -tearoff 0
    set count -1
    foreach col \
	    [list "#000000" "#000080" "#008000" "#800000" \
	    "#008080" "#800080" "#808000" "#808080" \
	    "#0000ff" "#00ff00" "#ff0000" "#00ffff" \
	    "#ff00ff" "#ffff00" "#ffffff"] {
	if {[incr count] == 4} {
	    set colbreak 1
	    set count 0
	} else {
	    set colbreak 0
	}

	$name.menu add command -label "   " \
		-background $col \
		-command "::dcColor::changeValue $name $col" \
		-columnbreak $colbreak \
		-hidemargin 1 \
		-font [font::get "_textField"] \
		-activebackground $col
    }
    $name.menu add command \
	    -label "..." \
	    -command "::dcColor::editColorValue $name" \
	    -font [font::get "_textField"] \
	    -hidemargin 1

    eval renew [list $name] $args
    setValue $name ""
}


proc dcColor::editColorValue {name} {
    oop::data $name
    set newcolor [tk_chooseColor -initialcolor $data(value) -parent $name]
    if {$newcolor != ""} {
	changeValue $name $newcolor
    }
}


proc dcColor::changeValue {name value} {
    oop::data $name
    setValue $name $value
    dc::changed $name
}


proc dcColor::getValue {name} {
    oop::data $name
    return $data(value)
}


proc dcColor::setValue {name value} {
    oop::data $name
    if [empty $value] {
        set value "\#000000"
    }
    set data(value) $value
    $name configure -background $value -activebackground $value
}


proc dcColor::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -enabled 1]]
    dc::renew $name $remainder
    $name configure -state [lindex "disabled normal" $arg(enabled)]
}


# --- dcPath ---------------------------------------------------------------------------

namespace eval dcPath {}

proc dcPath::create {name args} {
    dc::create $name
    oop::addClass $name dcPath

    frame $name
    dc::bindDestroy $name

    button $name.open -text "..." -command [list ::dcPath::editPathValue $name] -takefocus 0
    pack $name.open -side right

    label $name.pad -width 1
    pack $name.pad -side right

    entry $name.entry
    pack $name.entry -side right -fill x -expand true
    bind $name.entry <FocusOut> [list ::dc::changed $name]

    eval renew [list $name] $args
}


proc dcPath::renew {name args} {
    oop::data $name
    set remainder [partialHandleArgs [list -width 30 -relativeTo "" -filetypes "{{All Files} *}" -enabled 1 -mustExist 1]]
    set data(relativeTo) $arg(relativeTo)
    set data(filetypes) $arg(filetypes)
    set data(mustExist) $arg(mustExist)
    dc::renew $name $remainder
    $name.entry configure -width $arg(width) -state [lindex "disabled normal" $arg(enabled)]
    $name.open configure -state [lindex "disabled normal" $arg(enabled)]
}


proc dcPath::getValue {name} {
    return [$name.entry get]
}


proc dcPath::setValue {name value} {
    $name.entry delete 0 end
    $name.entry insert 0 $value
}


proc dcPath::editPathValue {name} {
    oop::data $name

    set initialdir [file dirname [file join $data(relativeTo) [getValue $name]]]
    if [equal $data(filetypes) "dir"] {
        set filename [tk_chooseDirectory -parent $name -initialdir $initialdir]
    } elseif {$data(mustExist)} {
        set filename [tk_getOpenFile -parent $name -filetypes $data(filetypes) -initialdir $initialdir]
    } else {
        set filename [tk_getSaveFile -parent $name -filetypes $data(filetypes) -initialdir $initialdir]
    }
    if {$filename == ""} {
       return
    }
    if ![empty $data(relativeTo)] {
        set filename [makeRelative $filename $data(relativeTo)]
    }
    changeValue $name $filename
}


proc dcPath::changeValue {name newValue} {
    setValue $name $newValue
    dc::changed $name
}


proc dcPath::makeRelative {path base} {
    set lPath [file split $path]
    set lBase [file split $base]
    while { [llength $lPath]>0 && [llength $lBase]>0 && [equal [lindex $lPath 0] [lindex $lBase 0]] } {
        set lPath [lreplace $lPath 0 0]
        set lBase [lreplace $lBase 0 0]
    }
    foreach dummy $lBase {
        set lPath [linsert $lPath 0 ".."]
    }
    if { [llength $lPath]==0 } {
        set lPath "."
    }
    set result [eval file join $lPath]
    return $result
}


# -------------------------------------------------------------------------------------

