
namespace eval ::derivedSync {}

# Dieses Modul implementiert die Funktion derivedSync::syncView, die abgeleitete 
# Baumstrukturen mit ihrer Basis-Struktur synchronisiert.


# --- Zugriff auf Benutzer-Optionen ----------------------------------------


proc derivedSync::getOption {option obj} {
    if {[info exists ::derivedSync::option([c::getContext $obj],$option)]} {
        return $::derivedSync::option([c::getContext $obj],$option)
    } else {
        return 1
    }
}


# --- Hauptfunktion ---------------------------------------------------------


proc derivedSync::computeModelToRep {repRoot modelToRepName} {
    upvar $modelToRepName modelToRep
    catch {unset modelToRep}
    set modelToRep(0x0) 0x0
    c::traverseTree $repRoot isStructureObject rep {
        if [c::getModel $rep model] {
            lappend modelToRep($model) $rep
            # former lappend2
        }
    }
}


proc derivedSync::syncView {viewName view} {

    # delete
    c::traverseTree $view isLanguageObject obj {
        if [removeAbandoned $obj] {
            set obj ""
        }
    }
    computeModelToRep $view modelToRep
    c::traverseTree $view isLanguageObject obj {
        if [removeMultipleRep $obj modelToRep] {
            set obj ""
        }
    }

    # move

    computeModelToRep $view modelToRep
    c::traverseTree $view isLanguageObject obj {

        if ![equal $obj $view] {
	
            moveToRightList $obj modelToRep
        }
    }

    # create
    c::traverseTree $view isAggregate obj {
        createMissingRep $obj $viewName
    }

    # non-structural
    c::traverseTree $view isAggregate obj {
        orderRep $obj
    }
    c::traverseTree $view isValue obj {
        syncValue $obj
    }
    computeModelToRep $view modelToRep
    c::traverseTree $view isReference obj {
        syncReference $obj modelToRep
    }
}



# --- Defaultfunktionen (knnen berschrieben werden) -----------------------------------


proc derivedSync::syncValue {obj} {
    # precondition: obj is a value object
    if { [getOption sync $obj] } {
        syncutil::syncValue $obj
    }
}


proc derivedSync::syncReference {obj modelToRepName} {
    # precondition: obj is a reference object
    upvar $modelToRepName modelToRep
    if { [getOption sync $obj] } {
        syncutil::syncReference $obj modelToRep
    }
}


proc derivedSync::moveToRightList {obj modelToRepName} {
    # precondition: obj is LanguageObject
    # postcondition: obj is in the "right" aggregate list, determined by the model

    upvar $modelToRepName modelToRep    
    if { [getOption moveToRightList $obj] } {
        syncutil::moveToRightList $obj modelToRep 
    }
}      


proc derivedSync::createMissingRep {rep viewName} {
    # precondition: rep is an aggreate object
    # creates missing subelements of "rep".
    if { [getOption createMissingRep $rep] } {
        syncutil::createMissingRep $rep $viewName
    }
}


proc derivedSync::orderRep {rep} {
    # precondition: rep is an aggreate object
    if { [getOption orderRep $rep] } {
        syncutil::orderRep $rep
    }
}


proc derivedSync::removeAbandoned {rep} {
    # precondition: rep is LanguageObject
    if { [getOption removeAbandoned $rep] } {
        return [syncutil::removeAbandoned $rep]
    } else {
        return 0
    }
}


proc derivedSync::removeMultipleRep {rep modelToRepName} {
    # precondition: rep is LanguageObject
    upvar $modelToRepName modelToRep
    if { [getOption removeMultipleRep $rep] } {
        return [syncutil::removeMultipleRep $rep modelToRep]
    } else {
        return 0
    }
}


# --- Ende -----------------------------------------------------------------------------



