
main::loadLib abstractView.tcl


xn::class LidoView -inherits AbstractView {

    xn::method init {root viewName} {
        super init $root

        frame $data(toplevel).mainFrame
        pack $data(toplevel).mainFrame -fill both -expand 1
    
        meToolbar::create $data(toplevel).mainFrame.toolbar $viewName $data(toplevel).mainFrame.activeCanvas $root
        pack $data(toplevel).mainFrame.toolbar -fill y -side left

        activeCanvas::create $data(toplevel).mainFrame.activeCanvas
        pack $data(toplevel).mainFrame.activeCanvas -fill both -expand 1

        set data(toolbar) $data(toplevel).mainFrame.toolbar
        set data(activeCanvas) $data(toplevel).mainFrame.activeCanvas
        set data(viewName) $viewName
        set data(viewType) $viewName

        $object update
    }


    xn::method destroy {} {
        activeCanvas::delete $data(activeCanvas)
    }


    xn::method activateButton {objectName} {
        meToolbar::activateButton $data(toplevel).mainFrame.toolbar $objectName
    }


    xn::method update {} {
        super update
        vlDraw::start $data(activeCanvas) $data(root)
        c::startProcessor $data(viewName) $data(root)
        vlDraw::end
        meToolbar::updateStatus $data(toolbar)
    }


    xn::method wincmd_selectNext {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            catch {
                set new  "obj-[c::getSuccessor $node]"
                activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
            }
        }
    }


    xn::method wincmd_selectPrev {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            catch {
                set new "obj-[c::getPredecessor $node]"
                activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
            }
        }
    }


    xn::method wincmd_selectParent {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            catch {
                set new "obj-[c::getParent [c::getParent $node]]"
                activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
            }
        }
    }


    xn::method wincmd_selectFirstChild {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            catch {
                set new "obj-[c::getFirst [lindex [c::getNonEmptyAggSubElements $node] 0]]"
                activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
            }
        }
    }


    xn::method wincmd_selectNextChild {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            catch {
                set current [c::getParent $node]
                set list [getNonEmptyAggSubElements [c::getParent $current]]
                set index [lsearch -exact $list $current]
                set newAgg [lindex $list [expr ($index + 1) % [llength $list]]]
                set new "obj-[c::getFirst $newAgg]"
                activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
            }
        }
    }


    xn::method wincmd_selectAll {} {
        activeCanvas::trySetSelectionAndScroll $data(activeCanvas) "obj-$data(root)"
    }


    xn::method wincmd_editProperties {} {
        if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
            treedraw::editAttributes $node
        }
    }


    xn::method wincmd_delete {} {
        set delObjects [activeCanvas::getMultiselection $data(activeCanvas)]
        set delNodes ""
        foreach object $delObjects {
            if [regexp "^obj-(.*)$" $object dummy node] {
                lappend delNodes $node
            }
        }
        if {$delNodes != ""} {
            edit::delete $delNodes
        }
    }


    xn::method wincmd_cut {} {
        set cutObjects [activeCanvas::getMultiselection $data(activeCanvas)]
        set cutNodes ""
        foreach object $cutObjects {
            if [regexp "^obj-(.*)$" $object dummy node] {
                lappend cutNodes $node
            }
        }
        if {$cutNodes != ""} {
            edit::cutIntoClipboard $cutNodes
        }
    }


    xn::method wincmd_copy {} {
        set copyObjects [activeCanvas::getMultiselection $data(activeCanvas)]
        set copyNodes ""
        foreach object $copyObjects {
            if [regexp "^obj-(.*)$" $object dummy node] {
                lappend copyNodes $node
            }
        }
        if {$copyNodes != ""} {
            edit::copyIntoClipboard $copyNodes
        }
    }


    xn::method wincmd_paste {} {
        meToolbar::on_pasteClipboard $data(toolbar)
    }


    xn::method wincmd_exportViewInteractive {} {
        global exportRepData
        if {![info exists exportRepData]} {
            set exportRepData(format,value) "eps"
            set exportRepData(format,label) [i18n::text "Export format"]
            set exportRepData(format,editor) "Radio -values {eps eps png png}"
            set exportRepData(format,check) ""
            
            set exportRepData(path,value) "/tmp/viewExport.eps"
            set exportRepData(path,label) [i18n::text "Target file"]
            set exportRepData(path,editor) "Path -mustExist 0"
            set exportRepData(path,check) ""
        }
        if [CDlg::stdInputDialog [i18n::text "Export view"] [list format path] exportRepData] {
            $object exportView $exportRepData(format,value) $exportRepData(path,value)
        }
    }

    
    xn::method exportView {type filename} {
        if {[equal $type "png"]} {
            set epsFilename "$filename.eps"
            $object saveAsPostscript $epsFilename
            exportHtml::convertEpsToPng $epsFilename $filename
            file delete $epsFilename
        } else {
            $object saveAsPostscript $filename
        }    
    }


    xn::method wincmd_zoomIn {} {
        activeCanvas::zoomIn $data(activeCanvas)
        edit::forceUpdate $data(root)
    }


    xn::method wincmd_zoomOut {} {
        activeCanvas::zoomOut $data(activeCanvas)
        edit::forceUpdate $data(root)
    }


    xn::method saveAsPostscript {filename} {
        activeCanvas::saveAsPostscript $data(activeCanvas) $filename
    }




    xn::method selectNode {node} {
        while { ![activeCanvas::containsGroup $data(activeCanvas) obj-$node] } {        
            set node [c::getParent $node]
        }
        MDIFrame::MakeVisible $data(client)
        activeCanvas::setSelectionWithEffect $data(activeCanvas) "obj-$node"
    }

}

