
# Event types:
#
# - XXX: for temporary debug purposes
# - process: main steps of processing
# - change: changes of the abstract structure
# - profile: profiling information


namespace eval ::log {
    set knownEvents "XXX process options change profile eventManager"
    set shownEvents "XXX process"
    set mdiOutput 0
}


proc log::getStamp {type} {
    # private!
    set clicks [expr [clock clicks -millisec] & 0xFFFFFF]
    set timestamp [format "%04u.%03u" [expr $clicks/1000] [expr $clicks%1000]]

    set namespace [namespace tail [uplevel 2 {namespace current}]]
    return "..$timestamp ($namespace:$type)"
}

proc log::printStackdump {} {
    # private!
    set startLevel [expr { [info level] - 1 }]
    puts "Call chain:"
    for {set i $startLevel} {$i>=1} {incr i -1} {
        puts "| [info level $i]"
    }
}


proc log::log {type msg} {

    if { $type eq "error" } {
        error $msg
        return
    }
    if { $type eq "warning" } {
        puts "[getStamp $type] $msg"
        return
    }    

    if { ![main::getParam debug] } {
        return
    }

    if { [lsearch -exact $::log::knownEvents $type]==-1 } {
        error "Unknown event type: $type"
        puts "Orignial message: $msg"
        return
    } else {
        puts "[getStamp $type] $msg"
    }
}


proc log::error {msg} {
    puts "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    puts "!!                      Error                       !!"
    puts "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    puts "[getStamp "error"] $msg"
    if { $log::mdiOutput } {
        ::MDIFrame::SetError 1
    }
    printStackdump
}


proc log::process {shortInfo longInfo} {
    log process $longInfo
    if { $log::mdiOutput } {
        MDIFrame::SetProgress $shortInfo
        update idletasks
    }
}


proc log::assert {expr} {
    set result [uplevel 1 [list expr $expr]]
    if { !$result } {
        log::error "Assertion failed: $expr"
    }
}


