
namespace eval meToolbar { }


proc meToolbar::create {obj viewName activeCanvas root} {
    # Creates a virtual widget with path $obj

    upvar 0 ::meToolbar::$obj data
    set data(outerFrame) $obj
    set data(frame) $obj.innerFrame
    set data(pressed) ""
    set data(activeCanvas) $activeCanvas
    set data(buttons) ""
    set data(menus) ""
    set data(menuButtons) ""
    set data(root) $root
    
    set data(actRow) 0
    set data(actColumn) 0

    frame $data(outerFrame)
    frame $data(frame)
    
    pack $data(frame) -side top -fill x
    
    if [info exists ::meToolbar::buttonColumns($viewName)] {
        set buttonColumns $::meToolbar::buttonColumns($viewName)
    } else {
        set buttonColumns 1
        log::log XXX "No buttonColumns in definiton"
    }
    
    if [info exists ::meToolbar::buttons($viewName)] {
        foreach {label type function infoText class secClass userAction group groupType} $::meToolbar::buttons($viewName) {
            
                        
            if { ![empty $group] }  {
            
                if {$groupType == "generator"} {
                    
                    set buttonId [createButton $obj $label $type $infoText $class $secClass $userAction $group $groupType $buttonColumns $function]
                    menu $data(frame).$buttonId.menu -tearoff 0
                    lappend data(buttons) $buttonId
                    set data(class,$buttonId) $class
                    set data(menuFunct,$buttonId) $group
                
                } else {

                    if {$groupType == "image"} {
                       if { [string equal -length 5 $group "img::"] } {
                            set group [string replace $group 0 4 ""]
                       }
                    }
                    
                    if {![info exists data(buttonGroups,$group-$groupType)] } {

                            set buttonId [createButton $obj $label $type $infoText $class $secClass $userAction $group $groupType $buttonColumns $function]
                            menu $data(frame).$buttonId.menu -tearoff 0
                            set data(buttonGroups,$group-$groupType) $buttonId
                            lappend data(menuButtons) $buttonId
                            set tmpId 0
                            
                    } else {
                            set tmpId [expr [$data(frame).$data(buttonGroups,$group-$groupType).menu index end] +1]
                    }


                    set menuId "$data(buttonGroups,$group-$groupType)@$tmpId"
                    lappend data(menus) $menuId
                    set data(class,$menuId) $class
                    set data(secClass,$menuId) $secClass
                    set data(userAction,$menuId) $userAction
                    
                    switch $function {
                            "insert" {
                                set functionCode [list meToolbar::on_menuEntryPress $obj $menuId]   
                            }
                            "connect" {
                                set functionCode [list meToolbar::on_connectionMenuEntryPress $obj $menuId $class $secClass]
                            }
                            default {
                                log::log error "Invalid button function: $function"
                            }
                    }
                    meToolbar::addMenuElement $obj $data(buttonGroups,$group-$groupType) $label $type $functionCode
             }

             } else {
                

                
                set buttonId [createButton $obj $label $type $infoText $class $secClass $userAction $group $groupType $buttonColumns $function]
                
                lappend data(buttons) $buttonId
                set data(class,$buttonId) $class
                set data(secClass,$buttonId) $secClass
                set data(userAction,$buttonId) $userAction                
                
             }
             set data(function,$buttonId) $function
        }
    }
    #bind all <ButtonPress-1> +[list meToolbar::on_mouseDown $obj]
}


proc meToolbar::addMenuElement {obj buttonId label type function {state disabled}} {

    upvar 0 ::meToolbar::$obj data
    switch $type {
            "text" {
                  $data(frame).$buttonId.menu add command -label $label -command $function -state $state
            }
            "image" {
                  $data(frame).$buttonId.menu add command -image [main::getValidImage $label] -command $function -state $state
            }
            default {
                 log::log error "Invalid button type: $type"
            }
    }
}






proc meToolbar::activateButton {obj objectName} {
  upvar 0 ::meToolbar::$obj data
  activeCanvas::startObjectInsertion $data(activeCanvas) \
        [list vlDraw::acceptDropPredicate_type $objectName] [list vlDraw::onDrop_insertNew $objectName ""]
}



proc meToolbar::createButton {obj label type infoText class secClass userAction group groupType buttonColumns function} {
upvar 0 ::meToolbar::$obj data

#puts "obj:$obj label:$label type:$type infoText:$infoText class:$class secClass:$secClass userAction:$userAction group:$group groupType:$groupType buttonColumns:$buttonColumns function:$function"
        set buttonId "button[newint]"

        
        switch $groupType {
        
            "none" {
                switch $function {
                    "insert" {
                        set functionCode [list meToolbar::on_buttonPress $obj $buttonId]
                    }
                    "connect" {
                        set functionCode [list meToolbar::onConnectionButtonPress $obj $buttonId $class $secClass]
                    }
                    default {
                        log::log error "Invalid button function: $function"
                    }
                }                
                
                switch $type {
                
                    "text" {
                        button $data(frame).$buttonId -text $label -command $functionCode -state disabled
                    }
                    "image" {
                        button $data(frame).$buttonId -image [main::getValidImage $label] -command $functionCode -state disabled
                    }   
                    default {
                        log::log error "Invalid button type: $type"
                    }
                }
            
            }
            "text" {
                menubutton $data(frame).$buttonId -text $group -menu $data(frame).$buttonId.menu -relief raised
            }
            "image" {
                menubutton $data(frame).$buttonId -image [main::getValidImage img::$group] -menu $data(frame).$buttonId.menu -relief raised
            }
            "generator" {

                switch $type {
                            
                    "text" {
                        button $data(frame).$buttonId -text $label -command [list meToolbar::on_dynamicMenuButtonPress $obj $buttonId] -state disabled
                    }
                    "image" {
                        button $data(frame).$buttonId -image [main::getValidImage $label] -command [list meToolbar::on_dynamicMenuButtonPress $obj $buttonId] -state disabled
                    }   
                    default {
                        log::log error "Invalid button type: $type"
                    }
                }
                
            }
            default {
                log::log error "Invalid group type: $groupType"
            }
        } 
        #End Swich $groupType
        
        grid $data(frame).$buttonId -column $data(actColumn) -row $data(actRow) -sticky ew
        
        set data(actColumn) [expr [expr $data(actColumn) + 1] % $buttonColumns]
        
        if {$data(actColumn) == 0} {
                incr data(actRow)
        }
        
        if { ![empty $infoText] } {
              balloon::defineMessage $data(frame).$buttonId $infoText
        } elseif { [info exists ::helpToolTip::modelDoc($class)] && ![equal $function "connect"] } {
            ::balloon::defineMessage $data(frame).$buttonId "$class: $::helpToolTip::modelDoc($class)"
        } elseif { ![empty $class] && [equal $type "image"] && ![equal $function "connect"] } {
            set xx $data(frame).$buttonId
            balloon::defineMessage $data(frame).$buttonId $class            
        }

        return $buttonId
}


proc meToolbar::releaseButtons {obj} {
    upvar 0 ::meToolbar::$obj data
    if ![empty $data(pressed)] {
        if ![equal $data(pressed) "CLIPBOARD"] {
            $data(frame).$data(pressed) configure -relief raised
        }       
        set data(pressed) ""
    }
}


proc meToolbar::canInsert {className dropContexts} {
    set typeId [c::getTypeIdFromName $className]
    foreach dropContext $dropContexts {
        if { [edit::canInsertType $dropContext $typeId] } {
            return 1
        }
    }
    return 0
}


proc meToolbar::updateStatus {obj} {
    upvar 0 ::meToolbar::$obj data
    set dropGroups [activeCanvas::getDropTargetGroups $data(activeCanvas)]

    # "inspoints" herausfiltern
    set dropContexts ""
    foreach dropGroup $dropGroups {
        if [equal [vlDraw::getType $dropGroup] "inspoint"] {
            lappend dropContexts [vlDraw::getTreeContext $dropGroup]
        }
    }

    foreach buttonId $data(buttons) {
        if { [canInsert $data(class,$buttonId) $dropContexts] } {
            $data(frame).$buttonId configure -state normal
        } else {
            $data(frame).$buttonId configure -state disabled
        }
        
        set found 0
        set sFound 0
        if {$data(function,$buttonId) == "connect"} {
            foreach dropGroup $dropGroups {
                if {$found < 2 && [equal [vlDraw::getType $dropGroup] "obj"]} {
                    if [equal [c::getClassName [vlDraw::getTreeContext $dropGroup]] $data(class,$buttonId)] {
                       set found 1     
                    }
                    if [equal [c::getClassName [vlDraw::getTreeContext $dropGroup]] $data(secClass,$buttonId)] {
                       set sFound 1
                    }
                }  
            }
            if {$found == 1 && $sFound == 1} {
                $data(frame).$buttonId configure -state normal
            } else {
                $data(frame).$buttonId configure -state disabled
            }
        }
    }
    
    
    foreach menuId $data(menus) {
        set buttonId [lindex [split $menuId @] 0]
        set menuIndex [lindex [split $menuId @] 1]
        if { [canInsert $data(class,$menuId) $dropContexts] } {
            $data(frame).$buttonId.menu entryconfigure $menuIndex -state normal
        } else {
            $data(frame).$buttonId.menu entryconfigure $menuIndex -state disabled
        }
        
        
        set found 0
        set sFound 0
        if {$data(function,$buttonId) == "connect"} {
            foreach dropGroup $dropGroups {
                if {$found < 2 && [equal [vlDraw::getType $dropGroup] "obj"]} {
                    if [equal [c::getClassName [vlDraw::getTreeContext $dropGroup]] $data(class,$menuId)] {
                       set found 1     
                    }
                    if [equal [c::getClassName [vlDraw::getTreeContext $dropGroup]] $data(secClass,$menuId)] {
                       set sFound 1
                    }
                }  
            }
            if {$found == 1 && $sFound == 1} {
                $data(frame).$buttonId.menu entryconfigure $menuIndex -state normal
            } else {
                $data(frame).$buttonId.menu entryconfigure $menuIndex -state disabled
            }
        }
    }
}


proc meToolbar::updateStatus_old {obj} {
    upvar 0 ::meToolbar::$obj data
    foreach buttonId $data(buttons) {
        if [activeCanvas::canInsert $data(activeCanvas) [list vlDraw::acceptDropPredicate_type $data(class,$buttonId)]] {
            $data(frame).$buttonId configure -state normal
        } else {
            $data(frame).$buttonId configure -state disabled
        }
        
    }

}



proc meToolbar::on_buttonPress {obj buttonId} {
     upvar 0 ::meToolbar::$obj data
    set data(pressed) $buttonId
    $data(frame).$buttonId configure -relief sunken
    activeCanvas::startObjectInsertion $data(activeCanvas) \
        [list vlDraw::acceptDropPredicate_type $data(class,$buttonId)] [list vlDraw::onDrop_insertNew $data(class,$buttonId) $data(userAction,$buttonId)] [list meToolbar::releaseButtons $obj]
}

proc meToolbar::onConnectionButtonPress {obj buttonId firstClass secondClass} {
    upvar 0 ::meToolbar::$obj data
    set data(pressed) $buttonId
    $data(frame).$buttonId configure -relief sunken
    activeCanvas::startConnectionInsertion $data(activeCanvas) \
          [list meToolbar::acceptConnection $firstClass] [list meToolbar::acceptConnection $secondClass] $data(userAction,$buttonId) [list meToolbar::releaseButtons $obj]
 
}

proc meToolbar::acceptConnection {className group} {
 if { ![equal [vlDraw::getType $group] "obj"] } {
     return 0
 }
 return [c::nodeIsSubtypeOf [vlDraw::getTreeContext $group] $className]
}



proc meToolbar::on_dynamicMenuButtonPress {obj buttonId} {
    upvar 0 ::meToolbar::$obj data

    if {[procExists $data(menuFunct,$buttonId)]} {

        set newMenu [eval "$data(menuFunct,$buttonId) $obj $data(root)"]
        $data(frame).$buttonId.menu delete 0 end
        
        foreach {menuLabel type class function} $newMenu {
            meToolbar::addMenuElement $obj $buttonId $menuLabel $type [list meToolbar::on_dynamicMenuEntryPress $obj $class $function] normal
        }
        
        set winX [winfo rootx $data(frame).$buttonId]
        set winY [expr [winfo rooty $data(frame).$buttonId] + [winfo height $data(frame).$buttonId]]
        tk_popup $data(frame).$buttonId.menu $winX $winY  
    } else {
        log::log XXX "Function $data(menuFunct,$buttonId) does not exist"
    }
 
}

proc meToolbar::on_menuEntryPress {obj menuId} {
     upvar 0 ::meToolbar::$obj data
     activeCanvas::startObjectInsertion $data(activeCanvas) \
     [list vlDraw::acceptDropPredicate_type $data(class,$menuId)] [list vlDraw::onDrop_insertNew $data(class,$menuId) $data(userAction,$menuId)] [list meToolbar::releaseButtons $obj]

}

proc meToolbar::on_connectionMenuEntryPress {obj menuId firstClass secondClass} {
     upvar 0 ::meToolbar::$obj data
    activeCanvas::startConnectionInsertion $data(activeCanvas) \
          [list meToolbar::acceptConnection $firstClass] [list meToolbar::acceptConnection $secondClass] $data(userAction,$menuId) [list meToolbar::releaseButtons $obj]


}

proc meToolbar::insertConnection {class from to conArea conList canvas firstConP secondConP} {
    set newNode [edit::createObject  $class [edit::getFile [vlDraw::getTreeContext $firstConP]]]
    set obj [vlDraw::getTreeContext $firstConP]
    edit::insert [c::icBelow [c::get "$obj.INCLUDING $conArea.$conList"]] $newNode
    edit::setAttr $newNode $from [vlDraw::getTreeContext $firstConP]
    edit::setAttr $newNode $to [vlDraw::getTreeContext $secondConP]
    activeCanvas::selectAfterUpdate $canvas "obj-$newNode"
}

proc meToolbar::on_dynamicMenuEntryPress {obj className userAct} {
     upvar 0 ::meToolbar::$obj data
     activeCanvas::startObjectInsertion $data(activeCanvas) \
     [list vlDraw::acceptDropPredicate_type $className] [list vlDraw::onDrop_insertNew $className $userAct] [list meToolbar::releaseButtons $obj]

}

proc meToolbar::on_pasteClipboard {obj} {
    upvar 0 ::meToolbar::$obj data
    set data(pressed) "CLIPBOARD"
    activeCanvas::startObjectInsertion $data(activeCanvas) \
        vlDraw::acceptDropPredicate_clipboard vlDraw::onDrop_insertClipboard
}

proc meToolbar::on_mouseDown {obj} {
    upvar 0 ::meToolbar::$obj data
    releaseButtons $obj
    activeCanvas::returnToObjectSelection $data(activeCanvas) 
}
