
# treeView
#
# Baumsicht auf das abstrakte Modell

main::loadLib abstractView.tcl


xn::class TreeView -inherits AbstractView {


    xn::method init {root} {
        super init $root
        frame $data(toplevel).sCanvas
        pack $data(toplevel).sCanvas -fill both -expand 1

        canvas $data(toplevel).sCanvas.workspace -background white
        CDlg::mkScrollableXY $data(toplevel).sCanvas workspace

        set data(canvas) "$data(toplevel).sCanvas.workspace"
        set data(viewType) "Tree view"
        set data(selection) ""
        $object update
    }

    
    xn::method update {} {
        super update
        $data(canvas) delete all
        $object paintStructure $data(root)
        foreach {x1 y1 x2 y2} [$data(canvas) bbox all] {
            $data(canvas) config -scrollregion "0 0 [expr $x2+10] [expr $y2+10]"
        }
    }


    xn::method highlightNode {node} {
        set n [c::getParent $node]
        while { ![nullptr $n] } {
            set data(showDetail,$n) 1
            set n [c::getParent $n]
        }
        set data(selection) $node
        $object update
    }
    

    xn::method toggleShowDetail {elem} {
        set data(showDetail,$elem) [expr ! $data(showDetail,$elem)]
        $object update
    }


    xn::method paintItem {elem color1 text1 color2 text2 showAlways} {
        if $showAlways {
            # do nothing
        } elseif { [llength [c::getSubElements $elem]]>0 } {
            if { $data(showDetail,$elem) } {
                $data(canvas) create image $data(xpos) $data(ypos) -anchor w -image img::minus -tag showDetail-$elem
                $data(canvas) bind showDetail-$elem <Button-1> [list $object toggleShowDetail $elem]
            } else {
                $data(canvas) create image $data(xpos) $data(ypos) -anchor w -image img::plus -tag showDetail-$elem
                $data(canvas) bind showDetail-$elem <Button-1> [list $object toggleShowDetail $elem]
            }
        } else {
            $data(canvas) create image $data(xpos) $data(ypos) -anchor w -image img::leer
        }

        set tid [$data(canvas) create text [expr $data(xpos)+$data(icondist)] $data(ypos) -anchor w -text $text1 -fill $color1 -font $data(font) -tag elem-$elem]
        if { [equal $elem $data(selection)] } {
            $data(canvas) create rectangle [ $data(canvas) bbox $tid] -fill yellow -outline {}
            $data(canvas) raise $tid
            set data(selection) ""
        }

        set width [font measure $data(font) "$text1 "]
        $data(canvas) create text [expr $data(xpos)+$data(icondist)+$width] $data(ypos) -anchor w -text $text2 -fill $color2 -font $data(font) -tag elem-$elem
        $data(canvas) bind elem-$elem <Button-3> "treedraw::contextMenu $elem %X %Y"

        incr data(ypos) $data(deltay)
    }



    xn::method paintStructureRecursive {elem} {
        set attrColor darkgray
        set objColor blue
        set valueColor red
    
        if { ![info exists data(showDetail,$elem)] } {
            set data(showDetail,$elem) 0
        }

        if { [c::isLanguageObject $elem] } {
            set showAlways 0
            $object paintItem $elem $objColor [c::getClassName $elem] $objColor "([treedraw::getShortName $elem])" $showAlways
        } elseif { [c::isValue $elem] } {
            set showAlways 0
            $object paintItem $elem $attrColor "[c::getRole $elem]:" $valueColor [treedraw::getValue $elem] $showAlways
        } elseif { [c::isReference $elem] } {
            set showAlways 0
            $object paintItem $elem $attrColor "[c::getRole $elem]:" $valueColor "REF [treedraw::getValue $elem]" $showAlways
        } elseif { [c::isAggregate $elem] } {
            set showAlways 0
            $object paintItem $elem $attrColor "[c::getRole $elem]:" $valueColor "..." $showAlways
        } else {
            error "impossible branch"
            set showAlways 0
            $object paintItem $elem $attrColor "[c::getRole $elem] ([c::getClassName $elem])" $valueColor "" $showAlways
        }

        if { !$data(showDetail,$elem) && !$showAlways } {
            return
        }

        if [c::isLanguageObject $elem] {
            set lineColor $objColor
        } else {
            set lineColor $attrColor
        }

        incr data(xpos) $data(deltax)
        set start [expr $data(ypos)-$data(deltay)/2]
        set end $start
        foreach sub [c::getSubElements $elem] {
            set end [expr $data(ypos)+[image height img::leer]/2]
            $object paintStructureRecursive $sub
        }
        set linex [expr $data(xpos)+[image width img::leer]/2]
        $data(canvas) lower [$data(canvas) create line $linex $start $linex $end -fill $lineColor -width 1] all
        incr data(xpos) -$data(deltax)
    }


    xn::method paintStructure {elem} {

        set data(font) [font::get "_treeView"]

        set fontx [font measure $data(font) "M"]
        set fonty [font metrics $data(font) -linespace]
        set iconx [image width img::leer]
        set icony [image height img::leer]
        
        set data(xpos) 10
        set data(ypos) 10
        set data(deltax) [expr 2*$fontx]
        set data(deltay) [expr $fonty*4/3]
        set data(icondist) [expr $iconx+$fontx/2]
        
        $object paintStructureRecursive $elem
    }
}
