
/* implementation of tree construction functions */

#include "codeGen_node.h"

#include "codeGen_nodecode.h"

#include "attrpredef.h"

#include "codeGen_visitmap.h"

#include "codeGen_treeact.h"

#ifdef MONITOR
#include "attr_mon_dapto.h"
#endif

#include <stdlib.h>

#define _USE_OBSTACK 1

/* use of obstack: */

#if _USE_OBSTACK

#include "obstack.h"
static struct obstack TreeSpace;
static void *_TreeBase;

#ifdef __cplusplus
void* codeGen_NODEPTR_SUB_struct::operator new(size_t size)
{
	return obstack_alloc(&TreeSpace, size);
}
#else
#if defined(__STDC__) || defined(__cplusplus)
char* codeGen_TreeNodeAlloc(int size)
#else
char* codeGen_TreeNodeAlloc(size) int size;
#endif
{
	return (char *)(obstack_alloc(&TreeSpace, size));
}
#endif

void codeGen_InitTree()
{
	obstack_init(&TreeSpace);
	_TreeBase=obstack_alloc(&TreeSpace,0);
}

void codeGen_FreeTree()
{
	obstack_free(&TreeSpace, _TreeBase);
	_TreeBase=obstack_alloc(&TreeSpace,0);
}

#else

#include <stdio.h>

#ifdef __cplusplus
void* NODEPTR_struct::operator new(size_t size)
{
	void *retval = malloc(size);
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
#else
#if defined(__STDC__) || defined(__cplusplus)
char* codeGen_TreeNodeAlloc(int size)
#else
char* codeGen_TreeNodeAlloc(size) int size;
#endif
{
	char *retval = (char *) malloc(size);
	if (retval) return retval;
	fprintf(stderr, "*** DEADLY: No more memory.\n");
	exit(1);
}
#endif

void codeGen_InitTree() { }

void codeGen_FreeTree() { }

#endif

#ifdef MONITOR
#define _SETCOORD(node) \
        node->_coord = _coordref ? *_coordref : NoCoord;
#define _COPYCOORD(node) \
        node->_coord = _currn->_desc1->_coord;
#else
#define _SETCOORD(node)
#define _COPYCOORD(node)
#endif
#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkVIEWROOT (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkVIEWROOT (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBVIEWROOT)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Root)) return (codeGen_MkpVIEWROOT_codeGen(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkVIEWROOT */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Root (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Root (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Root)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Root */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Root_webApp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Root_webApp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Root_webApp)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication)) return (codeGen_Mkrule_1(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Root_webApp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Root_library (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Root_library (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Root_library)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Root_library */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Library (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Library (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Library)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Library */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Library_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Library_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Library_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Library_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Library_navigationElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Library_navigationElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Library_navigationElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Library_navigationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationElement_code (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationElement_code (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationElement_code)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationElement_code */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_entrancePage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_entrancePage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_entrancePage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_entrancePage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_webZone (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_webZone (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_webZone)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone)) return (codeGen_Mkrule_2(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_webZone */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_navigationBar (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_navigationBar (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_navigationBar)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBar)) return (codeGen_Mkrule_3(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_navigationBar */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_styleSheet (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_styleSheet (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_styleSheet)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheet)) return (codeGen_Mkrule_4(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_styleSheet */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_frames (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_frames (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_frames)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_frames */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_meta (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_meta (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_meta)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformation)) return (codeGen_Mkrule_5(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_meta */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_database (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_database (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_database)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase)) return (codeGen_Mkrule_6(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_database */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_roles (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_roles (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_roles)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_roles */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_linksExtern (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_linksExtern (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_linksExtern)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_linksExtern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_imagePath (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_imagePath (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_imagePath)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_imagePath */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_packageName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_packageName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_packageName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_packageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_pageEncoding (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_pageEncoding (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_pageEncoding)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_pageEncoding */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_header (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_header (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_header)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_header */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_footer (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_footer (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_footer)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_footer */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_mailHost (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_mailHost (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_mailHost)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_mailHost */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_mailFromAddress (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_mailFromAddress (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_mailFromAddress)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_mailFromAddress */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebApplication_catalinaHome (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebApplication_catalinaHome (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebApplication_catalinaHome)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebApplication_catalinaHome */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RootWebZone (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RootWebZone (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RootWebZone */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBar (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBar (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBar)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBar */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheet (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheet (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheet)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheet */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_GlobalFrame (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_GlobalFrame (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_GlobalFrame)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_GlobalFrame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebRole (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebRole (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebRole)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebRole */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LinkTargetExtern (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LinkTargetExtern (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LinkTargetExtern)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LinkTargetExtern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RootWebZone_webTargets (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone_webTargets)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RootWebZone_webTargets */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RootWebZone_errorLink (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RootWebZone_errorLink (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone_errorLink)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RootWebZone_errorLink */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RootWebZone_roles (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RootWebZone_roles (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone_roles)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RootWebZone_roles */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RootWebZone_setSize (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RootWebZone_setSize (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RootWebZone_setSize)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RootWebZone_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebRoleRef (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebRoleRef (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebRoleRef)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebRoleRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_webTargets (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_webTargets)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_webTargets */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_errorLink (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_errorLink (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_errorLink)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_errorLink */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_roles (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_roles (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_roles)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_roles */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_setSize (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_setSize (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_setSize)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SubWebZone_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SubWebZone_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SubWebZone_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SubWebZone_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebRole_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebRole_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebRole_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebRole_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebRole_color (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebRole_color (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebRole_color)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebRole_color */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebRoleRef_role (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebRoleRef_role (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebRoleRef_role)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebRoleRef_role */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_formalParameter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_formalParameter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_formalParameter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_formalParameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_title (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_title (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_title)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_title */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_frame (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_frame (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_frame)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Frame)) return (codeGen_Mkrule_7(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_frame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebPage_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebPage_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebPage_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebPage_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeFormalParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeFormalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Frame (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Frame (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Frame)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Frame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Frame_components (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Frame_components (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Frame_components)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Frame_components */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputMessagesElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputMessagesElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputMessagesElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputMessagesElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Table (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Table (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Table)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Table */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Heading (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Heading (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Heading)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Heading */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Paragraph (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Paragraph (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Paragraph)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Paragraph */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FrameRef (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FrameRef (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FrameRef)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FrameRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_GlobalFrame_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_GlobalFrame_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_GlobalFrame_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_GlobalFrame_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_GlobalFrame_components (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_GlobalFrame_components)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_GlobalFrame_components */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_GlobalFrame_setSize (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_GlobalFrame_setSize (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_GlobalFrame_setSize)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_GlobalFrame_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBar_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBar_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBar_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBar_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBar_elements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBar_elements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBar_elements)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBar_elements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBar_alignment (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBar_alignment (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBar_alignment)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBar_alignment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBarElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBarElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBarElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBarElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBarElement_imageName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBarElement_imageName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBarElement_imageName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBarElement_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_NavigationBarElement_target (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_NavigationBarElement_target (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_NavigationBarElement_target)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_NavigationBarElement_target */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FrameRef_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FrameRef_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FrameRef_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FrameRef_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FrameRef_frame (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FrameRef_frame (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FrameRef_frame)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FrameRef_frame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformation_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformation_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformation_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformation_elements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformation_elements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformation_elements)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformation_elements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformationElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformationElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformationElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformationElement_property (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformationElement_property (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformationElement_property)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformationElement_property */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MetaInformationElement_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MetaInformationElement_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MetaInformationElement_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MetaInformationElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_userName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_userName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_userName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_userName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_userPassword (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_userPassword (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_userPassword)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_userPassword */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_hostName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_hostName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_hostName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_hostName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_port (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_port (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_port)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_port */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_dataBaseType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_dataBaseType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_dataBaseType)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_dataBaseType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_prefix (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_prefix (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_prefix)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_prefix */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_dataSources (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_dataSources)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_dataSources */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_dataSourceModifications (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_dataSourceModifications)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_dataSourceModifications */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_setSize (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_setSize (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_setSize)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBase_modifySetSize (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBase_modifySetSize (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBase_modifySetSize)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBase_modifySetSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderedDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderedDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderedDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderedDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceTable (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceTable (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceTable)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBColumn_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBColumn_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBColumn_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBColumn_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBColumn_type (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBColumn_type (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBColumn_type)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasOne)) return (codeGen_Mkrule_20(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasMany)) return (codeGen_Mkrule_19(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBString)) return (codeGen_Mkrule_18(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBText)) return (codeGen_Mkrule_17(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBInteger)) return (codeGen_Mkrule_16(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBLong)) return (codeGen_Mkrule_15(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBFloat)) return (codeGen_Mkrule_14(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDouble)) return (codeGen_Mkrule_13(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBigInt)) return (codeGen_Mkrule_12(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDate)) return (codeGen_Mkrule_11(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBTimeStamp)) return (codeGen_Mkrule_10(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBoolean)) return (codeGen_Mkrule_9(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBlob)) return (codeGen_Mkrule_8(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBColumn_type */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBColumn_notNull (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBColumn_notNull (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBColumn_notNull)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBColumn_notNull */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBColumn_primaryKey (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBColumn_primaryKey (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBColumn_primaryKey)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBColumn_primaryKey */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBBlob (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBBlob (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBlob)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBBlob */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBBoolean (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBBoolean (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBoolean)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBBoolean */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBTimeStamp (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBTimeStamp (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBTimeStamp)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBTimeStamp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBDate (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBDate (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDate)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBDate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBBigInt (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBBigInt (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBigInt)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBBigInt */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBDouble (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBDouble (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDouble)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBDouble */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBFloat (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBFloat (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBFloat)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBFloat */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBLong (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBLong (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBLong)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBLong */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBInteger (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBInteger (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBInteger)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBInteger */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBText (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBText (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBText)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBText */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBString (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBString (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBString)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBString */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBHasMany (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBHasMany (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasMany)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBHasMany */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBHasOne (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBHasOne (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasOne)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBHasOne */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBHasOne_dataSourceTable (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBHasOne_dataSourceTable (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasOne_dataSourceTable)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBHasOne_dataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBHasMany_dataSourceTable (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBHasMany_dataSourceTable (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasMany_dataSourceTable)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBHasMany_dataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DBBlob_contentType (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DBBlob_contentType (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBlob_contentType)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DBBlob_contentType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceTable_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceTable_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceTable_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceTable_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceTable_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceTable_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceTable_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceTable_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceTable_columns (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceTable_columns (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceTable_columns)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceTable_columns */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_formalParameters (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_formalParameters (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_formalParameters)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_formalParameters */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_sourceDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_sourceDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_sourceDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_condition (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_condition)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_condition */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceProjection_projectionColumns (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceProjection_projectionColumns)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceProjection_projectionColumns */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceFormalParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceFormalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ParameterValueExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ParameterValueExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ParameterValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ProjectionColumnValueExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ProjectionColumnValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColumnValueExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColumnValueExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColumnValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ConstantExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ConstantExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ConstantExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_PredefinedExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_PredefinedExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_PredefinedExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_UnaryExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_UnaryExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_UnaryExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BinaryExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BinaryExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BinaryExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationProjection (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationProjection (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationProjection)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColumnProjection (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColumnProjection (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnProjection)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColumnProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColumnProjection_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColumnProjection_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnProjection_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColumnProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColumnProjection_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColumnProjection_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnProjection_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColumnProjection_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationProjection_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationProjection_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationProjection_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationProjection_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationProjection_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationProjection_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationProjection_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationProjection_computation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationProjection_computation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationProjection_computation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationProjection_computation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_formalParameters (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_formalParameters (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_formalParameters)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_formalParameters */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_sourceDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_sourceDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_sourceDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_firstRow (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_firstRow (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_firstRow)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_28(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_27(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_26(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_25(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_24(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_23(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_22(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_21(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_firstRow */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LimitDataSource_maxRows (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LimitDataSource_maxRows (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LimitDataSource_maxRows)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_36(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_35(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_34(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_33(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_32(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_31(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_30(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_29(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LimitDataSource_maxRows */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderedDataSource_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderedDataSource_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderedDataSource_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderedDataSource_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderedDataSource_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderedDataSource_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderedDataSource_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderedDataSource_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderedDataSource_sourceDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderedDataSource_sourceDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderedDataSource_sourceDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderedDataSource_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderedDataSource_orderColumns (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderedDataSource_orderColumns (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderedDataSource_orderColumns)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderedDataSource_orderColumns */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderColumn_orderColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderColumn_orderColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderColumn_orderColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderColumn_orderColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OrderColumn_order (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OrderColumn_order (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OrderColumn_order)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OrderColumn_order */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BinaryExpression_leftOperand (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BinaryExpression_leftOperand (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression_leftOperand)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_44(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_43(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_42(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_41(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_40(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_39(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_38(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_37(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BinaryExpression_leftOperand */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BinaryExpression_rightOperand (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BinaryExpression_rightOperand (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression_rightOperand)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_52(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_51(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_50(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_49(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_48(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_47(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_46(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_45(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BinaryExpression_rightOperand */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BinaryExpression_operator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BinaryExpression_operator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression_operator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BinaryExpression_operator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_UnaryExpression_operand (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_UnaryExpression_operand (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression_operand)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_60(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_59(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_58(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_57(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_56(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_55(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_54(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_53(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_UnaryExpression_operand */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_UnaryExpression_operator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_UnaryExpression_operator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression_operator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_UnaryExpression_operator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_PredefinedExpression_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_PredefinedExpression_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_PredefinedExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ConstantExpression_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ConstantExpression_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ConstantExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColumnValueExpression_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColumnValueExpression_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColumnValueExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ProjectionColumnValueExpression_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ProjectionColumnValueExpression_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ProjectionColumnValueExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ParameterValueExpression_parameter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ParameterValueExpression_parameter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression_parameter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ParameterValueExpression_parameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationExpression_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationExpression_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ComputationExpression_computation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ComputationExpression_computation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression_computation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ComputationExpression_computation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceValue_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceValue_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceValue_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceValue_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceValue_expression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceValue_expression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceValue_expression)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_68(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_67(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_66(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_65(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_64(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_63(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_62(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_61(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceValue_expression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_dataSource1 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_dataSource1 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_dataSource1)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_dataSource1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_dataSource2 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_dataSource2 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_dataSource2)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_dataSource2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_columnRef1 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_columnRef1 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_columnRef1)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_columnRef1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_columnRef2 (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_columnRef2 (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_columnRef2)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_columnRef2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceJoin_joinOperator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceJoin_joinOperator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceJoin_joinOperator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceJoin_joinOperator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert_targetDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert_targetDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert_targetDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert_formalParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert_formalParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert_formalParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert_formalParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceInsert_values (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceInsert_values (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceInsert_values)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceInsert_values */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_targetDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_targetDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_targetDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_formalParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_formalParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_formalParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_formalParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_conditions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_conditions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_conditions)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_76(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_75(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_74(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_73(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_72(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_71(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_70(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_69(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_conditions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_values (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceUpdate_values (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceUpdate_values)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceUpdate_values */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete_targetDataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete_targetDataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete_targetDataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete_formalParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete_formalParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete_formalParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete_formalParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceDelete_conditions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceDelete_conditions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceDelete_conditions)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_84(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_83(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_82(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_81(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_80(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_79(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_78(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_77(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceDelete_conditions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceActualParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceActualParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceActualParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceActualParam_formalParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceActualParam_formalParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceActualParam_formalParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceActualParam_expressions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceActualParam_expressions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceActualParam_expressions)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeConstantExpression)) return (codeGen_Mkrule_87(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeVariableExpression)) return (codeGen_Mkrule_86(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeSessionExpression)) return (codeGen_Mkrule_85(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceActualParam_expressions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeSessionExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeSessionExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeVariableExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeVariableExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeConstantExpression)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeConstantExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceFormalParam_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceFormalParam_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam_type (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataSourceFormalParam_type (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataSourceFormalParam_type)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasOne)) return (codeGen_Mkrule_100(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasMany)) return (codeGen_Mkrule_99(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBString)) return (codeGen_Mkrule_98(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBText)) return (codeGen_Mkrule_97(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBInteger)) return (codeGen_Mkrule_96(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBLong)) return (codeGen_Mkrule_95(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBFloat)) return (codeGen_Mkrule_94(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDouble)) return (codeGen_Mkrule_93(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBigInt)) return (codeGen_Mkrule_92(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDate)) return (codeGen_Mkrule_91(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBTimeStamp)) return (codeGen_Mkrule_90(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBoolean)) return (codeGen_Mkrule_89(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBlob)) return (codeGen_Mkrule_88(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataSourceFormalParam_type */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Paragraph_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Paragraph_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Paragraph_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Paragraph_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Paragraph_fragments (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Paragraph_fragments)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Paragraph_fragments */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputFragment (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputFragment (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputFragment)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TextFragment (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TextFragment (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TextFragment)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TextFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TextFragment_link (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TextFragment_link (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TextFragment_link)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TextFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TextFragment_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TextFragment_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TextFragment_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TextFragment_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TextFragment_fontstyle (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TextFragment_fontstyle (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TextFragment_fontstyle)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TextFragment_fontstyle */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_link (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_link (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_link)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_width (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_width (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_width)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_width */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_height (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_height (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_height)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_height */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_border (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_border (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_border)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_border */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_verticalAlign (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_verticalAlign (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_verticalAlign)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_verticalAlign */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_float (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_float (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_float)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_float */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_text (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_text (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_text)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_text */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageFragment_imageName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageFragment_imageName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageFragment_imageName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageFragment_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Heading_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Heading_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Heading_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Heading_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Heading_text (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Heading_text (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Heading_text)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Heading_text */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Heading_level (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Heading_level (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Heading_level)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Heading_level */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Table_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Table_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Table_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Table_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Table_rows (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Table_rows (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Table_rows)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Table_rows */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Table_columns (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Table_columns (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Table_columns)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Table_columns */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableRow (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableRow (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableRow)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableRow */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableRow_cells (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableRow_cells (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableRow_cells)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableRow_cells */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableCell (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableCell (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableCell)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableCell */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableCell_columnRef (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableCell_columnRef (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableCell_columnRef)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableCell_columnRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_TableCell_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_TableCell_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_TableCell_content)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Frame)) return (codeGen_Mkrule_101(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_TableCell_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheet_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheet_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheet_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheet_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheet_custom (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheet_custom (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheet_custom)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheet_custom */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheet_default (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheet_default (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheet_default)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheet_default */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomStyleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CustomStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HtmlStyleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_HtmlStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock_pairs (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomStyleSheetBlock_pairs)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CustomStyleSheetBlock_pairs */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CustomStyleSheetBlock_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomStyleSheetBlock_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CustomStyleSheetBlock_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheetPair (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheetPair (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheetPair)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheetPair */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheetPair_category (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheetPair_category (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheetPair_category)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheetPair_category */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheetPair_attribute (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheetPair_attribute (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheetPair_attribute)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheetPair_attribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheetPair_valueCategory (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheetPair_valueCategory (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheetPair_valueCategory)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheetPair_valueCategory */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleSheetPair_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleSheetPair_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleSheetPair_value)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SizeValue)) return (codeGen_Mkrule_109(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColorValue)) return (codeGen_Mkrule_108(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FamilyValue)) return (codeGen_Mkrule_107(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleValue)) return (codeGen_Mkrule_106(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HeadingValue)) return (codeGen_Mkrule_105(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_AlignmentValue)) return (codeGen_Mkrule_104(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListValue)) return (codeGen_Mkrule_103(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomValue)) return (codeGen_Mkrule_102(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleSheetPair_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CustomValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CustomValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CustomValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_AlignmentValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_AlignmentValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_AlignmentValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_AlignmentValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_HeadingValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_HeadingValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HeadingValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_HeadingValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FamilyValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FamilyValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FamilyValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FamilyValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColorValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColorValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColorValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColorValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SizeValue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SizeValue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SizeValue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SizeValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SizeValue_measure (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SizeValue_measure (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SizeValue_measure)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SizeValue_measure */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SizeValue_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SizeValue_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SizeValue_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SizeValue_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ColorValue_color (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ColorValue_color (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColorValue_color)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ColorValue_color */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FamilyValue_family (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FamilyValue_family (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FamilyValue_family)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FamilyValue_family */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StyleValue_style (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StyleValue_style (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StyleValue_style)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StyleValue_style */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_HeadingValue_heading (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_HeadingValue_heading (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HeadingValue_heading)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_HeadingValue_heading */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_AlignmentValue_alignment (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_AlignmentValue_alignment (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_AlignmentValue_alignment)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_AlignmentValue_alignment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListValue_listed (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListValue_listed (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListValue_listed)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListValue_listed */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CustomValue_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CustomValue_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CustomValue_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CustomValue_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock_pairs (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HtmlStyleSheetBlock_pairs)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_HtmlStyleSheetBlock_pairs */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_HtmlStyleSheetBlock_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_HtmlStyleSheetBlock_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_HtmlStyleSheetBlock_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionHandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionHandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionHandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionHandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionHandler_targetpage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionHandler_targetpage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionHandler_targetpage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionHandler_targetpage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionHandler_params (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionHandler_params (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionHandler_params)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionHandler_params */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeActualParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeActualParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeActualParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction_formalParameter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction_formalParameter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction_formalParameter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction_formalParameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction_successpage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction_successpage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction_successpage)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionHandler)) return (codeGen_Mkrule_110(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction_successpage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction_operations (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction_operations)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction_operations */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebAction_position (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebAction_position (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebAction_position)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebAction_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LogoutOperation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LogoutOperation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LogoutOperation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LogoutOperation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JavaOperation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JavaOperation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JavaOperation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JavaOperation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation_errorhandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation_errorhandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation_errorhandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation_errorhandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation_successMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation_successMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation_successMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation_errorMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation_errorMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation_errorMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation_filter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation_filter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation_filter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation_filter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DataBaseOperation_params (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DataBaseOperation_params (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DataBaseOperation_params)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DataBaseOperation_params */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JavaOperation_errorhandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JavaOperation_errorhandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JavaOperation_errorhandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JavaOperation_errorhandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JavaOperation_successMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JavaOperation_successMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JavaOperation_successMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JavaOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JavaOperation_errorMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JavaOperation_errorMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JavaOperation_errorMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JavaOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JavaOperation_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JavaOperation_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JavaOperation_content)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JavaOperation_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_errorhandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_errorhandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_errorhandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_errorhandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_successMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_successMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_successMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_errorMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_errorMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_errorMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_mailTo (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_mailTo (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_mailTo)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_mailTo */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_subject (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_subject (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_subject)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_subject */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MailingOperation_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MailingOperation_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MailingOperation_content)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MailingOperation_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LogoutOperation_errorhandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LogoutOperation_errorhandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LogoutOperation_errorhandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LogoutOperation_errorhandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LogoutOperation_successMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LogoutOperation_successMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LogoutOperation_successMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LogoutOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LogoutOperation_errorMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LogoutOperation_errorMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LogoutOperation_errorMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LogoutOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LogoutOperation_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LogoutOperation_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LogoutOperation_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LogoutOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_errorhandler (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_errorhandler (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_errorhandler)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_errorhandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_successMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_successMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_successMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_errorMessage (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_errorMessage (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_errorMessage)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_userDBColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_userDBColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_userDBColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_userDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_passwdDBColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_passwdDBColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_passwdDBColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_passwdDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_rolesDBColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_rolesDBColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_rolesDBColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_rolesDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_userFormField (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_userFormField (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_userFormField)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_userFormField */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LoginOperation_passwdFormField (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LoginOperation_passwdFormField (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LoginOperation_passwdFormField)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LoginOperation_passwdFormField */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_action (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_action (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_action)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_action */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_dataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_dataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_dataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_dataSourceId (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_dataSourceId (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_dataSourceId)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_dataSourceId */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_elements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_elements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_elements)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_elements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_Formular_actualParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_Formular_actualParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_Formular_actualParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_Formular_actualParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_EmailValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_EmailValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_EmailValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_EmailValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_CreditCardValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_CreditCardValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_CreditCardValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_CreditCardValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RegExprValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RegExprValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RegExprValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RegExprValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_RegExprValidator_pattern (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_RegExprValidator_pattern (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_RegExprValidator_pattern)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_RegExprValidator_pattern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StringLengthValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StringLengthValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StringLengthValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StringLengthValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StringLengthValidator_min (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StringLengthValidator_min (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StringLengthValidator_min)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StringLengthValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_StringLengthValidator_max (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_StringLengthValidator_max (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_StringLengthValidator_max)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_StringLengthValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LongRangeValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LongRangeValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LongRangeValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LongRangeValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LongRangeValidator_min (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LongRangeValidator_min (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LongRangeValidator_min)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LongRangeValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LongRangeValidator_max (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LongRangeValidator_max (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LongRangeValidator_max)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LongRangeValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleRangeValidator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleRangeValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator_min (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator_min (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleRangeValidator_min)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleRangeValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator_max (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleRangeValidator_max (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleRangeValidator_max)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleRangeValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_IntegerNumberConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_IntegerNumberConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter_minIntegerDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter_minIntegerDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_IntegerNumberConverter_minIntegerDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_IntegerNumberConverter_minIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter_maxIntegerDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_IntegerNumberConverter_maxIntegerDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_IntegerNumberConverter_maxIntegerDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_IntegerNumberConverter_maxIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleNumberConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleNumberConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_minIntegerDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_minIntegerDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleNumberConverter_minIntegerDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleNumberConverter_minIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_maxIntegerDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_maxIntegerDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleNumberConverter_maxIntegerDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleNumberConverter_maxIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_minFractionDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_minFractionDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleNumberConverter_minFractionDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleNumberConverter_minFractionDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_maxFractionDigits (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleNumberConverter_maxFractionDigits (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleNumberConverter_maxFractionDigits)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleNumberConverter_maxFractionDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_IntegerConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_IntegerConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_IntegerConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_IntegerConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LongConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LongConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LongConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LongConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BigIntegerConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BigIntegerConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BigIntegerConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BigIntegerConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FloatConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FloatConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FloatConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FloatConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DoubleConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DoubleConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DoubleConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DoubleConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_BooleanConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_BooleanConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BooleanConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_BooleanConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_DateTimeConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_DateTimeConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DateTimeConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_DateTimeConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_PatternConverter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_PatternConverter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PatternConverter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_PatternConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_PatternConverter_pattern (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_PatternConverter_pattern (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PatternConverter_pattern)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_PatternConverter_pattern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_layout (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_layout (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_layout)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_layout */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_elements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_elements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_elements)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_elements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_checked (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioGroupElement_checked (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioGroupElement_checked)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioGroupElement_checked */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_rows (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularTextAreaElement_rows (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularTextAreaElement_rows)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularTextAreaElement_rows */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_redisplay (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularPasswordElement_redisplay (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularPasswordElement_redisplay)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularPasswordElement_redisplay */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDateElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDateElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDateElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_accept (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularFileUploadElement_accept (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularFileUploadElement_accept)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularFileUploadElement_accept */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_checked (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularCheckboxElement_checked (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularCheckboxElement_checked)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularCheckboxElement_checked */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_description (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_description (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_description)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_pvalue (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_pvalue (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_pvalue)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_required (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_required (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_required)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_required */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_validator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_validator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_validator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_converter (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_converter)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_converter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_options (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_options (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_options)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularConstantSelectOption)) return (codeGen_Mkrule_112(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption)) return (codeGen_Mkrule_111(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_options */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_size (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_size (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_size)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_size */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularSelectElement_multiple (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularSelectElement_multiple (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularSelectElement_multiple)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularSelectElement_multiple */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDataSourceSelectOption */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularConstantSelectOption (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularConstantSelectOption (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularConstantSelectOption)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularConstantSelectOption */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularConstantSelectOption_options (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularConstantSelectOption_options (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularConstantSelectOption_options)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularConstantSelectOption_options */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SelectOption (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SelectOption (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SelectOption)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SelectOption */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SelectOption_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SelectOption_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SelectOption_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SelectOption_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_SelectOption_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_SelectOption_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_SelectOption_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_SelectOption_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDataSourceSelectOption_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_dataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_dataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption_dataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDataSourceSelectOption_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDataSourceSelectOption_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularDataSourceSelectOption_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularDataSourceSelectOption_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularDataSourceSelectOption_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioElement_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioElement_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioElement_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_FormularRadioElement_label (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_FormularRadioElement_label (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_FormularRadioElement_label)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_FormularRadioElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeActualParam_formalParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeActualParam_formalParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeActualParam_formalParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeActualParam_expressions (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeActualParam_expressions (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeActualParam_expressions)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeConstantExpression)) return (codeGen_Mkrule_115(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeVariableExpression)) return (codeGen_Mkrule_114(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeSessionExpression)) return (codeGen_Mkrule_113(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeActualParam_expressions */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeFormalParam_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeFormalParam_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam_type (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeFormalParam_type (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeFormalParam_type)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasOne)) return (codeGen_Mkrule_128(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBHasMany)) return (codeGen_Mkrule_127(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBString)) return (codeGen_Mkrule_126(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBText)) return (codeGen_Mkrule_125(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBInteger)) return (codeGen_Mkrule_124(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBLong)) return (codeGen_Mkrule_123(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBFloat)) return (codeGen_Mkrule_122(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDouble)) return (codeGen_Mkrule_121(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBigInt)) return (codeGen_Mkrule_120(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBDate)) return (codeGen_Mkrule_119(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBTimeStamp)) return (codeGen_Mkrule_118(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBoolean)) return (codeGen_Mkrule_117(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_DBBlob)) return (codeGen_Mkrule_116(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeFormalParam_type */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MD5Modifier (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MD5Modifier (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MD5Modifier)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MD5Modifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JoinModifier (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JoinModifier (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JoinModifier)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JoinModifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_JoinModifier_separator (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_JoinModifier_separator (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_JoinModifier_separator)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_JoinModifier_separator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression_modifier (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeConstantExpression_modifier)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeConstantExpression_modifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeConstantExpression_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeConstantExpression_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeConstantExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression_modifier (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeVariableExpression_modifier)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeVariableExpression_modifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeVariableExpression_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeVariableExpression_content)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeVariableExpression_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression_modifier (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeSessionExpression_modifier)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeSessionExpression_modifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_WebNodeSessionExpression_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_WebNodeSessionExpression_content)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_WebNodeSessionExpression_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputFragment_link (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputFragment_link (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputFragment_link)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputFragment_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputFragment_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputFragment_content)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputFragment_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputMessagesElement_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputMessagesElement_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_errorStyleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_errorStyleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputMessagesElement_errorStyleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputMessagesElement_errorStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_infoStyleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_infoStyleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputMessagesElement_infoStyleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputMessagesElement_infoStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_layout (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_OutputMessagesElement_layout (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_OutputMessagesElement_layout)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_OutputMessagesElement_layout */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputActualParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputActualParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActualParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputActualParam_formalParam (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputActualParam_formalParam (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActualParam_formalParam)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputActualParam_expression (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputActualParam_expression (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActualParam_expression)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_BinaryExpression)) return (codeGen_Mkrule_136(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_UnaryExpression)) return (codeGen_Mkrule_135(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_PredefinedExpression)) return (codeGen_Mkrule_134(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ConstantExpression)) return (codeGen_Mkrule_133(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ColumnValueExpression)) return (codeGen_Mkrule_132(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) return (codeGen_Mkrule_131(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ParameterValueExpression)) return (codeGen_Mkrule_130(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ComputationExpression)) return (codeGen_Mkrule_129(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputActualParam_expression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_dataSourceActualParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_dataSourceActualParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_dataSourceActualParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_dataSourceActualParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_dataSource (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_dataSource (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_dataSource)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_maxNumberOfElements (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_maxNumberOfElements (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_maxNumberOfElements)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_maxNumberOfElements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_renderAs (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_renderAs (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_renderAs)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_renderAs */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_navigationElement (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_navigationElement (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_navigationElement)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_navigationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_columns (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_columns (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_columns)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_columns */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_headerStyleClass (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_headerStyleClass (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_headerStyleClass)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_headerStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_evenRowStyleClass (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_evenRowStyleClass (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_evenRowStyleClass)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_evenRowStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputElement_oddRowStyleClass (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputElement_oddRowStyleClass (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputElement_oddRowStyleClass)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputElement_oddRowStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputColumn (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputColumn (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputColumn)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputColumn_header (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputColumn_header (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputColumn_header)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputColumn_header */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputColumn_content (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputColumn_content (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputColumn_content)) return (_currn);
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputConstantContent)) return (codeGen_Mkrule_141(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputRefContent)) return (codeGen_Mkrule_140(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputProjectionContent)) return (codeGen_Mkrule_139(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputNumerationContent)) return (codeGen_Mkrule_138(_coordref, _currn));
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActionConnectionsContent)) return (codeGen_Mkrule_137(_coordref, _currn));
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputColumn_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputActionConnectionsContent (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputActionConnectionsContent (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActionConnectionsContent)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputActionConnectionsContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputNumerationContent (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputNumerationContent (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputNumerationContent)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputNumerationContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputProjectionContent (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputProjectionContent (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputProjectionContent)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputProjectionContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputRefContent (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputRefContent (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputRefContent)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputRefContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputConstantContent (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputConstantContent (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputConstantContent)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputConstantContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputConstantContent_value (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputConstantContent_value (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputConstantContent_value)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputConstantContent_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputRefContent_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputRefContent_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputRefContent_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputRefContent_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputProjectionContent_column (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputProjectionContent_column (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputProjectionContent_column)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputProjectionContent_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputNumerationContent_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputNumerationContent_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputNumerationContent_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputNumerationContent_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ListOutputActionConnectionsContent_actionConnections (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ListOutputActionConnectionsContent_actionConnections)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ListOutputActionConnectionsContent_actionConnections */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionConnection (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionConnection (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionConnection)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionConnection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionConnection_action (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionConnection_action (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionConnection_action)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionConnection_action */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionConnection_imageName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionConnection_imageName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionConnection_imageName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionConnection_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionConnection_text (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionConnection_text (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionConnection_text)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionConnection_text */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ActionConnection_actualParams (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ActionConnection_actualParams (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ActionConnection_actualParams)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ActionConnection_actualParams */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LinkTargetExtern_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LinkTargetExtern_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LinkTargetExtern_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LinkTargetExtern_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_LinkTargetExtern_url (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_LinkTargetExtern_url (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_LinkTargetExtern_url)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_LinkTargetExtern_url */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap_styleSheetBlock (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap_styleSheetBlock (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap_styleSheetBlock)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap_name (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap_name (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap_name)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap_imageName (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap_imageName (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap_imageName)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap_altText (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap_altText (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap_altText)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap_altText */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_ImageMap_mapAreas (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_ImageMap_mapAreas (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_ImageMap_mapAreas)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_ImageMap_mapAreas */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MapArea (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MapArea (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MapArea)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MapArea */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MapArea_altText (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MapArea_altText (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MapArea_altText)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MapArea_altText */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MapArea_coords (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MapArea_coords (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MapArea_coords)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MapArea_coords */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MapArea_link (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MapArea_link (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MapArea_link)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MapArea_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkcodeGen_MapArea_queryString (POSITION *_coordref, NODEPTR _currn)
#else
NODEPTR codeGen_MkcodeGen_MapArea_queryString (_coordref, _currn)
	POSITION *_coordref; NODEPTR _currn;
#endif
{
if (_currn == NULLNODEPTR) return NULLNODEPTR;
if (codeGen_IsSymb (_currn, codeGen_SYMBcodeGen_MapArea_queryString)) return (_currn);
return(NULLNODEPTR);
}/* codeGen_MkcodeGen_MapArea_queryString */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpEmpty (POSITION *_coordref)
#else
NODEPTR codeGen_MkpEmpty (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpEmpty _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpEmpty;
#else
_currn = (codeGen__TPPpEmpty) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpEmpty));
#endif
_currn->_prod = codeGen_RULEpEmpty;
_SETCOORD(_currn)
codeGen__TERMACT_pEmpty;
return ( (NODEPTR) _currn);
}/* codeGen_MkpEmpty */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpVIEWROOT_codeGen (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpVIEWROOT_codeGen (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpVIEWROOT_codeGen _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpVIEWROOT_codeGen;
#else
_currn = (codeGen__TPPpVIEWROOT_codeGen) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpVIEWROOT_codeGen));
#endif
_currn->_prod = codeGen_RULEpVIEWROOT_codeGen;
_currn->_desc1 = (codeGen__TSPcodeGen_Root) codeGen_MkcodeGen_Root (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pVIEWROOT_codeGen: root of subtree no. 1 can not be made a codeGen_Root node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_pVIEWROOT_codeGen;
return ( (NODEPTR) _currn);
}/* codeGen_MkpVIEWROOT_codeGen */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Root (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_Root (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_Root _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Root;
#else
_currn = (codeGen__TPPpcodeGen_Root) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Root));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Root;
_currn->_desc1 = (codeGen__TSPcodeGen_Root_webApp) codeGen_MkcodeGen_Root_webApp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Root: root of subtree no. 1 can not be made a codeGen_Root_webApp node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Root_library) codeGen_MkcodeGen_Root_library (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Root: root of subtree no. 2 can not be made a codeGen_Root_library node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Root;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Root */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LibrarypcodeGen_Root_library (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LibrarypcodeGen_Root_library (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LibrarypcodeGen_Root_library _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LibrarypcodeGen_Root_library;
#else
	_new = (codeGen__TPPLST_codeGen_LibrarypcodeGen_Root_library) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LibrarypcodeGen_Root_library));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LibrarypcodeGen_Root_library;
	_new->_desc1 = (codeGen__TSPcodeGen_Library)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Root_library)_new;
	{codeGen__TPPLST_codeGen_LibrarypcodeGen_Root_library _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LibrarypcodeGen_Root_library;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Root_library (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Root_library (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Library)) 
		return codeGen__MkcodeGen_LibrarypcodeGen_Root_library (_coordref, codeGen_MkcodeGen_Library (_coordref, t));
	message (DEADLY, "a pcodeGen_Root_library list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Root_library (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Root_library (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Root_library)
	l =_ElempcodeGen_Root_library (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Root_library)
	r = _ElempcodeGen_Root_library (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Root_library)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Root_library)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Root_library)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Root_library)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Root_library (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Root_library (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Root_library root;
	codeGen__TPPLST_0pcodeGen_Root_library list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Root_library)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Root_library)
		t = _ElempcodeGen_Root_library (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Root_library)(((codeGen__TPPLST_0pcodeGen_Root_library) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Root_library) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Root_library;
#else
list = (codeGen__TPPLST_0pcodeGen_Root_library) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Root_library));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Root_library;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Root_library)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LibrarypcodeGen_Root_library)
	{codeGen__TPPLST_codeGen_LibrarypcodeGen_Root_library _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LibrarypcodeGen_Root_library;
	} this code has been moved to _ElempcodeGen_Root_library*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Root_library;
#else
root = (codeGen__TPPpcodeGen_Root_library) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Root_library));
#endif
root->_prod = codeGen_RULEpcodeGen_Root_library;
root->_desc1 = (codeGen__TSPLST_codeGen_Root_library)list;
{ codeGen__TPPpcodeGen_Root_library _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Root_library;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Library (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_Library (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_Library _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Library;
#else
_currn = (codeGen__TPPpcodeGen_Library) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Library));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Library;
_currn->_desc1 = (codeGen__TSPcodeGen_Library_name) codeGen_MkcodeGen_Library_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Library: root of subtree no. 1 can not be made a codeGen_Library_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Library_navigationElement) codeGen_MkcodeGen_Library_navigationElement (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Library: root of subtree no. 2 can not be made a codeGen_Library_navigationElement node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Library;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Library */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Library_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Library_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Library_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Library_name;
#else
_currn = (codeGen__TPPpcodeGen_Library_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Library_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Library_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Library_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Library_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_NavigationElementpcodeGen_Library_navigationElement (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_NavigationElementpcodeGen_Library_navigationElement (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement;
#else
	_new = (codeGen__TPPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement));
#endif
	_new->_prod = codeGen_RULELST_codeGen_NavigationElementpcodeGen_Library_navigationElement;
	_new->_desc1 = (codeGen__TSPcodeGen_NavigationElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Library_navigationElement)_new;
	{codeGen__TPPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_NavigationElementpcodeGen_Library_navigationElement;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Library_navigationElement (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Library_navigationElement (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_NavigationElement)) 
		return codeGen__MkcodeGen_NavigationElementpcodeGen_Library_navigationElement (_coordref, codeGen_MkcodeGen_NavigationElement (_coordref, t));
	message (DEADLY, "a pcodeGen_Library_navigationElement list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Library_navigationElement (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Library_navigationElement (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Library_navigationElement)
	l =_ElempcodeGen_Library_navigationElement (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Library_navigationElement)
	r = _ElempcodeGen_Library_navigationElement (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Library_navigationElement)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Library_navigationElement)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Library_navigationElement)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Library_navigationElement)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Library_navigationElement (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Library_navigationElement (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Library_navigationElement root;
	codeGen__TPPLST_0pcodeGen_Library_navigationElement list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Library_navigationElement)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Library_navigationElement)
		t = _ElempcodeGen_Library_navigationElement (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Library_navigationElement)(((codeGen__TPPLST_0pcodeGen_Library_navigationElement) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Library_navigationElement) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Library_navigationElement;
#else
list = (codeGen__TPPLST_0pcodeGen_Library_navigationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Library_navigationElement));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Library_navigationElement;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Library_navigationElement)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_NavigationElementpcodeGen_Library_navigationElement)
	{codeGen__TPPLST_codeGen_NavigationElementpcodeGen_Library_navigationElement _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_NavigationElementpcodeGen_Library_navigationElement;
	} this code has been moved to _ElempcodeGen_Library_navigationElement*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Library_navigationElement;
#else
root = (codeGen__TPPpcodeGen_Library_navigationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Library_navigationElement));
#endif
root->_prod = codeGen_RULEpcodeGen_Library_navigationElement;
root->_desc1 = (codeGen__TSPLST_codeGen_Library_navigationElement)list;
{ codeGen__TPPpcodeGen_Library_navigationElement _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Library_navigationElement;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_NavigationElement (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_NavigationElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationElement;
#else
_currn = (codeGen__TPPpcodeGen_NavigationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationElement;
_currn->_desc1 = (codeGen__TSPcodeGen_NavigationElement_code) codeGen_MkcodeGen_NavigationElement_code (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationElement: root of subtree no. 1 can not be made a codeGen_NavigationElement_code node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_NavigationElement_name) codeGen_MkcodeGen_NavigationElement_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationElement: root of subtree no. 2 can not be made a codeGen_NavigationElement_name node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationElement_code (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationElement_code (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationElement_code _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationElement_code;
#else
_currn = (codeGen__TPPpcodeGen_NavigationElement_code) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationElement_code));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationElement_code;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationElement_code;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationElement_code */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationElement_name;
#else
_currn = (codeGen__TPPpcodeGen_NavigationElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9, NODEPTR _desc10, NODEPTR _desc11, NODEPTR _desc12, NODEPTR _desc13, NODEPTR _desc14, NODEPTR _desc15, NODEPTR _desc16, NODEPTR _desc17, NODEPTR _desc18)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9,_desc10,_desc11,_desc12,_desc13,_desc14,_desc15,_desc16,_desc17,_desc18)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
	NODEPTR _desc10;
	NODEPTR _desc11;
	NODEPTR _desc12;
	NODEPTR _desc13;
	NODEPTR _desc14;
	NODEPTR _desc15;
	NODEPTR _desc16;
	NODEPTR _desc17;
	NODEPTR _desc18;
#endif
{	codeGen__TPPpcodeGen_WebApplication _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication;
_currn->_desc1 = (codeGen__TSPcodeGen_WebApplication_name) codeGen_MkcodeGen_WebApplication_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 1 can not be made a codeGen_WebApplication_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebApplication_entrancePage) codeGen_MkcodeGen_WebApplication_entrancePage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 2 can not be made a codeGen_WebApplication_entrancePage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_WebApplication_webZone) codeGen_MkcodeGen_WebApplication_webZone (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 3 can not be made a codeGen_WebApplication_webZone node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_WebApplication_navigationBar) codeGen_MkcodeGen_WebApplication_navigationBar (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 4 can not be made a codeGen_WebApplication_navigationBar node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_WebApplication_styleSheet) codeGen_MkcodeGen_WebApplication_styleSheet (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 5 can not be made a codeGen_WebApplication_styleSheet node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_WebApplication_frames) codeGen_MkcodeGen_WebApplication_frames (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 6 can not be made a codeGen_WebApplication_frames node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_WebApplication_meta) codeGen_MkcodeGen_WebApplication_meta (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 7 can not be made a codeGen_WebApplication_meta node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_WebApplication_database) codeGen_MkcodeGen_WebApplication_database (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 8 can not be made a codeGen_WebApplication_database node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_WebApplication_roles) codeGen_MkcodeGen_WebApplication_roles (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 9 can not be made a codeGen_WebApplication_roles node ", 0, _coordref);
_currn->_desc10 = (codeGen__TSPcodeGen_WebApplication_linksExtern) codeGen_MkcodeGen_WebApplication_linksExtern (_coordref, _desc10);	
if (((NODEPTR)_currn->_desc10) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 10 can not be made a codeGen_WebApplication_linksExtern node ", 0, _coordref);
_currn->_desc11 = (codeGen__TSPcodeGen_WebApplication_imagePath) codeGen_MkcodeGen_WebApplication_imagePath (_coordref, _desc11);	
if (((NODEPTR)_currn->_desc11) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 11 can not be made a codeGen_WebApplication_imagePath node ", 0, _coordref);
_currn->_desc12 = (codeGen__TSPcodeGen_WebApplication_packageName) codeGen_MkcodeGen_WebApplication_packageName (_coordref, _desc12);	
if (((NODEPTR)_currn->_desc12) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 12 can not be made a codeGen_WebApplication_packageName node ", 0, _coordref);
_currn->_desc13 = (codeGen__TSPcodeGen_WebApplication_pageEncoding) codeGen_MkcodeGen_WebApplication_pageEncoding (_coordref, _desc13);	
if (((NODEPTR)_currn->_desc13) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 13 can not be made a codeGen_WebApplication_pageEncoding node ", 0, _coordref);
_currn->_desc14 = (codeGen__TSPcodeGen_WebApplication_header) codeGen_MkcodeGen_WebApplication_header (_coordref, _desc14);	
if (((NODEPTR)_currn->_desc14) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 14 can not be made a codeGen_WebApplication_header node ", 0, _coordref);
_currn->_desc15 = (codeGen__TSPcodeGen_WebApplication_footer) codeGen_MkcodeGen_WebApplication_footer (_coordref, _desc15);	
if (((NODEPTR)_currn->_desc15) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 15 can not be made a codeGen_WebApplication_footer node ", 0, _coordref);
_currn->_desc16 = (codeGen__TSPcodeGen_WebApplication_mailHost) codeGen_MkcodeGen_WebApplication_mailHost (_coordref, _desc16);	
if (((NODEPTR)_currn->_desc16) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 16 can not be made a codeGen_WebApplication_mailHost node ", 0, _coordref);
_currn->_desc17 = (codeGen__TSPcodeGen_WebApplication_mailFromAddress) codeGen_MkcodeGen_WebApplication_mailFromAddress (_coordref, _desc17);	
if (((NODEPTR)_currn->_desc17) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 17 can not be made a codeGen_WebApplication_mailFromAddress node ", 0, _coordref);
_currn->_desc18 = (codeGen__TSPcodeGen_WebApplication_catalinaHome) codeGen_MkcodeGen_WebApplication_catalinaHome (_coordref, _desc18);	
if (((NODEPTR)_currn->_desc18) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebApplication: root of subtree no. 18 can not be made a codeGen_WebApplication_catalinaHome node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_name;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_entrancePage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_entrancePage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_entrancePage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_entrancePage;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_entrancePage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_entrancePage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_entrancePage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_entrancePage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_entrancePage */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_GlobalFramepcodeGen_WebApplication_frames (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_GlobalFramepcodeGen_WebApplication_frames (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames;
#else
	_new = (codeGen__TPPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames));
#endif
	_new->_prod = codeGen_RULELST_codeGen_GlobalFramepcodeGen_WebApplication_frames;
	_new->_desc1 = (codeGen__TSPcodeGen_GlobalFrame)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebApplication_frames)_new;
	{codeGen__TPPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_GlobalFramepcodeGen_WebApplication_frames;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebApplication_frames (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebApplication_frames (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_GlobalFrame)) 
		return codeGen__MkcodeGen_GlobalFramepcodeGen_WebApplication_frames (_coordref, codeGen_MkcodeGen_GlobalFrame (_coordref, t));
	message (DEADLY, "a pcodeGen_WebApplication_frames list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebApplication_frames (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebApplication_frames (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebApplication_frames)
	l =_ElempcodeGen_WebApplication_frames (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebApplication_frames)
	r = _ElempcodeGen_WebApplication_frames (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebApplication_frames)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebApplication_frames)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebApplication_frames)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebApplication_frames)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_frames (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_frames (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebApplication_frames root;
	codeGen__TPPLST_0pcodeGen_WebApplication_frames list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_frames)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebApplication_frames)
		t = _ElempcodeGen_WebApplication_frames (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_frames)(((codeGen__TPPLST_0pcodeGen_WebApplication_frames) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebApplication_frames) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebApplication_frames;
#else
list = (codeGen__TPPLST_0pcodeGen_WebApplication_frames) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebApplication_frames));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebApplication_frames;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebApplication_frames)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_GlobalFramepcodeGen_WebApplication_frames)
	{codeGen__TPPLST_codeGen_GlobalFramepcodeGen_WebApplication_frames _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_GlobalFramepcodeGen_WebApplication_frames;
	} this code has been moved to _ElempcodeGen_WebApplication_frames*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebApplication_frames;
#else
root = (codeGen__TPPpcodeGen_WebApplication_frames) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_frames));
#endif
root->_prod = codeGen_RULEpcodeGen_WebApplication_frames;
root->_desc1 = (codeGen__TSPLST_codeGen_WebApplication_frames)list;
{ codeGen__TPPpcodeGen_WebApplication_frames _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebApplication_frames;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebRolepcodeGen_WebApplication_roles (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebRolepcodeGen_WebApplication_roles (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebRolepcodeGen_WebApplication_roles _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebRolepcodeGen_WebApplication_roles;
#else
	_new = (codeGen__TPPLST_codeGen_WebRolepcodeGen_WebApplication_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebRolepcodeGen_WebApplication_roles));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebRolepcodeGen_WebApplication_roles;
	_new->_desc1 = (codeGen__TSPcodeGen_WebRole)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebApplication_roles)_new;
	{codeGen__TPPLST_codeGen_WebRolepcodeGen_WebApplication_roles _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebRolepcodeGen_WebApplication_roles;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebApplication_roles (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebApplication_roles (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebRole)) 
		return codeGen__MkcodeGen_WebRolepcodeGen_WebApplication_roles (_coordref, codeGen_MkcodeGen_WebRole (_coordref, t));
	message (DEADLY, "a pcodeGen_WebApplication_roles list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebApplication_roles (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebApplication_roles (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebApplication_roles)
	l =_ElempcodeGen_WebApplication_roles (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebApplication_roles)
	r = _ElempcodeGen_WebApplication_roles (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebApplication_roles)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebApplication_roles)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebApplication_roles)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebApplication_roles)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_roles (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_roles (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebApplication_roles root;
	codeGen__TPPLST_0pcodeGen_WebApplication_roles list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_roles)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebApplication_roles)
		t = _ElempcodeGen_WebApplication_roles (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_roles)(((codeGen__TPPLST_0pcodeGen_WebApplication_roles) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebApplication_roles) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebApplication_roles;
#else
list = (codeGen__TPPLST_0pcodeGen_WebApplication_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebApplication_roles));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebApplication_roles;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebApplication_roles)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebRolepcodeGen_WebApplication_roles)
	{codeGen__TPPLST_codeGen_WebRolepcodeGen_WebApplication_roles _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebRolepcodeGen_WebApplication_roles;
	} this code has been moved to _ElempcodeGen_WebApplication_roles*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebApplication_roles;
#else
root = (codeGen__TPPpcodeGen_WebApplication_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_roles));
#endif
root->_prod = codeGen_RULEpcodeGen_WebApplication_roles;
root->_desc1 = (codeGen__TSPLST_codeGen_WebApplication_roles)list;
{ codeGen__TPPpcodeGen_WebApplication_roles _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebApplication_roles;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern;
#else
	_new = (codeGen__TPPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern;
	_new->_desc1 = (codeGen__TSPcodeGen_LinkTargetExtern)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebApplication_linksExtern)_new;
	{codeGen__TPPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebApplication_linksExtern (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebApplication_linksExtern (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LinkTargetExtern)) 
		return codeGen__MkcodeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern (_coordref, codeGen_MkcodeGen_LinkTargetExtern (_coordref, t));
	message (DEADLY, "a pcodeGen_WebApplication_linksExtern list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebApplication_linksExtern (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebApplication_linksExtern (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebApplication_linksExtern)
	l =_ElempcodeGen_WebApplication_linksExtern (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebApplication_linksExtern)
	r = _ElempcodeGen_WebApplication_linksExtern (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_linksExtern (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_linksExtern (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebApplication_linksExtern root;
	codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebApplication_linksExtern)
		t = _ElempcodeGen_WebApplication_linksExtern (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)(((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebApplication_linksExtern;
#else
list = (codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebApplication_linksExtern));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebApplication_linksExtern;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebApplication_linksExtern)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern)
	{codeGen__TPPLST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LinkTargetExternpcodeGen_WebApplication_linksExtern;
	} this code has been moved to _ElempcodeGen_WebApplication_linksExtern*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebApplication_linksExtern;
#else
root = (codeGen__TPPpcodeGen_WebApplication_linksExtern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_linksExtern));
#endif
root->_prod = codeGen_RULEpcodeGen_WebApplication_linksExtern;
root->_desc1 = (codeGen__TSPLST_codeGen_WebApplication_linksExtern)list;
{ codeGen__TPPpcodeGen_WebApplication_linksExtern _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebApplication_linksExtern;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_imagePath (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_imagePath (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_imagePath _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_imagePath;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_imagePath) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_imagePath));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_imagePath;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_imagePath;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_imagePath */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_packageName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_packageName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_packageName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_packageName;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_packageName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_packageName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_packageName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_packageName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_packageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_pageEncoding (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_pageEncoding (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_pageEncoding _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_pageEncoding;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_pageEncoding) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_pageEncoding));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_pageEncoding;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_pageEncoding;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_pageEncoding */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_header (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_header (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_header _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_header;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_header) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_header));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_header;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_header;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_header */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_footer (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_footer (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_footer _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_footer;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_footer) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_footer));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_footer;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_footer;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_footer */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_mailHost (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_mailHost (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_mailHost _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_mailHost;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_mailHost) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_mailHost));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_mailHost;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_mailHost;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_mailHost */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_mailFromAddress (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_mailFromAddress (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_mailFromAddress _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_mailFromAddress;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_mailFromAddress) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_mailFromAddress));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_mailFromAddress;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_mailFromAddress;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_mailFromAddress */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebApplication_catalinaHome (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebApplication_catalinaHome (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebApplication_catalinaHome _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebApplication_catalinaHome;
#else
_currn = (codeGen__TPPpcodeGen_WebApplication_catalinaHome) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebApplication_catalinaHome));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebApplication_catalinaHome;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebApplication_catalinaHome;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebApplication_catalinaHome */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RootWebZone (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_RootWebZone (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_RootWebZone _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_RootWebZone;
#else
_currn = (codeGen__TPPpcodeGen_RootWebZone) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RootWebZone));
#endif
_currn->_prod = codeGen_RULEpcodeGen_RootWebZone;
_currn->_desc1 = (codeGen__TSPcodeGen_RootWebZone_webTargets) codeGen_MkcodeGen_RootWebZone_webTargets (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_RootWebZone: root of subtree no. 1 can not be made a codeGen_RootWebZone_webTargets node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_RootWebZone_errorLink) codeGen_MkcodeGen_RootWebZone_errorLink (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_RootWebZone: root of subtree no. 2 can not be made a codeGen_RootWebZone_errorLink node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_RootWebZone_roles) codeGen_MkcodeGen_RootWebZone_roles (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_RootWebZone: root of subtree no. 3 can not be made a codeGen_RootWebZone_roles node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_RootWebZone_setSize) codeGen_MkcodeGen_RootWebZone_setSize (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_RootWebZone: root of subtree no. 4 can not be made a codeGen_RootWebZone_setSize node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_RootWebZone;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_RootWebZone */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_SubWebZonepcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_SubWebZonepcodeGen_RootWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_SubWebZone)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_RootWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebPagepcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebPagepcodeGen_RootWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebPagepcodeGen_RootWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_WebPage)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_RootWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebPagepcodeGen_RootWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebActionpcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebActionpcodeGen_RootWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebActionpcodeGen_RootWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_WebAction)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_RootWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebActionpcodeGen_RootWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_RootWebZone_webTargets (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_SubWebZone)) 
		return codeGen__MkcodeGen_SubWebZonepcodeGen_RootWebZone_webTargets (_coordref, codeGen_MkcodeGen_SubWebZone (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebPage)) 
		return codeGen__MkcodeGen_WebPagepcodeGen_RootWebZone_webTargets (_coordref, codeGen_MkcodeGen_WebPage (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebAction)) 
		return codeGen__MkcodeGen_WebActionpcodeGen_RootWebZone_webTargets (_coordref, codeGen_MkcodeGen_WebAction (_coordref, t));
	message (DEADLY, "a pcodeGen_RootWebZone_webTargets list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_RootWebZone_webTargets (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_webTargets)
	l =_ElempcodeGen_RootWebZone_webTargets (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_webTargets)
	r = _ElempcodeGen_RootWebZone_webTargets (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RootWebZone_webTargets (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_RootWebZone_webTargets (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_RootWebZone_webTargets root;
	codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_webTargets)
		t = _ElempcodeGen_RootWebZone_webTargets (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)(((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_RootWebZone_webTargets;
#else
list = (codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_RootWebZone_webTargets));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_RootWebZone_webTargets;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_RootWebZone_webTargets)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets)
	{codeGen__TPPLST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_SubWebZonepcodeGen_RootWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_RootWebZone_webTargets*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebPagepcodeGen_RootWebZone_webTargets)
	{codeGen__TPPLST_codeGen_WebPagepcodeGen_RootWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebPagepcodeGen_RootWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_RootWebZone_webTargets*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebActionpcodeGen_RootWebZone_webTargets)
	{codeGen__TPPLST_codeGen_WebActionpcodeGen_RootWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebActionpcodeGen_RootWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_RootWebZone_webTargets*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_RootWebZone_webTargets;
#else
root = (codeGen__TPPpcodeGen_RootWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RootWebZone_webTargets));
#endif
root->_prod = codeGen_RULEpcodeGen_RootWebZone_webTargets;
root->_desc1 = (codeGen__TSPLST_codeGen_RootWebZone_webTargets)list;
{ codeGen__TPPpcodeGen_RootWebZone_webTargets _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_RootWebZone_webTargets;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RootWebZone_errorLink (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_RootWebZone_errorLink (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_RootWebZone_errorLink _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_RootWebZone_errorLink;
#else
_currn = (codeGen__TPPpcodeGen_RootWebZone_errorLink) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RootWebZone_errorLink));
#endif
_currn->_prod = codeGen_RULEpcodeGen_RootWebZone_errorLink;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_RootWebZone_errorLink;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_RootWebZone_errorLink */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebRoleRefpcodeGen_RootWebZone_roles (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebRoleRefpcodeGen_RootWebZone_roles (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles;
#else
	_new = (codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles;
	_new->_desc1 = (codeGen__TSPcodeGen_WebRoleRef)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_RootWebZone_roles)_new;
	{codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_RootWebZone_roles (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_RootWebZone_roles (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebRoleRef)) 
		return codeGen__MkcodeGen_WebRoleRefpcodeGen_RootWebZone_roles (_coordref, codeGen_MkcodeGen_WebRoleRef (_coordref, t));
	message (DEADLY, "a pcodeGen_RootWebZone_roles list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_RootWebZone_roles (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_RootWebZone_roles (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_roles)
	l =_ElempcodeGen_RootWebZone_roles (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_roles)
	r = _ElempcodeGen_RootWebZone_roles (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_RootWebZone_roles)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_RootWebZone_roles)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_RootWebZone_roles)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_RootWebZone_roles)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RootWebZone_roles (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_RootWebZone_roles (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_RootWebZone_roles root;
	codeGen__TPPLST_0pcodeGen_RootWebZone_roles list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_RootWebZone_roles)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_RootWebZone_roles)
		t = _ElempcodeGen_RootWebZone_roles (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_RootWebZone_roles)(((codeGen__TPPLST_0pcodeGen_RootWebZone_roles) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_RootWebZone_roles) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_RootWebZone_roles;
#else
list = (codeGen__TPPLST_0pcodeGen_RootWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_RootWebZone_roles));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_RootWebZone_roles;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_RootWebZone_roles)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles)
	{codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebRoleRefpcodeGen_RootWebZone_roles;
	} this code has been moved to _ElempcodeGen_RootWebZone_roles*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_RootWebZone_roles;
#else
root = (codeGen__TPPpcodeGen_RootWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RootWebZone_roles));
#endif
root->_prod = codeGen_RULEpcodeGen_RootWebZone_roles;
root->_desc1 = (codeGen__TSPLST_codeGen_RootWebZone_roles)list;
{ codeGen__TPPpcodeGen_RootWebZone_roles _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_RootWebZone_roles;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RootWebZone_setSize (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_RootWebZone_setSize (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_RootWebZone_setSize _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_RootWebZone_setSize;
#else
_currn = (codeGen__TPPpcodeGen_RootWebZone_setSize) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RootWebZone_setSize));
#endif
_currn->_prod = codeGen_RULEpcodeGen_RootWebZone_setSize;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_RootWebZone_setSize;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_RootWebZone_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	codeGen__TPPpcodeGen_SubWebZone _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SubWebZone;
#else
_currn = (codeGen__TPPpcodeGen_SubWebZone) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SubWebZone;
_currn->_desc1 = (codeGen__TSPcodeGen_SubWebZone_name) codeGen_MkcodeGen_SubWebZone_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 1 can not be made a codeGen_SubWebZone_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_SubWebZone_webTargets) codeGen_MkcodeGen_SubWebZone_webTargets (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 2 can not be made a codeGen_SubWebZone_webTargets node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_SubWebZone_errorLink) codeGen_MkcodeGen_SubWebZone_errorLink (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 3 can not be made a codeGen_SubWebZone_errorLink node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_SubWebZone_roles) codeGen_MkcodeGen_SubWebZone_roles (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 4 can not be made a codeGen_SubWebZone_roles node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_SubWebZone_setSize) codeGen_MkcodeGen_SubWebZone_setSize (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 5 can not be made a codeGen_SubWebZone_setSize node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_SubWebZone_position) codeGen_MkcodeGen_SubWebZone_position (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SubWebZone: root of subtree no. 6 can not be made a codeGen_SubWebZone_position node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SubWebZone;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SubWebZone */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SubWebZone_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SubWebZone_name;
#else
_currn = (codeGen__TPPpcodeGen_SubWebZone_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SubWebZone_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SubWebZone_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SubWebZone_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_SubWebZonepcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_SubWebZonepcodeGen_SubWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_SubWebZone)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_SubWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebPagepcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebPagepcodeGen_SubWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebPagepcodeGen_SubWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_WebPage)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_SubWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebPagepcodeGen_SubWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebActionpcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebActionpcodeGen_SubWebZone_webTargets (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets;
#else
	_new = (codeGen__TPPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebActionpcodeGen_SubWebZone_webTargets;
	_new->_desc1 = (codeGen__TSPcodeGen_WebAction)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_SubWebZone_webTargets)_new;
	{codeGen__TPPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebActionpcodeGen_SubWebZone_webTargets;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_SubWebZone_webTargets (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_SubWebZone)) 
		return codeGen__MkcodeGen_SubWebZonepcodeGen_SubWebZone_webTargets (_coordref, codeGen_MkcodeGen_SubWebZone (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebPage)) 
		return codeGen__MkcodeGen_WebPagepcodeGen_SubWebZone_webTargets (_coordref, codeGen_MkcodeGen_WebPage (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebAction)) 
		return codeGen__MkcodeGen_WebActionpcodeGen_SubWebZone_webTargets (_coordref, codeGen_MkcodeGen_WebAction (_coordref, t));
	message (DEADLY, "a pcodeGen_SubWebZone_webTargets list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_SubWebZone_webTargets (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_webTargets)
	l =_ElempcodeGen_SubWebZone_webTargets (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_webTargets)
	r = _ElempcodeGen_SubWebZone_webTargets (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_webTargets (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_webTargets (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_SubWebZone_webTargets root;
	codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_webTargets)
		t = _ElempcodeGen_SubWebZone_webTargets (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)(((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_SubWebZone_webTargets;
#else
list = (codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_SubWebZone_webTargets));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_SubWebZone_webTargets;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_SubWebZone_webTargets)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets)
	{codeGen__TPPLST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_SubWebZonepcodeGen_SubWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_SubWebZone_webTargets*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebPagepcodeGen_SubWebZone_webTargets)
	{codeGen__TPPLST_codeGen_WebPagepcodeGen_SubWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebPagepcodeGen_SubWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_SubWebZone_webTargets*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebActionpcodeGen_SubWebZone_webTargets)
	{codeGen__TPPLST_codeGen_WebActionpcodeGen_SubWebZone_webTargets _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebActionpcodeGen_SubWebZone_webTargets;
	} this code has been moved to _ElempcodeGen_SubWebZone_webTargets*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_SubWebZone_webTargets;
#else
root = (codeGen__TPPpcodeGen_SubWebZone_webTargets) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_webTargets));
#endif
root->_prod = codeGen_RULEpcodeGen_SubWebZone_webTargets;
root->_desc1 = (codeGen__TSPLST_codeGen_SubWebZone_webTargets)list;
{ codeGen__TPPpcodeGen_SubWebZone_webTargets _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_SubWebZone_webTargets;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_errorLink (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_errorLink (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SubWebZone_errorLink _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SubWebZone_errorLink;
#else
_currn = (codeGen__TPPpcodeGen_SubWebZone_errorLink) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_errorLink));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SubWebZone_errorLink;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SubWebZone_errorLink;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SubWebZone_errorLink */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebRoleRefpcodeGen_SubWebZone_roles (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebRoleRefpcodeGen_SubWebZone_roles (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles;
#else
	_new = (codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles;
	_new->_desc1 = (codeGen__TSPcodeGen_WebRoleRef)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_SubWebZone_roles)_new;
	{codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_SubWebZone_roles (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_SubWebZone_roles (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebRoleRef)) 
		return codeGen__MkcodeGen_WebRoleRefpcodeGen_SubWebZone_roles (_coordref, codeGen_MkcodeGen_WebRoleRef (_coordref, t));
	message (DEADLY, "a pcodeGen_SubWebZone_roles list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_SubWebZone_roles (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_SubWebZone_roles (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_roles)
	l =_ElempcodeGen_SubWebZone_roles (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_roles)
	r = _ElempcodeGen_SubWebZone_roles (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_SubWebZone_roles)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_SubWebZone_roles)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_SubWebZone_roles)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_SubWebZone_roles)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_roles (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_roles (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_SubWebZone_roles root;
	codeGen__TPPLST_0pcodeGen_SubWebZone_roles list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_SubWebZone_roles)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_SubWebZone_roles)
		t = _ElempcodeGen_SubWebZone_roles (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_SubWebZone_roles)(((codeGen__TPPLST_0pcodeGen_SubWebZone_roles) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_SubWebZone_roles) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_SubWebZone_roles;
#else
list = (codeGen__TPPLST_0pcodeGen_SubWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_SubWebZone_roles));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_SubWebZone_roles;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_SubWebZone_roles)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles)
	{codeGen__TPPLST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebRoleRefpcodeGen_SubWebZone_roles;
	} this code has been moved to _ElempcodeGen_SubWebZone_roles*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_SubWebZone_roles;
#else
root = (codeGen__TPPpcodeGen_SubWebZone_roles) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_roles));
#endif
root->_prod = codeGen_RULEpcodeGen_SubWebZone_roles;
root->_desc1 = (codeGen__TSPLST_codeGen_SubWebZone_roles)list;
{ codeGen__TPPpcodeGen_SubWebZone_roles _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_SubWebZone_roles;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_setSize (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_setSize (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SubWebZone_setSize _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SubWebZone_setSize;
#else
_currn = (codeGen__TPPpcodeGen_SubWebZone_setSize) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_setSize));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SubWebZone_setSize;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SubWebZone_setSize;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SubWebZone_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SubWebZone_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SubWebZone_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SubWebZone_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SubWebZone_position;
#else
_currn = (codeGen__TPPpcodeGen_SubWebZone_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SubWebZone_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SubWebZone_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SubWebZone_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SubWebZone_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebRole (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebRole (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebRole _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebRole;
#else
_currn = (codeGen__TPPpcodeGen_WebRole) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebRole));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebRole;
_currn->_desc1 = (codeGen__TSPcodeGen_WebRole_name) codeGen_MkcodeGen_WebRole_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebRole: root of subtree no. 1 can not be made a codeGen_WebRole_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebRole_color) codeGen_MkcodeGen_WebRole_color (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebRole: root of subtree no. 2 can not be made a codeGen_WebRole_color node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebRole;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebRole */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebRole_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebRole_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebRole_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebRole_name;
#else
_currn = (codeGen__TPPpcodeGen_WebRole_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebRole_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebRole_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebRole_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebRole_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebRole_color (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebRole_color (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebRole_color _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebRole_color;
#else
_currn = (codeGen__TPPpcodeGen_WebRole_color) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebRole_color));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebRole_color;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebRole_color;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebRole_color */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebRoleRef (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_WebRoleRef (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_WebRoleRef _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebRoleRef;
#else
_currn = (codeGen__TPPpcodeGen_WebRoleRef) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebRoleRef));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebRoleRef;
_currn->_desc1 = (codeGen__TSPcodeGen_WebRoleRef_role) codeGen_MkcodeGen_WebRoleRef_role (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebRoleRef: root of subtree no. 1 can not be made a codeGen_WebRoleRef_role node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebRoleRef;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebRoleRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebRoleRef_role (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebRoleRef_role (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebRoleRef_role _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebRoleRef_role;
#else
_currn = (codeGen__TPPpcodeGen_WebRoleRef_role) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebRoleRef_role));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebRoleRef_role;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebRoleRef_role;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebRoleRef_role */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR codeGen_MkpcodeGen_WebPage (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	codeGen__TPPpcodeGen_WebPage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebPage;
#else
_currn = (codeGen__TPPpcodeGen_WebPage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebPage;
_currn->_desc1 = (codeGen__TSPcodeGen_WebPage_name) codeGen_MkcodeGen_WebPage_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 1 can not be made a codeGen_WebPage_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebPage_formalParameter) codeGen_MkcodeGen_WebPage_formalParameter (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 2 can not be made a codeGen_WebPage_formalParameter node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_WebPage_title) codeGen_MkcodeGen_WebPage_title (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 3 can not be made a codeGen_WebPage_title node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_WebPage_frame) codeGen_MkcodeGen_WebPage_frame (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 4 can not be made a codeGen_WebPage_frame node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_WebPage_styleSheetBlock) codeGen_MkcodeGen_WebPage_styleSheetBlock (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 5 can not be made a codeGen_WebPage_styleSheetBlock node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_WebPage_position) codeGen_MkcodeGen_WebPage_position (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebPage: root of subtree no. 6 can not be made a codeGen_WebPage_position node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebPage;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebPage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebPage_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebPage_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebPage_name;
#else
_currn = (codeGen__TPPpcodeGen_WebPage_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebPage_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebPage_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebPage_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter;
#else
	_new = (codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter;
	_new->_desc1 = (codeGen__TSPcodeGen_WebNodeFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebPage_formalParameter)_new;
	{codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebPage_formalParameter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebPage_formalParameter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebNodeFormalParam)) 
		return codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter (_coordref, codeGen_MkcodeGen_WebNodeFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_WebPage_formalParameter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebPage_formalParameter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebPage_formalParameter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebPage_formalParameter)
	l =_ElempcodeGen_WebPage_formalParameter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebPage_formalParameter)
	r = _ElempcodeGen_WebPage_formalParameter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage_formalParameter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebPage_formalParameter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebPage_formalParameter root;
	codeGen__TPPLST_0pcodeGen_WebPage_formalParameter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebPage_formalParameter)
		t = _ElempcodeGen_WebPage_formalParameter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)(((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebPage_formalParameter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebPage_formalParameter;
#else
list = (codeGen__TPPLST_0pcodeGen_WebPage_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebPage_formalParameter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebPage_formalParameter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebPage_formalParameter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter)
	{codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebNodeFormalParampcodeGen_WebPage_formalParameter;
	} this code has been moved to _ElempcodeGen_WebPage_formalParameter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebPage_formalParameter;
#else
root = (codeGen__TPPpcodeGen_WebPage_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage_formalParameter));
#endif
root->_prod = codeGen_RULEpcodeGen_WebPage_formalParameter;
root->_desc1 = (codeGen__TSPLST_codeGen_WebPage_formalParameter)list;
{ codeGen__TPPpcodeGen_WebPage_formalParameter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebPage_formalParameter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage_title (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebPage_title (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebPage_title _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebPage_title;
#else
_currn = (codeGen__TPPpcodeGen_WebPage_title) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage_title));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebPage_title;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebPage_title;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebPage_title */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebPage_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebPage_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebPage_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_WebPage_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebPage_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebPage_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebPage_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebPage_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebPage_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebPage_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebPage_position;
#else
_currn = (codeGen__TPPpcodeGen_WebPage_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebPage_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebPage_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebPage_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebPage_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Frame (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_Frame (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_Frame _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Frame;
#else
_currn = (codeGen__TPPpcodeGen_Frame) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Frame));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Frame;
_currn->_desc1 = (codeGen__TSPcodeGen_Frame_components) codeGen_MkcodeGen_Frame_components (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Frame: root of subtree no. 1 can not be made a codeGen_Frame_components node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Frame;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Frame */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FrameRefpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FrameRefpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FrameRefpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FrameRefpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_FrameRefpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FrameRefpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FrameRefpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_FrameRef)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_FrameRefpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FrameRefpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ParagraphpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ParagraphpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ParagraphpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ParagraphpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ParagraphpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ParagraphpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ParagraphpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Paragraph)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_ParagraphpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ParagraphpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_HeadingpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_HeadingpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_HeadingpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_HeadingpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_HeadingpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_HeadingpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_HeadingpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Heading)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_HeadingpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_HeadingpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TablepcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TablepcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TablepcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TablepcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_TablepcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TablepcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TablepcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Table)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_TablepcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TablepcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_FormularpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Formular)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_FormularpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_OutputMessagesElementpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_OutputMessagesElementpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_OutputMessagesElementpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_OutputMessagesElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_OutputMessagesElementpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ListOutputElementpcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ListOutputElementpcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ListOutputElementpcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ListOutputElementpcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ListOutputElementpcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_ListOutputElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ListOutputElementpcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ImageMappcodeGen_Frame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ImageMappcodeGen_Frame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ImageMappcodeGen_Frame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ImageMappcodeGen_Frame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ImageMappcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ImageMappcodeGen_Frame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ImageMappcodeGen_Frame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_ImageMap)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Frame_components)_new;
	{codeGen__TPPLST_codeGen_ImageMappcodeGen_Frame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ImageMappcodeGen_Frame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Frame_components (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Frame_components (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FrameRef)) 
		return codeGen__MkcodeGen_FrameRefpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_FrameRef (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Paragraph)) 
		return codeGen__MkcodeGen_ParagraphpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_Paragraph (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Heading)) 
		return codeGen__MkcodeGen_HeadingpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_Heading (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Table)) 
		return codeGen__MkcodeGen_TablepcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_Table (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Formular)) 
		return codeGen__MkcodeGen_FormularpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_Formular (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_OutputMessagesElement)) 
		return codeGen__MkcodeGen_OutputMessagesElementpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_OutputMessagesElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ListOutputElement)) 
		return codeGen__MkcodeGen_ListOutputElementpcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_ListOutputElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ImageMap)) 
		return codeGen__MkcodeGen_ImageMappcodeGen_Frame_components (_coordref, codeGen_MkcodeGen_ImageMap (_coordref, t));
	message (DEADLY, "a pcodeGen_Frame_components list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Frame_components (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Frame_components (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Frame_components)
	l =_ElempcodeGen_Frame_components (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Frame_components)
	r = _ElempcodeGen_Frame_components (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Frame_components)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Frame_components)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Frame_components)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Frame_components)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Frame_components (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Frame_components (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Frame_components root;
	codeGen__TPPLST_0pcodeGen_Frame_components list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Frame_components)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Frame_components)
		t = _ElempcodeGen_Frame_components (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Frame_components)(((codeGen__TPPLST_0pcodeGen_Frame_components) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Frame_components) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Frame_components;
#else
list = (codeGen__TPPLST_0pcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Frame_components));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Frame_components;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Frame_components)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FrameRefpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_FrameRefpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FrameRefpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ParagraphpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_ParagraphpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ParagraphpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_HeadingpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_HeadingpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_HeadingpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TablepcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_TablepcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TablepcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_FormularpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_OutputMessagesElementpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_OutputMessagesElementpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ListOutputElementpcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ListOutputElementpcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ImageMappcodeGen_Frame_components)
	{codeGen__TPPLST_codeGen_ImageMappcodeGen_Frame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ImageMappcodeGen_Frame_components;
	} this code has been moved to _ElempcodeGen_Frame_components*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Frame_components;
#else
root = (codeGen__TPPpcodeGen_Frame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Frame_components));
#endif
root->_prod = codeGen_RULEpcodeGen_Frame_components;
root->_desc1 = (codeGen__TSPLST_codeGen_Frame_components)list;
{ codeGen__TPPpcodeGen_Frame_components _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Frame_components;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_GlobalFrame (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_GlobalFrame (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_GlobalFrame _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_GlobalFrame;
#else
_currn = (codeGen__TPPpcodeGen_GlobalFrame) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_GlobalFrame));
#endif
_currn->_prod = codeGen_RULEpcodeGen_GlobalFrame;
_currn->_desc1 = (codeGen__TSPcodeGen_GlobalFrame_name) codeGen_MkcodeGen_GlobalFrame_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_GlobalFrame: root of subtree no. 1 can not be made a codeGen_GlobalFrame_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_GlobalFrame_components) codeGen_MkcodeGen_GlobalFrame_components (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_GlobalFrame: root of subtree no. 2 can not be made a codeGen_GlobalFrame_components node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_GlobalFrame_setSize) codeGen_MkcodeGen_GlobalFrame_setSize (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_GlobalFrame: root of subtree no. 3 can not be made a codeGen_GlobalFrame_setSize node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_GlobalFrame;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_GlobalFrame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_GlobalFrame_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_GlobalFrame_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_GlobalFrame_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_GlobalFrame_name;
#else
_currn = (codeGen__TPPpcodeGen_GlobalFrame_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_GlobalFrame_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_GlobalFrame_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_GlobalFrame_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_GlobalFrame_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FrameRefpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FrameRefpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FrameRefpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_FrameRef)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FrameRefpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ParagraphpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ParagraphpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ParagraphpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Paragraph)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ParagraphpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_HeadingpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_HeadingpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_HeadingpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_HeadingpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_HeadingpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_HeadingpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_HeadingpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Heading)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_HeadingpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_HeadingpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TablepcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TablepcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TablepcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TablepcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_TablepcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TablepcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TablepcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Table)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_TablepcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TablepcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_FormularpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_Formular)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_FormularpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_OutputMessagesElementpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_OutputMessagesElementpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_OutputMessagesElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ListOutputElementpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ListOutputElementpcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_ListOutputElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ImageMappcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ImageMappcodeGen_GlobalFrame_components (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ImageMappcodeGen_GlobalFrame_components _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ImageMappcodeGen_GlobalFrame_components;
#else
	_new = (codeGen__TPPLST_codeGen_ImageMappcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ImageMappcodeGen_GlobalFrame_components));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ImageMappcodeGen_GlobalFrame_components;
	_new->_desc1 = (codeGen__TSPcodeGen_ImageMap)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_GlobalFrame_components)_new;
	{codeGen__TPPLST_codeGen_ImageMappcodeGen_GlobalFrame_components _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ImageMappcodeGen_GlobalFrame_components;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_GlobalFrame_components (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FrameRef)) 
		return codeGen__MkcodeGen_FrameRefpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_FrameRef (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Paragraph)) 
		return codeGen__MkcodeGen_ParagraphpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_Paragraph (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Heading)) 
		return codeGen__MkcodeGen_HeadingpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_Heading (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Table)) 
		return codeGen__MkcodeGen_TablepcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_Table (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_Formular)) 
		return codeGen__MkcodeGen_FormularpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_Formular (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_OutputMessagesElement)) 
		return codeGen__MkcodeGen_OutputMessagesElementpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_OutputMessagesElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ListOutputElement)) 
		return codeGen__MkcodeGen_ListOutputElementpcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_ListOutputElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ImageMap)) 
		return codeGen__MkcodeGen_ImageMappcodeGen_GlobalFrame_components (_coordref, codeGen_MkcodeGen_ImageMap (_coordref, t));
	message (DEADLY, "a pcodeGen_GlobalFrame_components list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_GlobalFrame_components (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_GlobalFrame_components)
	l =_ElempcodeGen_GlobalFrame_components (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_GlobalFrame_components)
	r = _ElempcodeGen_GlobalFrame_components (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_GlobalFrame_components)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_GlobalFrame_components)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_GlobalFrame_components)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_GlobalFrame_components)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_GlobalFrame_components (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_GlobalFrame_components (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_GlobalFrame_components root;
	codeGen__TPPLST_0pcodeGen_GlobalFrame_components list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_GlobalFrame_components)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_GlobalFrame_components)
		t = _ElempcodeGen_GlobalFrame_components (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_GlobalFrame_components)(((codeGen__TPPLST_0pcodeGen_GlobalFrame_components) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_GlobalFrame_components) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_GlobalFrame_components;
#else
list = (codeGen__TPPLST_0pcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_GlobalFrame_components));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_GlobalFrame_components;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_GlobalFrame_components)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FrameRefpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_FrameRefpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FrameRefpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ParagraphpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_ParagraphpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ParagraphpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_HeadingpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_HeadingpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_HeadingpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TablepcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_TablepcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TablepcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_FormularpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_OutputMessagesElementpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ListOutputElementpcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ImageMappcodeGen_GlobalFrame_components)
	{codeGen__TPPLST_codeGen_ImageMappcodeGen_GlobalFrame_components _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ImageMappcodeGen_GlobalFrame_components;
	} this code has been moved to _ElempcodeGen_GlobalFrame_components*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_GlobalFrame_components;
#else
root = (codeGen__TPPpcodeGen_GlobalFrame_components) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_GlobalFrame_components));
#endif
root->_prod = codeGen_RULEpcodeGen_GlobalFrame_components;
root->_desc1 = (codeGen__TSPLST_codeGen_GlobalFrame_components)list;
{ codeGen__TPPpcodeGen_GlobalFrame_components _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_GlobalFrame_components;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_GlobalFrame_setSize (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_GlobalFrame_setSize (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_GlobalFrame_setSize _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_GlobalFrame_setSize;
#else
_currn = (codeGen__TPPpcodeGen_GlobalFrame_setSize) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_GlobalFrame_setSize));
#endif
_currn->_prod = codeGen_RULEpcodeGen_GlobalFrame_setSize;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_GlobalFrame_setSize;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_GlobalFrame_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBar (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBar (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_NavigationBar _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBar;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBar) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBar));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBar;
_currn->_desc1 = (codeGen__TSPcodeGen_NavigationBar_styleSheetBlock) codeGen_MkcodeGen_NavigationBar_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationBar: root of subtree no. 1 can not be made a codeGen_NavigationBar_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_NavigationBar_elements) codeGen_MkcodeGen_NavigationBar_elements (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationBar: root of subtree no. 2 can not be made a codeGen_NavigationBar_elements node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_NavigationBar_alignment) codeGen_MkcodeGen_NavigationBar_alignment (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationBar: root of subtree no. 3 can not be made a codeGen_NavigationBar_alignment node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBar;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBar */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBar_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBar_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationBar_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBar_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBar_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBar_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBar_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBar_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBar_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_NavigationBarElementpcodeGen_NavigationBar_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_NavigationBarElementpcodeGen_NavigationBar_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements;
#else
	_new = (codeGen__TPPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_NavigationBarElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_NavigationBar_elements)_new;
	{codeGen__TPPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_NavigationBar_elements (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_NavigationBar_elements (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_NavigationBarElement)) 
		return codeGen__MkcodeGen_NavigationBarElementpcodeGen_NavigationBar_elements (_coordref, codeGen_MkcodeGen_NavigationBarElement (_coordref, t));
	message (DEADLY, "a pcodeGen_NavigationBar_elements list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_NavigationBar_elements (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_NavigationBar_elements (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_NavigationBar_elements)
	l =_ElempcodeGen_NavigationBar_elements (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_NavigationBar_elements)
	r = _ElempcodeGen_NavigationBar_elements (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_NavigationBar_elements)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_NavigationBar_elements)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_NavigationBar_elements)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_NavigationBar_elements)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBar_elements (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBar_elements (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_NavigationBar_elements root;
	codeGen__TPPLST_0pcodeGen_NavigationBar_elements list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_NavigationBar_elements)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_NavigationBar_elements)
		t = _ElempcodeGen_NavigationBar_elements (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_NavigationBar_elements)(((codeGen__TPPLST_0pcodeGen_NavigationBar_elements) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_NavigationBar_elements) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_NavigationBar_elements;
#else
list = (codeGen__TPPLST_0pcodeGen_NavigationBar_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_NavigationBar_elements));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_NavigationBar_elements;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_NavigationBar_elements)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements)
	{codeGen__TPPLST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_NavigationBarElementpcodeGen_NavigationBar_elements;
	} this code has been moved to _ElempcodeGen_NavigationBar_elements*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_NavigationBar_elements;
#else
root = (codeGen__TPPpcodeGen_NavigationBar_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBar_elements));
#endif
root->_prod = codeGen_RULEpcodeGen_NavigationBar_elements;
root->_desc1 = (codeGen__TSPLST_codeGen_NavigationBar_elements)list;
{ codeGen__TPPpcodeGen_NavigationBar_elements _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_NavigationBar_elements;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBar_alignment (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBar_alignment (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationBar_alignment _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBar_alignment;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBar_alignment) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBar_alignment));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBar_alignment;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBar_alignment;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBar_alignment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBarElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBarElement (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_NavigationBarElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBarElement;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBarElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBarElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBarElement;
_currn->_desc1 = (codeGen__TSPcodeGen_NavigationBarElement_imageName) codeGen_MkcodeGen_NavigationBarElement_imageName (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationBarElement: root of subtree no. 1 can not be made a codeGen_NavigationBarElement_imageName node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_NavigationBarElement_target) codeGen_MkcodeGen_NavigationBarElement_target (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_NavigationBarElement: root of subtree no. 2 can not be made a codeGen_NavigationBarElement_target node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBarElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBarElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBarElement_imageName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBarElement_imageName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationBarElement_imageName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBarElement_imageName;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBarElement_imageName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBarElement_imageName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBarElement_imageName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBarElement_imageName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBarElement_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_NavigationBarElement_target (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_NavigationBarElement_target (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_NavigationBarElement_target _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_NavigationBarElement_target;
#else
_currn = (codeGen__TPPpcodeGen_NavigationBarElement_target) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_NavigationBarElement_target));
#endif
_currn->_prod = codeGen_RULEpcodeGen_NavigationBarElement_target;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_NavigationBarElement_target;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_NavigationBarElement_target */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FrameRef (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_FrameRef (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_FrameRef _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FrameRef;
#else
_currn = (codeGen__TPPpcodeGen_FrameRef) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FrameRef));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FrameRef;
_currn->_desc1 = (codeGen__TSPcodeGen_FrameRef_styleSheetBlock) codeGen_MkcodeGen_FrameRef_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FrameRef: root of subtree no. 1 can not be made a codeGen_FrameRef_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FrameRef_frame) codeGen_MkcodeGen_FrameRef_frame (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FrameRef: root of subtree no. 2 can not be made a codeGen_FrameRef_frame node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FrameRef;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FrameRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FrameRef_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FrameRef_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FrameRef_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FrameRef_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_FrameRef_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FrameRef_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FrameRef_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FrameRef_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FrameRef_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FrameRef_frame (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FrameRef_frame (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FrameRef_frame _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FrameRef_frame;
#else
_currn = (codeGen__TPPpcodeGen_FrameRef_frame) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FrameRef_frame));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FrameRef_frame;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FrameRef_frame;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FrameRef_frame */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformation (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_MetaInformation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MetaInformation;
#else
_currn = (codeGen__TPPpcodeGen_MetaInformation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MetaInformation;
_currn->_desc1 = (codeGen__TSPcodeGen_MetaInformation_name) codeGen_MkcodeGen_MetaInformation_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MetaInformation: root of subtree no. 1 can not be made a codeGen_MetaInformation_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_MetaInformation_elements) codeGen_MkcodeGen_MetaInformation_elements (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MetaInformation: root of subtree no. 2 can not be made a codeGen_MetaInformation_elements node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MetaInformation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MetaInformation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformation_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformation_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MetaInformation_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MetaInformation_name;
#else
_currn = (codeGen__TPPpcodeGen_MetaInformation_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformation_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MetaInformation_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MetaInformation_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MetaInformation_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MetaInformationElementpcodeGen_MetaInformation_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MetaInformationElementpcodeGen_MetaInformation_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements;
#else
	_new = (codeGen__TPPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_MetaInformationElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_MetaInformation_elements)_new;
	{codeGen__TPPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_MetaInformation_elements (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_MetaInformation_elements (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MetaInformationElement)) 
		return codeGen__MkcodeGen_MetaInformationElementpcodeGen_MetaInformation_elements (_coordref, codeGen_MkcodeGen_MetaInformationElement (_coordref, t));
	message (DEADLY, "a pcodeGen_MetaInformation_elements list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_MetaInformation_elements (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_MetaInformation_elements (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_MetaInformation_elements)
	l =_ElempcodeGen_MetaInformation_elements (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_MetaInformation_elements)
	r = _ElempcodeGen_MetaInformation_elements (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_MetaInformation_elements)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_MetaInformation_elements)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_MetaInformation_elements)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_MetaInformation_elements)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformation_elements (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformation_elements (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_MetaInformation_elements root;
	codeGen__TPPLST_0pcodeGen_MetaInformation_elements list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_MetaInformation_elements)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_MetaInformation_elements)
		t = _ElempcodeGen_MetaInformation_elements (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_MetaInformation_elements)(((codeGen__TPPLST_0pcodeGen_MetaInformation_elements) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_MetaInformation_elements) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_MetaInformation_elements;
#else
list = (codeGen__TPPLST_0pcodeGen_MetaInformation_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_MetaInformation_elements));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_MetaInformation_elements;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_MetaInformation_elements)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements)
	{codeGen__TPPLST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MetaInformationElementpcodeGen_MetaInformation_elements;
	} this code has been moved to _ElempcodeGen_MetaInformation_elements*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_MetaInformation_elements;
#else
root = (codeGen__TPPpcodeGen_MetaInformation_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformation_elements));
#endif
root->_prod = codeGen_RULEpcodeGen_MetaInformation_elements;
root->_desc1 = (codeGen__TSPLST_codeGen_MetaInformation_elements)list;
{ codeGen__TPPpcodeGen_MetaInformation_elements _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_MetaInformation_elements;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformationElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformationElement (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_MetaInformationElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MetaInformationElement;
#else
_currn = (codeGen__TPPpcodeGen_MetaInformationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformationElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MetaInformationElement;
_currn->_desc1 = (codeGen__TSPcodeGen_MetaInformationElement_property) codeGen_MkcodeGen_MetaInformationElement_property (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MetaInformationElement: root of subtree no. 1 can not be made a codeGen_MetaInformationElement_property node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_MetaInformationElement_value) codeGen_MkcodeGen_MetaInformationElement_value (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MetaInformationElement: root of subtree no. 2 can not be made a codeGen_MetaInformationElement_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MetaInformationElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MetaInformationElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformationElement_property (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformationElement_property (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MetaInformationElement_property _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MetaInformationElement_property;
#else
_currn = (codeGen__TPPpcodeGen_MetaInformationElement_property) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformationElement_property));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MetaInformationElement_property;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MetaInformationElement_property;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MetaInformationElement_property */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MetaInformationElement_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MetaInformationElement_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MetaInformationElement_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MetaInformationElement_value;
#else
_currn = (codeGen__TPPpcodeGen_MetaInformationElement_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MetaInformationElement_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MetaInformationElement_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MetaInformationElement_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MetaInformationElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9, NODEPTR _desc10, NODEPTR _desc11)
#else
NODEPTR codeGen_MkpcodeGen_DataBase (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9,_desc10,_desc11)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
	NODEPTR _desc10;
	NODEPTR _desc11;
#endif
{	codeGen__TPPpcodeGen_DataBase _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase;
#else
_currn = (codeGen__TPPpcodeGen_DataBase) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase;
_currn->_desc1 = (codeGen__TSPcodeGen_DataBase_name) codeGen_MkcodeGen_DataBase_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 1 can not be made a codeGen_DataBase_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataBase_userName) codeGen_MkcodeGen_DataBase_userName (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 2 can not be made a codeGen_DataBase_userName node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataBase_userPassword) codeGen_MkcodeGen_DataBase_userPassword (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 3 can not be made a codeGen_DataBase_userPassword node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataBase_hostName) codeGen_MkcodeGen_DataBase_hostName (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 4 can not be made a codeGen_DataBase_hostName node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataBase_port) codeGen_MkcodeGen_DataBase_port (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 5 can not be made a codeGen_DataBase_port node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_DataBase_dataBaseType) codeGen_MkcodeGen_DataBase_dataBaseType (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 6 can not be made a codeGen_DataBase_dataBaseType node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_DataBase_prefix) codeGen_MkcodeGen_DataBase_prefix (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 7 can not be made a codeGen_DataBase_prefix node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_DataBase_dataSources) codeGen_MkcodeGen_DataBase_dataSources (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 8 can not be made a codeGen_DataBase_dataSources node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_DataBase_dataSourceModifications) codeGen_MkcodeGen_DataBase_dataSourceModifications (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 9 can not be made a codeGen_DataBase_dataSourceModifications node ", 0, _coordref);
_currn->_desc10 = (codeGen__TSPcodeGen_DataBase_setSize) codeGen_MkcodeGen_DataBase_setSize (_coordref, _desc10);	
if (((NODEPTR)_currn->_desc10) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 10 can not be made a codeGen_DataBase_setSize node ", 0, _coordref);
_currn->_desc11 = (codeGen__TSPcodeGen_DataBase_modifySetSize) codeGen_MkcodeGen_DataBase_modifySetSize (_coordref, _desc11);	
if (((NODEPTR)_currn->_desc11) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBase: root of subtree no. 11 can not be made a codeGen_DataBase_modifySetSize node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_name;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_userName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_userName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_userName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_userName;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_userName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_userName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_userName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_userName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_userName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_userPassword (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_userPassword (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_userPassword _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_userPassword;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_userPassword) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_userPassword));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_userPassword;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_userPassword;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_userPassword */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_hostName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_hostName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_hostName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_hostName;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_hostName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_hostName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_hostName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_hostName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_hostName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_port (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_port (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_port _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_port;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_port) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_port));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_port;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_port;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_port */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_dataBaseType (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_dataBaseType (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_dataBaseType _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_dataBaseType;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_dataBaseType) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_dataBaseType));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_dataBaseType;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_dataBaseType;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_dataBaseType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_prefix (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_prefix (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_prefix _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_prefix;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_prefix) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_prefix));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_prefix;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_prefix;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_prefix */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceTablepcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceTablepcodeGen_DataBase_dataSources (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceTable)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSources)_new;
	{codeGen__TPPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceProjectionpcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceProjectionpcodeGen_DataBase_dataSources (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceProjection)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSources)_new;
	{codeGen__TPPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LimitDataSourcepcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LimitDataSourcepcodeGen_DataBase_dataSources (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources;
#else
	_new = (codeGen__TPPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources;
	_new->_desc1 = (codeGen__TSPcodeGen_LimitDataSource)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSources)_new;
	{codeGen__TPPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_OrderedDataSourcepcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_OrderedDataSourcepcodeGen_DataBase_dataSources (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources;
#else
	_new = (codeGen__TPPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources));
#endif
	_new->_prod = codeGen_RULELST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources;
	_new->_desc1 = (codeGen__TSPcodeGen_OrderedDataSource)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSources)_new;
	{codeGen__TPPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceJoinpcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceJoinpcodeGen_DataBase_dataSources (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceJoin)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSources)_new;
	{codeGen__TPPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataBase_dataSources (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceTable)) 
		return codeGen__MkcodeGen_DataSourceTablepcodeGen_DataBase_dataSources (_coordref, codeGen_MkcodeGen_DataSourceTable (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceProjection)) 
		return codeGen__MkcodeGen_DataSourceProjectionpcodeGen_DataBase_dataSources (_coordref, codeGen_MkcodeGen_DataSourceProjection (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LimitDataSource)) 
		return codeGen__MkcodeGen_LimitDataSourcepcodeGen_DataBase_dataSources (_coordref, codeGen_MkcodeGen_LimitDataSource (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_OrderedDataSource)) 
		return codeGen__MkcodeGen_OrderedDataSourcepcodeGen_DataBase_dataSources (_coordref, codeGen_MkcodeGen_OrderedDataSource (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceJoin)) 
		return codeGen__MkcodeGen_DataSourceJoinpcodeGen_DataBase_dataSources (_coordref, codeGen_MkcodeGen_DataSourceJoin (_coordref, t));
	message (DEADLY, "a pcodeGen_DataBase_dataSources list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataBase_dataSources (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSources)
	l =_ElempcodeGen_DataBase_dataSources (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSources)
	r = _ElempcodeGen_DataBase_dataSources (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataBase_dataSources)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataBase_dataSources)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataBase_dataSources)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataBase_dataSources)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_dataSources (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_dataSources (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataBase_dataSources root;
	codeGen__TPPLST_0pcodeGen_DataBase_dataSources list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataBase_dataSources)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSources)
		t = _ElempcodeGen_DataBase_dataSources (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataBase_dataSources)(((codeGen__TPPLST_0pcodeGen_DataBase_dataSources) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataBase_dataSources) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataBase_dataSources;
#else
list = (codeGen__TPPLST_0pcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataBase_dataSources));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataBase_dataSources;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataBase_dataSources)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources)
	{codeGen__TPPLST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceTablepcodeGen_DataBase_dataSources;
	} this code has been moved to _ElempcodeGen_DataBase_dataSources*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources)
	{codeGen__TPPLST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceProjectionpcodeGen_DataBase_dataSources;
	} this code has been moved to _ElempcodeGen_DataBase_dataSources*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources)
	{codeGen__TPPLST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LimitDataSourcepcodeGen_DataBase_dataSources;
	} this code has been moved to _ElempcodeGen_DataBase_dataSources*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources)
	{codeGen__TPPLST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_OrderedDataSourcepcodeGen_DataBase_dataSources;
	} this code has been moved to _ElempcodeGen_DataBase_dataSources*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources)
	{codeGen__TPPLST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceJoinpcodeGen_DataBase_dataSources;
	} this code has been moved to _ElempcodeGen_DataBase_dataSources*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataBase_dataSources;
#else
root = (codeGen__TPPpcodeGen_DataBase_dataSources) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_dataSources));
#endif
root->_prod = codeGen_RULEpcodeGen_DataBase_dataSources;
root->_desc1 = (codeGen__TSPLST_codeGen_DataBase_dataSources)list;
{ codeGen__TPPpcodeGen_DataBase_dataSources _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataBase_dataSources;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceInsert)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSourceModifications)_new;
	{codeGen__TPPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceUpdate)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSourceModifications)_new;
	{codeGen__TPPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceDelete)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBase_dataSourceModifications)_new;
	{codeGen__TPPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataBase_dataSourceModifications (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceInsert)) 
		return codeGen__MkcodeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications (_coordref, codeGen_MkcodeGen_DataSourceInsert (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceUpdate)) 
		return codeGen__MkcodeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications (_coordref, codeGen_MkcodeGen_DataSourceUpdate (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceDelete)) 
		return codeGen__MkcodeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications (_coordref, codeGen_MkcodeGen_DataSourceDelete (_coordref, t));
	message (DEADLY, "a pcodeGen_DataBase_dataSourceModifications list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataBase_dataSourceModifications (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSourceModifications)
	l =_ElempcodeGen_DataBase_dataSourceModifications (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSourceModifications)
	r = _ElempcodeGen_DataBase_dataSourceModifications (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_dataSourceModifications (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_dataSourceModifications (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataBase_dataSourceModifications root;
	codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataBase_dataSourceModifications)
		t = _ElempcodeGen_DataBase_dataSourceModifications (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)(((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataBase_dataSourceModifications;
#else
list = (codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataBase_dataSourceModifications));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataBase_dataSourceModifications;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataBase_dataSourceModifications)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications)
	{codeGen__TPPLST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceInsertpcodeGen_DataBase_dataSourceModifications;
	} this code has been moved to _ElempcodeGen_DataBase_dataSourceModifications*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications)
	{codeGen__TPPLST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceUpdatepcodeGen_DataBase_dataSourceModifications;
	} this code has been moved to _ElempcodeGen_DataBase_dataSourceModifications*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications)
	{codeGen__TPPLST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceDeletepcodeGen_DataBase_dataSourceModifications;
	} this code has been moved to _ElempcodeGen_DataBase_dataSourceModifications*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataBase_dataSourceModifications;
#else
root = (codeGen__TPPpcodeGen_DataBase_dataSourceModifications) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_dataSourceModifications));
#endif
root->_prod = codeGen_RULEpcodeGen_DataBase_dataSourceModifications;
root->_desc1 = (codeGen__TSPLST_codeGen_DataBase_dataSourceModifications)list;
{ codeGen__TPPpcodeGen_DataBase_dataSourceModifications _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataBase_dataSourceModifications;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_setSize (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_setSize (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_setSize _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_setSize;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_setSize) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_setSize));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_setSize;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_setSize;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_setSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBase_modifySetSize (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBase_modifySetSize (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBase_modifySetSize _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBase_modifySetSize;
#else
_currn = (codeGen__TPPpcodeGen_DataBase_modifySetSize) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBase_modifySetSize));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBase_modifySetSize;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBase_modifySetSize;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBase_modifySetSize */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBColumn (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_DBColumn (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_DBColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBColumn;
#else
_currn = (codeGen__TPPpcodeGen_DBColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBColumn;
_currn->_desc1 = (codeGen__TSPcodeGen_DBColumn_name) codeGen_MkcodeGen_DBColumn_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBColumn: root of subtree no. 1 can not be made a codeGen_DBColumn_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DBColumn_type) codeGen_MkcodeGen_DBColumn_type (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBColumn: root of subtree no. 2 can not be made a codeGen_DBColumn_type node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DBColumn_notNull) codeGen_MkcodeGen_DBColumn_notNull (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBColumn: root of subtree no. 3 can not be made a codeGen_DBColumn_notNull node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DBColumn_primaryKey) codeGen_MkcodeGen_DBColumn_primaryKey (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBColumn: root of subtree no. 4 can not be made a codeGen_DBColumn_primaryKey node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBColumn;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBColumn_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBColumn_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBColumn_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBColumn_name;
#else
_currn = (codeGen__TPPpcodeGen_DBColumn_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBColumn_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBColumn_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBColumn_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBColumn_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBColumn_notNull (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBColumn_notNull (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBColumn_notNull _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBColumn_notNull;
#else
_currn = (codeGen__TPPpcodeGen_DBColumn_notNull) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBColumn_notNull));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBColumn_notNull;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBColumn_notNull;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBColumn_notNull */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBColumn_primaryKey (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBColumn_primaryKey (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBColumn_primaryKey _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBColumn_primaryKey;
#else
_currn = (codeGen__TPPpcodeGen_DBColumn_primaryKey) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBColumn_primaryKey));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBColumn_primaryKey;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBColumn_primaryKey;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBColumn_primaryKey */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBHasOne (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_DBHasOne (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_DBHasOne _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBHasOne;
#else
_currn = (codeGen__TPPpcodeGen_DBHasOne) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBHasOne));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBHasOne;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasOne_dataSourceTable) codeGen_MkcodeGen_DBHasOne_dataSourceTable (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBHasOne: root of subtree no. 1 can not be made a codeGen_DBHasOne_dataSourceTable node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBHasOne;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBHasOne */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBHasOne_dataSourceTable (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBHasOne_dataSourceTable (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBHasOne_dataSourceTable _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBHasOne_dataSourceTable;
#else
_currn = (codeGen__TPPpcodeGen_DBHasOne_dataSourceTable) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBHasOne_dataSourceTable));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBHasOne_dataSourceTable;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBHasOne_dataSourceTable;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBHasOne_dataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBHasMany (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_DBHasMany (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_DBHasMany _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBHasMany;
#else
_currn = (codeGen__TPPpcodeGen_DBHasMany) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBHasMany));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBHasMany;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasMany_dataSourceTable) codeGen_MkcodeGen_DBHasMany_dataSourceTable (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBHasMany: root of subtree no. 1 can not be made a codeGen_DBHasMany_dataSourceTable node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBHasMany;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBHasMany */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBHasMany_dataSourceTable (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBHasMany_dataSourceTable (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBHasMany_dataSourceTable _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBHasMany_dataSourceTable;
#else
_currn = (codeGen__TPPpcodeGen_DBHasMany_dataSourceTable) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBHasMany_dataSourceTable));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBHasMany_dataSourceTable;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBHasMany_dataSourceTable;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBHasMany_dataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBString (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBString (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBString _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBString;
#else
_currn = (codeGen__TPPpcodeGen_DBString) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBString));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBString;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBString;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBString */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBText (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBText (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBText _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBText;
#else
_currn = (codeGen__TPPpcodeGen_DBText) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBText));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBText;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBText;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBText */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBInteger (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBInteger (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBInteger _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBInteger;
#else
_currn = (codeGen__TPPpcodeGen_DBInteger) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBInteger));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBInteger;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBInteger;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBInteger */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBLong (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBLong (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBLong _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBLong;
#else
_currn = (codeGen__TPPpcodeGen_DBLong) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBLong));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBLong;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBLong;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBLong */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBFloat (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBFloat (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBFloat _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBFloat;
#else
_currn = (codeGen__TPPpcodeGen_DBFloat) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBFloat));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBFloat;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBFloat;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBFloat */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBDouble (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBDouble (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBDouble _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBDouble;
#else
_currn = (codeGen__TPPpcodeGen_DBDouble) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBDouble));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBDouble;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBDouble;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBDouble */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBBigInt (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBBigInt (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBBigInt _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBBigInt;
#else
_currn = (codeGen__TPPpcodeGen_DBBigInt) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBBigInt));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBBigInt;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBBigInt;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBBigInt */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBDate (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBDate (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBDate _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBDate;
#else
_currn = (codeGen__TPPpcodeGen_DBDate) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBDate));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBDate;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBDate;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBDate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBTimeStamp (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBTimeStamp (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBTimeStamp _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBTimeStamp;
#else
_currn = (codeGen__TPPpcodeGen_DBTimeStamp) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBTimeStamp));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBTimeStamp;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBTimeStamp;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBTimeStamp */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBBoolean (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DBBoolean (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DBBoolean _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBBoolean;
#else
_currn = (codeGen__TPPpcodeGen_DBBoolean) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBBoolean));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBBoolean;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBBoolean;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBBoolean */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBBlob (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_DBBlob (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_DBBlob _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBBlob;
#else
_currn = (codeGen__TPPpcodeGen_DBBlob) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBBlob));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBBlob;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBlob_contentType) codeGen_MkcodeGen_DBBlob_contentType (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DBBlob: root of subtree no. 1 can not be made a codeGen_DBBlob_contentType node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBBlob;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBBlob */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DBBlob_contentType (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DBBlob_contentType (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DBBlob_contentType _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DBBlob_contentType;
#else
_currn = (codeGen__TPPpcodeGen_DBBlob_contentType) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DBBlob_contentType));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DBBlob_contentType;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DBBlob_contentType;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DBBlob_contentType */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceTable (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceTable (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_DataSourceTable _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceTable;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceTable) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceTable));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceTable;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceTable_name) codeGen_MkcodeGen_DataSourceTable_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceTable: root of subtree no. 1 can not be made a codeGen_DataSourceTable_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceTable_position) codeGen_MkcodeGen_DataSourceTable_position (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceTable: root of subtree no. 2 can not be made a codeGen_DataSourceTable_position node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceTable_columns) codeGen_MkcodeGen_DataSourceTable_columns (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceTable: root of subtree no. 3 can not be made a codeGen_DataSourceTable_columns node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceTable;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceTable */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceTable_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceTable_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceTable_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceTable_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceTable_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceTable_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceTable_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceTable_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceTable_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceTable_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceTable_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceTable_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceTable_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceTable_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceTable_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceTable_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceTable_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceTable_position */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DBColumnpcodeGen_DataSourceTable_columns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DBColumnpcodeGen_DataSourceTable_columns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns;
#else
	_new = (codeGen__TPPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DBColumnpcodeGen_DataSourceTable_columns;
	_new->_desc1 = (codeGen__TSPcodeGen_DBColumn)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceTable_columns)_new;
	{codeGen__TPPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DBColumnpcodeGen_DataSourceTable_columns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceTable_columns (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceTable_columns (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DBColumn)) 
		return codeGen__MkcodeGen_DBColumnpcodeGen_DataSourceTable_columns (_coordref, codeGen_MkcodeGen_DBColumn (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceTable_columns list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceTable_columns (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceTable_columns (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceTable_columns)
	l =_ElempcodeGen_DataSourceTable_columns (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceTable_columns)
	r = _ElempcodeGen_DataSourceTable_columns (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceTable_columns (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceTable_columns (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceTable_columns root;
	codeGen__TPPLST_0pcodeGen_DataSourceTable_columns list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceTable_columns)
		t = _ElempcodeGen_DataSourceTable_columns (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)(((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceTable_columns) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceTable_columns;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceTable_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceTable_columns));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceTable_columns;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceTable_columns)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DBColumnpcodeGen_DataSourceTable_columns)
	{codeGen__TPPLST_codeGen_DBColumnpcodeGen_DataSourceTable_columns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DBColumnpcodeGen_DataSourceTable_columns;
	} this code has been moved to _ElempcodeGen_DataSourceTable_columns*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceTable_columns;
#else
root = (codeGen__TPPpcodeGen_DataSourceTable_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceTable_columns));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceTable_columns;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceTable_columns)list;
{ codeGen__TPPpcodeGen_DataSourceTable_columns _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceTable_columns;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	codeGen__TPPpcodeGen_DataSourceProjection _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceProjection;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceProjection) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceProjection;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceProjection_name) codeGen_MkcodeGen_DataSourceProjection_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 1 can not be made a codeGen_DataSourceProjection_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceProjection_position) codeGen_MkcodeGen_DataSourceProjection_position (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 2 can not be made a codeGen_DataSourceProjection_position node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceProjection_formalParameters) codeGen_MkcodeGen_DataSourceProjection_formalParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 3 can not be made a codeGen_DataSourceProjection_formalParameters node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataSourceProjection_sourceDataSource) codeGen_MkcodeGen_DataSourceProjection_sourceDataSource (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 4 can not be made a codeGen_DataSourceProjection_sourceDataSource node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataSourceProjection_condition) codeGen_MkcodeGen_DataSourceProjection_condition (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 5 can not be made a codeGen_DataSourceProjection_condition node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_DataSourceProjection_projectionColumns) codeGen_MkcodeGen_DataSourceProjection_projectionColumns (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceProjection: root of subtree no. 6 can not be made a codeGen_DataSourceProjection_projectionColumns node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceProjection;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceProjection_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceProjection_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceProjection_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceProjection_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceProjection_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceProjection_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceProjection_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceProjection_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceProjection_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceProjection_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceProjection_position */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_formalParameters)_new;
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceProjection_formalParameters (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceProjection_formalParameters (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceFormalParam)) 
		return codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters (_coordref, codeGen_MkcodeGen_DataSourceFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceProjection_formalParameters list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_formalParameters (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_formalParameters (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_formalParameters)
	l =_ElempcodeGen_DataSourceProjection_formalParameters (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_formalParameters)
	r = _ElempcodeGen_DataSourceProjection_formalParameters (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_formalParameters (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_formalParameters (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceProjection_formalParameters root;
	codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_formalParameters)
		t = _ElempcodeGen_DataSourceProjection_formalParameters (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)(((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceProjection_formalParameters;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceProjection_formalParameters));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceProjection_formalParameters;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_formalParameters)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters)
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceProjection_formalParameters;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_formalParameters*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceProjection_formalParameters;
#else
root = (codeGen__TPPpcodeGen_DataSourceProjection_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_formalParameters));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceProjection_formalParameters;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceProjection_formalParameters)list;
{ codeGen__TPPpcodeGen_DataSourceProjection_formalParameters _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceProjection_formalParameters;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_sourceDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_sourceDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceProjection_sourceDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceProjection_sourceDataSource;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceProjection_sourceDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_sourceDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceProjection_sourceDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceProjection_sourceDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceProjection_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_BinaryExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_UnaryExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_ConstantExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition;
#else
	_new = (codeGen__TPPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition;
	_new->_desc1 = (codeGen__TSPcodeGen_ComputationExpression)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)_new;
	{codeGen__TPPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceProjection_condition (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BinaryExpression)) 
		return codeGen__MkcodeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_BinaryExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_UnaryExpression)) 
		return codeGen__MkcodeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_UnaryExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PredefinedExpression)) 
		return codeGen__MkcodeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_PredefinedExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ConstantExpression)) 
		return codeGen__MkcodeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_ConstantExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ColumnValueExpression)) 
		return codeGen__MkcodeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_ColumnValueExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ProjectionColumnValueExpression)) 
		return codeGen__MkcodeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ParameterValueExpression)) 
		return codeGen__MkcodeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_ParameterValueExpression (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ComputationExpression)) 
		return codeGen__MkcodeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition (_coordref, codeGen_MkcodeGen_ComputationExpression (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceProjection_condition list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_condition (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_condition)
	l =_ElempcodeGen_DataSourceProjection_condition (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_condition)
	r = _ElempcodeGen_DataSourceProjection_condition (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_condition (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_condition (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceProjection_condition root;
	codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_condition)
		t = _ElempcodeGen_DataSourceProjection_condition (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)(((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceProjection_condition;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceProjection_condition));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceProjection_condition;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_condition)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BinaryExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_UnaryExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PredefinedExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ConstantExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ProjectionColumnValueExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ParameterValueExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition)
	{codeGen__TPPLST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ComputationExpressionpcodeGen_DataSourceProjection_condition;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_condition*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceProjection_condition;
#else
root = (codeGen__TPPpcodeGen_DataSourceProjection_condition) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_condition));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceProjection_condition;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceProjection_condition)list;
{ codeGen__TPPpcodeGen_DataSourceProjection_condition _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceProjection_condition;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns;
#else
	_new = (codeGen__TPPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns;
	_new->_desc1 = (codeGen__TSPcodeGen_ColumnProjection)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_projectionColumns)_new;
	{codeGen__TPPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns;
#else
	_new = (codeGen__TPPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns;
	_new->_desc1 = (codeGen__TSPcodeGen_ComputationProjection)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceProjection_projectionColumns)_new;
	{codeGen__TPPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceProjection_projectionColumns (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ColumnProjection)) 
		return codeGen__MkcodeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns (_coordref, codeGen_MkcodeGen_ColumnProjection (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ComputationProjection)) 
		return codeGen__MkcodeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns (_coordref, codeGen_MkcodeGen_ComputationProjection (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceProjection_projectionColumns list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceProjection_projectionColumns (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_projectionColumns)
	l =_ElempcodeGen_DataSourceProjection_projectionColumns (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_projectionColumns)
	r = _ElempcodeGen_DataSourceProjection_projectionColumns (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_projectionColumns (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceProjection_projectionColumns (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceProjection_projectionColumns root;
	codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceProjection_projectionColumns)
		t = _ElempcodeGen_DataSourceProjection_projectionColumns (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)(((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceProjection_projectionColumns;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceProjection_projectionColumns));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceProjection_projectionColumns;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceProjection_projectionColumns)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns)
	{codeGen__TPPLST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ColumnProjectionpcodeGen_DataSourceProjection_projectionColumns;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_projectionColumns*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns)
	{codeGen__TPPLST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ComputationProjectionpcodeGen_DataSourceProjection_projectionColumns;
	} this code has been moved to _ElempcodeGen_DataSourceProjection_projectionColumns*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceProjection_projectionColumns;
#else
root = (codeGen__TPPpcodeGen_DataSourceProjection_projectionColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceProjection_projectionColumns));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceProjection_projectionColumns;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceProjection_projectionColumns)list;
{ codeGen__TPPpcodeGen_DataSourceProjection_projectionColumns _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceProjection_projectionColumns;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColumnProjection (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_ColumnProjection (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_ColumnProjection _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColumnProjection;
#else
_currn = (codeGen__TPPpcodeGen_ColumnProjection) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColumnProjection));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColumnProjection;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnProjection_name) codeGen_MkcodeGen_ColumnProjection_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ColumnProjection: root of subtree no. 1 can not be made a codeGen_ColumnProjection_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ColumnProjection_column) codeGen_MkcodeGen_ColumnProjection_column (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ColumnProjection: root of subtree no. 2 can not be made a codeGen_ColumnProjection_column node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColumnProjection;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColumnProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColumnProjection_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ColumnProjection_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ColumnProjection_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColumnProjection_name;
#else
_currn = (codeGen__TPPpcodeGen_ColumnProjection_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColumnProjection_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColumnProjection_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColumnProjection_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColumnProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColumnProjection_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ColumnProjection_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ColumnProjection_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColumnProjection_column;
#else
_currn = (codeGen__TPPpcodeGen_ColumnProjection_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColumnProjection_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColumnProjection_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColumnProjection_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColumnProjection_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationProjection (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_ComputationProjection (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_ComputationProjection _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationProjection;
#else
_currn = (codeGen__TPPpcodeGen_ComputationProjection) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationProjection));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationProjection;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationProjection_name) codeGen_MkcodeGen_ComputationProjection_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ComputationProjection: root of subtree no. 1 can not be made a codeGen_ComputationProjection_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ComputationProjection_column) codeGen_MkcodeGen_ComputationProjection_column (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ComputationProjection: root of subtree no. 2 can not be made a codeGen_ComputationProjection_column node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_ComputationProjection_computation) codeGen_MkcodeGen_ComputationProjection_computation (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ComputationProjection: root of subtree no. 3 can not be made a codeGen_ComputationProjection_computation node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationProjection;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationProjection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationProjection_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ComputationProjection_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ComputationProjection_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationProjection_name;
#else
_currn = (codeGen__TPPpcodeGen_ComputationProjection_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationProjection_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationProjection_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationProjection_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationProjection_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationProjection_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ComputationProjection_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ComputationProjection_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationProjection_column;
#else
_currn = (codeGen__TPPpcodeGen_ComputationProjection_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationProjection_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationProjection_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationProjection_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationProjection_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationProjection_computation (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ComputationProjection_computation (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ComputationProjection_computation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationProjection_computation;
#else
_currn = (codeGen__TPPpcodeGen_ComputationProjection_computation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationProjection_computation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationProjection_computation;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationProjection_computation;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationProjection_computation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LimitDataSource (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR codeGen_MkpcodeGen_LimitDataSource (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	codeGen__TPPpcodeGen_LimitDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LimitDataSource;
#else
_currn = (codeGen__TPPpcodeGen_LimitDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LimitDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LimitDataSource;
_currn->_desc1 = (codeGen__TSPcodeGen_LimitDataSource_name) codeGen_MkcodeGen_LimitDataSource_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 1 can not be made a codeGen_LimitDataSource_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_LimitDataSource_position) codeGen_MkcodeGen_LimitDataSource_position (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 2 can not be made a codeGen_LimitDataSource_position node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_LimitDataSource_formalParameters) codeGen_MkcodeGen_LimitDataSource_formalParameters (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 3 can not be made a codeGen_LimitDataSource_formalParameters node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_LimitDataSource_sourceDataSource) codeGen_MkcodeGen_LimitDataSource_sourceDataSource (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 4 can not be made a codeGen_LimitDataSource_sourceDataSource node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_LimitDataSource_firstRow) codeGen_MkcodeGen_LimitDataSource_firstRow (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 5 can not be made a codeGen_LimitDataSource_firstRow node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_LimitDataSource_maxRows) codeGen_MkcodeGen_LimitDataSource_maxRows (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LimitDataSource: root of subtree no. 6 can not be made a codeGen_LimitDataSource_maxRows node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LimitDataSource;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LimitDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LimitDataSource_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LimitDataSource_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LimitDataSource_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LimitDataSource_name;
#else
_currn = (codeGen__TPPpcodeGen_LimitDataSource_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LimitDataSource_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LimitDataSource_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LimitDataSource_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LimitDataSource_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LimitDataSource_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LimitDataSource_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LimitDataSource_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LimitDataSource_position;
#else
_currn = (codeGen__TPPpcodeGen_LimitDataSource_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LimitDataSource_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LimitDataSource_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LimitDataSource_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LimitDataSource_position */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_LimitDataSource_formalParameters)_new;
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_LimitDataSource_formalParameters (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_LimitDataSource_formalParameters (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceFormalParam)) 
		return codeGen__MkcodeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters (_coordref, codeGen_MkcodeGen_DataSourceFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_LimitDataSource_formalParameters list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_LimitDataSource_formalParameters (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_LimitDataSource_formalParameters (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_LimitDataSource_formalParameters)
	l =_ElempcodeGen_LimitDataSource_formalParameters (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_LimitDataSource_formalParameters)
	r = _ElempcodeGen_LimitDataSource_formalParameters (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LimitDataSource_formalParameters (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_LimitDataSource_formalParameters (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_LimitDataSource_formalParameters root;
	codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_LimitDataSource_formalParameters)
		t = _ElempcodeGen_LimitDataSource_formalParameters (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)(((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_LimitDataSource_formalParameters;
#else
list = (codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_LimitDataSource_formalParameters));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_LimitDataSource_formalParameters;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_LimitDataSource_formalParameters)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters)
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_LimitDataSource_formalParameters;
	} this code has been moved to _ElempcodeGen_LimitDataSource_formalParameters*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_LimitDataSource_formalParameters;
#else
root = (codeGen__TPPpcodeGen_LimitDataSource_formalParameters) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LimitDataSource_formalParameters));
#endif
root->_prod = codeGen_RULEpcodeGen_LimitDataSource_formalParameters;
root->_desc1 = (codeGen__TSPLST_codeGen_LimitDataSource_formalParameters)list;
{ codeGen__TPPpcodeGen_LimitDataSource_formalParameters _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_LimitDataSource_formalParameters;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LimitDataSource_sourceDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LimitDataSource_sourceDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LimitDataSource_sourceDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LimitDataSource_sourceDataSource;
#else
_currn = (codeGen__TPPpcodeGen_LimitDataSource_sourceDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LimitDataSource_sourceDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LimitDataSource_sourceDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LimitDataSource_sourceDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LimitDataSource_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderedDataSource (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_OrderedDataSource (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_OrderedDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderedDataSource;
#else
_currn = (codeGen__TPPpcodeGen_OrderedDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderedDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderedDataSource;
_currn->_desc1 = (codeGen__TSPcodeGen_OrderedDataSource_name) codeGen_MkcodeGen_OrderedDataSource_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderedDataSource: root of subtree no. 1 can not be made a codeGen_OrderedDataSource_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_OrderedDataSource_position) codeGen_MkcodeGen_OrderedDataSource_position (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderedDataSource: root of subtree no. 2 can not be made a codeGen_OrderedDataSource_position node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_OrderedDataSource_sourceDataSource) codeGen_MkcodeGen_OrderedDataSource_sourceDataSource (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderedDataSource: root of subtree no. 3 can not be made a codeGen_OrderedDataSource_sourceDataSource node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_OrderedDataSource_orderColumns) codeGen_MkcodeGen_OrderedDataSource_orderColumns (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderedDataSource: root of subtree no. 4 can not be made a codeGen_OrderedDataSource_orderColumns node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderedDataSource;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderedDataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OrderedDataSource_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderedDataSource_name;
#else
_currn = (codeGen__TPPpcodeGen_OrderedDataSource_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderedDataSource_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderedDataSource_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderedDataSource_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderedDataSource_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OrderedDataSource_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderedDataSource_position;
#else
_currn = (codeGen__TPPpcodeGen_OrderedDataSource_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderedDataSource_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderedDataSource_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderedDataSource_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderedDataSource_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_sourceDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_sourceDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OrderedDataSource_sourceDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderedDataSource_sourceDataSource;
#else
_currn = (codeGen__TPPpcodeGen_OrderedDataSource_sourceDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderedDataSource_sourceDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderedDataSource_sourceDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderedDataSource_sourceDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderedDataSource_sourceDataSource */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns;
#else
	_new = (codeGen__TPPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns;
	_new->_desc1 = (codeGen__TSPcodeGen_OrderColumn)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_OrderedDataSource_orderColumns)_new;
	{codeGen__TPPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_OrderedDataSource_orderColumns (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_OrderedDataSource_orderColumns (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_OrderColumn)) 
		return codeGen__MkcodeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns (_coordref, codeGen_MkcodeGen_OrderColumn (_coordref, t));
	message (DEADLY, "a pcodeGen_OrderedDataSource_orderColumns list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_OrderedDataSource_orderColumns (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_OrderedDataSource_orderColumns (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_OrderedDataSource_orderColumns)
	l =_ElempcodeGen_OrderedDataSource_orderColumns (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_OrderedDataSource_orderColumns)
	r = _ElempcodeGen_OrderedDataSource_orderColumns (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_orderColumns (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_OrderedDataSource_orderColumns (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_OrderedDataSource_orderColumns root;
	codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_OrderedDataSource_orderColumns)
		t = _ElempcodeGen_OrderedDataSource_orderColumns (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)(((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_OrderedDataSource_orderColumns;
#else
list = (codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_OrderedDataSource_orderColumns));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_OrderedDataSource_orderColumns;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_OrderedDataSource_orderColumns)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns)
	{codeGen__TPPLST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_OrderColumnpcodeGen_OrderedDataSource_orderColumns;
	} this code has been moved to _ElempcodeGen_OrderedDataSource_orderColumns*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_OrderedDataSource_orderColumns;
#else
root = (codeGen__TPPpcodeGen_OrderedDataSource_orderColumns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderedDataSource_orderColumns));
#endif
root->_prod = codeGen_RULEpcodeGen_OrderedDataSource_orderColumns;
root->_desc1 = (codeGen__TSPLST_codeGen_OrderedDataSource_orderColumns)list;
{ codeGen__TPPpcodeGen_OrderedDataSource_orderColumns _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_OrderedDataSource_orderColumns;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderColumn (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_OrderColumn (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_OrderColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderColumn;
#else
_currn = (codeGen__TPPpcodeGen_OrderColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderColumn;
_currn->_desc1 = (codeGen__TSPcodeGen_OrderColumn_orderColumn) codeGen_MkcodeGen_OrderColumn_orderColumn (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderColumn: root of subtree no. 1 can not be made a codeGen_OrderColumn_orderColumn node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_OrderColumn_order) codeGen_MkcodeGen_OrderColumn_order (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OrderColumn: root of subtree no. 2 can not be made a codeGen_OrderColumn_order node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderColumn;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderColumn_orderColumn (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OrderColumn_orderColumn (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OrderColumn_orderColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderColumn_orderColumn;
#else
_currn = (codeGen__TPPpcodeGen_OrderColumn_orderColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderColumn_orderColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderColumn_orderColumn;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderColumn_orderColumn;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderColumn_orderColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OrderColumn_order (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OrderColumn_order (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OrderColumn_order _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OrderColumn_order;
#else
_currn = (codeGen__TPPpcodeGen_OrderColumn_order) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OrderColumn_order));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OrderColumn_order;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OrderColumn_order;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OrderColumn_order */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_BinaryExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_BinaryExpression (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_BinaryExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_BinaryExpression;
#else
_currn = (codeGen__TPPpcodeGen_BinaryExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_BinaryExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_BinaryExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression_leftOperand) codeGen_MkcodeGen_BinaryExpression_leftOperand (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_BinaryExpression: root of subtree no. 1 can not be made a codeGen_BinaryExpression_leftOperand node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_BinaryExpression_rightOperand) codeGen_MkcodeGen_BinaryExpression_rightOperand (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_BinaryExpression: root of subtree no. 2 can not be made a codeGen_BinaryExpression_rightOperand node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_BinaryExpression_operator) codeGen_MkcodeGen_BinaryExpression_operator (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_BinaryExpression: root of subtree no. 3 can not be made a codeGen_BinaryExpression_operator node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_BinaryExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_BinaryExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_BinaryExpression_operator (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_BinaryExpression_operator (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_BinaryExpression_operator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_BinaryExpression_operator;
#else
_currn = (codeGen__TPPpcodeGen_BinaryExpression_operator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_BinaryExpression_operator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_BinaryExpression_operator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_BinaryExpression_operator;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_BinaryExpression_operator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_UnaryExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_UnaryExpression (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_UnaryExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_UnaryExpression;
#else
_currn = (codeGen__TPPpcodeGen_UnaryExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_UnaryExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_UnaryExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression_operand) codeGen_MkcodeGen_UnaryExpression_operand (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_UnaryExpression: root of subtree no. 1 can not be made a codeGen_UnaryExpression_operand node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_UnaryExpression_operator) codeGen_MkcodeGen_UnaryExpression_operator (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_UnaryExpression: root of subtree no. 2 can not be made a codeGen_UnaryExpression_operator node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_UnaryExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_UnaryExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_UnaryExpression_operator (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_UnaryExpression_operator (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_UnaryExpression_operator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_UnaryExpression_operator;
#else
_currn = (codeGen__TPPpcodeGen_UnaryExpression_operator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_UnaryExpression_operator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_UnaryExpression_operator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_UnaryExpression_operator;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_UnaryExpression_operator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_PredefinedExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_PredefinedExpression (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_PredefinedExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_PredefinedExpression;
#else
_currn = (codeGen__TPPpcodeGen_PredefinedExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_PredefinedExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_PredefinedExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression_value) codeGen_MkcodeGen_PredefinedExpression_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_PredefinedExpression: root of subtree no. 1 can not be made a codeGen_PredefinedExpression_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_PredefinedExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_PredefinedExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_PredefinedExpression_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_PredefinedExpression_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_PredefinedExpression_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_PredefinedExpression_value;
#else
_currn = (codeGen__TPPpcodeGen_PredefinedExpression_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_PredefinedExpression_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_PredefinedExpression_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_PredefinedExpression_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_PredefinedExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ConstantExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ConstantExpression (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ConstantExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ConstantExpression;
#else
_currn = (codeGen__TPPpcodeGen_ConstantExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ConstantExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ConstantExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression_value) codeGen_MkcodeGen_ConstantExpression_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ConstantExpression: root of subtree no. 1 can not be made a codeGen_ConstantExpression_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ConstantExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ConstantExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ConstantExpression_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ConstantExpression_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ConstantExpression_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ConstantExpression_value;
#else
_currn = (codeGen__TPPpcodeGen_ConstantExpression_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ConstantExpression_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ConstantExpression_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ConstantExpression_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ConstantExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColumnValueExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ColumnValueExpression (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ColumnValueExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColumnValueExpression;
#else
_currn = (codeGen__TPPpcodeGen_ColumnValueExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColumnValueExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColumnValueExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression_column) codeGen_MkcodeGen_ColumnValueExpression_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ColumnValueExpression: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression_column node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColumnValueExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColumnValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColumnValueExpression_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ColumnValueExpression_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ColumnValueExpression_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColumnValueExpression_column;
#else
_currn = (codeGen__TPPpcodeGen_ColumnValueExpression_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColumnValueExpression_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColumnValueExpression_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColumnValueExpression_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColumnValueExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ProjectionColumnValueExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ProjectionColumnValueExpression (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ProjectionColumnValueExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ProjectionColumnValueExpression;
#else
_currn = (codeGen__TPPpcodeGen_ProjectionColumnValueExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ProjectionColumnValueExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ProjectionColumnValueExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression_column) codeGen_MkcodeGen_ProjectionColumnValueExpression_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ProjectionColumnValueExpression: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression_column node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ProjectionColumnValueExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ProjectionColumnValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ProjectionColumnValueExpression_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ProjectionColumnValueExpression_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ProjectionColumnValueExpression_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ProjectionColumnValueExpression_column;
#else
_currn = (codeGen__TPPpcodeGen_ProjectionColumnValueExpression_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ProjectionColumnValueExpression_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ProjectionColumnValueExpression_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ProjectionColumnValueExpression_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ProjectionColumnValueExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ParameterValueExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ParameterValueExpression (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ParameterValueExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ParameterValueExpression;
#else
_currn = (codeGen__TPPpcodeGen_ParameterValueExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ParameterValueExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ParameterValueExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression_parameter) codeGen_MkcodeGen_ParameterValueExpression_parameter (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ParameterValueExpression: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression_parameter node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ParameterValueExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ParameterValueExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ParameterValueExpression_parameter (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ParameterValueExpression_parameter (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ParameterValueExpression_parameter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ParameterValueExpression_parameter;
#else
_currn = (codeGen__TPPpcodeGen_ParameterValueExpression_parameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ParameterValueExpression_parameter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ParameterValueExpression_parameter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ParameterValueExpression_parameter;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ParameterValueExpression_parameter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_ComputationExpression (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_ComputationExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationExpression;
#else
_currn = (codeGen__TPPpcodeGen_ComputationExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression_column) codeGen_MkcodeGen_ComputationExpression_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ComputationExpression: root of subtree no. 1 can not be made a codeGen_ComputationExpression_column node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ComputationExpression_computation) codeGen_MkcodeGen_ComputationExpression_computation (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ComputationExpression: root of subtree no. 2 can not be made a codeGen_ComputationExpression_computation node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationExpression */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationExpression_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ComputationExpression_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ComputationExpression_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationExpression_column;
#else
_currn = (codeGen__TPPpcodeGen_ComputationExpression_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationExpression_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationExpression_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationExpression_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationExpression_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ComputationExpression_computation (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ComputationExpression_computation (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ComputationExpression_computation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ComputationExpression_computation;
#else
_currn = (codeGen__TPPpcodeGen_ComputationExpression_computation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ComputationExpression_computation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ComputationExpression_computation;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ComputationExpression_computation;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ComputationExpression_computation */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceValue (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_DataSourceValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceValue;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceValue;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceValue_column) codeGen_MkcodeGen_DataSourceValue_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceValue: root of subtree no. 1 can not be made a codeGen_DataSourceValue_column node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceValue_expression) codeGen_MkcodeGen_DataSourceValue_expression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceValue: root of subtree no. 2 can not be made a codeGen_DataSourceValue_expression node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceValue_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceValue_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceValue_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceValue_column;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceValue_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceValue_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceValue_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceValue_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceValue_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceJoin_name) codeGen_MkcodeGen_DataSourceJoin_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 1 can not be made a codeGen_DataSourceJoin_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceJoin_position) codeGen_MkcodeGen_DataSourceJoin_position (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 2 can not be made a codeGen_DataSourceJoin_position node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceJoin_dataSource1) codeGen_MkcodeGen_DataSourceJoin_dataSource1 (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 3 can not be made a codeGen_DataSourceJoin_dataSource1 node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataSourceJoin_dataSource2) codeGen_MkcodeGen_DataSourceJoin_dataSource2 (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 4 can not be made a codeGen_DataSourceJoin_dataSource2 node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataSourceJoin_columnRef1) codeGen_MkcodeGen_DataSourceJoin_columnRef1 (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 5 can not be made a codeGen_DataSourceJoin_columnRef1 node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_DataSourceJoin_columnRef2) codeGen_MkcodeGen_DataSourceJoin_columnRef2 (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 6 can not be made a codeGen_DataSourceJoin_columnRef2 node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_DataSourceJoin_joinOperator) codeGen_MkcodeGen_DataSourceJoin_joinOperator (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceJoin: root of subtree no. 7 can not be made a codeGen_DataSourceJoin_joinOperator node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_dataSource1 (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_dataSource1 (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_dataSource1 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_dataSource1;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_dataSource1) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_dataSource1));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_dataSource1;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_dataSource1;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_dataSource1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_dataSource2 (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_dataSource2 (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_dataSource2 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_dataSource2;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_dataSource2) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_dataSource2));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_dataSource2;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_dataSource2;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_dataSource2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_columnRef1 (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_columnRef1 (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_columnRef1 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_columnRef1;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_columnRef1) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_columnRef1));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_columnRef1;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_columnRef1;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_columnRef1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_columnRef2 (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_columnRef2 (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_columnRef2 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_columnRef2;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_columnRef2) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_columnRef2));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_columnRef2;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_columnRef2;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_columnRef2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_joinOperator (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceJoin_joinOperator (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceJoin_joinOperator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceJoin_joinOperator;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceJoin_joinOperator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceJoin_joinOperator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceJoin_joinOperator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceJoin_joinOperator;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceJoin_joinOperator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	codeGen__TPPpcodeGen_DataSourceInsert _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceInsert;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceInsert) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceInsert;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceInsert_targetDataSource) codeGen_MkcodeGen_DataSourceInsert_targetDataSource (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceInsert: root of subtree no. 1 can not be made a codeGen_DataSourceInsert_targetDataSource node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceInsert_formalParams) codeGen_MkcodeGen_DataSourceInsert_formalParams (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceInsert: root of subtree no. 2 can not be made a codeGen_DataSourceInsert_formalParams node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceInsert_name) codeGen_MkcodeGen_DataSourceInsert_name (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceInsert: root of subtree no. 3 can not be made a codeGen_DataSourceInsert_name node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataSourceInsert_position) codeGen_MkcodeGen_DataSourceInsert_position (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceInsert: root of subtree no. 4 can not be made a codeGen_DataSourceInsert_position node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataSourceInsert_values) codeGen_MkcodeGen_DataSourceInsert_values (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceInsert: root of subtree no. 5 can not be made a codeGen_DataSourceInsert_values node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceInsert;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceInsert */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_targetDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_targetDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceInsert_targetDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceInsert_targetDataSource;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceInsert_targetDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert_targetDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceInsert_targetDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceInsert_targetDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceInsert_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceInsert_formalParams)_new;
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceInsert_formalParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceInsert_formalParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceFormalParam)) 
		return codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams (_coordref, codeGen_MkcodeGen_DataSourceFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceInsert_formalParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceInsert_formalParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceInsert_formalParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_formalParams)
	l =_ElempcodeGen_DataSourceInsert_formalParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_formalParams)
	r = _ElempcodeGen_DataSourceInsert_formalParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_formalParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_formalParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceInsert_formalParams root;
	codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_formalParams)
		t = _ElempcodeGen_DataSourceInsert_formalParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)(((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceInsert_formalParams;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceInsert_formalParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceInsert_formalParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_formalParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams)
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceInsert_formalParams;
	} this code has been moved to _ElempcodeGen_DataSourceInsert_formalParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceInsert_formalParams;
#else
root = (codeGen__TPPpcodeGen_DataSourceInsert_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert_formalParams));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceInsert_formalParams;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceInsert_formalParams)list;
{ codeGen__TPPpcodeGen_DataSourceInsert_formalParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceInsert_formalParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceInsert_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceInsert_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceInsert_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceInsert_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceInsert_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceInsert_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceInsert_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceInsert_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceInsert_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceInsert_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceInsert_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceInsert_position */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceInsert_values (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceInsert_values (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceValue)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceInsert_values)_new;
	{codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceInsert_values (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceInsert_values (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceValue)) 
		return codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceInsert_values (_coordref, codeGen_MkcodeGen_DataSourceValue (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceInsert_values list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceInsert_values (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceInsert_values (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_values)
	l =_ElempcodeGen_DataSourceInsert_values (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_values)
	r = _ElempcodeGen_DataSourceInsert_values (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_values (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceInsert_values (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceInsert_values root;
	codeGen__TPPLST_0pcodeGen_DataSourceInsert_values list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceInsert_values)
		t = _ElempcodeGen_DataSourceInsert_values (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)(((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceInsert_values) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceInsert_values;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceInsert_values));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceInsert_values;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceInsert_values)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values)
	{codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceValuepcodeGen_DataSourceInsert_values;
	} this code has been moved to _ElempcodeGen_DataSourceInsert_values*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceInsert_values;
#else
root = (codeGen__TPPpcodeGen_DataSourceInsert_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceInsert_values));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceInsert_values;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceInsert_values)list;
{ codeGen__TPPpcodeGen_DataSourceInsert_values _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceInsert_values;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
#endif
{	codeGen__TPPpcodeGen_DataSourceUpdate _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceUpdate;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceUpdate) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceUpdate;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceUpdate_targetDataSource) codeGen_MkcodeGen_DataSourceUpdate_targetDataSource (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 1 can not be made a codeGen_DataSourceUpdate_targetDataSource node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceUpdate_formalParams) codeGen_MkcodeGen_DataSourceUpdate_formalParams (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 2 can not be made a codeGen_DataSourceUpdate_formalParams node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceUpdate_name) codeGen_MkcodeGen_DataSourceUpdate_name (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 3 can not be made a codeGen_DataSourceUpdate_name node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataSourceUpdate_position) codeGen_MkcodeGen_DataSourceUpdate_position (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 4 can not be made a codeGen_DataSourceUpdate_position node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataSourceUpdate_conditions) codeGen_MkcodeGen_DataSourceUpdate_conditions (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 5 can not be made a codeGen_DataSourceUpdate_conditions node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_DataSourceUpdate_values) codeGen_MkcodeGen_DataSourceUpdate_values (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceUpdate: root of subtree no. 6 can not be made a codeGen_DataSourceUpdate_values node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceUpdate;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceUpdate */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_targetDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_targetDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceUpdate_targetDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceUpdate_targetDataSource;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceUpdate_targetDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate_targetDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceUpdate_targetDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceUpdate_targetDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceUpdate_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceUpdate_formalParams)_new;
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceUpdate_formalParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceUpdate_formalParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceFormalParam)) 
		return codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams (_coordref, codeGen_MkcodeGen_DataSourceFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceUpdate_formalParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceUpdate_formalParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceUpdate_formalParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_formalParams)
	l =_ElempcodeGen_DataSourceUpdate_formalParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_formalParams)
	r = _ElempcodeGen_DataSourceUpdate_formalParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_formalParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_formalParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceUpdate_formalParams root;
	codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_formalParams)
		t = _ElempcodeGen_DataSourceUpdate_formalParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)(((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceUpdate_formalParams;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceUpdate_formalParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceUpdate_formalParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_formalParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams)
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceUpdate_formalParams;
	} this code has been moved to _ElempcodeGen_DataSourceUpdate_formalParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceUpdate_formalParams;
#else
root = (codeGen__TPPpcodeGen_DataSourceUpdate_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate_formalParams));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceUpdate_formalParams;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceUpdate_formalParams)list;
{ codeGen__TPPpcodeGen_DataSourceUpdate_formalParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceUpdate_formalParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceUpdate_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceUpdate_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceUpdate_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceUpdate_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceUpdate_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceUpdate_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceUpdate_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceUpdate_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceUpdate_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceUpdate_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceUpdate_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceUpdate_position */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceUpdate_values (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceUpdate_values (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceValue)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceUpdate_values)_new;
	{codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceUpdate_values (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceUpdate_values (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceValue)) 
		return codeGen__MkcodeGen_DataSourceValuepcodeGen_DataSourceUpdate_values (_coordref, codeGen_MkcodeGen_DataSourceValue (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceUpdate_values list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceUpdate_values (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceUpdate_values (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_values)
	l =_ElempcodeGen_DataSourceUpdate_values (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_values)
	r = _ElempcodeGen_DataSourceUpdate_values (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_values (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceUpdate_values (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceUpdate_values root;
	codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceUpdate_values)
		t = _ElempcodeGen_DataSourceUpdate_values (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)(((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceUpdate_values;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceUpdate_values));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceUpdate_values;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceUpdate_values)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values)
	{codeGen__TPPLST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceValuepcodeGen_DataSourceUpdate_values;
	} this code has been moved to _ElempcodeGen_DataSourceUpdate_values*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceUpdate_values;
#else
root = (codeGen__TPPpcodeGen_DataSourceUpdate_values) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceUpdate_values));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceUpdate_values;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceUpdate_values)list;
{ codeGen__TPPpcodeGen_DataSourceUpdate_values _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceUpdate_values;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceDelete (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceDelete (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	codeGen__TPPpcodeGen_DataSourceDelete _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceDelete;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceDelete) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceDelete));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceDelete;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceDelete_targetDataSource) codeGen_MkcodeGen_DataSourceDelete_targetDataSource (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceDelete: root of subtree no. 1 can not be made a codeGen_DataSourceDelete_targetDataSource node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceDelete_formalParams) codeGen_MkcodeGen_DataSourceDelete_formalParams (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceDelete: root of subtree no. 2 can not be made a codeGen_DataSourceDelete_formalParams node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataSourceDelete_name) codeGen_MkcodeGen_DataSourceDelete_name (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceDelete: root of subtree no. 3 can not be made a codeGen_DataSourceDelete_name node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataSourceDelete_position) codeGen_MkcodeGen_DataSourceDelete_position (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceDelete: root of subtree no. 4 can not be made a codeGen_DataSourceDelete_position node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataSourceDelete_conditions) codeGen_MkcodeGen_DataSourceDelete_conditions (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceDelete: root of subtree no. 5 can not be made a codeGen_DataSourceDelete_conditions node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceDelete;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceDelete */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_targetDataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_targetDataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceDelete_targetDataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceDelete_targetDataSource;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceDelete_targetDataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceDelete_targetDataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceDelete_targetDataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceDelete_targetDataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceDelete_targetDataSource */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataSourceDelete_formalParams)_new;
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataSourceDelete_formalParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataSourceDelete_formalParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceFormalParam)) 
		return codeGen__MkcodeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams (_coordref, codeGen_MkcodeGen_DataSourceFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_DataSourceDelete_formalParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataSourceDelete_formalParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataSourceDelete_formalParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataSourceDelete_formalParams)
	l =_ElempcodeGen_DataSourceDelete_formalParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataSourceDelete_formalParams)
	r = _ElempcodeGen_DataSourceDelete_formalParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_formalParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_formalParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataSourceDelete_formalParams root;
	codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataSourceDelete_formalParams)
		t = _ElempcodeGen_DataSourceDelete_formalParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)(((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataSourceDelete_formalParams;
#else
list = (codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataSourceDelete_formalParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataSourceDelete_formalParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataSourceDelete_formalParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams)
	{codeGen__TPPLST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceFormalParampcodeGen_DataSourceDelete_formalParams;
	} this code has been moved to _ElempcodeGen_DataSourceDelete_formalParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataSourceDelete_formalParams;
#else
root = (codeGen__TPPpcodeGen_DataSourceDelete_formalParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceDelete_formalParams));
#endif
root->_prod = codeGen_RULEpcodeGen_DataSourceDelete_formalParams;
root->_desc1 = (codeGen__TSPLST_codeGen_DataSourceDelete_formalParams)list;
{ codeGen__TPPpcodeGen_DataSourceDelete_formalParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataSourceDelete_formalParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceDelete_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceDelete_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceDelete_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceDelete_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceDelete_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceDelete_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceDelete_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceDelete_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceDelete_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceDelete_position;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceDelete_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceDelete_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceDelete_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceDelete_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceDelete_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceActualParam (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceActualParam (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_DataSourceActualParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceActualParam;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceActualParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceActualParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceActualParam;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceActualParam_formalParam) codeGen_MkcodeGen_DataSourceActualParam_formalParam (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceActualParam: root of subtree no. 1 can not be made a codeGen_DataSourceActualParam_formalParam node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceActualParam_expressions) codeGen_MkcodeGen_DataSourceActualParam_expressions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceActualParam: root of subtree no. 2 can not be made a codeGen_DataSourceActualParam_expressions node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceActualParam;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceActualParam_formalParam (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceActualParam_formalParam (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceActualParam_formalParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceActualParam_formalParam;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceActualParam_formalParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceActualParam_formalParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceActualParam_formalParam;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceActualParam_formalParam;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceFormalParam (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceFormalParam (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_DataSourceFormalParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceFormalParam;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceFormalParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceFormalParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceFormalParam;
_currn->_desc1 = (codeGen__TSPcodeGen_DataSourceFormalParam_name) codeGen_MkcodeGen_DataSourceFormalParam_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceFormalParam: root of subtree no. 1 can not be made a codeGen_DataSourceFormalParam_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataSourceFormalParam_type) codeGen_MkcodeGen_DataSourceFormalParam_type (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataSourceFormalParam: root of subtree no. 2 can not be made a codeGen_DataSourceFormalParam_type node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceFormalParam;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceFormalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataSourceFormalParam_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataSourceFormalParam_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataSourceFormalParam_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataSourceFormalParam_name;
#else
_currn = (codeGen__TPPpcodeGen_DataSourceFormalParam_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataSourceFormalParam_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataSourceFormalParam_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataSourceFormalParam_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataSourceFormalParam_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Paragraph (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_Paragraph (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_Paragraph _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Paragraph;
#else
_currn = (codeGen__TPPpcodeGen_Paragraph) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Paragraph));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Paragraph;
_currn->_desc1 = (codeGen__TSPcodeGen_Paragraph_styleSheetBlock) codeGen_MkcodeGen_Paragraph_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Paragraph: root of subtree no. 1 can not be made a codeGen_Paragraph_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Paragraph_fragments) codeGen_MkcodeGen_Paragraph_fragments (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Paragraph: root of subtree no. 2 can not be made a codeGen_Paragraph_fragments node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Paragraph;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Paragraph */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Paragraph_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Paragraph_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Paragraph_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Paragraph_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_Paragraph_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Paragraph_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Paragraph_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Paragraph_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Paragraph_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TextFragmentpcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TextFragmentpcodeGen_Paragraph_fragments (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments;
#else
	_new = (codeGen__TPPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TextFragmentpcodeGen_Paragraph_fragments;
	_new->_desc1 = (codeGen__TSPcodeGen_TextFragment)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Paragraph_fragments)_new;
	{codeGen__TPPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TextFragmentpcodeGen_Paragraph_fragments;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ImageFragmentpcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ImageFragmentpcodeGen_Paragraph_fragments (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments;
#else
	_new = (codeGen__TPPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments;
	_new->_desc1 = (codeGen__TSPcodeGen_ImageFragment)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Paragraph_fragments)_new;
	{codeGen__TPPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_OutputFragmentpcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_OutputFragmentpcodeGen_Paragraph_fragments (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments;
#else
	_new = (codeGen__TPPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments));
#endif
	_new->_prod = codeGen_RULELST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments;
	_new->_desc1 = (codeGen__TSPcodeGen_OutputFragment)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Paragraph_fragments)_new;
	{codeGen__TPPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Paragraph_fragments (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_TextFragment)) 
		return codeGen__MkcodeGen_TextFragmentpcodeGen_Paragraph_fragments (_coordref, codeGen_MkcodeGen_TextFragment (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ImageFragment)) 
		return codeGen__MkcodeGen_ImageFragmentpcodeGen_Paragraph_fragments (_coordref, codeGen_MkcodeGen_ImageFragment (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_OutputFragment)) 
		return codeGen__MkcodeGen_OutputFragmentpcodeGen_Paragraph_fragments (_coordref, codeGen_MkcodeGen_OutputFragment (_coordref, t));
	message (DEADLY, "a pcodeGen_Paragraph_fragments list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Paragraph_fragments (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Paragraph_fragments)
	l =_ElempcodeGen_Paragraph_fragments (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Paragraph_fragments)
	r = _ElempcodeGen_Paragraph_fragments (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Paragraph_fragments)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Paragraph_fragments)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Paragraph_fragments)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Paragraph_fragments)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Paragraph_fragments (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Paragraph_fragments (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Paragraph_fragments root;
	codeGen__TPPLST_0pcodeGen_Paragraph_fragments list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Paragraph_fragments)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Paragraph_fragments)
		t = _ElempcodeGen_Paragraph_fragments (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Paragraph_fragments)(((codeGen__TPPLST_0pcodeGen_Paragraph_fragments) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Paragraph_fragments) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Paragraph_fragments;
#else
list = (codeGen__TPPLST_0pcodeGen_Paragraph_fragments) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Paragraph_fragments));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Paragraph_fragments;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Paragraph_fragments)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TextFragmentpcodeGen_Paragraph_fragments)
	{codeGen__TPPLST_codeGen_TextFragmentpcodeGen_Paragraph_fragments _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TextFragmentpcodeGen_Paragraph_fragments;
	} this code has been moved to _ElempcodeGen_Paragraph_fragments*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments)
	{codeGen__TPPLST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ImageFragmentpcodeGen_Paragraph_fragments;
	} this code has been moved to _ElempcodeGen_Paragraph_fragments*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments)
	{codeGen__TPPLST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_OutputFragmentpcodeGen_Paragraph_fragments;
	} this code has been moved to _ElempcodeGen_Paragraph_fragments*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Paragraph_fragments;
#else
root = (codeGen__TPPpcodeGen_Paragraph_fragments) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Paragraph_fragments));
#endif
root->_prod = codeGen_RULEpcodeGen_Paragraph_fragments;
root->_desc1 = (codeGen__TSPLST_codeGen_Paragraph_fragments)list;
{ codeGen__TPPpcodeGen_Paragraph_fragments _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Paragraph_fragments;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TextFragment (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_TextFragment (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_TextFragment _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TextFragment;
#else
_currn = (codeGen__TPPpcodeGen_TextFragment) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TextFragment));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TextFragment;
_currn->_desc1 = (codeGen__TSPcodeGen_TextFragment_link) codeGen_MkcodeGen_TextFragment_link (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TextFragment: root of subtree no. 1 can not be made a codeGen_TextFragment_link node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_TextFragment_value) codeGen_MkcodeGen_TextFragment_value (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TextFragment: root of subtree no. 2 can not be made a codeGen_TextFragment_value node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_TextFragment_fontstyle) codeGen_MkcodeGen_TextFragment_fontstyle (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TextFragment: root of subtree no. 3 can not be made a codeGen_TextFragment_fontstyle node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TextFragment;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TextFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TextFragment_link (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_TextFragment_link (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_TextFragment_link _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TextFragment_link;
#else
_currn = (codeGen__TPPpcodeGen_TextFragment_link) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TextFragment_link));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TextFragment_link;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TextFragment_link;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TextFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TextFragment_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_TextFragment_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_TextFragment_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TextFragment_value;
#else
_currn = (codeGen__TPPpcodeGen_TextFragment_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TextFragment_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TextFragment_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TextFragment_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TextFragment_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TextFragment_fontstyle (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_TextFragment_fontstyle (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_TextFragment_fontstyle _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TextFragment_fontstyle;
#else
_currn = (codeGen__TPPpcodeGen_TextFragment_fontstyle) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TextFragment_fontstyle));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TextFragment_fontstyle;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TextFragment_fontstyle;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TextFragment_fontstyle */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
#endif
{	codeGen__TPPpcodeGen_ImageFragment _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment;
_currn->_desc1 = (codeGen__TSPcodeGen_ImageFragment_link) codeGen_MkcodeGen_ImageFragment_link (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 1 can not be made a codeGen_ImageFragment_link node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ImageFragment_width) codeGen_MkcodeGen_ImageFragment_width (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 2 can not be made a codeGen_ImageFragment_width node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_ImageFragment_height) codeGen_MkcodeGen_ImageFragment_height (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 3 can not be made a codeGen_ImageFragment_height node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_ImageFragment_border) codeGen_MkcodeGen_ImageFragment_border (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 4 can not be made a codeGen_ImageFragment_border node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_ImageFragment_verticalAlign) codeGen_MkcodeGen_ImageFragment_verticalAlign (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 5 can not be made a codeGen_ImageFragment_verticalAlign node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_ImageFragment_float) codeGen_MkcodeGen_ImageFragment_float (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 6 can not be made a codeGen_ImageFragment_float node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_ImageFragment_text) codeGen_MkcodeGen_ImageFragment_text (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 7 can not be made a codeGen_ImageFragment_text node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_ImageFragment_imageName) codeGen_MkcodeGen_ImageFragment_imageName (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageFragment: root of subtree no. 8 can not be made a codeGen_ImageFragment_imageName node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_link (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_link (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_link _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_link;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_link) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_link));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_link;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_link;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_width (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_width (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_width _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_width;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_width) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_width));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_width;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_width;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_width */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_height (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_height (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_height _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_height;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_height) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_height));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_height;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_height;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_height */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_border (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_border (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_border _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_border;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_border) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_border));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_border;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_border;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_border */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_verticalAlign (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_verticalAlign (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_verticalAlign _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_verticalAlign;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_verticalAlign) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_verticalAlign));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_verticalAlign;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_verticalAlign;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_verticalAlign */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_float (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_float (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_float _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_float;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_float) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_float));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_float;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_float;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_float */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_text (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_text (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_text _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_text;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_text) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_text));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_text;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_text;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_text */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageFragment_imageName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageFragment_imageName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageFragment_imageName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageFragment_imageName;
#else
_currn = (codeGen__TPPpcodeGen_ImageFragment_imageName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageFragment_imageName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageFragment_imageName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageFragment_imageName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageFragment_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Heading (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_Heading (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_Heading _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Heading;
#else
_currn = (codeGen__TPPpcodeGen_Heading) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Heading));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Heading;
_currn->_desc1 = (codeGen__TSPcodeGen_Heading_styleSheetBlock) codeGen_MkcodeGen_Heading_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Heading: root of subtree no. 1 can not be made a codeGen_Heading_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Heading_text) codeGen_MkcodeGen_Heading_text (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Heading: root of subtree no. 2 can not be made a codeGen_Heading_text node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_Heading_level) codeGen_MkcodeGen_Heading_level (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Heading: root of subtree no. 3 can not be made a codeGen_Heading_level node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Heading;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Heading */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Heading_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Heading_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Heading_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Heading_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_Heading_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Heading_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Heading_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Heading_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Heading_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Heading_text (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Heading_text (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Heading_text _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Heading_text;
#else
_currn = (codeGen__TPPpcodeGen_Heading_text) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Heading_text));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Heading_text;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Heading_text;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Heading_text */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Heading_level (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Heading_level (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Heading_level _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Heading_level;
#else
_currn = (codeGen__TPPpcodeGen_Heading_level) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Heading_level));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Heading_level;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Heading_level;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Heading_level */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Table (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_Table (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_Table _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Table;
#else
_currn = (codeGen__TPPpcodeGen_Table) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Table));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Table;
_currn->_desc1 = (codeGen__TSPcodeGen_Table_styleSheetBlock) codeGen_MkcodeGen_Table_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Table: root of subtree no. 1 can not be made a codeGen_Table_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Table_rows) codeGen_MkcodeGen_Table_rows (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Table: root of subtree no. 2 can not be made a codeGen_Table_rows node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_Table_columns) codeGen_MkcodeGen_Table_columns (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Table: root of subtree no. 3 can not be made a codeGen_Table_columns node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Table;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Table */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Table_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Table_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Table_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Table_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_Table_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Table_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Table_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Table_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Table_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TableRowpcodeGen_Table_rows (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TableRowpcodeGen_Table_rows (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TableRowpcodeGen_Table_rows _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TableRowpcodeGen_Table_rows;
#else
	_new = (codeGen__TPPLST_codeGen_TableRowpcodeGen_Table_rows) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TableRowpcodeGen_Table_rows));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TableRowpcodeGen_Table_rows;
	_new->_desc1 = (codeGen__TSPcodeGen_TableRow)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Table_rows)_new;
	{codeGen__TPPLST_codeGen_TableRowpcodeGen_Table_rows _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TableRowpcodeGen_Table_rows;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Table_rows (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Table_rows (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_TableRow)) 
		return codeGen__MkcodeGen_TableRowpcodeGen_Table_rows (_coordref, codeGen_MkcodeGen_TableRow (_coordref, t));
	message (DEADLY, "a pcodeGen_Table_rows list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Table_rows (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Table_rows (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Table_rows)
	l =_ElempcodeGen_Table_rows (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Table_rows)
	r = _ElempcodeGen_Table_rows (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Table_rows)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Table_rows)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Table_rows)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Table_rows)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Table_rows (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Table_rows (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Table_rows root;
	codeGen__TPPLST_0pcodeGen_Table_rows list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Table_rows)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Table_rows)
		t = _ElempcodeGen_Table_rows (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Table_rows)(((codeGen__TPPLST_0pcodeGen_Table_rows) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Table_rows) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Table_rows;
#else
list = (codeGen__TPPLST_0pcodeGen_Table_rows) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Table_rows));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Table_rows;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Table_rows)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TableRowpcodeGen_Table_rows)
	{codeGen__TPPLST_codeGen_TableRowpcodeGen_Table_rows _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TableRowpcodeGen_Table_rows;
	} this code has been moved to _ElempcodeGen_Table_rows*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Table_rows;
#else
root = (codeGen__TPPpcodeGen_Table_rows) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Table_rows));
#endif
root->_prod = codeGen_RULEpcodeGen_Table_rows;
root->_desc1 = (codeGen__TSPLST_codeGen_Table_rows)list;
{ codeGen__TPPpcodeGen_Table_rows _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Table_rows;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TableColumnpcodeGen_Table_columns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TableColumnpcodeGen_Table_columns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TableColumnpcodeGen_Table_columns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TableColumnpcodeGen_Table_columns;
#else
	_new = (codeGen__TPPLST_codeGen_TableColumnpcodeGen_Table_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TableColumnpcodeGen_Table_columns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TableColumnpcodeGen_Table_columns;
	_new->_desc1 = (codeGen__TSPcodeGen_TableColumn)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Table_columns)_new;
	{codeGen__TPPLST_codeGen_TableColumnpcodeGen_Table_columns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TableColumnpcodeGen_Table_columns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Table_columns (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Table_columns (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_TableColumn)) 
		return codeGen__MkcodeGen_TableColumnpcodeGen_Table_columns (_coordref, codeGen_MkcodeGen_TableColumn (_coordref, t));
	message (DEADLY, "a pcodeGen_Table_columns list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Table_columns (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Table_columns (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Table_columns)
	l =_ElempcodeGen_Table_columns (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Table_columns)
	r = _ElempcodeGen_Table_columns (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Table_columns)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Table_columns)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Table_columns)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Table_columns)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Table_columns (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Table_columns (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Table_columns root;
	codeGen__TPPLST_0pcodeGen_Table_columns list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Table_columns)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Table_columns)
		t = _ElempcodeGen_Table_columns (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Table_columns)(((codeGen__TPPLST_0pcodeGen_Table_columns) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Table_columns) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Table_columns;
#else
list = (codeGen__TPPLST_0pcodeGen_Table_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Table_columns));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Table_columns;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Table_columns)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TableColumnpcodeGen_Table_columns)
	{codeGen__TPPLST_codeGen_TableColumnpcodeGen_Table_columns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TableColumnpcodeGen_Table_columns;
	} this code has been moved to _ElempcodeGen_Table_columns*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Table_columns;
#else
root = (codeGen__TPPpcodeGen_Table_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Table_columns));
#endif
root->_prod = codeGen_RULEpcodeGen_Table_columns;
root->_desc1 = (codeGen__TSPLST_codeGen_Table_columns)list;
{ codeGen__TPPpcodeGen_Table_columns _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Table_columns;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TableRow (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_TableRow (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_TableRow _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TableRow;
#else
_currn = (codeGen__TPPpcodeGen_TableRow) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TableRow));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TableRow;
_currn->_desc1 = (codeGen__TSPcodeGen_TableRow_cells) codeGen_MkcodeGen_TableRow_cells (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TableRow: root of subtree no. 1 can not be made a codeGen_TableRow_cells node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TableRow;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TableRow */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_TableCellpcodeGen_TableRow_cells (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_TableCellpcodeGen_TableRow_cells (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_TableCellpcodeGen_TableRow_cells _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_TableCellpcodeGen_TableRow_cells;
#else
	_new = (codeGen__TPPLST_codeGen_TableCellpcodeGen_TableRow_cells) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_TableCellpcodeGen_TableRow_cells));
#endif
	_new->_prod = codeGen_RULELST_codeGen_TableCellpcodeGen_TableRow_cells;
	_new->_desc1 = (codeGen__TSPcodeGen_TableCell)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_TableRow_cells)_new;
	{codeGen__TPPLST_codeGen_TableCellpcodeGen_TableRow_cells _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_TableCellpcodeGen_TableRow_cells;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_TableRow_cells (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_TableRow_cells (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_TableCell)) 
		return codeGen__MkcodeGen_TableCellpcodeGen_TableRow_cells (_coordref, codeGen_MkcodeGen_TableCell (_coordref, t));
	message (DEADLY, "a pcodeGen_TableRow_cells list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_TableRow_cells (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_TableRow_cells (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_TableRow_cells)
	l =_ElempcodeGen_TableRow_cells (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_TableRow_cells)
	r = _ElempcodeGen_TableRow_cells (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_TableRow_cells)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_TableRow_cells)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_TableRow_cells)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_TableRow_cells)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TableRow_cells (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_TableRow_cells (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_TableRow_cells root;
	codeGen__TPPLST_0pcodeGen_TableRow_cells list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_TableRow_cells)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_TableRow_cells)
		t = _ElempcodeGen_TableRow_cells (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_TableRow_cells)(((codeGen__TPPLST_0pcodeGen_TableRow_cells) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_TableRow_cells) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_TableRow_cells;
#else
list = (codeGen__TPPLST_0pcodeGen_TableRow_cells) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_TableRow_cells));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_TableRow_cells;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_TableRow_cells)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_TableCellpcodeGen_TableRow_cells)
	{codeGen__TPPLST_codeGen_TableCellpcodeGen_TableRow_cells _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_TableCellpcodeGen_TableRow_cells;
	} this code has been moved to _ElempcodeGen_TableRow_cells*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_TableRow_cells;
#else
root = (codeGen__TPPpcodeGen_TableRow_cells) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TableRow_cells));
#endif
root->_prod = codeGen_RULEpcodeGen_TableRow_cells;
root->_desc1 = (codeGen__TSPLST_codeGen_TableRow_cells)list;
{ codeGen__TPPpcodeGen_TableRow_cells _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_TableRow_cells;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TableColumn (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_TableColumn (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_TableColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TableColumn;
#else
_currn = (codeGen__TPPpcodeGen_TableColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TableColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TableColumn;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TableColumn;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TableColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TableCell (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_TableCell (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_TableCell _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TableCell;
#else
_currn = (codeGen__TPPpcodeGen_TableCell) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TableCell));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TableCell;
_currn->_desc1 = (codeGen__TSPcodeGen_TableCell_columnRef) codeGen_MkcodeGen_TableCell_columnRef (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TableCell: root of subtree no. 1 can not be made a codeGen_TableCell_columnRef node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_TableCell_content) codeGen_MkcodeGen_TableCell_content (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_TableCell: root of subtree no. 2 can not be made a codeGen_TableCell_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TableCell;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TableCell */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_TableCell_columnRef (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_TableCell_columnRef (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_TableCell_columnRef _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_TableCell_columnRef;
#else
_currn = (codeGen__TPPpcodeGen_TableCell_columnRef) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_TableCell_columnRef));
#endif
_currn->_prod = codeGen_RULEpcodeGen_TableCell_columnRef;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_TableCell_columnRef;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_TableCell_columnRef */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheet (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheet (_coordref, _TERM1,_desc1,_desc2,_desc3)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
#endif
{	codeGen__TPPpcodeGen_StyleSheet _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheet;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheet) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheet));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheet;
_currn->_desc1 = (codeGen__TSPcodeGen_StyleSheet_name) codeGen_MkcodeGen_StyleSheet_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheet: root of subtree no. 1 can not be made a codeGen_StyleSheet_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_StyleSheet_custom) codeGen_MkcodeGen_StyleSheet_custom (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheet: root of subtree no. 2 can not be made a codeGen_StyleSheet_custom node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_StyleSheet_default) codeGen_MkcodeGen_StyleSheet_default (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheet: root of subtree no. 3 can not be made a codeGen_StyleSheet_default node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheet;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheet */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheet_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheet_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StyleSheet_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheet_name;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheet_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheet_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheet_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheet_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheet_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom;
#else
	_new = (codeGen__TPPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom;
	_new->_desc1 = (codeGen__TSPcodeGen_CustomStyleSheetBlock)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_StyleSheet_custom)_new;
	{codeGen__TPPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_StyleSheet_custom (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_StyleSheet_custom (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CustomStyleSheetBlock)) 
		return codeGen__MkcodeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom (_coordref, codeGen_MkcodeGen_CustomStyleSheetBlock (_coordref, t));
	message (DEADLY, "a pcodeGen_StyleSheet_custom list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_StyleSheet_custom (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_StyleSheet_custom (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_custom)
	l =_ElempcodeGen_StyleSheet_custom (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_custom)
	r = _ElempcodeGen_StyleSheet_custom (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_StyleSheet_custom)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_StyleSheet_custom)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_StyleSheet_custom)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_StyleSheet_custom)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheet_custom (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheet_custom (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_StyleSheet_custom root;
	codeGen__TPPLST_0pcodeGen_StyleSheet_custom list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_StyleSheet_custom)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_custom)
		t = _ElempcodeGen_StyleSheet_custom (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_StyleSheet_custom)(((codeGen__TPPLST_0pcodeGen_StyleSheet_custom) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_StyleSheet_custom) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_StyleSheet_custom;
#else
list = (codeGen__TPPLST_0pcodeGen_StyleSheet_custom) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_StyleSheet_custom));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_StyleSheet_custom;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_StyleSheet_custom)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom)
	{codeGen__TPPLST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CustomStyleSheetBlockpcodeGen_StyleSheet_custom;
	} this code has been moved to _ElempcodeGen_StyleSheet_custom*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_StyleSheet_custom;
#else
root = (codeGen__TPPpcodeGen_StyleSheet_custom) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheet_custom));
#endif
root->_prod = codeGen_RULEpcodeGen_StyleSheet_custom;
root->_desc1 = (codeGen__TSPLST_codeGen_StyleSheet_custom)list;
{ codeGen__TPPpcodeGen_StyleSheet_custom _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_StyleSheet_custom;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default;
#else
	_new = (codeGen__TPPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default));
#endif
	_new->_prod = codeGen_RULELST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default;
	_new->_desc1 = (codeGen__TSPcodeGen_HtmlStyleSheetBlock)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_StyleSheet_default)_new;
	{codeGen__TPPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_StyleSheet_default (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_StyleSheet_default (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_HtmlStyleSheetBlock)) 
		return codeGen__MkcodeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default (_coordref, codeGen_MkcodeGen_HtmlStyleSheetBlock (_coordref, t));
	message (DEADLY, "a pcodeGen_StyleSheet_default list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_StyleSheet_default (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_StyleSheet_default (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_default)
	l =_ElempcodeGen_StyleSheet_default (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_default)
	r = _ElempcodeGen_StyleSheet_default (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_StyleSheet_default)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_StyleSheet_default)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_StyleSheet_default)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_StyleSheet_default)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheet_default (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheet_default (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_StyleSheet_default root;
	codeGen__TPPLST_0pcodeGen_StyleSheet_default list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_StyleSheet_default)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_StyleSheet_default)
		t = _ElempcodeGen_StyleSheet_default (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_StyleSheet_default)(((codeGen__TPPLST_0pcodeGen_StyleSheet_default) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_StyleSheet_default) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_StyleSheet_default;
#else
list = (codeGen__TPPLST_0pcodeGen_StyleSheet_default) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_StyleSheet_default));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_StyleSheet_default;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_StyleSheet_default)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default)
	{codeGen__TPPLST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_HtmlStyleSheetBlockpcodeGen_StyleSheet_default;
	} this code has been moved to _ElempcodeGen_StyleSheet_default*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_StyleSheet_default;
#else
root = (codeGen__TPPpcodeGen_StyleSheet_default) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheet_default));
#endif
root->_prod = codeGen_RULEpcodeGen_StyleSheet_default;
root->_desc1 = (codeGen__TSPLST_codeGen_StyleSheet_default)list;
{ codeGen__TPPpcodeGen_StyleSheet_default _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_StyleSheet_default;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_CustomStyleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_CustomStyleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_CustomStyleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CustomStyleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_CustomStyleSheetBlock;
_currn->_desc1 = (codeGen__TSPcodeGen_CustomStyleSheetBlock_pairs) codeGen_MkcodeGen_CustomStyleSheetBlock_pairs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_CustomStyleSheetBlock: root of subtree no. 1 can not be made a codeGen_CustomStyleSheetBlock_pairs node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_CustomStyleSheetBlock_name) codeGen_MkcodeGen_CustomStyleSheetBlock_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_CustomStyleSheetBlock: root of subtree no. 2 can not be made a codeGen_CustomStyleSheetBlock_name node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_CustomStyleSheetBlock;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_CustomStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs;
#else
	_new = (codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs;
	_new->_desc1 = (codeGen__TSPcodeGen_StyleSheetPair)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_CustomStyleSheetBlock_pairs)_new;
	{codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_CustomStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_CustomStyleSheetBlock_pairs (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StyleSheetPair)) 
		return codeGen__MkcodeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs (_coordref, codeGen_MkcodeGen_StyleSheetPair (_coordref, t));
	message (DEADLY, "a pcodeGen_CustomStyleSheetBlock_pairs list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_CustomStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_CustomStyleSheetBlock_pairs (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_CustomStyleSheetBlock_pairs)
	l =_ElempcodeGen_CustomStyleSheetBlock_pairs (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_CustomStyleSheetBlock_pairs)
	r = _ElempcodeGen_CustomStyleSheetBlock_pairs (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock_pairs (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_CustomStyleSheetBlock_pairs root;
	codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_CustomStyleSheetBlock_pairs)
		t = _ElempcodeGen_CustomStyleSheetBlock_pairs (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)(((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_CustomStyleSheetBlock_pairs;
#else
list = (codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_CustomStyleSheetBlock_pairs));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_CustomStyleSheetBlock_pairs;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_CustomStyleSheetBlock_pairs)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs)
	{codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StyleSheetPairpcodeGen_CustomStyleSheetBlock_pairs;
	} this code has been moved to _ElempcodeGen_CustomStyleSheetBlock_pairs*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_CustomStyleSheetBlock_pairs;
#else
root = (codeGen__TPPpcodeGen_CustomStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CustomStyleSheetBlock_pairs));
#endif
root->_prod = codeGen_RULEpcodeGen_CustomStyleSheetBlock_pairs;
root->_desc1 = (codeGen__TSPLST_codeGen_CustomStyleSheetBlock_pairs)list;
{ codeGen__TPPpcodeGen_CustomStyleSheetBlock_pairs _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_CustomStyleSheetBlock_pairs;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_CustomStyleSheetBlock_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_CustomStyleSheetBlock_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_CustomStyleSheetBlock_name;
#else
_currn = (codeGen__TPPpcodeGen_CustomStyleSheetBlock_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CustomStyleSheetBlock_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_CustomStyleSheetBlock_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_CustomStyleSheetBlock_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_CustomStyleSheetBlock_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheetPair (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheetPair (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_StyleSheetPair _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheetPair;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheetPair) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheetPair));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheetPair;
_currn->_desc1 = (codeGen__TSPcodeGen_StyleSheetPair_category) codeGen_MkcodeGen_StyleSheetPair_category (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheetPair: root of subtree no. 1 can not be made a codeGen_StyleSheetPair_category node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_StyleSheetPair_attribute) codeGen_MkcodeGen_StyleSheetPair_attribute (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheetPair: root of subtree no. 2 can not be made a codeGen_StyleSheetPair_attribute node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_StyleSheetPair_valueCategory) codeGen_MkcodeGen_StyleSheetPair_valueCategory (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheetPair: root of subtree no. 3 can not be made a codeGen_StyleSheetPair_valueCategory node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_StyleSheetPair_value) codeGen_MkcodeGen_StyleSheetPair_value (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleSheetPair: root of subtree no. 4 can not be made a codeGen_StyleSheetPair_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheetPair;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheetPair */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_category (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_category (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StyleSheetPair_category _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheetPair_category;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheetPair_category) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheetPair_category));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheetPair_category;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheetPair_category;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheetPair_category */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_attribute (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_attribute (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StyleSheetPair_attribute _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheetPair_attribute;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheetPair_attribute) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheetPair_attribute));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheetPair_attribute;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheetPair_attribute;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheetPair_attribute */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_valueCategory (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StyleSheetPair_valueCategory (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StyleSheetPair_valueCategory _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleSheetPair_valueCategory;
#else
_currn = (codeGen__TPPpcodeGen_StyleSheetPair_valueCategory) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleSheetPair_valueCategory));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleSheetPair_valueCategory;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleSheetPair_valueCategory;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleSheetPair_valueCategory */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SizeValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_SizeValue (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_SizeValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SizeValue;
#else
_currn = (codeGen__TPPpcodeGen_SizeValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SizeValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SizeValue;
_currn->_desc1 = (codeGen__TSPcodeGen_SizeValue_measure) codeGen_MkcodeGen_SizeValue_measure (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SizeValue: root of subtree no. 1 can not be made a codeGen_SizeValue_measure node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_SizeValue_value) codeGen_MkcodeGen_SizeValue_value (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SizeValue: root of subtree no. 2 can not be made a codeGen_SizeValue_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SizeValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SizeValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SizeValue_measure (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SizeValue_measure (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SizeValue_measure _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SizeValue_measure;
#else
_currn = (codeGen__TPPpcodeGen_SizeValue_measure) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SizeValue_measure));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SizeValue_measure;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SizeValue_measure;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SizeValue_measure */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SizeValue_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SizeValue_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SizeValue_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SizeValue_value;
#else
_currn = (codeGen__TPPpcodeGen_SizeValue_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SizeValue_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SizeValue_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SizeValue_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SizeValue_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColorValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ColorValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ColorValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColorValue;
#else
_currn = (codeGen__TPPpcodeGen_ColorValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColorValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColorValue;
_currn->_desc1 = (codeGen__TSPcodeGen_ColorValue_color) codeGen_MkcodeGen_ColorValue_color (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ColorValue: root of subtree no. 1 can not be made a codeGen_ColorValue_color node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColorValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColorValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ColorValue_color (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ColorValue_color (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ColorValue_color _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ColorValue_color;
#else
_currn = (codeGen__TPPpcodeGen_ColorValue_color) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ColorValue_color));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ColorValue_color;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ColorValue_color;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ColorValue_color */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FamilyValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_FamilyValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_FamilyValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FamilyValue;
#else
_currn = (codeGen__TPPpcodeGen_FamilyValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FamilyValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FamilyValue;
_currn->_desc1 = (codeGen__TSPcodeGen_FamilyValue_family) codeGen_MkcodeGen_FamilyValue_family (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FamilyValue: root of subtree no. 1 can not be made a codeGen_FamilyValue_family node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FamilyValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FamilyValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FamilyValue_family (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FamilyValue_family (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FamilyValue_family _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FamilyValue_family;
#else
_currn = (codeGen__TPPpcodeGen_FamilyValue_family) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FamilyValue_family));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FamilyValue_family;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FamilyValue_family;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FamilyValue_family */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_StyleValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_StyleValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleValue;
#else
_currn = (codeGen__TPPpcodeGen_StyleValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleValue;
_currn->_desc1 = (codeGen__TSPcodeGen_StyleValue_style) codeGen_MkcodeGen_StyleValue_style (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StyleValue: root of subtree no. 1 can not be made a codeGen_StyleValue_style node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StyleValue_style (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StyleValue_style (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StyleValue_style _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StyleValue_style;
#else
_currn = (codeGen__TPPpcodeGen_StyleValue_style) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StyleValue_style));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StyleValue_style;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StyleValue_style;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StyleValue_style */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_HeadingValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_HeadingValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_HeadingValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_HeadingValue;
#else
_currn = (codeGen__TPPpcodeGen_HeadingValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_HeadingValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_HeadingValue;
_currn->_desc1 = (codeGen__TSPcodeGen_HeadingValue_heading) codeGen_MkcodeGen_HeadingValue_heading (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_HeadingValue: root of subtree no. 1 can not be made a codeGen_HeadingValue_heading node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_HeadingValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_HeadingValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_HeadingValue_heading (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_HeadingValue_heading (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_HeadingValue_heading _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_HeadingValue_heading;
#else
_currn = (codeGen__TPPpcodeGen_HeadingValue_heading) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_HeadingValue_heading));
#endif
_currn->_prod = codeGen_RULEpcodeGen_HeadingValue_heading;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_HeadingValue_heading;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_HeadingValue_heading */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_AlignmentValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_AlignmentValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_AlignmentValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_AlignmentValue;
#else
_currn = (codeGen__TPPpcodeGen_AlignmentValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_AlignmentValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_AlignmentValue;
_currn->_desc1 = (codeGen__TSPcodeGen_AlignmentValue_alignment) codeGen_MkcodeGen_AlignmentValue_alignment (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_AlignmentValue: root of subtree no. 1 can not be made a codeGen_AlignmentValue_alignment node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_AlignmentValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_AlignmentValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_AlignmentValue_alignment (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_AlignmentValue_alignment (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_AlignmentValue_alignment _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_AlignmentValue_alignment;
#else
_currn = (codeGen__TPPpcodeGen_AlignmentValue_alignment) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_AlignmentValue_alignment));
#endif
_currn->_prod = codeGen_RULEpcodeGen_AlignmentValue_alignment;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_AlignmentValue_alignment;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_AlignmentValue_alignment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListValue;
#else
_currn = (codeGen__TPPpcodeGen_ListValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListValue;
_currn->_desc1 = (codeGen__TSPcodeGen_ListValue_listed) codeGen_MkcodeGen_ListValue_listed (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListValue: root of subtree no. 1 can not be made a codeGen_ListValue_listed node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListValue_listed (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListValue_listed (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListValue_listed _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListValue_listed;
#else
_currn = (codeGen__TPPpcodeGen_ListValue_listed) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListValue_listed));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListValue_listed;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListValue_listed;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListValue_listed */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CustomValue (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_CustomValue (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_CustomValue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_CustomValue;
#else
_currn = (codeGen__TPPpcodeGen_CustomValue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CustomValue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_CustomValue;
_currn->_desc1 = (codeGen__TSPcodeGen_CustomValue_value) codeGen_MkcodeGen_CustomValue_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_CustomValue: root of subtree no. 1 can not be made a codeGen_CustomValue_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_CustomValue;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_CustomValue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CustomValue_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_CustomValue_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_CustomValue_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_CustomValue_value;
#else
_currn = (codeGen__TPPpcodeGen_CustomValue_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CustomValue_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_CustomValue_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_CustomValue_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_CustomValue_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_HtmlStyleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_HtmlStyleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_HtmlStyleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_HtmlStyleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_HtmlStyleSheetBlock;
_currn->_desc1 = (codeGen__TSPcodeGen_HtmlStyleSheetBlock_pairs) codeGen_MkcodeGen_HtmlStyleSheetBlock_pairs (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_HtmlStyleSheetBlock: root of subtree no. 1 can not be made a codeGen_HtmlStyleSheetBlock_pairs node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_HtmlStyleSheetBlock_name) codeGen_MkcodeGen_HtmlStyleSheetBlock_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_HtmlStyleSheetBlock: root of subtree no. 2 can not be made a codeGen_HtmlStyleSheetBlock_name node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_HtmlStyleSheetBlock;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_HtmlStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs;
#else
	_new = (codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs;
	_new->_desc1 = (codeGen__TSPcodeGen_StyleSheetPair)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_HtmlStyleSheetBlock_pairs)_new;
	{codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_HtmlStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_HtmlStyleSheetBlock_pairs (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StyleSheetPair)) 
		return codeGen__MkcodeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs (_coordref, codeGen_MkcodeGen_StyleSheetPair (_coordref, t));
	message (DEADLY, "a pcodeGen_HtmlStyleSheetBlock_pairs list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_HtmlStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_HtmlStyleSheetBlock_pairs (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_HtmlStyleSheetBlock_pairs)
	l =_ElempcodeGen_HtmlStyleSheetBlock_pairs (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_HtmlStyleSheetBlock_pairs)
	r = _ElempcodeGen_HtmlStyleSheetBlock_pairs (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock_pairs (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock_pairs (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_HtmlStyleSheetBlock_pairs root;
	codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_HtmlStyleSheetBlock_pairs)
		t = _ElempcodeGen_HtmlStyleSheetBlock_pairs (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)(((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_HtmlStyleSheetBlock_pairs;
#else
list = (codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_HtmlStyleSheetBlock_pairs));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_HtmlStyleSheetBlock_pairs;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_HtmlStyleSheetBlock_pairs)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs)
	{codeGen__TPPLST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StyleSheetPairpcodeGen_HtmlStyleSheetBlock_pairs;
	} this code has been moved to _ElempcodeGen_HtmlStyleSheetBlock_pairs*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_HtmlStyleSheetBlock_pairs;
#else
root = (codeGen__TPPpcodeGen_HtmlStyleSheetBlock_pairs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_HtmlStyleSheetBlock_pairs));
#endif
root->_prod = codeGen_RULEpcodeGen_HtmlStyleSheetBlock_pairs;
root->_desc1 = (codeGen__TSPLST_codeGen_HtmlStyleSheetBlock_pairs)list;
{ codeGen__TPPpcodeGen_HtmlStyleSheetBlock_pairs _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_HtmlStyleSheetBlock_pairs;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_HtmlStyleSheetBlock_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_HtmlStyleSheetBlock_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_HtmlStyleSheetBlock_name;
#else
_currn = (codeGen__TPPpcodeGen_HtmlStyleSheetBlock_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_HtmlStyleSheetBlock_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_HtmlStyleSheetBlock_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_HtmlStyleSheetBlock_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_HtmlStyleSheetBlock_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionHandler (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_ActionHandler (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_ActionHandler _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionHandler;
#else
_currn = (codeGen__TPPpcodeGen_ActionHandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionHandler));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionHandler;
_currn->_desc1 = (codeGen__TSPcodeGen_ActionHandler_targetpage) codeGen_MkcodeGen_ActionHandler_targetpage (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionHandler: root of subtree no. 1 can not be made a codeGen_ActionHandler_targetpage node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ActionHandler_params) codeGen_MkcodeGen_ActionHandler_params (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionHandler: root of subtree no. 2 can not be made a codeGen_ActionHandler_params node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionHandler;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionHandler */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionHandler_targetpage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ActionHandler_targetpage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ActionHandler_targetpage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionHandler_targetpage;
#else
_currn = (codeGen__TPPpcodeGen_ActionHandler_targetpage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionHandler_targetpage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionHandler_targetpage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionHandler_targetpage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionHandler_targetpage */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebNodeActualParampcodeGen_ActionHandler_params (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebNodeActualParampcodeGen_ActionHandler_params (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params;
#else
	_new = (codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params;
	_new->_desc1 = (codeGen__TSPcodeGen_WebNodeActualParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ActionHandler_params)_new;
	{codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ActionHandler_params (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ActionHandler_params (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebNodeActualParam)) 
		return codeGen__MkcodeGen_WebNodeActualParampcodeGen_ActionHandler_params (_coordref, codeGen_MkcodeGen_WebNodeActualParam (_coordref, t));
	message (DEADLY, "a pcodeGen_ActionHandler_params list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ActionHandler_params (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ActionHandler_params (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ActionHandler_params)
	l =_ElempcodeGen_ActionHandler_params (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ActionHandler_params)
	r = _ElempcodeGen_ActionHandler_params (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ActionHandler_params)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ActionHandler_params)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ActionHandler_params)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ActionHandler_params)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionHandler_params (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ActionHandler_params (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ActionHandler_params root;
	codeGen__TPPLST_0pcodeGen_ActionHandler_params list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ActionHandler_params)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ActionHandler_params)
		t = _ElempcodeGen_ActionHandler_params (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ActionHandler_params)(((codeGen__TPPLST_0pcodeGen_ActionHandler_params) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ActionHandler_params) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ActionHandler_params;
#else
list = (codeGen__TPPLST_0pcodeGen_ActionHandler_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ActionHandler_params));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ActionHandler_params;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ActionHandler_params)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params)
	{codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebNodeActualParampcodeGen_ActionHandler_params;
	} this code has been moved to _ElempcodeGen_ActionHandler_params*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ActionHandler_params;
#else
root = (codeGen__TPPpcodeGen_ActionHandler_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionHandler_params));
#endif
root->_prod = codeGen_RULEpcodeGen_ActionHandler_params;
root->_desc1 = (codeGen__TSPLST_codeGen_ActionHandler_params)list;
{ codeGen__TPPpcodeGen_ActionHandler_params _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ActionHandler_params;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebAction (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR codeGen_MkpcodeGen_WebAction (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	codeGen__TPPpcodeGen_WebAction _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebAction;
#else
_currn = (codeGen__TPPpcodeGen_WebAction) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebAction));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebAction;
_currn->_desc1 = (codeGen__TSPcodeGen_WebAction_name) codeGen_MkcodeGen_WebAction_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebAction: root of subtree no. 1 can not be made a codeGen_WebAction_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebAction_formalParameter) codeGen_MkcodeGen_WebAction_formalParameter (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebAction: root of subtree no. 2 can not be made a codeGen_WebAction_formalParameter node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_WebAction_successpage) codeGen_MkcodeGen_WebAction_successpage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebAction: root of subtree no. 3 can not be made a codeGen_WebAction_successpage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_WebAction_operations) codeGen_MkcodeGen_WebAction_operations (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebAction: root of subtree no. 4 can not be made a codeGen_WebAction_operations node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_WebAction_position) codeGen_MkcodeGen_WebAction_position (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebAction: root of subtree no. 5 can not be made a codeGen_WebAction_position node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebAction;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebAction */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebAction_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebAction_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebAction_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebAction_name;
#else
_currn = (codeGen__TPPpcodeGen_WebAction_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebAction_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebAction_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebAction_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebAction_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter;
#else
	_new = (codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter;
	_new->_desc1 = (codeGen__TSPcodeGen_WebNodeFormalParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_formalParameter)_new;
	{codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebAction_formalParameter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebAction_formalParameter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebNodeFormalParam)) 
		return codeGen__MkcodeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter (_coordref, codeGen_MkcodeGen_WebNodeFormalParam (_coordref, t));
	message (DEADLY, "a pcodeGen_WebAction_formalParameter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebAction_formalParameter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebAction_formalParameter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebAction_formalParameter)
	l =_ElempcodeGen_WebAction_formalParameter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebAction_formalParameter)
	r = _ElempcodeGen_WebAction_formalParameter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebAction_formalParameter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebAction_formalParameter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebAction_formalParameter root;
	codeGen__TPPLST_0pcodeGen_WebAction_formalParameter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebAction_formalParameter)
		t = _ElempcodeGen_WebAction_formalParameter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)(((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebAction_formalParameter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebAction_formalParameter;
#else
list = (codeGen__TPPLST_0pcodeGen_WebAction_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebAction_formalParameter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebAction_formalParameter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebAction_formalParameter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter)
	{codeGen__TPPLST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebNodeFormalParampcodeGen_WebAction_formalParameter;
	} this code has been moved to _ElempcodeGen_WebAction_formalParameter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebAction_formalParameter;
#else
root = (codeGen__TPPpcodeGen_WebAction_formalParameter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebAction_formalParameter));
#endif
root->_prod = codeGen_RULEpcodeGen_WebAction_formalParameter;
root->_desc1 = (codeGen__TSPLST_codeGen_WebAction_formalParameter)list;
{ codeGen__TPPpcodeGen_WebAction_formalParameter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebAction_formalParameter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataBaseOperationpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataBaseOperationpcodeGen_WebAction_operations (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations;
#else
	_new = (codeGen__TPPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataBaseOperationpcodeGen_WebAction_operations;
	_new->_desc1 = (codeGen__TSPcodeGen_DataBaseOperation)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_operations)_new;
	{codeGen__TPPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataBaseOperationpcodeGen_WebAction_operations;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_JavaOperationpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_JavaOperationpcodeGen_WebAction_operations (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_JavaOperationpcodeGen_WebAction_operations _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_JavaOperationpcodeGen_WebAction_operations;
#else
	_new = (codeGen__TPPLST_codeGen_JavaOperationpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_JavaOperationpcodeGen_WebAction_operations));
#endif
	_new->_prod = codeGen_RULELST_codeGen_JavaOperationpcodeGen_WebAction_operations;
	_new->_desc1 = (codeGen__TSPcodeGen_JavaOperation)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_operations)_new;
	{codeGen__TPPLST_codeGen_JavaOperationpcodeGen_WebAction_operations _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_JavaOperationpcodeGen_WebAction_operations;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MailingOperationpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MailingOperationpcodeGen_WebAction_operations (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MailingOperationpcodeGen_WebAction_operations _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MailingOperationpcodeGen_WebAction_operations;
#else
	_new = (codeGen__TPPLST_codeGen_MailingOperationpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MailingOperationpcodeGen_WebAction_operations));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MailingOperationpcodeGen_WebAction_operations;
	_new->_desc1 = (codeGen__TSPcodeGen_MailingOperation)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_operations)_new;
	{codeGen__TPPLST_codeGen_MailingOperationpcodeGen_WebAction_operations _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MailingOperationpcodeGen_WebAction_operations;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LogoutOperationpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LogoutOperationpcodeGen_WebAction_operations (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations;
#else
	_new = (codeGen__TPPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LogoutOperationpcodeGen_WebAction_operations;
	_new->_desc1 = (codeGen__TSPcodeGen_LogoutOperation)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_operations)_new;
	{codeGen__TPPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LogoutOperationpcodeGen_WebAction_operations;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LoginOperationpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LoginOperationpcodeGen_WebAction_operations (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LoginOperationpcodeGen_WebAction_operations _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LoginOperationpcodeGen_WebAction_operations;
#else
	_new = (codeGen__TPPLST_codeGen_LoginOperationpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LoginOperationpcodeGen_WebAction_operations));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LoginOperationpcodeGen_WebAction_operations;
	_new->_desc1 = (codeGen__TSPcodeGen_LoginOperation)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebAction_operations)_new;
	{codeGen__TPPLST_codeGen_LoginOperationpcodeGen_WebAction_operations _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LoginOperationpcodeGen_WebAction_operations;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebAction_operations (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataBaseOperation)) 
		return codeGen__MkcodeGen_DataBaseOperationpcodeGen_WebAction_operations (_coordref, codeGen_MkcodeGen_DataBaseOperation (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_JavaOperation)) 
		return codeGen__MkcodeGen_JavaOperationpcodeGen_WebAction_operations (_coordref, codeGen_MkcodeGen_JavaOperation (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MailingOperation)) 
		return codeGen__MkcodeGen_MailingOperationpcodeGen_WebAction_operations (_coordref, codeGen_MkcodeGen_MailingOperation (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LogoutOperation)) 
		return codeGen__MkcodeGen_LogoutOperationpcodeGen_WebAction_operations (_coordref, codeGen_MkcodeGen_LogoutOperation (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LoginOperation)) 
		return codeGen__MkcodeGen_LoginOperationpcodeGen_WebAction_operations (_coordref, codeGen_MkcodeGen_LoginOperation (_coordref, t));
	message (DEADLY, "a pcodeGen_WebAction_operations list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebAction_operations (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebAction_operations)
	l =_ElempcodeGen_WebAction_operations (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebAction_operations)
	r = _ElempcodeGen_WebAction_operations (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebAction_operations)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebAction_operations)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebAction_operations)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebAction_operations)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebAction_operations (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebAction_operations (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebAction_operations root;
	codeGen__TPPLST_0pcodeGen_WebAction_operations list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebAction_operations)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebAction_operations)
		t = _ElempcodeGen_WebAction_operations (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebAction_operations)(((codeGen__TPPLST_0pcodeGen_WebAction_operations) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebAction_operations) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebAction_operations;
#else
list = (codeGen__TPPLST_0pcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebAction_operations));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebAction_operations;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebAction_operations)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataBaseOperationpcodeGen_WebAction_operations)
	{codeGen__TPPLST_codeGen_DataBaseOperationpcodeGen_WebAction_operations _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataBaseOperationpcodeGen_WebAction_operations;
	} this code has been moved to _ElempcodeGen_WebAction_operations*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_JavaOperationpcodeGen_WebAction_operations)
	{codeGen__TPPLST_codeGen_JavaOperationpcodeGen_WebAction_operations _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_JavaOperationpcodeGen_WebAction_operations;
	} this code has been moved to _ElempcodeGen_WebAction_operations*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MailingOperationpcodeGen_WebAction_operations)
	{codeGen__TPPLST_codeGen_MailingOperationpcodeGen_WebAction_operations _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MailingOperationpcodeGen_WebAction_operations;
	} this code has been moved to _ElempcodeGen_WebAction_operations*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LogoutOperationpcodeGen_WebAction_operations)
	{codeGen__TPPLST_codeGen_LogoutOperationpcodeGen_WebAction_operations _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LogoutOperationpcodeGen_WebAction_operations;
	} this code has been moved to _ElempcodeGen_WebAction_operations*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LoginOperationpcodeGen_WebAction_operations)
	{codeGen__TPPLST_codeGen_LoginOperationpcodeGen_WebAction_operations _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LoginOperationpcodeGen_WebAction_operations;
	} this code has been moved to _ElempcodeGen_WebAction_operations*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebAction_operations;
#else
root = (codeGen__TPPpcodeGen_WebAction_operations) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebAction_operations));
#endif
root->_prod = codeGen_RULEpcodeGen_WebAction_operations;
root->_desc1 = (codeGen__TSPLST_codeGen_WebAction_operations)list;
{ codeGen__TPPpcodeGen_WebAction_operations _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebAction_operations;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebAction_position (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebAction_position (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebAction_position _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebAction_position;
#else
_currn = (codeGen__TPPpcodeGen_WebAction_position) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebAction_position));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebAction_position;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebAction_position;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebAction_position */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	codeGen__TPPpcodeGen_DataBaseOperation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBaseOperation;
#else
_currn = (codeGen__TPPpcodeGen_DataBaseOperation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBaseOperation;
_currn->_desc1 = (codeGen__TSPcodeGen_DataBaseOperation_errorhandler) codeGen_MkcodeGen_DataBaseOperation_errorhandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBaseOperation: root of subtree no. 1 can not be made a codeGen_DataBaseOperation_errorhandler node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DataBaseOperation_successMessage) codeGen_MkcodeGen_DataBaseOperation_successMessage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBaseOperation: root of subtree no. 2 can not be made a codeGen_DataBaseOperation_successMessage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DataBaseOperation_errorMessage) codeGen_MkcodeGen_DataBaseOperation_errorMessage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBaseOperation: root of subtree no. 3 can not be made a codeGen_DataBaseOperation_errorMessage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DataBaseOperation_filter) codeGen_MkcodeGen_DataBaseOperation_filter (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBaseOperation: root of subtree no. 4 can not be made a codeGen_DataBaseOperation_filter node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_DataBaseOperation_params) codeGen_MkcodeGen_DataBaseOperation_params (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DataBaseOperation: root of subtree no. 5 can not be made a codeGen_DataBaseOperation_params node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBaseOperation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBaseOperation */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler;
#else
	_new = (codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionHandler)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBaseOperation_errorhandler)_new;
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataBaseOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataBaseOperation_errorhandler (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionHandler)) 
		return codeGen__MkcodeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler (_coordref, codeGen_MkcodeGen_ActionHandler (_coordref, t));
	message (DEADLY, "a pcodeGen_DataBaseOperation_errorhandler list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataBaseOperation_errorhandler (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataBaseOperation_errorhandler (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_errorhandler)
	l =_ElempcodeGen_DataBaseOperation_errorhandler (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_errorhandler)
	r = _ElempcodeGen_DataBaseOperation_errorhandler (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_errorhandler (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataBaseOperation_errorhandler root;
	codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_errorhandler)
		t = _ElempcodeGen_DataBaseOperation_errorhandler (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)(((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataBaseOperation_errorhandler;
#else
list = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataBaseOperation_errorhandler));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataBaseOperation_errorhandler;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_errorhandler)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler)
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_DataBaseOperation_errorhandler;
	} this code has been moved to _ElempcodeGen_DataBaseOperation_errorhandler*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataBaseOperation_errorhandler;
#else
root = (codeGen__TPPpcodeGen_DataBaseOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation_errorhandler));
#endif
root->_prod = codeGen_RULEpcodeGen_DataBaseOperation_errorhandler;
root->_desc1 = (codeGen__TSPLST_codeGen_DataBaseOperation_errorhandler)list;
{ codeGen__TPPpcodeGen_DataBaseOperation_errorhandler _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataBaseOperation_errorhandler;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_successMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_successMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBaseOperation_successMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBaseOperation_successMessage;
#else
_currn = (codeGen__TPPpcodeGen_DataBaseOperation_successMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation_successMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBaseOperation_successMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBaseOperation_successMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBaseOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_errorMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_errorMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBaseOperation_errorMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBaseOperation_errorMessage;
#else
_currn = (codeGen__TPPpcodeGen_DataBaseOperation_errorMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation_errorMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBaseOperation_errorMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBaseOperation_errorMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBaseOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_filter (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_filter (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DataBaseOperation_filter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DataBaseOperation_filter;
#else
_currn = (codeGen__TPPpcodeGen_DataBaseOperation_filter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation_filter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DataBaseOperation_filter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DataBaseOperation_filter;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DataBaseOperation_filter */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceActualParampcodeGen_DataBaseOperation_params (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceActualParampcodeGen_DataBaseOperation_params (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceActualParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_DataBaseOperation_params)_new;
	{codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_DataBaseOperation_params (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_DataBaseOperation_params (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceActualParam)) 
		return codeGen__MkcodeGen_DataSourceActualParampcodeGen_DataBaseOperation_params (_coordref, codeGen_MkcodeGen_DataSourceActualParam (_coordref, t));
	message (DEADLY, "a pcodeGen_DataBaseOperation_params list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_DataBaseOperation_params (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_DataBaseOperation_params (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_params)
	l =_ElempcodeGen_DataBaseOperation_params (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_params)
	r = _ElempcodeGen_DataBaseOperation_params (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_params (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_DataBaseOperation_params (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_DataBaseOperation_params root;
	codeGen__TPPLST_0pcodeGen_DataBaseOperation_params list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_DataBaseOperation_params)
		t = _ElempcodeGen_DataBaseOperation_params (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)(((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_DataBaseOperation_params) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_DataBaseOperation_params;
#else
list = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_DataBaseOperation_params));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_DataBaseOperation_params;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_DataBaseOperation_params)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params)
	{codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceActualParampcodeGen_DataBaseOperation_params;
	} this code has been moved to _ElempcodeGen_DataBaseOperation_params*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_DataBaseOperation_params;
#else
root = (codeGen__TPPpcodeGen_DataBaseOperation_params) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DataBaseOperation_params));
#endif
root->_prod = codeGen_RULEpcodeGen_DataBaseOperation_params;
root->_desc1 = (codeGen__TSPLST_codeGen_DataBaseOperation_params)list;
{ codeGen__TPPpcodeGen_DataBaseOperation_params _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_DataBaseOperation_params;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JavaOperation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_JavaOperation (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_JavaOperation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JavaOperation;
#else
_currn = (codeGen__TPPpcodeGen_JavaOperation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JavaOperation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JavaOperation;
_currn->_desc1 = (codeGen__TSPcodeGen_JavaOperation_errorhandler) codeGen_MkcodeGen_JavaOperation_errorhandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_JavaOperation: root of subtree no. 1 can not be made a codeGen_JavaOperation_errorhandler node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_JavaOperation_successMessage) codeGen_MkcodeGen_JavaOperation_successMessage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_JavaOperation: root of subtree no. 2 can not be made a codeGen_JavaOperation_successMessage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_JavaOperation_errorMessage) codeGen_MkcodeGen_JavaOperation_errorMessage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_JavaOperation: root of subtree no. 3 can not be made a codeGen_JavaOperation_errorMessage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_JavaOperation_content) codeGen_MkcodeGen_JavaOperation_content (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_JavaOperation: root of subtree no. 4 can not be made a codeGen_JavaOperation_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JavaOperation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JavaOperation */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler;
#else
	_new = (codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionHandler)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_JavaOperation_errorhandler)_new;
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_JavaOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_JavaOperation_errorhandler (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionHandler)) 
		return codeGen__MkcodeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler (_coordref, codeGen_MkcodeGen_ActionHandler (_coordref, t));
	message (DEADLY, "a pcodeGen_JavaOperation_errorhandler list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_JavaOperation_errorhandler (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_JavaOperation_errorhandler (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_JavaOperation_errorhandler)
	l =_ElempcodeGen_JavaOperation_errorhandler (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_JavaOperation_errorhandler)
	r = _ElempcodeGen_JavaOperation_errorhandler (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JavaOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_JavaOperation_errorhandler (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_JavaOperation_errorhandler root;
	codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_JavaOperation_errorhandler)
		t = _ElempcodeGen_JavaOperation_errorhandler (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)(((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_JavaOperation_errorhandler;
#else
list = (codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_JavaOperation_errorhandler));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_JavaOperation_errorhandler;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_JavaOperation_errorhandler)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler)
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_JavaOperation_errorhandler;
	} this code has been moved to _ElempcodeGen_JavaOperation_errorhandler*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_JavaOperation_errorhandler;
#else
root = (codeGen__TPPpcodeGen_JavaOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JavaOperation_errorhandler));
#endif
root->_prod = codeGen_RULEpcodeGen_JavaOperation_errorhandler;
root->_desc1 = (codeGen__TSPLST_codeGen_JavaOperation_errorhandler)list;
{ codeGen__TPPpcodeGen_JavaOperation_errorhandler _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_JavaOperation_errorhandler;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JavaOperation_successMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_JavaOperation_successMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_JavaOperation_successMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JavaOperation_successMessage;
#else
_currn = (codeGen__TPPpcodeGen_JavaOperation_successMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JavaOperation_successMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JavaOperation_successMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JavaOperation_successMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JavaOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JavaOperation_errorMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_JavaOperation_errorMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_JavaOperation_errorMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JavaOperation_errorMessage;
#else
_currn = (codeGen__TPPpcodeGen_JavaOperation_errorMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JavaOperation_errorMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JavaOperation_errorMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JavaOperation_errorMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JavaOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JavaOperation_content (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_JavaOperation_content (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_JavaOperation_content _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JavaOperation_content;
#else
_currn = (codeGen__TPPpcodeGen_JavaOperation_content) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JavaOperation_content));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JavaOperation_content;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JavaOperation_content;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JavaOperation_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	codeGen__TPPpcodeGen_MailingOperation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation;
_currn->_desc1 = (codeGen__TSPcodeGen_MailingOperation_errorhandler) codeGen_MkcodeGen_MailingOperation_errorhandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 1 can not be made a codeGen_MailingOperation_errorhandler node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_MailingOperation_successMessage) codeGen_MkcodeGen_MailingOperation_successMessage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 2 can not be made a codeGen_MailingOperation_successMessage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_MailingOperation_errorMessage) codeGen_MkcodeGen_MailingOperation_errorMessage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 3 can not be made a codeGen_MailingOperation_errorMessage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_MailingOperation_name) codeGen_MkcodeGen_MailingOperation_name (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 4 can not be made a codeGen_MailingOperation_name node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_MailingOperation_mailTo) codeGen_MkcodeGen_MailingOperation_mailTo (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 5 can not be made a codeGen_MailingOperation_mailTo node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_MailingOperation_subject) codeGen_MkcodeGen_MailingOperation_subject (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 6 can not be made a codeGen_MailingOperation_subject node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_MailingOperation_content) codeGen_MkcodeGen_MailingOperation_content (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MailingOperation: root of subtree no. 7 can not be made a codeGen_MailingOperation_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler;
#else
	_new = (codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionHandler)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_MailingOperation_errorhandler)_new;
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_MailingOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_MailingOperation_errorhandler (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionHandler)) 
		return codeGen__MkcodeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler (_coordref, codeGen_MkcodeGen_ActionHandler (_coordref, t));
	message (DEADLY, "a pcodeGen_MailingOperation_errorhandler list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_MailingOperation_errorhandler (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_MailingOperation_errorhandler (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_MailingOperation_errorhandler)
	l =_ElempcodeGen_MailingOperation_errorhandler (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_MailingOperation_errorhandler)
	r = _ElempcodeGen_MailingOperation_errorhandler (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_errorhandler (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_MailingOperation_errorhandler root;
	codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_MailingOperation_errorhandler)
		t = _ElempcodeGen_MailingOperation_errorhandler (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)(((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_MailingOperation_errorhandler;
#else
list = (codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_MailingOperation_errorhandler));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_MailingOperation_errorhandler;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_MailingOperation_errorhandler)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler)
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_MailingOperation_errorhandler;
	} this code has been moved to _ElempcodeGen_MailingOperation_errorhandler*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_MailingOperation_errorhandler;
#else
root = (codeGen__TPPpcodeGen_MailingOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_errorhandler));
#endif
root->_prod = codeGen_RULEpcodeGen_MailingOperation_errorhandler;
root->_desc1 = (codeGen__TSPLST_codeGen_MailingOperation_errorhandler)list;
{ codeGen__TPPpcodeGen_MailingOperation_errorhandler _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_MailingOperation_errorhandler;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_successMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_successMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_successMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_successMessage;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_successMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_successMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_successMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_successMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_errorMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_errorMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_errorMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_errorMessage;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_errorMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_errorMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_errorMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_errorMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_name;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_mailTo (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_mailTo (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_mailTo _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_mailTo;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_mailTo) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_mailTo));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_mailTo;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_mailTo;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_mailTo */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_subject (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_subject (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_subject _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_subject;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_subject) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_subject));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_subject;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_subject;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_subject */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MailingOperation_content (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MailingOperation_content (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MailingOperation_content _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MailingOperation_content;
#else
_currn = (codeGen__TPPpcodeGen_MailingOperation_content) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MailingOperation_content));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MailingOperation_content;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MailingOperation_content;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MailingOperation_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LogoutOperation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_LogoutOperation (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_LogoutOperation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LogoutOperation;
#else
_currn = (codeGen__TPPpcodeGen_LogoutOperation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LogoutOperation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LogoutOperation;
_currn->_desc1 = (codeGen__TSPcodeGen_LogoutOperation_errorhandler) codeGen_MkcodeGen_LogoutOperation_errorhandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LogoutOperation: root of subtree no. 1 can not be made a codeGen_LogoutOperation_errorhandler node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_LogoutOperation_successMessage) codeGen_MkcodeGen_LogoutOperation_successMessage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LogoutOperation: root of subtree no. 2 can not be made a codeGen_LogoutOperation_successMessage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_LogoutOperation_errorMessage) codeGen_MkcodeGen_LogoutOperation_errorMessage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LogoutOperation: root of subtree no. 3 can not be made a codeGen_LogoutOperation_errorMessage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_LogoutOperation_name) codeGen_MkcodeGen_LogoutOperation_name (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LogoutOperation: root of subtree no. 4 can not be made a codeGen_LogoutOperation_name node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LogoutOperation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LogoutOperation */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler;
#else
	_new = (codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionHandler)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_LogoutOperation_errorhandler)_new;
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_LogoutOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_LogoutOperation_errorhandler (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionHandler)) 
		return codeGen__MkcodeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler (_coordref, codeGen_MkcodeGen_ActionHandler (_coordref, t));
	message (DEADLY, "a pcodeGen_LogoutOperation_errorhandler list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_LogoutOperation_errorhandler (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_LogoutOperation_errorhandler (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_LogoutOperation_errorhandler)
	l =_ElempcodeGen_LogoutOperation_errorhandler (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_LogoutOperation_errorhandler)
	r = _ElempcodeGen_LogoutOperation_errorhandler (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LogoutOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_LogoutOperation_errorhandler (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_LogoutOperation_errorhandler root;
	codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_LogoutOperation_errorhandler)
		t = _ElempcodeGen_LogoutOperation_errorhandler (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)(((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_LogoutOperation_errorhandler;
#else
list = (codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_LogoutOperation_errorhandler));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_LogoutOperation_errorhandler;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_LogoutOperation_errorhandler)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler)
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_LogoutOperation_errorhandler;
	} this code has been moved to _ElempcodeGen_LogoutOperation_errorhandler*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_LogoutOperation_errorhandler;
#else
root = (codeGen__TPPpcodeGen_LogoutOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LogoutOperation_errorhandler));
#endif
root->_prod = codeGen_RULEpcodeGen_LogoutOperation_errorhandler;
root->_desc1 = (codeGen__TSPLST_codeGen_LogoutOperation_errorhandler)list;
{ codeGen__TPPpcodeGen_LogoutOperation_errorhandler _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_LogoutOperation_errorhandler;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LogoutOperation_successMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LogoutOperation_successMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LogoutOperation_successMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LogoutOperation_successMessage;
#else
_currn = (codeGen__TPPpcodeGen_LogoutOperation_successMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LogoutOperation_successMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LogoutOperation_successMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LogoutOperation_successMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LogoutOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LogoutOperation_errorMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LogoutOperation_errorMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LogoutOperation_errorMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LogoutOperation_errorMessage;
#else
_currn = (codeGen__TPPpcodeGen_LogoutOperation_errorMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LogoutOperation_errorMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LogoutOperation_errorMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LogoutOperation_errorMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LogoutOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LogoutOperation_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LogoutOperation_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LogoutOperation_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LogoutOperation_name;
#else
_currn = (codeGen__TPPpcodeGen_LogoutOperation_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LogoutOperation_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LogoutOperation_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LogoutOperation_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LogoutOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
#endif
{	codeGen__TPPpcodeGen_LoginOperation _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation;
_currn->_desc1 = (codeGen__TSPcodeGen_LoginOperation_errorhandler) codeGen_MkcodeGen_LoginOperation_errorhandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 1 can not be made a codeGen_LoginOperation_errorhandler node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_LoginOperation_successMessage) codeGen_MkcodeGen_LoginOperation_successMessage (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 2 can not be made a codeGen_LoginOperation_successMessage node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_LoginOperation_errorMessage) codeGen_MkcodeGen_LoginOperation_errorMessage (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 3 can not be made a codeGen_LoginOperation_errorMessage node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_LoginOperation_name) codeGen_MkcodeGen_LoginOperation_name (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 4 can not be made a codeGen_LoginOperation_name node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_LoginOperation_userDBColumn) codeGen_MkcodeGen_LoginOperation_userDBColumn (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 5 can not be made a codeGen_LoginOperation_userDBColumn node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_LoginOperation_passwdDBColumn) codeGen_MkcodeGen_LoginOperation_passwdDBColumn (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 6 can not be made a codeGen_LoginOperation_passwdDBColumn node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_LoginOperation_rolesDBColumn) codeGen_MkcodeGen_LoginOperation_rolesDBColumn (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 7 can not be made a codeGen_LoginOperation_rolesDBColumn node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_LoginOperation_userFormField) codeGen_MkcodeGen_LoginOperation_userFormField (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 8 can not be made a codeGen_LoginOperation_userFormField node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_LoginOperation_passwdFormField) codeGen_MkcodeGen_LoginOperation_passwdFormField (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LoginOperation: root of subtree no. 9 can not be made a codeGen_LoginOperation_passwdFormField node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler;
#else
	_new = (codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionHandler)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_LoginOperation_errorhandler)_new;
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_LoginOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_LoginOperation_errorhandler (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionHandler)) 
		return codeGen__MkcodeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler (_coordref, codeGen_MkcodeGen_ActionHandler (_coordref, t));
	message (DEADLY, "a pcodeGen_LoginOperation_errorhandler list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_LoginOperation_errorhandler (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_LoginOperation_errorhandler (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_LoginOperation_errorhandler)
	l =_ElempcodeGen_LoginOperation_errorhandler (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_LoginOperation_errorhandler)
	r = _ElempcodeGen_LoginOperation_errorhandler (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_errorhandler (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_errorhandler (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_LoginOperation_errorhandler root;
	codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_LoginOperation_errorhandler)
		t = _ElempcodeGen_LoginOperation_errorhandler (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)(((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_LoginOperation_errorhandler;
#else
list = (codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_LoginOperation_errorhandler));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_LoginOperation_errorhandler;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_LoginOperation_errorhandler)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler)
	{codeGen__TPPLST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionHandlerpcodeGen_LoginOperation_errorhandler;
	} this code has been moved to _ElempcodeGen_LoginOperation_errorhandler*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_LoginOperation_errorhandler;
#else
root = (codeGen__TPPpcodeGen_LoginOperation_errorhandler) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_errorhandler));
#endif
root->_prod = codeGen_RULEpcodeGen_LoginOperation_errorhandler;
root->_desc1 = (codeGen__TSPLST_codeGen_LoginOperation_errorhandler)list;
{ codeGen__TPPpcodeGen_LoginOperation_errorhandler _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_LoginOperation_errorhandler;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_successMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_successMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_successMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_successMessage;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_successMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_successMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_successMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_successMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_successMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_errorMessage (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_errorMessage (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_errorMessage _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_errorMessage;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_errorMessage) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_errorMessage));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_errorMessage;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_errorMessage;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_errorMessage */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_name;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_userDBColumn (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_userDBColumn (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_userDBColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_userDBColumn;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_userDBColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_userDBColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_userDBColumn;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_userDBColumn;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_userDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_passwdDBColumn (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_passwdDBColumn (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_passwdDBColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_passwdDBColumn;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_passwdDBColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_passwdDBColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_passwdDBColumn;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_passwdDBColumn;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_passwdDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_rolesDBColumn (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_rolesDBColumn (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_rolesDBColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_rolesDBColumn;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_rolesDBColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_rolesDBColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_rolesDBColumn;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_rolesDBColumn;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_rolesDBColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_userFormField (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_userFormField (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_userFormField _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_userFormField;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_userFormField) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_userFormField));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_userFormField;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_userFormField;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_userFormField */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LoginOperation_passwdFormField (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LoginOperation_passwdFormField (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LoginOperation_passwdFormField _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LoginOperation_passwdFormField;
#else
_currn = (codeGen__TPPpcodeGen_LoginOperation_passwdFormField) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LoginOperation_passwdFormField));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LoginOperation_passwdFormField;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LoginOperation_passwdFormField;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LoginOperation_passwdFormField */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR codeGen_MkpcodeGen_Formular (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	codeGen__TPPpcodeGen_Formular _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular;
#else
_currn = (codeGen__TPPpcodeGen_Formular) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular;
_currn->_desc1 = (codeGen__TSPcodeGen_Formular_styleSheetBlock) codeGen_MkcodeGen_Formular_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 1 can not be made a codeGen_Formular_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_Formular_name) codeGen_MkcodeGen_Formular_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 2 can not be made a codeGen_Formular_name node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_Formular_action) codeGen_MkcodeGen_Formular_action (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 3 can not be made a codeGen_Formular_action node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_Formular_dataSource) codeGen_MkcodeGen_Formular_dataSource (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 4 can not be made a codeGen_Formular_dataSource node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_Formular_dataSourceId) codeGen_MkcodeGen_Formular_dataSourceId (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 5 can not be made a codeGen_Formular_dataSourceId node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_Formular_elements) codeGen_MkcodeGen_Formular_elements (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 6 can not be made a codeGen_Formular_elements node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_Formular_actualParams) codeGen_MkcodeGen_Formular_actualParams (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_Formular: root of subtree no. 7 can not be made a codeGen_Formular_actualParams node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Formular_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Formular_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_Formular_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Formular_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Formular_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular_name;
#else
_currn = (codeGen__TPPpcodeGen_Formular_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_action (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Formular_action (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Formular_action _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular_action;
#else
_currn = (codeGen__TPPpcodeGen_Formular_action) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_action));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular_action;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular_action;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular_action */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_dataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Formular_dataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Formular_dataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular_dataSource;
#else
_currn = (codeGen__TPPpcodeGen_Formular_dataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_dataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular_dataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular_dataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_dataSourceId (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_Formular_dataSourceId (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_Formular_dataSourceId _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_Formular_dataSourceId;
#else
_currn = (codeGen__TPPpcodeGen_Formular_dataSourceId) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_dataSourceId));
#endif
_currn->_prod = codeGen_RULEpcodeGen_Formular_dataSourceId;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_Formular_dataSourceId;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_Formular_dataSourceId */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularRadioGroupElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularRadioGroupElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularRadioGroupElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularTextElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularTextElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularTextElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularTextElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularTextElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularTextElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularTextElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularTextElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularTextElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularTextElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularTextAreaElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularTextAreaElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularTextAreaElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularPasswordElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularPasswordElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularPasswordElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularPasswordElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularPasswordElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularDateElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularDateElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularDateElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularDateElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularDateElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularDateElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularDateElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularDateElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularDateElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularDateElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularFileUploadElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularFileUploadElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularFileUploadElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularCheckboxElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularCheckboxElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularCheckboxElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularSelectElementpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularSelectElementpcodeGen_Formular_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularSelectElementpcodeGen_Formular_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularSelectElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_elements)_new;
	{codeGen__TPPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularSelectElementpcodeGen_Formular_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Formular_elements (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Formular_elements (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularRadioGroupElement)) 
		return codeGen__MkcodeGen_FormularRadioGroupElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularRadioGroupElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularTextElement)) 
		return codeGen__MkcodeGen_FormularTextElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularTextElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularTextAreaElement)) 
		return codeGen__MkcodeGen_FormularTextAreaElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularTextAreaElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularPasswordElement)) 
		return codeGen__MkcodeGen_FormularPasswordElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularPasswordElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularDateElement)) 
		return codeGen__MkcodeGen_FormularDateElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularDateElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularFileUploadElement)) 
		return codeGen__MkcodeGen_FormularFileUploadElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularFileUploadElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularCheckboxElement)) 
		return codeGen__MkcodeGen_FormularCheckboxElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularCheckboxElement (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularSelectElement)) 
		return codeGen__MkcodeGen_FormularSelectElementpcodeGen_Formular_elements (_coordref, codeGen_MkcodeGen_FormularSelectElement (_coordref, t));
	message (DEADLY, "a pcodeGen_Formular_elements list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Formular_elements (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Formular_elements (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Formular_elements)
	l =_ElempcodeGen_Formular_elements (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Formular_elements)
	r = _ElempcodeGen_Formular_elements (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Formular_elements)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Formular_elements)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Formular_elements)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Formular_elements)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_elements (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Formular_elements (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Formular_elements root;
	codeGen__TPPLST_0pcodeGen_Formular_elements list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Formular_elements)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Formular_elements)
		t = _ElempcodeGen_Formular_elements (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Formular_elements)(((codeGen__TPPLST_0pcodeGen_Formular_elements) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Formular_elements) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Formular_elements;
#else
list = (codeGen__TPPLST_0pcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Formular_elements));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Formular_elements;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Formular_elements)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularRadioGroupElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularTextElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularTextElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularTextElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularTextAreaElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularPasswordElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularPasswordElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularPasswordElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularDateElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularDateElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularDateElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularFileUploadElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularCheckboxElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularSelectElementpcodeGen_Formular_elements)
	{codeGen__TPPLST_codeGen_FormularSelectElementpcodeGen_Formular_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularSelectElementpcodeGen_Formular_elements;
	} this code has been moved to _ElempcodeGen_Formular_elements*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Formular_elements;
#else
root = (codeGen__TPPpcodeGen_Formular_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_elements));
#endif
root->_prod = codeGen_RULEpcodeGen_Formular_elements;
root->_desc1 = (codeGen__TSPLST_codeGen_Formular_elements)list;
{ codeGen__TPPpcodeGen_Formular_elements _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Formular_elements;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_WebNodeActualParampcodeGen_Formular_actualParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_WebNodeActualParampcodeGen_Formular_actualParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams;
#else
	_new = (codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams;
	_new->_desc1 = (codeGen__TSPcodeGen_WebNodeActualParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_Formular_actualParams)_new;
	{codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_Formular_actualParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_Formular_actualParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_WebNodeActualParam)) 
		return codeGen__MkcodeGen_WebNodeActualParampcodeGen_Formular_actualParams (_coordref, codeGen_MkcodeGen_WebNodeActualParam (_coordref, t));
	message (DEADLY, "a pcodeGen_Formular_actualParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_Formular_actualParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_Formular_actualParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_Formular_actualParams)
	l =_ElempcodeGen_Formular_actualParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_Formular_actualParams)
	r = _ElempcodeGen_Formular_actualParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_Formular_actualParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_Formular_actualParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_Formular_actualParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_Formular_actualParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_Formular_actualParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_Formular_actualParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_Formular_actualParams root;
	codeGen__TPPLST_0pcodeGen_Formular_actualParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_Formular_actualParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_Formular_actualParams)
		t = _ElempcodeGen_Formular_actualParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_Formular_actualParams)(((codeGen__TPPLST_0pcodeGen_Formular_actualParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_Formular_actualParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_Formular_actualParams;
#else
list = (codeGen__TPPLST_0pcodeGen_Formular_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_Formular_actualParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_Formular_actualParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_Formular_actualParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams)
	{codeGen__TPPLST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_WebNodeActualParampcodeGen_Formular_actualParams;
	} this code has been moved to _ElempcodeGen_Formular_actualParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_Formular_actualParams;
#else
root = (codeGen__TPPpcodeGen_Formular_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_Formular_actualParams));
#endif
root->_prod = codeGen_RULEpcodeGen_Formular_actualParams;
root->_desc1 = (codeGen__TSPLST_codeGen_Formular_actualParams)list;
{ codeGen__TPPpcodeGen_Formular_actualParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_Formular_actualParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_EmailValidator (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_EmailValidator (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_EmailValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_EmailValidator;
#else
_currn = (codeGen__TPPpcodeGen_EmailValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_EmailValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_EmailValidator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_EmailValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_EmailValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_CreditCardValidator (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_CreditCardValidator (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_CreditCardValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_CreditCardValidator;
#else
_currn = (codeGen__TPPpcodeGen_CreditCardValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_CreditCardValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_CreditCardValidator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_CreditCardValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_CreditCardValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RegExprValidator (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_RegExprValidator (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_RegExprValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_RegExprValidator;
#else
_currn = (codeGen__TPPpcodeGen_RegExprValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RegExprValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_RegExprValidator;
_currn->_desc1 = (codeGen__TSPcodeGen_RegExprValidator_pattern) codeGen_MkcodeGen_RegExprValidator_pattern (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_RegExprValidator: root of subtree no. 1 can not be made a codeGen_RegExprValidator_pattern node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_RegExprValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_RegExprValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_RegExprValidator_pattern (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_RegExprValidator_pattern (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_RegExprValidator_pattern _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_RegExprValidator_pattern;
#else
_currn = (codeGen__TPPpcodeGen_RegExprValidator_pattern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_RegExprValidator_pattern));
#endif
_currn->_prod = codeGen_RULEpcodeGen_RegExprValidator_pattern;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_RegExprValidator_pattern;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_RegExprValidator_pattern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StringLengthValidator (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_StringLengthValidator (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_StringLengthValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StringLengthValidator;
#else
_currn = (codeGen__TPPpcodeGen_StringLengthValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StringLengthValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StringLengthValidator;
_currn->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator_min) codeGen_MkcodeGen_StringLengthValidator_min (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StringLengthValidator: root of subtree no. 1 can not be made a codeGen_StringLengthValidator_min node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_StringLengthValidator_max) codeGen_MkcodeGen_StringLengthValidator_max (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_StringLengthValidator: root of subtree no. 2 can not be made a codeGen_StringLengthValidator_max node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StringLengthValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StringLengthValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StringLengthValidator_min (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StringLengthValidator_min (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StringLengthValidator_min _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StringLengthValidator_min;
#else
_currn = (codeGen__TPPpcodeGen_StringLengthValidator_min) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StringLengthValidator_min));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StringLengthValidator_min;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StringLengthValidator_min;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StringLengthValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_StringLengthValidator_max (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_StringLengthValidator_max (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_StringLengthValidator_max _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_StringLengthValidator_max;
#else
_currn = (codeGen__TPPpcodeGen_StringLengthValidator_max) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_StringLengthValidator_max));
#endif
_currn->_prod = codeGen_RULEpcodeGen_StringLengthValidator_max;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_StringLengthValidator_max;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_StringLengthValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LongRangeValidator (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_LongRangeValidator (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_LongRangeValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LongRangeValidator;
#else
_currn = (codeGen__TPPpcodeGen_LongRangeValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LongRangeValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LongRangeValidator;
_currn->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator_min) codeGen_MkcodeGen_LongRangeValidator_min (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LongRangeValidator: root of subtree no. 1 can not be made a codeGen_LongRangeValidator_min node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_LongRangeValidator_max) codeGen_MkcodeGen_LongRangeValidator_max (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LongRangeValidator: root of subtree no. 2 can not be made a codeGen_LongRangeValidator_max node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LongRangeValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LongRangeValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LongRangeValidator_min (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LongRangeValidator_min (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LongRangeValidator_min _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LongRangeValidator_min;
#else
_currn = (codeGen__TPPpcodeGen_LongRangeValidator_min) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LongRangeValidator_min));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LongRangeValidator_min;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LongRangeValidator_min;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LongRangeValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LongRangeValidator_max (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LongRangeValidator_max (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LongRangeValidator_max _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LongRangeValidator_max;
#else
_currn = (codeGen__TPPpcodeGen_LongRangeValidator_max) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LongRangeValidator_max));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LongRangeValidator_max;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LongRangeValidator_max;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LongRangeValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_DoubleRangeValidator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleRangeValidator;
#else
_currn = (codeGen__TPPpcodeGen_DoubleRangeValidator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleRangeValidator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleRangeValidator;
_currn->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator_min) codeGen_MkcodeGen_DoubleRangeValidator_min (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleRangeValidator: root of subtree no. 1 can not be made a codeGen_DoubleRangeValidator_min node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DoubleRangeValidator_max) codeGen_MkcodeGen_DoubleRangeValidator_max (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleRangeValidator: root of subtree no. 2 can not be made a codeGen_DoubleRangeValidator_max node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleRangeValidator;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleRangeValidator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator_min (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator_min (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleRangeValidator_min _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleRangeValidator_min;
#else
_currn = (codeGen__TPPpcodeGen_DoubleRangeValidator_min) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleRangeValidator_min));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleRangeValidator_min;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleRangeValidator_min;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleRangeValidator_min */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator_max (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleRangeValidator_max (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleRangeValidator_max _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleRangeValidator_max;
#else
_currn = (codeGen__TPPpcodeGen_DoubleRangeValidator_max) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleRangeValidator_max));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleRangeValidator_max;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleRangeValidator_max;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleRangeValidator_max */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_IntegerNumberConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_IntegerNumberConverter;
#else
_currn = (codeGen__TPPpcodeGen_IntegerNumberConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_IntegerNumberConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_IntegerNumberConverter;
_currn->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter_minIntegerDigits) codeGen_MkcodeGen_IntegerNumberConverter_minIntegerDigits (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_IntegerNumberConverter: root of subtree no. 1 can not be made a codeGen_IntegerNumberConverter_minIntegerDigits node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_IntegerNumberConverter_maxIntegerDigits) codeGen_MkcodeGen_IntegerNumberConverter_maxIntegerDigits (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_IntegerNumberConverter: root of subtree no. 2 can not be made a codeGen_IntegerNumberConverter_maxIntegerDigits node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_IntegerNumberConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_IntegerNumberConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter_minIntegerDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter_minIntegerDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_IntegerNumberConverter_minIntegerDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_IntegerNumberConverter_minIntegerDigits;
#else
_currn = (codeGen__TPPpcodeGen_IntegerNumberConverter_minIntegerDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_IntegerNumberConverter_minIntegerDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_IntegerNumberConverter_minIntegerDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_IntegerNumberConverter_minIntegerDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_IntegerNumberConverter_minIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter_maxIntegerDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_IntegerNumberConverter_maxIntegerDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_IntegerNumberConverter_maxIntegerDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_IntegerNumberConverter_maxIntegerDigits;
#else
_currn = (codeGen__TPPpcodeGen_IntegerNumberConverter_maxIntegerDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_IntegerNumberConverter_maxIntegerDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_IntegerNumberConverter_maxIntegerDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_IntegerNumberConverter_maxIntegerDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_IntegerNumberConverter_maxIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_DoubleNumberConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleNumberConverter;
#else
_currn = (codeGen__TPPpcodeGen_DoubleNumberConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleNumberConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleNumberConverter;
_currn->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter_minIntegerDigits) codeGen_MkcodeGen_DoubleNumberConverter_minIntegerDigits (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleNumberConverter: root of subtree no. 1 can not be made a codeGen_DoubleNumberConverter_minIntegerDigits node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_DoubleNumberConverter_maxIntegerDigits) codeGen_MkcodeGen_DoubleNumberConverter_maxIntegerDigits (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleNumberConverter: root of subtree no. 2 can not be made a codeGen_DoubleNumberConverter_maxIntegerDigits node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_DoubleNumberConverter_minFractionDigits) codeGen_MkcodeGen_DoubleNumberConverter_minFractionDigits (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleNumberConverter: root of subtree no. 3 can not be made a codeGen_DoubleNumberConverter_minFractionDigits node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_DoubleNumberConverter_maxFractionDigits) codeGen_MkcodeGen_DoubleNumberConverter_maxFractionDigits (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_DoubleNumberConverter: root of subtree no. 4 can not be made a codeGen_DoubleNumberConverter_maxFractionDigits node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleNumberConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleNumberConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_minIntegerDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_minIntegerDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleNumberConverter_minIntegerDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleNumberConverter_minIntegerDigits;
#else
_currn = (codeGen__TPPpcodeGen_DoubleNumberConverter_minIntegerDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleNumberConverter_minIntegerDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleNumberConverter_minIntegerDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleNumberConverter_minIntegerDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleNumberConverter_minIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_maxIntegerDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_maxIntegerDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleNumberConverter_maxIntegerDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleNumberConverter_maxIntegerDigits;
#else
_currn = (codeGen__TPPpcodeGen_DoubleNumberConverter_maxIntegerDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleNumberConverter_maxIntegerDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleNumberConverter_maxIntegerDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleNumberConverter_maxIntegerDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleNumberConverter_maxIntegerDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_minFractionDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_minFractionDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleNumberConverter_minFractionDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleNumberConverter_minFractionDigits;
#else
_currn = (codeGen__TPPpcodeGen_DoubleNumberConverter_minFractionDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleNumberConverter_minFractionDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleNumberConverter_minFractionDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleNumberConverter_minFractionDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleNumberConverter_minFractionDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_maxFractionDigits (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_DoubleNumberConverter_maxFractionDigits (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_DoubleNumberConverter_maxFractionDigits _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleNumberConverter_maxFractionDigits;
#else
_currn = (codeGen__TPPpcodeGen_DoubleNumberConverter_maxFractionDigits) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleNumberConverter_maxFractionDigits));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleNumberConverter_maxFractionDigits;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleNumberConverter_maxFractionDigits;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleNumberConverter_maxFractionDigits */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_IntegerConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_IntegerConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_IntegerConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_IntegerConverter;
#else
_currn = (codeGen__TPPpcodeGen_IntegerConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_IntegerConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_IntegerConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_IntegerConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_IntegerConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LongConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_LongConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_LongConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LongConverter;
#else
_currn = (codeGen__TPPpcodeGen_LongConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LongConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LongConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LongConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LongConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_BigIntegerConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_BigIntegerConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_BigIntegerConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_BigIntegerConverter;
#else
_currn = (codeGen__TPPpcodeGen_BigIntegerConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_BigIntegerConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_BigIntegerConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_BigIntegerConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_BigIntegerConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FloatConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_FloatConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_FloatConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FloatConverter;
#else
_currn = (codeGen__TPPpcodeGen_FloatConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FloatConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FloatConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FloatConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FloatConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DoubleConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DoubleConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DoubleConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DoubleConverter;
#else
_currn = (codeGen__TPPpcodeGen_DoubleConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DoubleConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DoubleConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DoubleConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DoubleConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_BooleanConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_BooleanConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_BooleanConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_BooleanConverter;
#else
_currn = (codeGen__TPPpcodeGen_BooleanConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_BooleanConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_BooleanConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_BooleanConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_BooleanConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_DateTimeConverter (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_DateTimeConverter (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_DateTimeConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_DateTimeConverter;
#else
_currn = (codeGen__TPPpcodeGen_DateTimeConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_DateTimeConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_DateTimeConverter;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_DateTimeConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_DateTimeConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_PatternConverter (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_PatternConverter (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_PatternConverter _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_PatternConverter;
#else
_currn = (codeGen__TPPpcodeGen_PatternConverter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_PatternConverter));
#endif
_currn->_prod = codeGen_RULEpcodeGen_PatternConverter;
_currn->_desc1 = (codeGen__TSPcodeGen_PatternConverter_pattern) codeGen_MkcodeGen_PatternConverter_pattern (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_PatternConverter: root of subtree no. 1 can not be made a codeGen_PatternConverter_pattern node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_PatternConverter;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_PatternConverter */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_PatternConverter_pattern (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_PatternConverter_pattern (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_PatternConverter_pattern _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_PatternConverter_pattern;
#else
_currn = (codeGen__TPPpcodeGen_PatternConverter_pattern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_PatternConverter_pattern));
#endif
_currn->_prod = codeGen_RULEpcodeGen_PatternConverter_pattern;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_PatternConverter_pattern;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_PatternConverter_pattern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9, NODEPTR _desc10)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9,_desc10)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
	NODEPTR _desc10;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularRadioGroupElement_name) codeGen_MkcodeGen_FormularRadioGroupElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 1 can not be made a codeGen_FormularRadioGroupElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularRadioGroupElement_label) codeGen_MkcodeGen_FormularRadioGroupElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 2 can not be made a codeGen_FormularRadioGroupElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularRadioGroupElement_description) codeGen_MkcodeGen_FormularRadioGroupElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 3 can not be made a codeGen_FormularRadioGroupElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularRadioGroupElement_pvalue) codeGen_MkcodeGen_FormularRadioGroupElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 4 can not be made a codeGen_FormularRadioGroupElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularRadioGroupElement_required) codeGen_MkcodeGen_FormularRadioGroupElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 5 can not be made a codeGen_FormularRadioGroupElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularRadioGroupElement_validator) codeGen_MkcodeGen_FormularRadioGroupElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 6 can not be made a codeGen_FormularRadioGroupElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularRadioGroupElement_converter) codeGen_MkcodeGen_FormularRadioGroupElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 7 can not be made a codeGen_FormularRadioGroupElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularRadioGroupElement_layout) codeGen_MkcodeGen_FormularRadioGroupElement_layout (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 8 can not be made a codeGen_FormularRadioGroupElement_layout node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_FormularRadioGroupElement_elements) codeGen_MkcodeGen_FormularRadioGroupElement_elements (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 9 can not be made a codeGen_FormularRadioGroupElement_elements node ", 0, _coordref);
_currn->_desc10 = (codeGen__TSPcodeGen_FormularRadioGroupElement_checked) codeGen_MkcodeGen_FormularRadioGroupElement_checked (_coordref, _desc10);	
if (((NODEPTR)_currn->_desc10) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioGroupElement: root of subtree no. 10 can not be made a codeGen_FormularRadioGroupElement_checked node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularRadioGroupElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_validator)
	l =_ElempcodeGen_FormularRadioGroupElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_validator)
	r = _ElempcodeGen_FormularRadioGroupElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularRadioGroupElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_validator)
		t = _ElempcodeGen_FormularRadioGroupElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularRadioGroupElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularRadioGroupElement_validator;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularRadioGroupElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularRadioGroupElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_validator)list;
{ codeGen__TPPpcodeGen_FormularRadioGroupElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularRadioGroupElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_converter)
	l =_ElempcodeGen_FormularRadioGroupElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_converter)
	r = _ElempcodeGen_FormularRadioGroupElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularRadioGroupElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_converter)
		t = _ElempcodeGen_FormularRadioGroupElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularRadioGroupElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularRadioGroupElement_converter;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularRadioGroupElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularRadioGroupElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_converter)list;
{ codeGen__TPPpcodeGen_FormularRadioGroupElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_layout (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_layout (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_layout _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_layout;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_layout) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_layout));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_layout;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_layout;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_layout */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements;
#else
	_new = (codeGen__TPPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements;
	_new->_desc1 = (codeGen__TSPcodeGen_FormularRadioElement)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_elements)_new;
	{codeGen__TPPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_elements (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularRadioGroupElement_elements (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FormularRadioElement)) 
		return codeGen__MkcodeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements (_coordref, codeGen_MkcodeGen_FormularRadioElement (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularRadioGroupElement_elements list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_elements (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularRadioGroupElement_elements (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_elements)
	l =_ElempcodeGen_FormularRadioGroupElement_elements (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_elements)
	r = _ElempcodeGen_FormularRadioGroupElement_elements (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_elements (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_elements (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularRadioGroupElement_elements root;
	codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularRadioGroupElement_elements)
		t = _ElempcodeGen_FormularRadioGroupElement_elements (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)(((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_elements;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularRadioGroupElement_elements));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularRadioGroupElement_elements;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularRadioGroupElement_elements)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements)
	{codeGen__TPPLST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FormularRadioElementpcodeGen_FormularRadioGroupElement_elements;
	} this code has been moved to _ElempcodeGen_FormularRadioGroupElement_elements*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularRadioGroupElement_elements;
#else
root = (codeGen__TPPpcodeGen_FormularRadioGroupElement_elements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_elements));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_elements;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularRadioGroupElement_elements)list;
{ codeGen__TPPpcodeGen_FormularRadioGroupElement_elements _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_elements;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_checked (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioGroupElement_checked (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioGroupElement_checked _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioGroupElement_checked;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioGroupElement_checked) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioGroupElement_checked));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioGroupElement_checked;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioGroupElement_checked;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioGroupElement_checked */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularTextElement_name) codeGen_MkcodeGen_FormularTextElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 1 can not be made a codeGen_FormularTextElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularTextElement_label) codeGen_MkcodeGen_FormularTextElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 2 can not be made a codeGen_FormularTextElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularTextElement_description) codeGen_MkcodeGen_FormularTextElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 3 can not be made a codeGen_FormularTextElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularTextElement_pvalue) codeGen_MkcodeGen_FormularTextElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 4 can not be made a codeGen_FormularTextElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularTextElement_required) codeGen_MkcodeGen_FormularTextElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 5 can not be made a codeGen_FormularTextElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularTextElement_validator) codeGen_MkcodeGen_FormularTextElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 6 can not be made a codeGen_FormularTextElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularTextElement_converter) codeGen_MkcodeGen_FormularTextElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextElement: root of subtree no. 7 can not be made a codeGen_FormularTextElement_converter node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularTextElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularTextElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularTextElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_validator)
	l =_ElempcodeGen_FormularTextElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_validator)
	r = _ElempcodeGen_FormularTextElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularTextElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularTextElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_validator)
		t = _ElempcodeGen_FormularTextElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularTextElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularTextElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularTextElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularTextElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularTextElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularTextElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularTextElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularTextElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularTextElement_validator)list;
{ codeGen__TPPpcodeGen_FormularTextElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularTextElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularTextElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularTextElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularTextElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_converter)
	l =_ElempcodeGen_FormularTextElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_converter)
	r = _ElempcodeGen_FormularTextElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularTextElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularTextElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularTextElement_converter)
		t = _ElempcodeGen_FormularTextElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularTextElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularTextElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularTextElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularTextElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularTextElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularTextElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularTextElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularTextElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularTextElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularTextElement_converter)list;
{ codeGen__TPPpcodeGen_FormularTextElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularTextElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularTextAreaElement_name) codeGen_MkcodeGen_FormularTextAreaElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 1 can not be made a codeGen_FormularTextAreaElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularTextAreaElement_label) codeGen_MkcodeGen_FormularTextAreaElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 2 can not be made a codeGen_FormularTextAreaElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularTextAreaElement_description) codeGen_MkcodeGen_FormularTextAreaElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 3 can not be made a codeGen_FormularTextAreaElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularTextAreaElement_pvalue) codeGen_MkcodeGen_FormularTextAreaElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 4 can not be made a codeGen_FormularTextAreaElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularTextAreaElement_required) codeGen_MkcodeGen_FormularTextAreaElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 5 can not be made a codeGen_FormularTextAreaElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularTextAreaElement_validator) codeGen_MkcodeGen_FormularTextAreaElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 6 can not be made a codeGen_FormularTextAreaElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularTextAreaElement_converter) codeGen_MkcodeGen_FormularTextAreaElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 7 can not be made a codeGen_FormularTextAreaElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularTextAreaElement_rows) codeGen_MkcodeGen_FormularTextAreaElement_rows (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularTextAreaElement: root of subtree no. 8 can not be made a codeGen_FormularTextAreaElement_rows node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularTextAreaElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularTextAreaElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularTextAreaElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_validator)
	l =_ElempcodeGen_FormularTextAreaElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_validator)
	r = _ElempcodeGen_FormularTextAreaElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularTextAreaElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_validator)
		t = _ElempcodeGen_FormularTextAreaElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularTextAreaElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularTextAreaElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularTextAreaElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularTextAreaElement_validator;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularTextAreaElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularTextAreaElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_validator)list;
{ codeGen__TPPpcodeGen_FormularTextAreaElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularTextAreaElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularTextAreaElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularTextAreaElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularTextAreaElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_converter)
	l =_ElempcodeGen_FormularTextAreaElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_converter)
	r = _ElempcodeGen_FormularTextAreaElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularTextAreaElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularTextAreaElement_converter)
		t = _ElempcodeGen_FormularTextAreaElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularTextAreaElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularTextAreaElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularTextAreaElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularTextAreaElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularTextAreaElement_converter;
	} this code has been moved to _ElempcodeGen_FormularTextAreaElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularTextAreaElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularTextAreaElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularTextAreaElement_converter)list;
{ codeGen__TPPpcodeGen_FormularTextAreaElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularTextAreaElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_rows (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularTextAreaElement_rows (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularTextAreaElement_rows _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularTextAreaElement_rows;
#else
_currn = (codeGen__TPPpcodeGen_FormularTextAreaElement_rows) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularTextAreaElement_rows));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularTextAreaElement_rows;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularTextAreaElement_rows;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularTextAreaElement_rows */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularPasswordElement_name) codeGen_MkcodeGen_FormularPasswordElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 1 can not be made a codeGen_FormularPasswordElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularPasswordElement_label) codeGen_MkcodeGen_FormularPasswordElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 2 can not be made a codeGen_FormularPasswordElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularPasswordElement_description) codeGen_MkcodeGen_FormularPasswordElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 3 can not be made a codeGen_FormularPasswordElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularPasswordElement_pvalue) codeGen_MkcodeGen_FormularPasswordElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 4 can not be made a codeGen_FormularPasswordElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularPasswordElement_required) codeGen_MkcodeGen_FormularPasswordElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 5 can not be made a codeGen_FormularPasswordElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularPasswordElement_validator) codeGen_MkcodeGen_FormularPasswordElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 6 can not be made a codeGen_FormularPasswordElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularPasswordElement_converter) codeGen_MkcodeGen_FormularPasswordElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 7 can not be made a codeGen_FormularPasswordElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularPasswordElement_redisplay) codeGen_MkcodeGen_FormularPasswordElement_redisplay (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularPasswordElement: root of subtree no. 8 can not be made a codeGen_FormularPasswordElement_redisplay node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularPasswordElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularPasswordElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularPasswordElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_validator)
	l =_ElempcodeGen_FormularPasswordElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_validator)
	r = _ElempcodeGen_FormularPasswordElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularPasswordElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_validator)
		t = _ElempcodeGen_FormularPasswordElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularPasswordElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularPasswordElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularPasswordElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularPasswordElement_validator;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularPasswordElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularPasswordElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularPasswordElement_validator)list;
{ codeGen__TPPpcodeGen_FormularPasswordElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularPasswordElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularPasswordElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularPasswordElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularPasswordElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularPasswordElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularPasswordElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_converter)
	l =_ElempcodeGen_FormularPasswordElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_converter)
	r = _ElempcodeGen_FormularPasswordElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularPasswordElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularPasswordElement_converter)
		t = _ElempcodeGen_FormularPasswordElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularPasswordElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularPasswordElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularPasswordElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularPasswordElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularPasswordElement_converter;
	} this code has been moved to _ElempcodeGen_FormularPasswordElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularPasswordElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularPasswordElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularPasswordElement_converter)list;
{ codeGen__TPPpcodeGen_FormularPasswordElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularPasswordElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_redisplay (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularPasswordElement_redisplay (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularPasswordElement_redisplay _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularPasswordElement_redisplay;
#else
_currn = (codeGen__TPPpcodeGen_FormularPasswordElement_redisplay) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularPasswordElement_redisplay));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularPasswordElement_redisplay;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularPasswordElement_redisplay;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularPasswordElement_redisplay */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularDateElement_name) codeGen_MkcodeGen_FormularDateElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 1 can not be made a codeGen_FormularDateElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularDateElement_label) codeGen_MkcodeGen_FormularDateElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 2 can not be made a codeGen_FormularDateElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularDateElement_description) codeGen_MkcodeGen_FormularDateElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 3 can not be made a codeGen_FormularDateElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularDateElement_pvalue) codeGen_MkcodeGen_FormularDateElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 4 can not be made a codeGen_FormularDateElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularDateElement_required) codeGen_MkcodeGen_FormularDateElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 5 can not be made a codeGen_FormularDateElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularDateElement_validator) codeGen_MkcodeGen_FormularDateElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 6 can not be made a codeGen_FormularDateElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularDateElement_converter) codeGen_MkcodeGen_FormularDateElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDateElement: root of subtree no. 7 can not be made a codeGen_FormularDateElement_converter node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDateElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDateElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularDateElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDateElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDateElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDateElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularDateElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularDateElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularDateElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_validator)
	l =_ElempcodeGen_FormularDateElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_validator)
	r = _ElempcodeGen_FormularDateElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularDateElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularDateElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_validator)
		t = _ElempcodeGen_FormularDateElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularDateElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularDateElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularDateElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularDateElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularDateElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularDateElement_validator;
	} this code has been moved to _ElempcodeGen_FormularDateElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularDateElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularDateElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularDateElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularDateElement_validator)list;
{ codeGen__TPPpcodeGen_FormularDateElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularDateElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularDateElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularDateElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularDateElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularDateElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularDateElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_converter)
	l =_ElempcodeGen_FormularDateElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_converter)
	r = _ElempcodeGen_FormularDateElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDateElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularDateElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularDateElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularDateElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularDateElement_converter)
		t = _ElempcodeGen_FormularDateElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularDateElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularDateElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularDateElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularDateElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularDateElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularDateElement_converter;
	} this code has been moved to _ElempcodeGen_FormularDateElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularDateElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularDateElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDateElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularDateElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularDateElement_converter)list;
{ codeGen__TPPpcodeGen_FormularDateElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularDateElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularFileUploadElement_name) codeGen_MkcodeGen_FormularFileUploadElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 1 can not be made a codeGen_FormularFileUploadElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularFileUploadElement_label) codeGen_MkcodeGen_FormularFileUploadElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 2 can not be made a codeGen_FormularFileUploadElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularFileUploadElement_description) codeGen_MkcodeGen_FormularFileUploadElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 3 can not be made a codeGen_FormularFileUploadElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularFileUploadElement_pvalue) codeGen_MkcodeGen_FormularFileUploadElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 4 can not be made a codeGen_FormularFileUploadElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularFileUploadElement_required) codeGen_MkcodeGen_FormularFileUploadElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 5 can not be made a codeGen_FormularFileUploadElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularFileUploadElement_validator) codeGen_MkcodeGen_FormularFileUploadElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 6 can not be made a codeGen_FormularFileUploadElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularFileUploadElement_converter) codeGen_MkcodeGen_FormularFileUploadElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 7 can not be made a codeGen_FormularFileUploadElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularFileUploadElement_accept) codeGen_MkcodeGen_FormularFileUploadElement_accept (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularFileUploadElement: root of subtree no. 8 can not be made a codeGen_FormularFileUploadElement_accept node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularFileUploadElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularFileUploadElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularFileUploadElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_validator)
	l =_ElempcodeGen_FormularFileUploadElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_validator)
	r = _ElempcodeGen_FormularFileUploadElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularFileUploadElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_validator)
		t = _ElempcodeGen_FormularFileUploadElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularFileUploadElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularFileUploadElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularFileUploadElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularFileUploadElement_validator;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularFileUploadElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularFileUploadElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_validator)list;
{ codeGen__TPPpcodeGen_FormularFileUploadElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularFileUploadElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularFileUploadElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularFileUploadElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularFileUploadElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_converter)
	l =_ElempcodeGen_FormularFileUploadElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_converter)
	r = _ElempcodeGen_FormularFileUploadElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularFileUploadElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularFileUploadElement_converter)
		t = _ElempcodeGen_FormularFileUploadElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularFileUploadElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularFileUploadElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularFileUploadElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularFileUploadElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularFileUploadElement_converter;
	} this code has been moved to _ElempcodeGen_FormularFileUploadElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularFileUploadElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularFileUploadElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularFileUploadElement_converter)list;
{ codeGen__TPPpcodeGen_FormularFileUploadElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularFileUploadElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_accept (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularFileUploadElement_accept (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularFileUploadElement_accept _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularFileUploadElement_accept;
#else
_currn = (codeGen__TPPpcodeGen_FormularFileUploadElement_accept) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularFileUploadElement_accept));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularFileUploadElement_accept;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularFileUploadElement_accept;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularFileUploadElement_accept */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularCheckboxElement_name) codeGen_MkcodeGen_FormularCheckboxElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 1 can not be made a codeGen_FormularCheckboxElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularCheckboxElement_label) codeGen_MkcodeGen_FormularCheckboxElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 2 can not be made a codeGen_FormularCheckboxElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularCheckboxElement_description) codeGen_MkcodeGen_FormularCheckboxElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 3 can not be made a codeGen_FormularCheckboxElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularCheckboxElement_pvalue) codeGen_MkcodeGen_FormularCheckboxElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 4 can not be made a codeGen_FormularCheckboxElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularCheckboxElement_required) codeGen_MkcodeGen_FormularCheckboxElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 5 can not be made a codeGen_FormularCheckboxElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularCheckboxElement_validator) codeGen_MkcodeGen_FormularCheckboxElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 6 can not be made a codeGen_FormularCheckboxElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularCheckboxElement_converter) codeGen_MkcodeGen_FormularCheckboxElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 7 can not be made a codeGen_FormularCheckboxElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularCheckboxElement_value) codeGen_MkcodeGen_FormularCheckboxElement_value (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 8 can not be made a codeGen_FormularCheckboxElement_value node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_FormularCheckboxElement_checked) codeGen_MkcodeGen_FormularCheckboxElement_checked (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularCheckboxElement: root of subtree no. 9 can not be made a codeGen_FormularCheckboxElement_checked node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularCheckboxElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularCheckboxElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularCheckboxElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_validator)
	l =_ElempcodeGen_FormularCheckboxElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_validator)
	r = _ElempcodeGen_FormularCheckboxElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularCheckboxElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_validator)
		t = _ElempcodeGen_FormularCheckboxElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularCheckboxElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularCheckboxElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularCheckboxElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularCheckboxElement_validator;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularCheckboxElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularCheckboxElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_validator)list;
{ codeGen__TPPpcodeGen_FormularCheckboxElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularCheckboxElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularCheckboxElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularCheckboxElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularCheckboxElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_converter)
	l =_ElempcodeGen_FormularCheckboxElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_converter)
	r = _ElempcodeGen_FormularCheckboxElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularCheckboxElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularCheckboxElement_converter)
		t = _ElempcodeGen_FormularCheckboxElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularCheckboxElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularCheckboxElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularCheckboxElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularCheckboxElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularCheckboxElement_converter;
	} this code has been moved to _ElempcodeGen_FormularCheckboxElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularCheckboxElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularCheckboxElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularCheckboxElement_converter)list;
{ codeGen__TPPpcodeGen_FormularCheckboxElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularCheckboxElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_value;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_checked (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularCheckboxElement_checked (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularCheckboxElement_checked _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularCheckboxElement_checked;
#else
_currn = (codeGen__TPPpcodeGen_FormularCheckboxElement_checked) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularCheckboxElement_checked));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularCheckboxElement_checked;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularCheckboxElement_checked;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularCheckboxElement_checked */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9, NODEPTR _desc10)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9,_desc10)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
	NODEPTR _desc10;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularSelectElement_name) codeGen_MkcodeGen_FormularSelectElement_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 1 can not be made a codeGen_FormularSelectElement_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularSelectElement_label) codeGen_MkcodeGen_FormularSelectElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 2 can not be made a codeGen_FormularSelectElement_label node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularSelectElement_description) codeGen_MkcodeGen_FormularSelectElement_description (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 3 can not be made a codeGen_FormularSelectElement_description node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularSelectElement_pvalue) codeGen_MkcodeGen_FormularSelectElement_pvalue (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 4 can not be made a codeGen_FormularSelectElement_pvalue node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_FormularSelectElement_required) codeGen_MkcodeGen_FormularSelectElement_required (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 5 can not be made a codeGen_FormularSelectElement_required node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_FormularSelectElement_validator) codeGen_MkcodeGen_FormularSelectElement_validator (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 6 can not be made a codeGen_FormularSelectElement_validator node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_FormularSelectElement_converter) codeGen_MkcodeGen_FormularSelectElement_converter (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 7 can not be made a codeGen_FormularSelectElement_converter node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_FormularSelectElement_options) codeGen_MkcodeGen_FormularSelectElement_options (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 8 can not be made a codeGen_FormularSelectElement_options node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_FormularSelectElement_size) codeGen_MkcodeGen_FormularSelectElement_size (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 9 can not be made a codeGen_FormularSelectElement_size node ", 0, _coordref);
_currn->_desc10 = (codeGen__TSPcodeGen_FormularSelectElement_multiple) codeGen_MkcodeGen_FormularSelectElement_multiple (_coordref, _desc10);	
if (((NODEPTR)_currn->_desc10) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularSelectElement: root of subtree no. 10 can not be made a codeGen_FormularSelectElement_multiple node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_description (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_description (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_description _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_description;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_description) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_description));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_description;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_description;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_description */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_pvalue (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_pvalue (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_pvalue _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_pvalue;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_pvalue) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_pvalue));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_pvalue;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_pvalue;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_pvalue */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_required (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_required (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_required _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_required;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_required) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_required));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_required;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_required;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_required */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_EmailValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_CreditCardValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_RegExprValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_StringLengthValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_LongRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleRangeValidator)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)_new;
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularSelectElement_validator (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_EmailValidator)) 
		return codeGen__MkcodeGen_EmailValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_EmailValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_CreditCardValidator)) 
		return codeGen__MkcodeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_CreditCardValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_RegExprValidator)) 
		return codeGen__MkcodeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_RegExprValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_StringLengthValidator)) 
		return codeGen__MkcodeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_StringLengthValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongRangeValidator)) 
		return codeGen__MkcodeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_LongRangeValidator (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleRangeValidator)) 
		return codeGen__MkcodeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator (_coordref, codeGen_MkcodeGen_DoubleRangeValidator (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularSelectElement_validator list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularSelectElement_validator (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_validator)
	l =_ElempcodeGen_FormularSelectElement_validator (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_validator)
	r = _ElempcodeGen_FormularSelectElement_validator (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_validator (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_validator (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularSelectElement_validator root;
	codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_validator)
		t = _ElempcodeGen_FormularSelectElement_validator (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)(((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularSelectElement_validator;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularSelectElement_validator));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularSelectElement_validator;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_validator)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_EmailValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_CreditCardValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_RegExprValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_StringLengthValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongRangeValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator)
	{codeGen__TPPLST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleRangeValidatorpcodeGen_FormularSelectElement_validator;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_validator*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularSelectElement_validator;
#else
root = (codeGen__TPPpcodeGen_FormularSelectElement_validator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_validator));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularSelectElement_validator;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularSelectElement_validator)list;
{ codeGen__TPPpcodeGen_FormularSelectElement_validator _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularSelectElement_validator;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleNumberConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_IntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_LongConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_LongConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BigIntegerConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_FloatConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_FloatConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DoubleConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_BooleanConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_DateTimeConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_PatternConverterpcodeGen_FormularSelectElement_converter (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter;
#else
	_new = (codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter));
#endif
	_new->_prod = codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter;
	_new->_desc1 = (codeGen__TSPcodeGen_PatternConverter)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)_new;
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularSelectElement_converter (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerNumberConverter)) 
		return codeGen__MkcodeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_IntegerNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleNumberConverter)) 
		return codeGen__MkcodeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_DoubleNumberConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_IntegerConverter)) 
		return codeGen__MkcodeGen_IntegerConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_IntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_LongConverter)) 
		return codeGen__MkcodeGen_LongConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_LongConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BigIntegerConverter)) 
		return codeGen__MkcodeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_BigIntegerConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_FloatConverter)) 
		return codeGen__MkcodeGen_FloatConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_FloatConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DoubleConverter)) 
		return codeGen__MkcodeGen_DoubleConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_DoubleConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_BooleanConverter)) 
		return codeGen__MkcodeGen_BooleanConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_BooleanConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DateTimeConverter)) 
		return codeGen__MkcodeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_DateTimeConverter (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_PatternConverter)) 
		return codeGen__MkcodeGen_PatternConverterpcodeGen_FormularSelectElement_converter (_coordref, codeGen_MkcodeGen_PatternConverter (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularSelectElement_converter list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularSelectElement_converter (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_converter)
	l =_ElempcodeGen_FormularSelectElement_converter (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_converter)
	r = _ElempcodeGen_FormularSelectElement_converter (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_converter (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_converter (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularSelectElement_converter root;
	codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularSelectElement_converter)
		t = _ElempcodeGen_FormularSelectElement_converter (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)(((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularSelectElement_converter;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularSelectElement_converter));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularSelectElement_converter;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularSelectElement_converter)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerNumberConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleNumberConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_IntegerConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_LongConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BigIntegerConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_FloatConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DoubleConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_BooleanConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DateTimeConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter)
	{codeGen__TPPLST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_PatternConverterpcodeGen_FormularSelectElement_converter;
	} this code has been moved to _ElempcodeGen_FormularSelectElement_converter*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularSelectElement_converter;
#else
root = (codeGen__TPPpcodeGen_FormularSelectElement_converter) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_converter));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularSelectElement_converter;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularSelectElement_converter)list;
{ codeGen__TPPpcodeGen_FormularSelectElement_converter _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularSelectElement_converter;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_size (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_size (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_size _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_size;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_size) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_size));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_size;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_size;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_size */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_multiple (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularSelectElement_multiple (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularSelectElement_multiple _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularSelectElement_multiple;
#else
_currn = (codeGen__TPPpcodeGen_FormularSelectElement_multiple) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularSelectElement_multiple));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularSelectElement_multiple;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularSelectElement_multiple;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularSelectElement_multiple */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularConstantSelectOption (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_FormularConstantSelectOption (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_FormularConstantSelectOption _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularConstantSelectOption;
#else
_currn = (codeGen__TPPpcodeGen_FormularConstantSelectOption) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularConstantSelectOption));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularConstantSelectOption;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularConstantSelectOption_options) codeGen_MkcodeGen_FormularConstantSelectOption_options (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularConstantSelectOption: root of subtree no. 1 can not be made a codeGen_FormularConstantSelectOption_options node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularConstantSelectOption;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularConstantSelectOption */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options;
#else
	_new = (codeGen__TPPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options));
#endif
	_new->_prod = codeGen_RULELST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options;
	_new->_desc1 = (codeGen__TSPcodeGen_SelectOption)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_FormularConstantSelectOption_options)_new;
	{codeGen__TPPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_FormularConstantSelectOption_options (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_FormularConstantSelectOption_options (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_SelectOption)) 
		return codeGen__MkcodeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options (_coordref, codeGen_MkcodeGen_SelectOption (_coordref, t));
	message (DEADLY, "a pcodeGen_FormularConstantSelectOption_options list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_FormularConstantSelectOption_options (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_FormularConstantSelectOption_options (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_FormularConstantSelectOption_options)
	l =_ElempcodeGen_FormularConstantSelectOption_options (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_FormularConstantSelectOption_options)
	r = _ElempcodeGen_FormularConstantSelectOption_options (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularConstantSelectOption_options (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_FormularConstantSelectOption_options (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_FormularConstantSelectOption_options root;
	codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_FormularConstantSelectOption_options)
		t = _ElempcodeGen_FormularConstantSelectOption_options (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)(((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_FormularConstantSelectOption_options;
#else
list = (codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_FormularConstantSelectOption_options));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_FormularConstantSelectOption_options;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_FormularConstantSelectOption_options)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options)
	{codeGen__TPPLST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_SelectOptionpcodeGen_FormularConstantSelectOption_options;
	} this code has been moved to _ElempcodeGen_FormularConstantSelectOption_options*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_FormularConstantSelectOption_options;
#else
root = (codeGen__TPPpcodeGen_FormularConstantSelectOption_options) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularConstantSelectOption_options));
#endif
root->_prod = codeGen_RULEpcodeGen_FormularConstantSelectOption_options;
root->_desc1 = (codeGen__TSPLST_codeGen_FormularConstantSelectOption_options)list;
{ codeGen__TPPpcodeGen_FormularConstantSelectOption_options _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_FormularConstantSelectOption_options;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SelectOption (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_SelectOption (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_SelectOption _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SelectOption;
#else
_currn = (codeGen__TPPpcodeGen_SelectOption) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SelectOption));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SelectOption;
_currn->_desc1 = (codeGen__TSPcodeGen_SelectOption_value) codeGen_MkcodeGen_SelectOption_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SelectOption: root of subtree no. 1 can not be made a codeGen_SelectOption_value node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_SelectOption_label) codeGen_MkcodeGen_SelectOption_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_SelectOption: root of subtree no. 2 can not be made a codeGen_SelectOption_label node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SelectOption;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SelectOption */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SelectOption_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SelectOption_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SelectOption_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SelectOption_value;
#else
_currn = (codeGen__TPPpcodeGen_SelectOption_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SelectOption_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SelectOption_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SelectOption_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SelectOption_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_SelectOption_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_SelectOption_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_SelectOption_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_SelectOption_label;
#else
_currn = (codeGen__TPPpcodeGen_SelectOption_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_SelectOption_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_SelectOption_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_SelectOption_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_SelectOption_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_FormularDataSourceSelectOption _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDataSourceSelectOption;
#else
_currn = (codeGen__TPPpcodeGen_FormularDataSourceSelectOption) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDataSourceSelectOption));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDataSourceSelectOption;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularDataSourceSelectOption_name) codeGen_MkcodeGen_FormularDataSourceSelectOption_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDataSourceSelectOption: root of subtree no. 1 can not be made a codeGen_FormularDataSourceSelectOption_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularDataSourceSelectOption_dataSource) codeGen_MkcodeGen_FormularDataSourceSelectOption_dataSource (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDataSourceSelectOption: root of subtree no. 2 can not be made a codeGen_FormularDataSourceSelectOption_dataSource node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_FormularDataSourceSelectOption_value) codeGen_MkcodeGen_FormularDataSourceSelectOption_value (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDataSourceSelectOption: root of subtree no. 3 can not be made a codeGen_FormularDataSourceSelectOption_value node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_FormularDataSourceSelectOption_label) codeGen_MkcodeGen_FormularDataSourceSelectOption_label (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularDataSourceSelectOption: root of subtree no. 4 can not be made a codeGen_FormularDataSourceSelectOption_label node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDataSourceSelectOption;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDataSourceSelectOption */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDataSourceSelectOption_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDataSourceSelectOption_name;
#else
_currn = (codeGen__TPPpcodeGen_FormularDataSourceSelectOption_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDataSourceSelectOption_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDataSourceSelectOption_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDataSourceSelectOption_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDataSourceSelectOption_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_dataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_dataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDataSourceSelectOption_dataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDataSourceSelectOption_dataSource;
#else
_currn = (codeGen__TPPpcodeGen_FormularDataSourceSelectOption_dataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDataSourceSelectOption_dataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDataSourceSelectOption_dataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDataSourceSelectOption_dataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDataSourceSelectOption_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDataSourceSelectOption_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDataSourceSelectOption_value;
#else
_currn = (codeGen__TPPpcodeGen_FormularDataSourceSelectOption_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDataSourceSelectOption_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDataSourceSelectOption_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDataSourceSelectOption_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDataSourceSelectOption_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularDataSourceSelectOption_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularDataSourceSelectOption_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularDataSourceSelectOption_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularDataSourceSelectOption_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularDataSourceSelectOption_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularDataSourceSelectOption_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularDataSourceSelectOption_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularDataSourceSelectOption_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioElement (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_FormularRadioElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioElement;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioElement;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularRadioElement_value) codeGen_MkcodeGen_FormularRadioElement_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioElement: root of subtree no. 1 can not be made a codeGen_FormularRadioElement_value node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_FormularRadioElement_label) codeGen_MkcodeGen_FormularRadioElement_label (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_FormularRadioElement: root of subtree no. 2 can not be made a codeGen_FormularRadioElement_label node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioElement_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioElement_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioElement_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioElement_value;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioElement_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioElement_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioElement_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioElement_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioElement_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_FormularRadioElement_label (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_FormularRadioElement_label (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_FormularRadioElement_label _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_FormularRadioElement_label;
#else
_currn = (codeGen__TPPpcodeGen_FormularRadioElement_label) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_FormularRadioElement_label));
#endif
_currn->_prod = codeGen_RULEpcodeGen_FormularRadioElement_label;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_FormularRadioElement_label;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_FormularRadioElement_label */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeActualParam (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeActualParam (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebNodeActualParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeActualParam;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeActualParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeActualParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeActualParam;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeActualParam_formalParam) codeGen_MkcodeGen_WebNodeActualParam_formalParam (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeActualParam: root of subtree no. 1 can not be made a codeGen_WebNodeActualParam_formalParam node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebNodeActualParam_expressions) codeGen_MkcodeGen_WebNodeActualParam_expressions (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeActualParam: root of subtree no. 2 can not be made a codeGen_WebNodeActualParam_expressions node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeActualParam;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeActualParam_formalParam (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeActualParam_formalParam (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebNodeActualParam_formalParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeActualParam_formalParam;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeActualParam_formalParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeActualParam_formalParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeActualParam_formalParam;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeActualParam_formalParam;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeFormalParam (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeFormalParam (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebNodeFormalParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeFormalParam;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeFormalParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeFormalParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeFormalParam;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeFormalParam_name) codeGen_MkcodeGen_WebNodeFormalParam_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeFormalParam: root of subtree no. 1 can not be made a codeGen_WebNodeFormalParam_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebNodeFormalParam_type) codeGen_MkcodeGen_WebNodeFormalParam_type (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeFormalParam: root of subtree no. 2 can not be made a codeGen_WebNodeFormalParam_type node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeFormalParam;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeFormalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeFormalParam_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeFormalParam_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebNodeFormalParam_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeFormalParam_name;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeFormalParam_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeFormalParam_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeFormalParam_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeFormalParam_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeFormalParam_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MD5Modifier (POSITION *_coordref, int _TERM1)
#else
NODEPTR codeGen_MkpcodeGen_MD5Modifier (_coordref, _TERM1)
	POSITION *_coordref;
	int _TERM1;
#endif
{	codeGen__TPPpcodeGen_MD5Modifier _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MD5Modifier;
#else
_currn = (codeGen__TPPpcodeGen_MD5Modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MD5Modifier));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MD5Modifier;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MD5Modifier;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MD5Modifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JoinModifier (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_JoinModifier (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_JoinModifier _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JoinModifier;
#else
_currn = (codeGen__TPPpcodeGen_JoinModifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JoinModifier));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JoinModifier;
_currn->_desc1 = (codeGen__TSPcodeGen_JoinModifier_separator) codeGen_MkcodeGen_JoinModifier_separator (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_JoinModifier: root of subtree no. 1 can not be made a codeGen_JoinModifier_separator node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JoinModifier;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JoinModifier */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_JoinModifier_separator (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_JoinModifier_separator (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_JoinModifier_separator _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_JoinModifier_separator;
#else
_currn = (codeGen__TPPpcodeGen_JoinModifier_separator) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_JoinModifier_separator));
#endif
_currn->_prod = codeGen_RULEpcodeGen_JoinModifier_separator;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_JoinModifier_separator;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_JoinModifier_separator */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebNodeConstantExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeConstantExpression;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeConstantExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeConstantExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeConstantExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeConstantExpression_modifier) codeGen_MkcodeGen_WebNodeConstantExpression_modifier (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeConstantExpression: root of subtree no. 1 can not be made a codeGen_WebNodeConstantExpression_modifier node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebNodeConstantExpression_value) codeGen_MkcodeGen_WebNodeConstantExpression_value (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeConstantExpression: root of subtree no. 2 can not be made a codeGen_WebNodeConstantExpression_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeConstantExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeConstantExpression */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_MD5Modifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeConstantExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_JoinModifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeConstantExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebNodeConstantExpression_modifier (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MD5Modifier)) 
		return codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier (_coordref, codeGen_MkcodeGen_MD5Modifier (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_JoinModifier)) 
		return codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier (_coordref, codeGen_MkcodeGen_JoinModifier (_coordref, t));
	message (DEADLY, "a pcodeGen_WebNodeConstantExpression_modifier list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebNodeConstantExpression_modifier (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebNodeConstantExpression_modifier)
	l =_ElempcodeGen_WebNodeConstantExpression_modifier (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebNodeConstantExpression_modifier)
	r = _ElempcodeGen_WebNodeConstantExpression_modifier (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression_modifier (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebNodeConstantExpression_modifier root;
	codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebNodeConstantExpression_modifier)
		t = _ElempcodeGen_WebNodeConstantExpression_modifier (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)(((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebNodeConstantExpression_modifier;
#else
list = (codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebNodeConstantExpression_modifier));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebNodeConstantExpression_modifier;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebNodeConstantExpression_modifier)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier)
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeConstantExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeConstantExpression_modifier*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier)
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeConstantExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeConstantExpression_modifier*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebNodeConstantExpression_modifier;
#else
root = (codeGen__TPPpcodeGen_WebNodeConstantExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeConstantExpression_modifier));
#endif
root->_prod = codeGen_RULEpcodeGen_WebNodeConstantExpression_modifier;
root->_desc1 = (codeGen__TSPLST_codeGen_WebNodeConstantExpression_modifier)list;
{ codeGen__TPPpcodeGen_WebNodeConstantExpression_modifier _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebNodeConstantExpression_modifier;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeConstantExpression_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebNodeConstantExpression_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeConstantExpression_value;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeConstantExpression_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeConstantExpression_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeConstantExpression_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeConstantExpression_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeConstantExpression_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebNodeVariableExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeVariableExpression;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeVariableExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeVariableExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeVariableExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeVariableExpression_modifier) codeGen_MkcodeGen_WebNodeVariableExpression_modifier (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeVariableExpression: root of subtree no. 1 can not be made a codeGen_WebNodeVariableExpression_modifier node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebNodeVariableExpression_content) codeGen_MkcodeGen_WebNodeVariableExpression_content (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeVariableExpression: root of subtree no. 2 can not be made a codeGen_WebNodeVariableExpression_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeVariableExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeVariableExpression */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_MD5Modifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeVariableExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_JoinModifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeVariableExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebNodeVariableExpression_modifier (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MD5Modifier)) 
		return codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier (_coordref, codeGen_MkcodeGen_MD5Modifier (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_JoinModifier)) 
		return codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier (_coordref, codeGen_MkcodeGen_JoinModifier (_coordref, t));
	message (DEADLY, "a pcodeGen_WebNodeVariableExpression_modifier list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebNodeVariableExpression_modifier (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebNodeVariableExpression_modifier)
	l =_ElempcodeGen_WebNodeVariableExpression_modifier (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebNodeVariableExpression_modifier)
	r = _ElempcodeGen_WebNodeVariableExpression_modifier (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression_modifier (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebNodeVariableExpression_modifier root;
	codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebNodeVariableExpression_modifier)
		t = _ElempcodeGen_WebNodeVariableExpression_modifier (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)(((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebNodeVariableExpression_modifier;
#else
list = (codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebNodeVariableExpression_modifier));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebNodeVariableExpression_modifier;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebNodeVariableExpression_modifier)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier)
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeVariableExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeVariableExpression_modifier*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier)
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeVariableExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeVariableExpression_modifier*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebNodeVariableExpression_modifier;
#else
root = (codeGen__TPPpcodeGen_WebNodeVariableExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeVariableExpression_modifier));
#endif
root->_prod = codeGen_RULEpcodeGen_WebNodeVariableExpression_modifier;
root->_desc1 = (codeGen__TSPLST_codeGen_WebNodeVariableExpression_modifier)list;
{ codeGen__TPPpcodeGen_WebNodeVariableExpression_modifier _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebNodeVariableExpression_modifier;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression_content (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeVariableExpression_content (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebNodeVariableExpression_content _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeVariableExpression_content;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeVariableExpression_content) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeVariableExpression_content));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeVariableExpression_content;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeVariableExpression_content;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeVariableExpression_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_WebNodeSessionExpression _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeSessionExpression;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeSessionExpression) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeSessionExpression));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeSessionExpression;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeSessionExpression_modifier) codeGen_MkcodeGen_WebNodeSessionExpression_modifier (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeSessionExpression: root of subtree no. 1 can not be made a codeGen_WebNodeSessionExpression_modifier node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_WebNodeSessionExpression_content) codeGen_MkcodeGen_WebNodeSessionExpression_content (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_WebNodeSessionExpression: root of subtree no. 2 can not be made a codeGen_WebNodeSessionExpression_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeSessionExpression;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeSessionExpression */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_MD5Modifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeSessionExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier;
#else
	_new = (codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier));
#endif
	_new->_prod = codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier;
	_new->_desc1 = (codeGen__TSPcodeGen_JoinModifier)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_WebNodeSessionExpression_modifier)_new;
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_WebNodeSessionExpression_modifier (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MD5Modifier)) 
		return codeGen__MkcodeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier (_coordref, codeGen_MkcodeGen_MD5Modifier (_coordref, t));
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_JoinModifier)) 
		return codeGen__MkcodeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier (_coordref, codeGen_MkcodeGen_JoinModifier (_coordref, t));
	message (DEADLY, "a pcodeGen_WebNodeSessionExpression_modifier list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_WebNodeSessionExpression_modifier (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_WebNodeSessionExpression_modifier)
	l =_ElempcodeGen_WebNodeSessionExpression_modifier (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_WebNodeSessionExpression_modifier)
	r = _ElempcodeGen_WebNodeSessionExpression_modifier (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression_modifier (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression_modifier (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_WebNodeSessionExpression_modifier root;
	codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_WebNodeSessionExpression_modifier)
		t = _ElempcodeGen_WebNodeSessionExpression_modifier (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)(((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_WebNodeSessionExpression_modifier;
#else
list = (codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_WebNodeSessionExpression_modifier));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_WebNodeSessionExpression_modifier;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_WebNodeSessionExpression_modifier)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier)
	{codeGen__TPPLST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MD5ModifierpcodeGen_WebNodeSessionExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeSessionExpression_modifier*/
	/*if (_currn->_prod == codeGen_RULELST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier)
	{codeGen__TPPLST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_JoinModifierpcodeGen_WebNodeSessionExpression_modifier;
	} this code has been moved to _ElempcodeGen_WebNodeSessionExpression_modifier*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_WebNodeSessionExpression_modifier;
#else
root = (codeGen__TPPpcodeGen_WebNodeSessionExpression_modifier) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeSessionExpression_modifier));
#endif
root->_prod = codeGen_RULEpcodeGen_WebNodeSessionExpression_modifier;
root->_desc1 = (codeGen__TSPLST_codeGen_WebNodeSessionExpression_modifier)list;
{ codeGen__TPPpcodeGen_WebNodeSessionExpression_modifier _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_WebNodeSessionExpression_modifier;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression_content (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_WebNodeSessionExpression_content (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_WebNodeSessionExpression_content _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_WebNodeSessionExpression_content;
#else
_currn = (codeGen__TPPpcodeGen_WebNodeSessionExpression_content) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_WebNodeSessionExpression_content));
#endif
_currn->_prod = codeGen_RULEpcodeGen_WebNodeSessionExpression_content;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_WebNodeSessionExpression_content;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_WebNodeSessionExpression_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputFragment (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_OutputFragment (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_OutputFragment _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputFragment;
#else
_currn = (codeGen__TPPpcodeGen_OutputFragment) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputFragment));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputFragment;
_currn->_desc1 = (codeGen__TSPcodeGen_OutputFragment_link) codeGen_MkcodeGen_OutputFragment_link (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputFragment: root of subtree no. 1 can not be made a codeGen_OutputFragment_link node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_OutputFragment_content) codeGen_MkcodeGen_OutputFragment_content (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputFragment: root of subtree no. 2 can not be made a codeGen_OutputFragment_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputFragment;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputFragment */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputFragment_link (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputFragment_link (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputFragment_link _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputFragment_link;
#else
_currn = (codeGen__TPPpcodeGen_OutputFragment_link) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputFragment_link));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputFragment_link;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputFragment_link;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputFragment_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputFragment_content (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputFragment_content (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputFragment_content _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputFragment_content;
#else
_currn = (codeGen__TPPpcodeGen_OutputFragment_content) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputFragment_content));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputFragment_content;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputFragment_content;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputFragment_content */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_OutputMessagesElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputMessagesElement;
#else
_currn = (codeGen__TPPpcodeGen_OutputMessagesElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputMessagesElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputMessagesElement;
_currn->_desc1 = (codeGen__TSPcodeGen_OutputMessagesElement_styleSheetBlock) codeGen_MkcodeGen_OutputMessagesElement_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputMessagesElement: root of subtree no. 1 can not be made a codeGen_OutputMessagesElement_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_OutputMessagesElement_errorStyleSheetBlock) codeGen_MkcodeGen_OutputMessagesElement_errorStyleSheetBlock (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputMessagesElement: root of subtree no. 2 can not be made a codeGen_OutputMessagesElement_errorStyleSheetBlock node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_OutputMessagesElement_infoStyleSheetBlock) codeGen_MkcodeGen_OutputMessagesElement_infoStyleSheetBlock (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputMessagesElement: root of subtree no. 3 can not be made a codeGen_OutputMessagesElement_infoStyleSheetBlock node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_OutputMessagesElement_layout) codeGen_MkcodeGen_OutputMessagesElement_layout (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_OutputMessagesElement: root of subtree no. 4 can not be made a codeGen_OutputMessagesElement_layout node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputMessagesElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputMessagesElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputMessagesElement_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputMessagesElement_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_OutputMessagesElement_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputMessagesElement_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputMessagesElement_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputMessagesElement_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputMessagesElement_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_errorStyleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_errorStyleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputMessagesElement_errorStyleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputMessagesElement_errorStyleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_OutputMessagesElement_errorStyleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputMessagesElement_errorStyleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputMessagesElement_errorStyleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputMessagesElement_errorStyleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputMessagesElement_errorStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_infoStyleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_infoStyleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputMessagesElement_infoStyleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputMessagesElement_infoStyleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_OutputMessagesElement_infoStyleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputMessagesElement_infoStyleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputMessagesElement_infoStyleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputMessagesElement_infoStyleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputMessagesElement_infoStyleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_layout (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_OutputMessagesElement_layout (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_OutputMessagesElement_layout _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_OutputMessagesElement_layout;
#else
_currn = (codeGen__TPPpcodeGen_OutputMessagesElement_layout) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_OutputMessagesElement_layout));
#endif
_currn->_prod = codeGen_RULEpcodeGen_OutputMessagesElement_layout;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_OutputMessagesElement_layout;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_OutputMessagesElement_layout */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputActualParam (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputActualParam (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_ListOutputActualParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputActualParam;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputActualParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputActualParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputActualParam;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputActualParam_formalParam) codeGen_MkcodeGen_ListOutputActualParam_formalParam (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputActualParam: root of subtree no. 1 can not be made a codeGen_ListOutputActualParam_formalParam node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ListOutputActualParam_expression) codeGen_MkcodeGen_ListOutputActualParam_expression (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputActualParam: root of subtree no. 2 can not be made a codeGen_ListOutputActualParam_expression node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputActualParam;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputActualParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputActualParam_formalParam (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputActualParam_formalParam (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputActualParam_formalParam _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputActualParam_formalParam;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputActualParam_formalParam) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputActualParam_formalParam));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputActualParam_formalParam;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputActualParam_formalParam;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputActualParam_formalParam */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5, NODEPTR _desc6, NODEPTR _desc7, NODEPTR _desc8, NODEPTR _desc9, NODEPTR _desc10, NODEPTR _desc11)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5,_desc6,_desc7,_desc8,_desc9,_desc10,_desc11)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
	NODEPTR _desc6;
	NODEPTR _desc7;
	NODEPTR _desc8;
	NODEPTR _desc9;
	NODEPTR _desc10;
	NODEPTR _desc11;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputElement_styleSheetBlock) codeGen_MkcodeGen_ListOutputElement_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 1 can not be made a codeGen_ListOutputElement_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ListOutputElement_name) codeGen_MkcodeGen_ListOutputElement_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 2 can not be made a codeGen_ListOutputElement_name node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_ListOutputElement_dataSourceActualParams) codeGen_MkcodeGen_ListOutputElement_dataSourceActualParams (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 3 can not be made a codeGen_ListOutputElement_dataSourceActualParams node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_ListOutputElement_dataSource) codeGen_MkcodeGen_ListOutputElement_dataSource (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 4 can not be made a codeGen_ListOutputElement_dataSource node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_ListOutputElement_maxNumberOfElements) codeGen_MkcodeGen_ListOutputElement_maxNumberOfElements (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 5 can not be made a codeGen_ListOutputElement_maxNumberOfElements node ", 0, _coordref);
_currn->_desc6 = (codeGen__TSPcodeGen_ListOutputElement_renderAs) codeGen_MkcodeGen_ListOutputElement_renderAs (_coordref, _desc6);	
if (((NODEPTR)_currn->_desc6) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 6 can not be made a codeGen_ListOutputElement_renderAs node ", 0, _coordref);
_currn->_desc7 = (codeGen__TSPcodeGen_ListOutputElement_navigationElement) codeGen_MkcodeGen_ListOutputElement_navigationElement (_coordref, _desc7);	
if (((NODEPTR)_currn->_desc7) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 7 can not be made a codeGen_ListOutputElement_navigationElement node ", 0, _coordref);
_currn->_desc8 = (codeGen__TSPcodeGen_ListOutputElement_columns) codeGen_MkcodeGen_ListOutputElement_columns (_coordref, _desc8);	
if (((NODEPTR)_currn->_desc8) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 8 can not be made a codeGen_ListOutputElement_columns node ", 0, _coordref);
_currn->_desc9 = (codeGen__TSPcodeGen_ListOutputElement_headerStyleClass) codeGen_MkcodeGen_ListOutputElement_headerStyleClass (_coordref, _desc9);	
if (((NODEPTR)_currn->_desc9) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 9 can not be made a codeGen_ListOutputElement_headerStyleClass node ", 0, _coordref);
_currn->_desc10 = (codeGen__TSPcodeGen_ListOutputElement_evenRowStyleClass) codeGen_MkcodeGen_ListOutputElement_evenRowStyleClass (_coordref, _desc10);	
if (((NODEPTR)_currn->_desc10) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 10 can not be made a codeGen_ListOutputElement_evenRowStyleClass node ", 0, _coordref);
_currn->_desc11 = (codeGen__TSPcodeGen_ListOutputElement_oddRowStyleClass) codeGen_MkcodeGen_ListOutputElement_oddRowStyleClass (_coordref, _desc11);	
if (((NODEPTR)_currn->_desc11) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputElement: root of subtree no. 11 can not be made a codeGen_ListOutputElement_oddRowStyleClass node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_name;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_name */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams;
#else
	_new = (codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams;
	_new->_desc1 = (codeGen__TSPcodeGen_DataSourceActualParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ListOutputElement_dataSourceActualParams)_new;
	{codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ListOutputElement_dataSourceActualParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ListOutputElement_dataSourceActualParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_DataSourceActualParam)) 
		return codeGen__MkcodeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams (_coordref, codeGen_MkcodeGen_DataSourceActualParam (_coordref, t));
	message (DEADLY, "a pcodeGen_ListOutputElement_dataSourceActualParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ListOutputElement_dataSourceActualParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ListOutputElement_dataSourceActualParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_dataSourceActualParams)
	l =_ElempcodeGen_ListOutputElement_dataSourceActualParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_dataSourceActualParams)
	r = _ElempcodeGen_ListOutputElement_dataSourceActualParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_dataSourceActualParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_dataSourceActualParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ListOutputElement_dataSourceActualParams root;
	codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_dataSourceActualParams)
		t = _ElempcodeGen_ListOutputElement_dataSourceActualParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)(((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ListOutputElement_dataSourceActualParams;
#else
list = (codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ListOutputElement_dataSourceActualParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ListOutputElement_dataSourceActualParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ListOutputElement_dataSourceActualParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams)
	{codeGen__TPPLST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_DataSourceActualParampcodeGen_ListOutputElement_dataSourceActualParams;
	} this code has been moved to _ElempcodeGen_ListOutputElement_dataSourceActualParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ListOutputElement_dataSourceActualParams;
#else
root = (codeGen__TPPpcodeGen_ListOutputElement_dataSourceActualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_dataSourceActualParams));
#endif
root->_prod = codeGen_RULEpcodeGen_ListOutputElement_dataSourceActualParams;
root->_desc1 = (codeGen__TSPLST_codeGen_ListOutputElement_dataSourceActualParams)list;
{ codeGen__TPPpcodeGen_ListOutputElement_dataSourceActualParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ListOutputElement_dataSourceActualParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_dataSource (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_dataSource (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_dataSource _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_dataSource;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_dataSource) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_dataSource));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_dataSource;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_dataSource;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_dataSource */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_maxNumberOfElements (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_maxNumberOfElements (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_maxNumberOfElements _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_maxNumberOfElements;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_maxNumberOfElements) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_maxNumberOfElements));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_maxNumberOfElements;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_maxNumberOfElements;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_maxNumberOfElements */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_renderAs (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_renderAs (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_renderAs _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_renderAs;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_renderAs) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_renderAs));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_renderAs;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_renderAs;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_renderAs */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_navigationElement (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_navigationElement (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_navigationElement _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_navigationElement;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_navigationElement) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_navigationElement));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_navigationElement;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_navigationElement;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_navigationElement */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ListOutputColumnpcodeGen_ListOutputElement_columns (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ListOutputColumnpcodeGen_ListOutputElement_columns (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns;
#else
	_new = (codeGen__TPPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns;
	_new->_desc1 = (codeGen__TSPcodeGen_ListOutputColumn)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ListOutputElement_columns)_new;
	{codeGen__TPPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ListOutputElement_columns (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ListOutputElement_columns (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ListOutputColumn)) 
		return codeGen__MkcodeGen_ListOutputColumnpcodeGen_ListOutputElement_columns (_coordref, codeGen_MkcodeGen_ListOutputColumn (_coordref, t));
	message (DEADLY, "a pcodeGen_ListOutputElement_columns list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ListOutputElement_columns (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ListOutputElement_columns (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_columns)
	l =_ElempcodeGen_ListOutputElement_columns (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_columns)
	r = _ElempcodeGen_ListOutputElement_columns (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_columns (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_columns (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ListOutputElement_columns root;
	codeGen__TPPLST_0pcodeGen_ListOutputElement_columns list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ListOutputElement_columns)
		t = _ElempcodeGen_ListOutputElement_columns (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)(((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ListOutputElement_columns) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ListOutputElement_columns;
#else
list = (codeGen__TPPLST_0pcodeGen_ListOutputElement_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ListOutputElement_columns));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ListOutputElement_columns;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ListOutputElement_columns)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns)
	{codeGen__TPPLST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ListOutputColumnpcodeGen_ListOutputElement_columns;
	} this code has been moved to _ElempcodeGen_ListOutputElement_columns*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ListOutputElement_columns;
#else
root = (codeGen__TPPpcodeGen_ListOutputElement_columns) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_columns));
#endif
root->_prod = codeGen_RULEpcodeGen_ListOutputElement_columns;
root->_desc1 = (codeGen__TSPLST_codeGen_ListOutputElement_columns)list;
{ codeGen__TPPpcodeGen_ListOutputElement_columns _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ListOutputElement_columns;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_headerStyleClass (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_headerStyleClass (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_headerStyleClass _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_headerStyleClass;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_headerStyleClass) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_headerStyleClass));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_headerStyleClass;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_headerStyleClass;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_headerStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_evenRowStyleClass (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_evenRowStyleClass (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_evenRowStyleClass _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_evenRowStyleClass;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_evenRowStyleClass) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_evenRowStyleClass));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_evenRowStyleClass;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_evenRowStyleClass;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_evenRowStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputElement_oddRowStyleClass (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputElement_oddRowStyleClass (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputElement_oddRowStyleClass _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputElement_oddRowStyleClass;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputElement_oddRowStyleClass) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputElement_oddRowStyleClass));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputElement_oddRowStyleClass;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputElement_oddRowStyleClass;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputElement_oddRowStyleClass */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputColumn (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputColumn (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_ListOutputColumn _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputColumn;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputColumn) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputColumn));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputColumn;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputColumn_header) codeGen_MkcodeGen_ListOutputColumn_header (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputColumn: root of subtree no. 1 can not be made a codeGen_ListOutputColumn_header node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ListOutputColumn_content) codeGen_MkcodeGen_ListOutputColumn_content (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputColumn: root of subtree no. 2 can not be made a codeGen_ListOutputColumn_content node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputColumn;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputColumn */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputColumn_header (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputColumn_header (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputColumn_header _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputColumn_header;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputColumn_header) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputColumn_header));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputColumn_header;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputColumn_header;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputColumn_header */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputConstantContent (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputConstantContent (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListOutputConstantContent _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputConstantContent;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputConstantContent) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputConstantContent));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputConstantContent;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputConstantContent_value) codeGen_MkcodeGen_ListOutputConstantContent_value (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputConstantContent: root of subtree no. 1 can not be made a codeGen_ListOutputConstantContent_value node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputConstantContent;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputConstantContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputConstantContent_value (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputConstantContent_value (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputConstantContent_value _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputConstantContent_value;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputConstantContent_value) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputConstantContent_value));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputConstantContent_value;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputConstantContent_value;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputConstantContent_value */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputRefContent (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputRefContent (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListOutputRefContent _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputRefContent;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputRefContent) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputRefContent));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputRefContent;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputRefContent_column) codeGen_MkcodeGen_ListOutputRefContent_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputRefContent: root of subtree no. 1 can not be made a codeGen_ListOutputRefContent_column node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputRefContent;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputRefContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputRefContent_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputRefContent_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputRefContent_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputRefContent_column;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputRefContent_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputRefContent_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputRefContent_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputRefContent_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputRefContent_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputProjectionContent (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputProjectionContent (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListOutputProjectionContent _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputProjectionContent;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputProjectionContent) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputProjectionContent));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputProjectionContent;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputProjectionContent_column) codeGen_MkcodeGen_ListOutputProjectionContent_column (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputProjectionContent: root of subtree no. 1 can not be made a codeGen_ListOutputProjectionContent_column node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputProjectionContent;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputProjectionContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputProjectionContent_column (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputProjectionContent_column (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputProjectionContent_column _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputProjectionContent_column;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputProjectionContent_column) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputProjectionContent_column));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputProjectionContent_column;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputProjectionContent_column;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputProjectionContent_column */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputNumerationContent (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputNumerationContent (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListOutputNumerationContent _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputNumerationContent;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputNumerationContent) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputNumerationContent));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputNumerationContent;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputNumerationContent_name) codeGen_MkcodeGen_ListOutputNumerationContent_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputNumerationContent: root of subtree no. 1 can not be made a codeGen_ListOutputNumerationContent_name node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputNumerationContent;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputNumerationContent */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputNumerationContent_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputNumerationContent_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ListOutputNumerationContent_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputNumerationContent_name;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputNumerationContent_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputNumerationContent_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputNumerationContent_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputNumerationContent_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputNumerationContent_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputActionConnectionsContent (POSITION *_coordref, int _TERM1, NODEPTR _desc1)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputActionConnectionsContent (_coordref, _TERM1,_desc1)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
#endif
{	codeGen__TPPpcodeGen_ListOutputActionConnectionsContent _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ListOutputActionConnectionsContent;
#else
_currn = (codeGen__TPPpcodeGen_ListOutputActionConnectionsContent) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputActionConnectionsContent));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ListOutputActionConnectionsContent;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputActionConnectionsContent_actionConnections) codeGen_MkcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ListOutputActionConnectionsContent: root of subtree no. 1 can not be made a codeGen_ListOutputActionConnectionsContent_actionConnections node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ListOutputActionConnectionsContent;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ListOutputActionConnectionsContent */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections;
#else
	_new = (codeGen__TPPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections;
	_new->_desc1 = (codeGen__TSPcodeGen_ActionConnection)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ListOutputActionConnectionsContent_actionConnections)_new;
	{codeGen__TPPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ListOutputActionConnectionsContent_actionConnections (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ActionConnection)) 
		return codeGen__MkcodeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, codeGen_MkcodeGen_ActionConnection (_coordref, t));
	message (DEADLY, "a pcodeGen_ListOutputActionConnectionsContent_actionConnections list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ListOutputActionConnectionsContent_actionConnections (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ListOutputActionConnectionsContent_actionConnections)
	l =_ElempcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ListOutputActionConnectionsContent_actionConnections)
	r = _ElempcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ListOutputActionConnectionsContent_actionConnections (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ListOutputActionConnectionsContent_actionConnections root;
	codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ListOutputActionConnectionsContent_actionConnections)
		t = _ElempcodeGen_ListOutputActionConnectionsContent_actionConnections (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)(((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections;
#else
list = (codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ListOutputActionConnectionsContent_actionConnections)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections)
	{codeGen__TPPLST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ActionConnectionpcodeGen_ListOutputActionConnectionsContent_actionConnections;
	} this code has been moved to _ElempcodeGen_ListOutputActionConnectionsContent_actionConnections*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ListOutputActionConnectionsContent_actionConnections;
#else
root = (codeGen__TPPpcodeGen_ListOutputActionConnectionsContent_actionConnections) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ListOutputActionConnectionsContent_actionConnections));
#endif
root->_prod = codeGen_RULEpcodeGen_ListOutputActionConnectionsContent_actionConnections;
root->_desc1 = (codeGen__TSPLST_codeGen_ListOutputActionConnectionsContent_actionConnections)list;
{ codeGen__TPPpcodeGen_ListOutputActionConnectionsContent_actionConnections _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ListOutputActionConnectionsContent_actionConnections;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionConnection (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_ActionConnection (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_ActionConnection _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionConnection;
#else
_currn = (codeGen__TPPpcodeGen_ActionConnection) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionConnection));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionConnection;
_currn->_desc1 = (codeGen__TSPcodeGen_ActionConnection_action) codeGen_MkcodeGen_ActionConnection_action (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionConnection: root of subtree no. 1 can not be made a codeGen_ActionConnection_action node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ActionConnection_imageName) codeGen_MkcodeGen_ActionConnection_imageName (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionConnection: root of subtree no. 2 can not be made a codeGen_ActionConnection_imageName node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_ActionConnection_text) codeGen_MkcodeGen_ActionConnection_text (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionConnection: root of subtree no. 3 can not be made a codeGen_ActionConnection_text node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_ActionConnection_actualParams) codeGen_MkcodeGen_ActionConnection_actualParams (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ActionConnection: root of subtree no. 4 can not be made a codeGen_ActionConnection_actualParams node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionConnection;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionConnection */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionConnection_action (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ActionConnection_action (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ActionConnection_action _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionConnection_action;
#else
_currn = (codeGen__TPPpcodeGen_ActionConnection_action) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionConnection_action));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionConnection_action;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionConnection_action;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionConnection_action */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionConnection_imageName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ActionConnection_imageName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ActionConnection_imageName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionConnection_imageName;
#else
_currn = (codeGen__TPPpcodeGen_ActionConnection_imageName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionConnection_imageName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionConnection_imageName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionConnection_imageName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionConnection_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionConnection_text (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ActionConnection_text (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ActionConnection_text _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ActionConnection_text;
#else
_currn = (codeGen__TPPpcodeGen_ActionConnection_text) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionConnection_text));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ActionConnection_text;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ActionConnection_text;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ActionConnection_text */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams;
#else
	_new = (codeGen__TPPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams));
#endif
	_new->_prod = codeGen_RULELST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams;
	_new->_desc1 = (codeGen__TSPcodeGen_ListOutputActualParam)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ActionConnection_actualParams)_new;
	{codeGen__TPPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ActionConnection_actualParams (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ActionConnection_actualParams (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_ListOutputActualParam)) 
		return codeGen__MkcodeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams (_coordref, codeGen_MkcodeGen_ListOutputActualParam (_coordref, t));
	message (DEADLY, "a pcodeGen_ActionConnection_actualParams list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ActionConnection_actualParams (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ActionConnection_actualParams (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ActionConnection_actualParams)
	l =_ElempcodeGen_ActionConnection_actualParams (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ActionConnection_actualParams)
	r = _ElempcodeGen_ActionConnection_actualParams (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ActionConnection_actualParams (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ActionConnection_actualParams (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ActionConnection_actualParams root;
	codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ActionConnection_actualParams)
		t = _ElempcodeGen_ActionConnection_actualParams (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)(((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ActionConnection_actualParams;
#else
list = (codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ActionConnection_actualParams));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ActionConnection_actualParams;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ActionConnection_actualParams)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams)
	{codeGen__TPPLST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_ListOutputActualParampcodeGen_ActionConnection_actualParams;
	} this code has been moved to _ElempcodeGen_ActionConnection_actualParams*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ActionConnection_actualParams;
#else
root = (codeGen__TPPpcodeGen_ActionConnection_actualParams) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ActionConnection_actualParams));
#endif
root->_prod = codeGen_RULEpcodeGen_ActionConnection_actualParams;
root->_desc1 = (codeGen__TSPLST_codeGen_ActionConnection_actualParams)list;
{ codeGen__TPPpcodeGen_ActionConnection_actualParams _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ActionConnection_actualParams;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2)
#else
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern (_coordref, _TERM1,_desc1,_desc2)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
#endif
{	codeGen__TPPpcodeGen_LinkTargetExtern _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LinkTargetExtern;
#else
_currn = (codeGen__TPPpcodeGen_LinkTargetExtern) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LinkTargetExtern));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LinkTargetExtern;
_currn->_desc1 = (codeGen__TSPcodeGen_LinkTargetExtern_name) codeGen_MkcodeGen_LinkTargetExtern_name (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LinkTargetExtern: root of subtree no. 1 can not be made a codeGen_LinkTargetExtern_name node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_LinkTargetExtern_url) codeGen_MkcodeGen_LinkTargetExtern_url (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_LinkTargetExtern: root of subtree no. 2 can not be made a codeGen_LinkTargetExtern_url node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LinkTargetExtern;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LinkTargetExtern */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LinkTargetExtern_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LinkTargetExtern_name;
#else
_currn = (codeGen__TPPpcodeGen_LinkTargetExtern_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LinkTargetExtern_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LinkTargetExtern_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LinkTargetExtern_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LinkTargetExtern_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern_url (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_LinkTargetExtern_url (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_LinkTargetExtern_url _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_LinkTargetExtern_url;
#else
_currn = (codeGen__TPPpcodeGen_LinkTargetExtern_url) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_LinkTargetExtern_url));
#endif
_currn->_prod = codeGen_RULEpcodeGen_LinkTargetExtern_url;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_LinkTargetExtern_url;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_LinkTargetExtern_url */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4, NODEPTR _desc5)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4,_desc5)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
	NODEPTR _desc5;
#endif
{	codeGen__TPPpcodeGen_ImageMap _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageMap;
#else
_currn = (codeGen__TPPpcodeGen_ImageMap) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageMap;
_currn->_desc1 = (codeGen__TSPcodeGen_ImageMap_styleSheetBlock) codeGen_MkcodeGen_ImageMap_styleSheetBlock (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageMap: root of subtree no. 1 can not be made a codeGen_ImageMap_styleSheetBlock node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_ImageMap_name) codeGen_MkcodeGen_ImageMap_name (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageMap: root of subtree no. 2 can not be made a codeGen_ImageMap_name node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_ImageMap_imageName) codeGen_MkcodeGen_ImageMap_imageName (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageMap: root of subtree no. 3 can not be made a codeGen_ImageMap_imageName node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_ImageMap_altText) codeGen_MkcodeGen_ImageMap_altText (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageMap: root of subtree no. 4 can not be made a codeGen_ImageMap_altText node ", 0, _coordref);
_currn->_desc5 = (codeGen__TSPcodeGen_ImageMap_mapAreas) codeGen_MkcodeGen_ImageMap_mapAreas (_coordref, _desc5);	
if (((NODEPTR)_currn->_desc5) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_ImageMap: root of subtree no. 5 can not be made a codeGen_ImageMap_mapAreas node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageMap;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageMap */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap_styleSheetBlock (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap_styleSheetBlock (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageMap_styleSheetBlock _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageMap_styleSheetBlock;
#else
_currn = (codeGen__TPPpcodeGen_ImageMap_styleSheetBlock) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap_styleSheetBlock));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageMap_styleSheetBlock;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageMap_styleSheetBlock;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageMap_styleSheetBlock */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap_name (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap_name (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageMap_name _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageMap_name;
#else
_currn = (codeGen__TPPpcodeGen_ImageMap_name) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap_name));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageMap_name;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageMap_name;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageMap_name */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap_imageName (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap_imageName (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageMap_imageName _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageMap_imageName;
#else
_currn = (codeGen__TPPpcodeGen_ImageMap_imageName) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap_imageName));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageMap_imageName;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageMap_imageName;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageMap_imageName */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap_altText (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap_altText (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_ImageMap_altText _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_ImageMap_altText;
#else
_currn = (codeGen__TPPpcodeGen_ImageMap_altText) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap_altText));
#endif
_currn->_prod = codeGen_RULEpcodeGen_ImageMap_altText;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_ImageMap_altText;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_ImageMap_altText */

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR codeGen__MkcodeGen_MapAreapcodeGen_ImageMap_mapAreas (POSITION *_coordref, NODEPTR elem)
#else
static NODEPTR codeGen__MkcodeGen_MapAreapcodeGen_ImageMap_mapAreas (_coordref, elem)
	POSITION *_coordref; NODEPTR elem;
#endif
{
	codeGen__TPPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas _new;
#ifdef __cplusplus
	_new = new codeGen__TPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas;
#else
	_new = (codeGen__TPPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas));
#endif
	_new->_prod = codeGen_RULELST_codeGen_MapAreapcodeGen_ImageMap_mapAreas;
	_new->_desc1 = (codeGen__TSPcodeGen_MapArea)elem;
	_new->_desc2 = (codeGen__TSPLST_codeGen_ImageMap_mapAreas)_new;
	{codeGen__TPPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas _currn = _new;
	 _SETCOORD(_currn)
	 codeGen__TERMACT_LST_codeGen_MapAreapcodeGen_ImageMap_mapAreas;
	}
	return (NODEPTR)_new;
}

#if defined(__STDC__) || defined(__cplusplus)
static NODEPTR _ElempcodeGen_ImageMap_mapAreas (POSITION *_coordref, NODEPTR t)
#else
static NODEPTR _ElempcodeGen_ImageMap_mapAreas (_coordref, t)
	POSITION *_coordref; NODEPTR t;
#endif
/* coerces t to a list element and returns a single circular list */
{	NODEPTR elem;
	/* check first without then with coercion: */
	if (codeGen_IsSymb (t, codeGen_SYMBcodeGen_MapArea)) 
		return codeGen__MkcodeGen_MapAreapcodeGen_ImageMap_mapAreas (_coordref, codeGen_MkcodeGen_MapArea (_coordref, t));
	message (DEADLY, "a pcodeGen_ImageMap_mapAreas list element node can not be generated ",
		0, _coordref);
	return t; /* never reached */
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mk2pcodeGen_ImageMap_mapAreas (POSITION *_coordref, NODEPTR l, NODEPTR r)
#else
NODEPTR codeGen_Mk2pcodeGen_ImageMap_mapAreas (_coordref, l, r) POSITION *_coordref; NODEPTR l, r;
#endif
/* make a list of l and r, which may be null, elements, or lists */
{	NODEPTR last;
if (l == NULLNODEPTR) return r;
if (r == NULLNODEPTR) return l;
/* ensure l and r to be lists: */
if (codeGen_LHSMAP[l->_prod] != codeGen_SYMBLST_codeGen_ImageMap_mapAreas)
	l =_ElempcodeGen_ImageMap_mapAreas (_coordref, l);
if (codeGen_LHSMAP[r->_prod] != codeGen_SYMBLST_codeGen_ImageMap_mapAreas)
	r = _ElempcodeGen_ImageMap_mapAreas (_coordref, r);
/* concatenate two lists: */
last = ((NODEPTR)((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)r)->_desc2);
((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)r)->_desc2 = ((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)l)->_desc2;
((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)l)->_desc2 = last;
return l;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_ImageMap_mapAreas (POSITION *_coordref, NODEPTR t)
#else
NODEPTR codeGen_MkpcodeGen_ImageMap_mapAreas (_coordref, t) POSITION *_coordref; NODEPTR t;
#endif
{	NODEPTR _currn; codeGen__TPPpcodeGen_ImageMap_mapAreas root;
	codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas list, remain, p;
/* argument t may be empty, a list element, or a list: */
if (t == NULLNODEPTR)
	remain = (codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)NULLNODEPTR;
else {	if (codeGen_LHSMAP[t->_prod] != codeGen_SYMBLST_codeGen_ImageMap_mapAreas)
		t = _ElempcodeGen_ImageMap_mapAreas (_coordref, t);
	remain = (codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)(((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas) t)->_desc2);
	((codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas) t)->_desc2 = NULLNODEPTR;
}
/* remain is a possibly empty list in reverse order */
/* finalize the list by an empty context: */
#ifdef __cplusplus
list = new codeGen__TPLST_0pcodeGen_ImageMap_mapAreas;
#else
list = (codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPLST_0pcodeGen_ImageMap_mapAreas));
#endif
list->_prod = codeGen_RULELST_0pcodeGen_ImageMap_mapAreas;
list->_desc1 = NULLNODEPTR;
list->_desc2 = NULLNODEPTR;
_SETCOORD(list)
_currn = (NODEPTR)list;
/* reverse the list and visit its nodes: */
while (remain) {
	p = (codeGen__TPPLST_0pcodeGen_ImageMap_mapAreas)(remain->_desc2);
	remain->_desc2 = (NODEPTR)list;
	list = remain;
	remain = p;
	_currn = (NODEPTR)list;
	/*if (_currn->_prod == codeGen_RULELST_codeGen_MapAreapcodeGen_ImageMap_mapAreas)
	{codeGen__TPPLST_codeGen_MapAreapcodeGen_ImageMap_mapAreas _currn = _currn;
	 codeGen__TERMACT_LST_codeGen_MapAreapcodeGen_ImageMap_mapAreas;
	} this code has been moved to _ElempcodeGen_ImageMap_mapAreas*/
}
/* make root node and visit it: */
#ifdef __cplusplus
root = new codeGen__TPpcodeGen_ImageMap_mapAreas;
#else
root = (codeGen__TPPpcodeGen_ImageMap_mapAreas) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_ImageMap_mapAreas));
#endif
root->_prod = codeGen_RULEpcodeGen_ImageMap_mapAreas;
root->_desc1 = (codeGen__TSPLST_codeGen_ImageMap_mapAreas)list;
{ codeGen__TPPpcodeGen_ImageMap_mapAreas _currn = root;
  _SETCOORD(_currn)
  codeGen__TERMACT_pcodeGen_ImageMap_mapAreas;
}
_currn = (NODEPTR)root;
return (NODEPTR)_currn;
}

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MapArea (POSITION *_coordref, int _TERM1, NODEPTR _desc1, NODEPTR _desc2, NODEPTR _desc3, NODEPTR _desc4)
#else
NODEPTR codeGen_MkpcodeGen_MapArea (_coordref, _TERM1,_desc1,_desc2,_desc3,_desc4)
	POSITION *_coordref;
	int _TERM1;
	NODEPTR _desc1;
	NODEPTR _desc2;
	NODEPTR _desc3;
	NODEPTR _desc4;
#endif
{	codeGen__TPPpcodeGen_MapArea _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MapArea;
#else
_currn = (codeGen__TPPpcodeGen_MapArea) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MapArea));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MapArea;
_currn->_desc1 = (codeGen__TSPcodeGen_MapArea_altText) codeGen_MkcodeGen_MapArea_altText (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MapArea: root of subtree no. 1 can not be made a codeGen_MapArea_altText node ", 0, _coordref);
_currn->_desc2 = (codeGen__TSPcodeGen_MapArea_coords) codeGen_MkcodeGen_MapArea_coords (_coordref, _desc2);	
if (((NODEPTR)_currn->_desc2) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MapArea: root of subtree no. 2 can not be made a codeGen_MapArea_coords node ", 0, _coordref);
_currn->_desc3 = (codeGen__TSPcodeGen_MapArea_link) codeGen_MkcodeGen_MapArea_link (_coordref, _desc3);	
if (((NODEPTR)_currn->_desc3) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MapArea: root of subtree no. 3 can not be made a codeGen_MapArea_link node ", 0, _coordref);
_currn->_desc4 = (codeGen__TSPcodeGen_MapArea_queryString) codeGen_MkcodeGen_MapArea_queryString (_coordref, _desc4);	
if (((NODEPTR)_currn->_desc4) == NULLNODEPTR)	
	message (DEADLY, "RULE pcodeGen_MapArea: root of subtree no. 4 can not be made a codeGen_MapArea_queryString node ", 0, _coordref);
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MapArea;
#ifdef MONITOR
_dapto_term_int(((NODEPTR)_currn), "ID", _TERM1);
#endif

return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MapArea */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MapArea_altText (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MapArea_altText (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MapArea_altText _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MapArea_altText;
#else
_currn = (codeGen__TPPpcodeGen_MapArea_altText) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MapArea_altText));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MapArea_altText;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MapArea_altText;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MapArea_altText */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MapArea_coords (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MapArea_coords (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MapArea_coords _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MapArea_coords;
#else
_currn = (codeGen__TPPpcodeGen_MapArea_coords) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MapArea_coords));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MapArea_coords;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MapArea_coords;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MapArea_coords */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MapArea_link (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MapArea_link (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MapArea_link _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MapArea_link;
#else
_currn = (codeGen__TPPpcodeGen_MapArea_link) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MapArea_link));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MapArea_link;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MapArea_link;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MapArea_link */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_MkpcodeGen_MapArea_queryString (POSITION *_coordref)
#else
NODEPTR codeGen_MkpcodeGen_MapArea_queryString (_coordref)
	POSITION *_coordref;
#endif
{	codeGen__TPPpcodeGen_MapArea_queryString _currn;
#ifdef __cplusplus
_currn = new codeGen__TPpcodeGen_MapArea_queryString;
#else
_currn = (codeGen__TPPpcodeGen_MapArea_queryString) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPpcodeGen_MapArea_queryString));
#endif
_currn->_prod = codeGen_RULEpcodeGen_MapArea_queryString;
_SETCOORD(_currn)
codeGen__TERMACT_pcodeGen_MapArea_queryString;
return ( (NODEPTR) _currn);
}/* codeGen_MkpcodeGen_MapArea_queryString */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_1 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_1 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_1 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_1;
#else
_currn = (codeGen__TPPrule_1) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_1));
#endif
_currn->_prod = codeGen_RULErule_1;
_currn->_desc1 = (codeGen__TSPcodeGen_WebApplication) codeGen_MkcodeGen_WebApplication (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_1: root of subtree no. 1 can not be made a codeGen_WebApplication node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_1;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_1 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_2 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_2 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_2 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_2;
#else
_currn = (codeGen__TPPrule_2) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_2));
#endif
_currn->_prod = codeGen_RULErule_2;
_currn->_desc1 = (codeGen__TSPcodeGen_RootWebZone) codeGen_MkcodeGen_RootWebZone (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_2: root of subtree no. 1 can not be made a codeGen_RootWebZone node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_2;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_2 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_3 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_3 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_3 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_3;
#else
_currn = (codeGen__TPPrule_3) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_3));
#endif
_currn->_prod = codeGen_RULErule_3;
_currn->_desc1 = (codeGen__TSPcodeGen_NavigationBar) codeGen_MkcodeGen_NavigationBar (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_3: root of subtree no. 1 can not be made a codeGen_NavigationBar node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_3;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_3 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_4 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_4 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_4 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_4;
#else
_currn = (codeGen__TPPrule_4) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_4));
#endif
_currn->_prod = codeGen_RULErule_4;
_currn->_desc1 = (codeGen__TSPcodeGen_StyleSheet) codeGen_MkcodeGen_StyleSheet (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_4: root of subtree no. 1 can not be made a codeGen_StyleSheet node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_4;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_4 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_5 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_5 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_5 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_5;
#else
_currn = (codeGen__TPPrule_5) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_5));
#endif
_currn->_prod = codeGen_RULErule_5;
_currn->_desc1 = (codeGen__TSPcodeGen_MetaInformation) codeGen_MkcodeGen_MetaInformation (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_5: root of subtree no. 1 can not be made a codeGen_MetaInformation node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_5;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_5 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_6 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_6 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_6 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_6;
#else
_currn = (codeGen__TPPrule_6) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_6));
#endif
_currn->_prod = codeGen_RULErule_6;
_currn->_desc1 = (codeGen__TSPcodeGen_DataBase) codeGen_MkcodeGen_DataBase (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_6: root of subtree no. 1 can not be made a codeGen_DataBase node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_6;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_6 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_7 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_7 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_7 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_7;
#else
_currn = (codeGen__TPPrule_7) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_7));
#endif
_currn->_prod = codeGen_RULErule_7;
_currn->_desc1 = (codeGen__TSPcodeGen_Frame) codeGen_MkcodeGen_Frame (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_7: root of subtree no. 1 can not be made a codeGen_Frame node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_7;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_7 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_8 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_8 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_8 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_8;
#else
_currn = (codeGen__TPPrule_8) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_8));
#endif
_currn->_prod = codeGen_RULErule_8;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBlob) codeGen_MkcodeGen_DBBlob (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_8: root of subtree no. 1 can not be made a codeGen_DBBlob node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_8;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_8 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_9 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_9 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_9 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_9;
#else
_currn = (codeGen__TPPrule_9) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_9));
#endif
_currn->_prod = codeGen_RULErule_9;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBoolean) codeGen_MkcodeGen_DBBoolean (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_9: root of subtree no. 1 can not be made a codeGen_DBBoolean node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_9;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_9 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_10 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_10 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_10 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_10;
#else
_currn = (codeGen__TPPrule_10) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_10));
#endif
_currn->_prod = codeGen_RULErule_10;
_currn->_desc1 = (codeGen__TSPcodeGen_DBTimeStamp) codeGen_MkcodeGen_DBTimeStamp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_10: root of subtree no. 1 can not be made a codeGen_DBTimeStamp node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_10;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_10 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_11 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_11 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_11 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_11;
#else
_currn = (codeGen__TPPrule_11) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_11));
#endif
_currn->_prod = codeGen_RULErule_11;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDate) codeGen_MkcodeGen_DBDate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_11: root of subtree no. 1 can not be made a codeGen_DBDate node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_11;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_11 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_12 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_12 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_12 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_12;
#else
_currn = (codeGen__TPPrule_12) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_12));
#endif
_currn->_prod = codeGen_RULErule_12;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBigInt) codeGen_MkcodeGen_DBBigInt (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_12: root of subtree no. 1 can not be made a codeGen_DBBigInt node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_12;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_12 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_13 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_13 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_13 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_13;
#else
_currn = (codeGen__TPPrule_13) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_13));
#endif
_currn->_prod = codeGen_RULErule_13;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDouble) codeGen_MkcodeGen_DBDouble (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_13: root of subtree no. 1 can not be made a codeGen_DBDouble node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_13;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_13 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_14 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_14 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_14 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_14;
#else
_currn = (codeGen__TPPrule_14) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_14));
#endif
_currn->_prod = codeGen_RULErule_14;
_currn->_desc1 = (codeGen__TSPcodeGen_DBFloat) codeGen_MkcodeGen_DBFloat (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_14: root of subtree no. 1 can not be made a codeGen_DBFloat node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_14;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_14 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_15 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_15 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_15 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_15;
#else
_currn = (codeGen__TPPrule_15) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_15));
#endif
_currn->_prod = codeGen_RULErule_15;
_currn->_desc1 = (codeGen__TSPcodeGen_DBLong) codeGen_MkcodeGen_DBLong (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_15: root of subtree no. 1 can not be made a codeGen_DBLong node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_15;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_15 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_16 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_16 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_16 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_16;
#else
_currn = (codeGen__TPPrule_16) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_16));
#endif
_currn->_prod = codeGen_RULErule_16;
_currn->_desc1 = (codeGen__TSPcodeGen_DBInteger) codeGen_MkcodeGen_DBInteger (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_16: root of subtree no. 1 can not be made a codeGen_DBInteger node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_16;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_16 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_17 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_17 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_17 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_17;
#else
_currn = (codeGen__TPPrule_17) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_17));
#endif
_currn->_prod = codeGen_RULErule_17;
_currn->_desc1 = (codeGen__TSPcodeGen_DBText) codeGen_MkcodeGen_DBText (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_17: root of subtree no. 1 can not be made a codeGen_DBText node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_17;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_17 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_18 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_18 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_18 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_18;
#else
_currn = (codeGen__TPPrule_18) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_18));
#endif
_currn->_prod = codeGen_RULErule_18;
_currn->_desc1 = (codeGen__TSPcodeGen_DBString) codeGen_MkcodeGen_DBString (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_18: root of subtree no. 1 can not be made a codeGen_DBString node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_18;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_18 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_19 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_19 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_19 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_19;
#else
_currn = (codeGen__TPPrule_19) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_19));
#endif
_currn->_prod = codeGen_RULErule_19;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasMany) codeGen_MkcodeGen_DBHasMany (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_19: root of subtree no. 1 can not be made a codeGen_DBHasMany node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_19;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_19 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_20 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_20 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_20 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_20;
#else
_currn = (codeGen__TPPrule_20) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_20));
#endif
_currn->_prod = codeGen_RULErule_20;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasOne) codeGen_MkcodeGen_DBHasOne (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_20: root of subtree no. 1 can not be made a codeGen_DBHasOne node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_20;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_20 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_21 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_21 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_21 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_21;
#else
_currn = (codeGen__TPPrule_21) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_21));
#endif
_currn->_prod = codeGen_RULErule_21;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_21: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_21;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_21 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_22 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_22 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_22 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_22;
#else
_currn = (codeGen__TPPrule_22) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_22));
#endif
_currn->_prod = codeGen_RULErule_22;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_22: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_22;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_22 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_23 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_23 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_23 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_23;
#else
_currn = (codeGen__TPPrule_23) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_23));
#endif
_currn->_prod = codeGen_RULErule_23;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_23: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_23;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_23 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_24 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_24 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_24 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_24;
#else
_currn = (codeGen__TPPrule_24) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_24));
#endif
_currn->_prod = codeGen_RULErule_24;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_24: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_24;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_24 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_25 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_25 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_25 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_25;
#else
_currn = (codeGen__TPPrule_25) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_25));
#endif
_currn->_prod = codeGen_RULErule_25;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_25: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_25;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_25 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_26 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_26 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_26 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_26;
#else
_currn = (codeGen__TPPrule_26) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_26));
#endif
_currn->_prod = codeGen_RULErule_26;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_26: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_26;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_26 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_27 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_27 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_27 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_27;
#else
_currn = (codeGen__TPPrule_27) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_27));
#endif
_currn->_prod = codeGen_RULErule_27;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_27: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_27;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_27 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_28 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_28 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_28 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_28;
#else
_currn = (codeGen__TPPrule_28) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_28));
#endif
_currn->_prod = codeGen_RULErule_28;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_28: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_28;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_28 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_29 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_29 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_29 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_29;
#else
_currn = (codeGen__TPPrule_29) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_29));
#endif
_currn->_prod = codeGen_RULErule_29;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_29: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_29;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_29 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_30 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_30 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_30 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_30;
#else
_currn = (codeGen__TPPrule_30) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_30));
#endif
_currn->_prod = codeGen_RULErule_30;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_30: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_30;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_30 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_31 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_31 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_31 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_31;
#else
_currn = (codeGen__TPPrule_31) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_31));
#endif
_currn->_prod = codeGen_RULErule_31;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_31: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_31;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_31 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_32 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_32 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_32 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_32;
#else
_currn = (codeGen__TPPrule_32) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_32));
#endif
_currn->_prod = codeGen_RULErule_32;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_32: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_32;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_32 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_33 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_33 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_33 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_33;
#else
_currn = (codeGen__TPPrule_33) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_33));
#endif
_currn->_prod = codeGen_RULErule_33;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_33: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_33;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_33 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_34 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_34 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_34 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_34;
#else
_currn = (codeGen__TPPrule_34) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_34));
#endif
_currn->_prod = codeGen_RULErule_34;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_34: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_34;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_34 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_35 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_35 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_35 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_35;
#else
_currn = (codeGen__TPPrule_35) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_35));
#endif
_currn->_prod = codeGen_RULErule_35;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_35: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_35;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_35 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_36 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_36 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_36 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_36;
#else
_currn = (codeGen__TPPrule_36) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_36));
#endif
_currn->_prod = codeGen_RULErule_36;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_36: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_36;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_36 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_37 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_37 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_37 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_37;
#else
_currn = (codeGen__TPPrule_37) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_37));
#endif
_currn->_prod = codeGen_RULErule_37;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_37: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_37;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_37 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_38 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_38 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_38 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_38;
#else
_currn = (codeGen__TPPrule_38) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_38));
#endif
_currn->_prod = codeGen_RULErule_38;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_38: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_38;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_38 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_39 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_39 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_39 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_39;
#else
_currn = (codeGen__TPPrule_39) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_39));
#endif
_currn->_prod = codeGen_RULErule_39;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_39: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_39;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_39 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_40 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_40 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_40 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_40;
#else
_currn = (codeGen__TPPrule_40) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_40));
#endif
_currn->_prod = codeGen_RULErule_40;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_40: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_40;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_40 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_41 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_41 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_41 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_41;
#else
_currn = (codeGen__TPPrule_41) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_41));
#endif
_currn->_prod = codeGen_RULErule_41;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_41: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_41;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_41 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_42 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_42 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_42 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_42;
#else
_currn = (codeGen__TPPrule_42) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_42));
#endif
_currn->_prod = codeGen_RULErule_42;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_42: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_42;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_42 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_43 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_43 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_43 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_43;
#else
_currn = (codeGen__TPPrule_43) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_43));
#endif
_currn->_prod = codeGen_RULErule_43;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_43: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_43;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_43 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_44 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_44 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_44 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_44;
#else
_currn = (codeGen__TPPrule_44) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_44));
#endif
_currn->_prod = codeGen_RULErule_44;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_44: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_44;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_44 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_45 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_45 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_45 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_45;
#else
_currn = (codeGen__TPPrule_45) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_45));
#endif
_currn->_prod = codeGen_RULErule_45;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_45: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_45;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_45 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_46 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_46 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_46 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_46;
#else
_currn = (codeGen__TPPrule_46) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_46));
#endif
_currn->_prod = codeGen_RULErule_46;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_46: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_46;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_46 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_47 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_47 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_47 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_47;
#else
_currn = (codeGen__TPPrule_47) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_47));
#endif
_currn->_prod = codeGen_RULErule_47;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_47: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_47;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_47 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_48 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_48 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_48 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_48;
#else
_currn = (codeGen__TPPrule_48) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_48));
#endif
_currn->_prod = codeGen_RULErule_48;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_48: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_48;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_48 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_49 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_49 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_49 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_49;
#else
_currn = (codeGen__TPPrule_49) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_49));
#endif
_currn->_prod = codeGen_RULErule_49;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_49: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_49;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_49 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_50 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_50 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_50 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_50;
#else
_currn = (codeGen__TPPrule_50) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_50));
#endif
_currn->_prod = codeGen_RULErule_50;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_50: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_50;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_50 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_51 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_51 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_51 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_51;
#else
_currn = (codeGen__TPPrule_51) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_51));
#endif
_currn->_prod = codeGen_RULErule_51;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_51: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_51;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_51 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_52 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_52 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_52 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_52;
#else
_currn = (codeGen__TPPrule_52) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_52));
#endif
_currn->_prod = codeGen_RULErule_52;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_52: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_52;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_52 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_53 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_53 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_53 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_53;
#else
_currn = (codeGen__TPPrule_53) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_53));
#endif
_currn->_prod = codeGen_RULErule_53;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_53: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_53;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_53 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_54 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_54 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_54 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_54;
#else
_currn = (codeGen__TPPrule_54) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_54));
#endif
_currn->_prod = codeGen_RULErule_54;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_54: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_54;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_54 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_55 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_55 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_55 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_55;
#else
_currn = (codeGen__TPPrule_55) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_55));
#endif
_currn->_prod = codeGen_RULErule_55;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_55: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_55;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_55 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_56 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_56 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_56 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_56;
#else
_currn = (codeGen__TPPrule_56) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_56));
#endif
_currn->_prod = codeGen_RULErule_56;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_56: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_56;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_56 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_57 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_57 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_57 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_57;
#else
_currn = (codeGen__TPPrule_57) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_57));
#endif
_currn->_prod = codeGen_RULErule_57;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_57: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_57;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_57 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_58 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_58 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_58 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_58;
#else
_currn = (codeGen__TPPrule_58) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_58));
#endif
_currn->_prod = codeGen_RULErule_58;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_58: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_58;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_58 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_59 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_59 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_59 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_59;
#else
_currn = (codeGen__TPPrule_59) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_59));
#endif
_currn->_prod = codeGen_RULErule_59;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_59: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_59;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_59 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_60 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_60 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_60 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_60;
#else
_currn = (codeGen__TPPrule_60) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_60));
#endif
_currn->_prod = codeGen_RULErule_60;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_60: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_60;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_60 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_61 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_61 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_61 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_61;
#else
_currn = (codeGen__TPPrule_61) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_61));
#endif
_currn->_prod = codeGen_RULErule_61;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_61: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_61;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_61 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_62 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_62 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_62 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_62;
#else
_currn = (codeGen__TPPrule_62) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_62));
#endif
_currn->_prod = codeGen_RULErule_62;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_62: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_62;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_62 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_63 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_63 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_63 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_63;
#else
_currn = (codeGen__TPPrule_63) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_63));
#endif
_currn->_prod = codeGen_RULErule_63;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_63: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_63;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_63 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_64 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_64 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_64 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_64;
#else
_currn = (codeGen__TPPrule_64) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_64));
#endif
_currn->_prod = codeGen_RULErule_64;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_64: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_64;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_64 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_65 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_65 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_65 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_65;
#else
_currn = (codeGen__TPPrule_65) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_65));
#endif
_currn->_prod = codeGen_RULErule_65;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_65: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_65;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_65 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_66 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_66 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_66 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_66;
#else
_currn = (codeGen__TPPrule_66) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_66));
#endif
_currn->_prod = codeGen_RULErule_66;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_66: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_66;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_66 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_67 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_67 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_67 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_67;
#else
_currn = (codeGen__TPPrule_67) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_67));
#endif
_currn->_prod = codeGen_RULErule_67;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_67: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_67;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_67 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_68 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_68 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_68 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_68;
#else
_currn = (codeGen__TPPrule_68) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_68));
#endif
_currn->_prod = codeGen_RULErule_68;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_68: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_68;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_68 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_69 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_69 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_69 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_69;
#else
_currn = (codeGen__TPPrule_69) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_69));
#endif
_currn->_prod = codeGen_RULErule_69;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_69: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_69;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_69 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_70 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_70 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_70 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_70;
#else
_currn = (codeGen__TPPrule_70) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_70));
#endif
_currn->_prod = codeGen_RULErule_70;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_70: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_70;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_70 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_71 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_71 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_71 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_71;
#else
_currn = (codeGen__TPPrule_71) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_71));
#endif
_currn->_prod = codeGen_RULErule_71;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_71: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_71;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_71 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_72 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_72 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_72 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_72;
#else
_currn = (codeGen__TPPrule_72) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_72));
#endif
_currn->_prod = codeGen_RULErule_72;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_72: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_72;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_72 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_73 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_73 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_73 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_73;
#else
_currn = (codeGen__TPPrule_73) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_73));
#endif
_currn->_prod = codeGen_RULErule_73;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_73: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_73;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_73 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_74 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_74 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_74 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_74;
#else
_currn = (codeGen__TPPrule_74) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_74));
#endif
_currn->_prod = codeGen_RULErule_74;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_74: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_74;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_74 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_75 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_75 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_75 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_75;
#else
_currn = (codeGen__TPPrule_75) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_75));
#endif
_currn->_prod = codeGen_RULErule_75;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_75: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_75;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_75 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_76 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_76 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_76 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_76;
#else
_currn = (codeGen__TPPrule_76) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_76));
#endif
_currn->_prod = codeGen_RULErule_76;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_76: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_76;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_76 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_77 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_77 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_77 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_77;
#else
_currn = (codeGen__TPPrule_77) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_77));
#endif
_currn->_prod = codeGen_RULErule_77;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_77: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_77;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_77 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_78 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_78 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_78 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_78;
#else
_currn = (codeGen__TPPrule_78) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_78));
#endif
_currn->_prod = codeGen_RULErule_78;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_78: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_78;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_78 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_79 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_79 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_79 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_79;
#else
_currn = (codeGen__TPPrule_79) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_79));
#endif
_currn->_prod = codeGen_RULErule_79;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_79: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_79;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_79 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_80 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_80 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_80 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_80;
#else
_currn = (codeGen__TPPrule_80) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_80));
#endif
_currn->_prod = codeGen_RULErule_80;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_80: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_80;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_80 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_81 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_81 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_81 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_81;
#else
_currn = (codeGen__TPPrule_81) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_81));
#endif
_currn->_prod = codeGen_RULErule_81;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_81: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_81;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_81 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_82 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_82 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_82 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_82;
#else
_currn = (codeGen__TPPrule_82) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_82));
#endif
_currn->_prod = codeGen_RULErule_82;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_82: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_82;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_82 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_83 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_83 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_83 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_83;
#else
_currn = (codeGen__TPPrule_83) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_83));
#endif
_currn->_prod = codeGen_RULErule_83;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_83: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_83;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_83 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_84 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_84 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_84 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_84;
#else
_currn = (codeGen__TPPrule_84) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_84));
#endif
_currn->_prod = codeGen_RULErule_84;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_84: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_84;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_84 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_85 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_85 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_85 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_85;
#else
_currn = (codeGen__TPPrule_85) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_85));
#endif
_currn->_prod = codeGen_RULErule_85;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeSessionExpression) codeGen_MkcodeGen_WebNodeSessionExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_85: root of subtree no. 1 can not be made a codeGen_WebNodeSessionExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_85;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_85 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_86 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_86 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_86 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_86;
#else
_currn = (codeGen__TPPrule_86) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_86));
#endif
_currn->_prod = codeGen_RULErule_86;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeVariableExpression) codeGen_MkcodeGen_WebNodeVariableExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_86: root of subtree no. 1 can not be made a codeGen_WebNodeVariableExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_86;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_86 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_87 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_87 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_87 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_87;
#else
_currn = (codeGen__TPPrule_87) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_87));
#endif
_currn->_prod = codeGen_RULErule_87;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeConstantExpression) codeGen_MkcodeGen_WebNodeConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_87: root of subtree no. 1 can not be made a codeGen_WebNodeConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_87;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_87 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_88 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_88 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_88 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_88;
#else
_currn = (codeGen__TPPrule_88) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_88));
#endif
_currn->_prod = codeGen_RULErule_88;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBlob) codeGen_MkcodeGen_DBBlob (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_88: root of subtree no. 1 can not be made a codeGen_DBBlob node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_88;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_88 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_89 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_89 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_89 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_89;
#else
_currn = (codeGen__TPPrule_89) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_89));
#endif
_currn->_prod = codeGen_RULErule_89;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBoolean) codeGen_MkcodeGen_DBBoolean (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_89: root of subtree no. 1 can not be made a codeGen_DBBoolean node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_89;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_89 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_90 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_90 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_90 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_90;
#else
_currn = (codeGen__TPPrule_90) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_90));
#endif
_currn->_prod = codeGen_RULErule_90;
_currn->_desc1 = (codeGen__TSPcodeGen_DBTimeStamp) codeGen_MkcodeGen_DBTimeStamp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_90: root of subtree no. 1 can not be made a codeGen_DBTimeStamp node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_90;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_90 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_91 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_91 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_91 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_91;
#else
_currn = (codeGen__TPPrule_91) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_91));
#endif
_currn->_prod = codeGen_RULErule_91;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDate) codeGen_MkcodeGen_DBDate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_91: root of subtree no. 1 can not be made a codeGen_DBDate node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_91;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_91 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_92 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_92 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_92 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_92;
#else
_currn = (codeGen__TPPrule_92) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_92));
#endif
_currn->_prod = codeGen_RULErule_92;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBigInt) codeGen_MkcodeGen_DBBigInt (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_92: root of subtree no. 1 can not be made a codeGen_DBBigInt node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_92;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_92 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_93 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_93 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_93 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_93;
#else
_currn = (codeGen__TPPrule_93) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_93));
#endif
_currn->_prod = codeGen_RULErule_93;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDouble) codeGen_MkcodeGen_DBDouble (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_93: root of subtree no. 1 can not be made a codeGen_DBDouble node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_93;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_93 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_94 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_94 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_94 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_94;
#else
_currn = (codeGen__TPPrule_94) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_94));
#endif
_currn->_prod = codeGen_RULErule_94;
_currn->_desc1 = (codeGen__TSPcodeGen_DBFloat) codeGen_MkcodeGen_DBFloat (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_94: root of subtree no. 1 can not be made a codeGen_DBFloat node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_94;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_94 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_95 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_95 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_95 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_95;
#else
_currn = (codeGen__TPPrule_95) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_95));
#endif
_currn->_prod = codeGen_RULErule_95;
_currn->_desc1 = (codeGen__TSPcodeGen_DBLong) codeGen_MkcodeGen_DBLong (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_95: root of subtree no. 1 can not be made a codeGen_DBLong node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_95;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_95 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_96 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_96 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_96 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_96;
#else
_currn = (codeGen__TPPrule_96) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_96));
#endif
_currn->_prod = codeGen_RULErule_96;
_currn->_desc1 = (codeGen__TSPcodeGen_DBInteger) codeGen_MkcodeGen_DBInteger (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_96: root of subtree no. 1 can not be made a codeGen_DBInteger node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_96;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_96 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_97 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_97 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_97 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_97;
#else
_currn = (codeGen__TPPrule_97) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_97));
#endif
_currn->_prod = codeGen_RULErule_97;
_currn->_desc1 = (codeGen__TSPcodeGen_DBText) codeGen_MkcodeGen_DBText (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_97: root of subtree no. 1 can not be made a codeGen_DBText node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_97;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_97 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_98 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_98 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_98 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_98;
#else
_currn = (codeGen__TPPrule_98) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_98));
#endif
_currn->_prod = codeGen_RULErule_98;
_currn->_desc1 = (codeGen__TSPcodeGen_DBString) codeGen_MkcodeGen_DBString (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_98: root of subtree no. 1 can not be made a codeGen_DBString node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_98;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_98 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_99 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_99 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_99 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_99;
#else
_currn = (codeGen__TPPrule_99) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_99));
#endif
_currn->_prod = codeGen_RULErule_99;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasMany) codeGen_MkcodeGen_DBHasMany (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_99: root of subtree no. 1 can not be made a codeGen_DBHasMany node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_99;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_99 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_100 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_100 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_100 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_100;
#else
_currn = (codeGen__TPPrule_100) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_100));
#endif
_currn->_prod = codeGen_RULErule_100;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasOne) codeGen_MkcodeGen_DBHasOne (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_100: root of subtree no. 1 can not be made a codeGen_DBHasOne node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_100;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_100 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_101 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_101 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_101 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_101;
#else
_currn = (codeGen__TPPrule_101) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_101));
#endif
_currn->_prod = codeGen_RULErule_101;
_currn->_desc1 = (codeGen__TSPcodeGen_Frame) codeGen_MkcodeGen_Frame (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_101: root of subtree no. 1 can not be made a codeGen_Frame node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_101;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_101 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_102 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_102 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_102 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_102;
#else
_currn = (codeGen__TPPrule_102) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_102));
#endif
_currn->_prod = codeGen_RULErule_102;
_currn->_desc1 = (codeGen__TSPcodeGen_CustomValue) codeGen_MkcodeGen_CustomValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_102: root of subtree no. 1 can not be made a codeGen_CustomValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_102;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_102 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_103 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_103 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_103 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_103;
#else
_currn = (codeGen__TPPrule_103) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_103));
#endif
_currn->_prod = codeGen_RULErule_103;
_currn->_desc1 = (codeGen__TSPcodeGen_ListValue) codeGen_MkcodeGen_ListValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_103: root of subtree no. 1 can not be made a codeGen_ListValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_103;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_103 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_104 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_104 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_104 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_104;
#else
_currn = (codeGen__TPPrule_104) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_104));
#endif
_currn->_prod = codeGen_RULErule_104;
_currn->_desc1 = (codeGen__TSPcodeGen_AlignmentValue) codeGen_MkcodeGen_AlignmentValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_104: root of subtree no. 1 can not be made a codeGen_AlignmentValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_104;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_104 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_105 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_105 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_105 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_105;
#else
_currn = (codeGen__TPPrule_105) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_105));
#endif
_currn->_prod = codeGen_RULErule_105;
_currn->_desc1 = (codeGen__TSPcodeGen_HeadingValue) codeGen_MkcodeGen_HeadingValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_105: root of subtree no. 1 can not be made a codeGen_HeadingValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_105;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_105 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_106 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_106 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_106 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_106;
#else
_currn = (codeGen__TPPrule_106) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_106));
#endif
_currn->_prod = codeGen_RULErule_106;
_currn->_desc1 = (codeGen__TSPcodeGen_StyleValue) codeGen_MkcodeGen_StyleValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_106: root of subtree no. 1 can not be made a codeGen_StyleValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_106;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_106 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_107 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_107 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_107 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_107;
#else
_currn = (codeGen__TPPrule_107) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_107));
#endif
_currn->_prod = codeGen_RULErule_107;
_currn->_desc1 = (codeGen__TSPcodeGen_FamilyValue) codeGen_MkcodeGen_FamilyValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_107: root of subtree no. 1 can not be made a codeGen_FamilyValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_107;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_107 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_108 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_108 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_108 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_108;
#else
_currn = (codeGen__TPPrule_108) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_108));
#endif
_currn->_prod = codeGen_RULErule_108;
_currn->_desc1 = (codeGen__TSPcodeGen_ColorValue) codeGen_MkcodeGen_ColorValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_108: root of subtree no. 1 can not be made a codeGen_ColorValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_108;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_108 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_109 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_109 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_109 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_109;
#else
_currn = (codeGen__TPPrule_109) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_109));
#endif
_currn->_prod = codeGen_RULErule_109;
_currn->_desc1 = (codeGen__TSPcodeGen_SizeValue) codeGen_MkcodeGen_SizeValue (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_109: root of subtree no. 1 can not be made a codeGen_SizeValue node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_109;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_109 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_110 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_110 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_110 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_110;
#else
_currn = (codeGen__TPPrule_110) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_110));
#endif
_currn->_prod = codeGen_RULErule_110;
_currn->_desc1 = (codeGen__TSPcodeGen_ActionHandler) codeGen_MkcodeGen_ActionHandler (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_110: root of subtree no. 1 can not be made a codeGen_ActionHandler node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_110;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_110 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_111 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_111 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_111 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_111;
#else
_currn = (codeGen__TPPrule_111) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_111));
#endif
_currn->_prod = codeGen_RULErule_111;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularDataSourceSelectOption) codeGen_MkcodeGen_FormularDataSourceSelectOption (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_111: root of subtree no. 1 can not be made a codeGen_FormularDataSourceSelectOption node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_111;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_111 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_112 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_112 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_112 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_112;
#else
_currn = (codeGen__TPPrule_112) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_112));
#endif
_currn->_prod = codeGen_RULErule_112;
_currn->_desc1 = (codeGen__TSPcodeGen_FormularConstantSelectOption) codeGen_MkcodeGen_FormularConstantSelectOption (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_112: root of subtree no. 1 can not be made a codeGen_FormularConstantSelectOption node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_112;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_112 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_113 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_113 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_113 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_113;
#else
_currn = (codeGen__TPPrule_113) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_113));
#endif
_currn->_prod = codeGen_RULErule_113;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeSessionExpression) codeGen_MkcodeGen_WebNodeSessionExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_113: root of subtree no. 1 can not be made a codeGen_WebNodeSessionExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_113;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_113 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_114 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_114 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_114 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_114;
#else
_currn = (codeGen__TPPrule_114) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_114));
#endif
_currn->_prod = codeGen_RULErule_114;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeVariableExpression) codeGen_MkcodeGen_WebNodeVariableExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_114: root of subtree no. 1 can not be made a codeGen_WebNodeVariableExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_114;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_114 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_115 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_115 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_115 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_115;
#else
_currn = (codeGen__TPPrule_115) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_115));
#endif
_currn->_prod = codeGen_RULErule_115;
_currn->_desc1 = (codeGen__TSPcodeGen_WebNodeConstantExpression) codeGen_MkcodeGen_WebNodeConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_115: root of subtree no. 1 can not be made a codeGen_WebNodeConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_115;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_115 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_116 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_116 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_116 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_116;
#else
_currn = (codeGen__TPPrule_116) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_116));
#endif
_currn->_prod = codeGen_RULErule_116;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBlob) codeGen_MkcodeGen_DBBlob (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_116: root of subtree no. 1 can not be made a codeGen_DBBlob node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_116;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_116 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_117 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_117 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_117 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_117;
#else
_currn = (codeGen__TPPrule_117) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_117));
#endif
_currn->_prod = codeGen_RULErule_117;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBoolean) codeGen_MkcodeGen_DBBoolean (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_117: root of subtree no. 1 can not be made a codeGen_DBBoolean node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_117;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_117 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_118 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_118 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_118 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_118;
#else
_currn = (codeGen__TPPrule_118) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_118));
#endif
_currn->_prod = codeGen_RULErule_118;
_currn->_desc1 = (codeGen__TSPcodeGen_DBTimeStamp) codeGen_MkcodeGen_DBTimeStamp (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_118: root of subtree no. 1 can not be made a codeGen_DBTimeStamp node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_118;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_118 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_119 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_119 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_119 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_119;
#else
_currn = (codeGen__TPPrule_119) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_119));
#endif
_currn->_prod = codeGen_RULErule_119;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDate) codeGen_MkcodeGen_DBDate (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_119: root of subtree no. 1 can not be made a codeGen_DBDate node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_119;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_119 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_120 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_120 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_120 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_120;
#else
_currn = (codeGen__TPPrule_120) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_120));
#endif
_currn->_prod = codeGen_RULErule_120;
_currn->_desc1 = (codeGen__TSPcodeGen_DBBigInt) codeGen_MkcodeGen_DBBigInt (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_120: root of subtree no. 1 can not be made a codeGen_DBBigInt node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_120;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_120 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_121 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_121 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_121 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_121;
#else
_currn = (codeGen__TPPrule_121) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_121));
#endif
_currn->_prod = codeGen_RULErule_121;
_currn->_desc1 = (codeGen__TSPcodeGen_DBDouble) codeGen_MkcodeGen_DBDouble (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_121: root of subtree no. 1 can not be made a codeGen_DBDouble node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_121;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_121 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_122 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_122 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_122 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_122;
#else
_currn = (codeGen__TPPrule_122) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_122));
#endif
_currn->_prod = codeGen_RULErule_122;
_currn->_desc1 = (codeGen__TSPcodeGen_DBFloat) codeGen_MkcodeGen_DBFloat (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_122: root of subtree no. 1 can not be made a codeGen_DBFloat node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_122;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_122 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_123 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_123 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_123 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_123;
#else
_currn = (codeGen__TPPrule_123) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_123));
#endif
_currn->_prod = codeGen_RULErule_123;
_currn->_desc1 = (codeGen__TSPcodeGen_DBLong) codeGen_MkcodeGen_DBLong (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_123: root of subtree no. 1 can not be made a codeGen_DBLong node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_123;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_123 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_124 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_124 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_124 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_124;
#else
_currn = (codeGen__TPPrule_124) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_124));
#endif
_currn->_prod = codeGen_RULErule_124;
_currn->_desc1 = (codeGen__TSPcodeGen_DBInteger) codeGen_MkcodeGen_DBInteger (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_124: root of subtree no. 1 can not be made a codeGen_DBInteger node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_124;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_124 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_125 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_125 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_125 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_125;
#else
_currn = (codeGen__TPPrule_125) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_125));
#endif
_currn->_prod = codeGen_RULErule_125;
_currn->_desc1 = (codeGen__TSPcodeGen_DBText) codeGen_MkcodeGen_DBText (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_125: root of subtree no. 1 can not be made a codeGen_DBText node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_125;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_125 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_126 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_126 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_126 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_126;
#else
_currn = (codeGen__TPPrule_126) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_126));
#endif
_currn->_prod = codeGen_RULErule_126;
_currn->_desc1 = (codeGen__TSPcodeGen_DBString) codeGen_MkcodeGen_DBString (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_126: root of subtree no. 1 can not be made a codeGen_DBString node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_126;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_126 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_127 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_127 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_127 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_127;
#else
_currn = (codeGen__TPPrule_127) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_127));
#endif
_currn->_prod = codeGen_RULErule_127;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasMany) codeGen_MkcodeGen_DBHasMany (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_127: root of subtree no. 1 can not be made a codeGen_DBHasMany node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_127;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_127 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_128 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_128 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_128 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_128;
#else
_currn = (codeGen__TPPrule_128) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_128));
#endif
_currn->_prod = codeGen_RULErule_128;
_currn->_desc1 = (codeGen__TSPcodeGen_DBHasOne) codeGen_MkcodeGen_DBHasOne (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_128: root of subtree no. 1 can not be made a codeGen_DBHasOne node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_128;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_128 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_129 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_129 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_129 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_129;
#else
_currn = (codeGen__TPPrule_129) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_129));
#endif
_currn->_prod = codeGen_RULErule_129;
_currn->_desc1 = (codeGen__TSPcodeGen_ComputationExpression) codeGen_MkcodeGen_ComputationExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_129: root of subtree no. 1 can not be made a codeGen_ComputationExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_129;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_129 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_130 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_130 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_130 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_130;
#else
_currn = (codeGen__TPPrule_130) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_130));
#endif
_currn->_prod = codeGen_RULErule_130;
_currn->_desc1 = (codeGen__TSPcodeGen_ParameterValueExpression) codeGen_MkcodeGen_ParameterValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_130: root of subtree no. 1 can not be made a codeGen_ParameterValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_130;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_130 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_131 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_131 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_131 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_131;
#else
_currn = (codeGen__TPPrule_131) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_131));
#endif
_currn->_prod = codeGen_RULErule_131;
_currn->_desc1 = (codeGen__TSPcodeGen_ProjectionColumnValueExpression) codeGen_MkcodeGen_ProjectionColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_131: root of subtree no. 1 can not be made a codeGen_ProjectionColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_131;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_131 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_132 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_132 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_132 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_132;
#else
_currn = (codeGen__TPPrule_132) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_132));
#endif
_currn->_prod = codeGen_RULErule_132;
_currn->_desc1 = (codeGen__TSPcodeGen_ColumnValueExpression) codeGen_MkcodeGen_ColumnValueExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_132: root of subtree no. 1 can not be made a codeGen_ColumnValueExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_132;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_132 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_133 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_133 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_133 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_133;
#else
_currn = (codeGen__TPPrule_133) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_133));
#endif
_currn->_prod = codeGen_RULErule_133;
_currn->_desc1 = (codeGen__TSPcodeGen_ConstantExpression) codeGen_MkcodeGen_ConstantExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_133: root of subtree no. 1 can not be made a codeGen_ConstantExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_133;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_133 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_134 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_134 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_134 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_134;
#else
_currn = (codeGen__TPPrule_134) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_134));
#endif
_currn->_prod = codeGen_RULErule_134;
_currn->_desc1 = (codeGen__TSPcodeGen_PredefinedExpression) codeGen_MkcodeGen_PredefinedExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_134: root of subtree no. 1 can not be made a codeGen_PredefinedExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_134;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_134 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_135 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_135 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_135 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_135;
#else
_currn = (codeGen__TPPrule_135) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_135));
#endif
_currn->_prod = codeGen_RULErule_135;
_currn->_desc1 = (codeGen__TSPcodeGen_UnaryExpression) codeGen_MkcodeGen_UnaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_135: root of subtree no. 1 can not be made a codeGen_UnaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_135;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_135 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_136 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_136 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_136 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_136;
#else
_currn = (codeGen__TPPrule_136) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_136));
#endif
_currn->_prod = codeGen_RULErule_136;
_currn->_desc1 = (codeGen__TSPcodeGen_BinaryExpression) codeGen_MkcodeGen_BinaryExpression (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_136: root of subtree no. 1 can not be made a codeGen_BinaryExpression node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_136;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_136 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_137 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_137 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_137 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_137;
#else
_currn = (codeGen__TPPrule_137) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_137));
#endif
_currn->_prod = codeGen_RULErule_137;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputActionConnectionsContent) codeGen_MkcodeGen_ListOutputActionConnectionsContent (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_137: root of subtree no. 1 can not be made a codeGen_ListOutputActionConnectionsContent node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_137;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_137 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_138 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_138 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_138 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_138;
#else
_currn = (codeGen__TPPrule_138) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_138));
#endif
_currn->_prod = codeGen_RULErule_138;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputNumerationContent) codeGen_MkcodeGen_ListOutputNumerationContent (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_138: root of subtree no. 1 can not be made a codeGen_ListOutputNumerationContent node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_138;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_138 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_139 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_139 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_139 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_139;
#else
_currn = (codeGen__TPPrule_139) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_139));
#endif
_currn->_prod = codeGen_RULErule_139;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputProjectionContent) codeGen_MkcodeGen_ListOutputProjectionContent (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_139: root of subtree no. 1 can not be made a codeGen_ListOutputProjectionContent node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_139;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_139 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_140 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_140 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_140 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_140;
#else
_currn = (codeGen__TPPrule_140) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_140));
#endif
_currn->_prod = codeGen_RULErule_140;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputRefContent) codeGen_MkcodeGen_ListOutputRefContent (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_140: root of subtree no. 1 can not be made a codeGen_ListOutputRefContent node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_140;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_140 */

#if defined(__STDC__) || defined(__cplusplus)
NODEPTR codeGen_Mkrule_141 (POSITION *_coordref, NODEPTR _desc1)
#else
NODEPTR codeGen_Mkrule_141 (_coordref,_desc1)
	POSITION *_coordref;
	NODEPTR _desc1;
#endif
{	codeGen__TPPrule_141 _currn;
#ifdef __cplusplus
_currn = new codeGen__TPrule_141;
#else
_currn = (codeGen__TPPrule_141) codeGen_TreeNodeAlloc (sizeof (struct codeGen__TPrule_141));
#endif
_currn->_prod = codeGen_RULErule_141;
_currn->_desc1 = (codeGen__TSPcodeGen_ListOutputConstantContent) codeGen_MkcodeGen_ListOutputConstantContent (_coordref, _desc1);	
if (((NODEPTR)_currn->_desc1) == NULLNODEPTR)	
	message (DEADLY, "RULE rule_141: root of subtree no. 1 can not be made a codeGen_ListOutputConstantContent node ", 0, _coordref);
_COPYCOORD(_currn)
codeGen__TERMACT_rule_141;
return ( (NODEPTR) _currn);
}/* codeGen_Mkrule_141 */
