#include <stdio.h>
#include <stdlib.h>

#include "ptg_gen.h"
#include "obstack.h"

/* -------------------------------------------------------- */
/*                      Memory Management                   */
/* -------------------------------------------------------- */

static Obstack _PTGObstack;
static void *_PTGFirstObj = NULL;

static void _PTGInit()
{
	if (_PTGFirstObj)
		return;
	obstack_init(&_PTGObstack);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}

void PTGFree()
{
	obstack_free(&_PTGObstack, _PTGFirstObj);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}


#if defined(__STDC__) || defined(__cplusplus)
static void *MALLOC(int size)
#else
static void *MALLOC(size) int size;
#endif
{
	if (! _PTGFirstObj)
		_PTGInit();
	return (obstack_alloc(&_PTGObstack, size));
}

/* -------------------------------------------------------- */
/*                       Output-functions                   */
/* -------------------------------------------------------- */

static PTG_OUTPUT_FILE f;
static char buffer[40];

#if PTG_OUTPUT_DEFAULT
#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOut (PTGNode r)
#else
PTGNode PTGOut (r)
	PTGNode r;
#endif
{
	f = stdout;
	if (r) {
		(* (r->_print)) (r);
	}
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFile (char *fn, PTGNode r)
#else
PTGNode PTGOutFile (fn, r)
	char * fn; PTGNode r;
#endif
{
	if ((f = fopen(fn, "w"))  == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFile: output file '%s' can't be opened.\n",fn);
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	fclose(f);
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFPtr(FILE *fptr, PTGNode r)
#else
PTGNode PTGOutFPtr(fptr, r)
	FILE *fptr; PTGNode r;
#endif
{
	if ((f = fptr) == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFPtr: output file not open.\n");
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	return (r);
}

#endif

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGProcess(PTG_OUTPUT_FILE file, PTGNode r)
#else
PTGNode PTGProcess(file, r)
	PTG_OUTPUT_FILE file; PTGNode r;
#endif
{
	f = file;
	if (r)
		(* (r->_print)) (r);
	return (r);
}
/* -------------------------------------------------------- */
/*                            PTGNULL                       */
/* -------------------------------------------------------- */

/* Define PTGNULL as a PTGNode that prints nothing. */

#if defined(__STDC__) || defined(__cplusplus)
static void _PrPTGNULL(_PPTG0 n)
#else
static void _PrPTGNULL(n)
_PPTG0 n;
#endif
{(void)n; /* function printing nothing */}

struct _SPTG0   _PTGNULL = { (_PTGProc) _PrPTGNULL };

/* -------------------------------------------------------- */
/*          Node-Construction and Print-functions           */
/* -------------------------------------------------------- */


/* Implementation of Pattern AddNL */

typedef struct _SPTGAddNL{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGAddNL;

#ifdef PROTO_OK
static void _PrPTGAddNL(_PPTGAddNL n)
#else
static void _PrPTGAddNL(n)
	_PPTGAddNL n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGAddNL(PTGNode p1)
#else
PTGNode PTGAddNL(p1)
PTGNode p1;
#endif
{
	_PPTGAddNL n;

	n = (_PPTGAddNL)MALLOC(sizeof(struct _SPTGAddNL));
	n->_print = (_PTGProc)_PrPTGAddNL;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Id */

typedef struct _SPTGId{
	_PTGProc _print;
	int p1;
} * _PPTGId;

#ifdef PROTO_OK
static void _PrPTGId(_PPTGId n)
#else
static void _PrPTGId(n)
	_PPTGId n;
#endif
{
	PtgOutId(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGId(int p1)
#else
PTGNode PTGId(p1)
int p1;
#endif
{
	_PPTGId n;

	n = (_PPTGId)MALLOC(sizeof(struct _SPTGId));
	n->_print = (_PTGProc)_PrPTGId;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern AsIs */

typedef struct _SPTGAsIs{
	_PTGProc _print;
	char * p1;
} * _PPTGAsIs;

#ifdef PROTO_OK
static void _PrPTGAsIs(_PPTGAsIs n)
#else
static void _PrPTGAsIs(n)
	_PPTGAsIs n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGAsIs(char * p1)
#else
PTGNode PTGAsIs(p1)
char * p1;
#endif
{
	_PPTGAsIs n;

	n = (_PPTGAsIs)MALLOC(sizeof(struct _SPTGAsIs));
	n->_print = (_PTGProc)_PrPTGAsIs;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Numb */

typedef struct _SPTGNumb{
	_PTGProc _print;
	int p1;
} * _PPTGNumb;

#ifdef PROTO_OK
static void _PrPTGNumb(_PPTGNumb n)
#else
static void _PrPTGNumb(n)
	_PPTGNumb n;
#endif
{
	PTG_OUTPUT_INT(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGNumb(int p1)
#else
PTGNode PTGNumb(p1)
int p1;
#endif
{
	_PPTGNumb n;

	n = (_PPTGNumb)MALLOC(sizeof(struct _SPTGNumb));
	n->_print = (_PTGProc)_PrPTGNumb;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CString */

typedef struct _SPTGCString{
	_PTGProc _print;
	char * p1;
} * _PPTGCString;

#ifdef PROTO_OK
static void _PrPTGCString(_PPTGCString n)
#else
static void _PrPTGCString(n)
	_PPTGCString n;
#endif
{
	CPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCString(char * p1)
#else
PTGNode PTGCString(p1)
char * p1;
#endif
{
	_PPTGCString n;

	n = (_PPTGCString)MALLOC(sizeof(struct _SPTGCString));
	n->_print = (_PTGProc)_PrPTGCString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CChar */

typedef struct _SPTGCChar{
	_PTGProc _print;
	int p1;
} * _PPTGCChar;

#ifdef PROTO_OK
static void _PrPTGCChar(_PPTGCChar n)
#else
static void _PrPTGCChar(n)
	_PPTGCChar n;
#endif
{
	CPtgOutchar(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCChar(int p1)
#else
PTGNode PTGCChar(p1)
int p1;
#endif
{
	_PPTGCChar n;

	n = (_PPTGCChar)MALLOC(sizeof(struct _SPTGCChar));
	n->_print = (_PTGProc)_PrPTGCChar;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern PString */

typedef struct _SPTGPString{
	_PTGProc _print;
	char * p1;
} * _PPTGPString;

#ifdef PROTO_OK
static void _PrPTGPString(_PPTGPString n)
#else
static void _PrPTGPString(n)
	_PPTGPString n;
#endif
{
	PPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGPString(char * p1)
#else
PTGNode PTGPString(p1)
char * p1;
#endif
{
	_PPTGPString n;

	n = (_PPTGPString)MALLOC(sizeof(struct _SPTGPString));
	n->_print = (_PTGProc)_PrPTGPString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Seq */

typedef struct _SPTGSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGSeq;

#ifdef PROTO_OK
static void _PrPTGSeq(_PPTGSeq n)
#else
static void _PrPTGSeq(n)
	_PPTGSeq n;
#endif
{
	n->p1->_print(n->p1);
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGSeq n;

	n = (_PPTGSeq)MALLOC(sizeof(struct _SPTGSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern CommaSeq */

typedef struct _SPTGCommaSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGCommaSeq;

#ifdef PROTO_OK
static void _PrPTGCommaSeq(_PPTGCommaSeq n)
#else
static void _PrPTGCommaSeq(n)
	_PPTGCommaSeq n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, ", ");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGCommaSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGCommaSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGCommaSeq n;

	n = (_PPTGCommaSeq)MALLOC(sizeof(struct _SPTGCommaSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGCommaSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern Eol needed */
/* See AddNL for implementation */

/* Implementation of Pattern DotFrame */

typedef struct _SPTGDotFrame{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	int p3;
	int p4;
} * _PPTGDotFrame;

#ifdef PROTO_OK
static void _PrPTGDotFrame(_PPTGDotFrame n)
#else
static void _PrPTGDotFrame(n)
	_PPTGDotFrame n;
#endif
{
	PTG_OUTPUT_STRING(f, "digraph G {\n edge [dir=none];\n");
	PTG_OUTPUT_STRING(f, " graph [ratio = \"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"];\n");
	PTG_OUTPUT_STRING(f, " graph [size = \"");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, ",");
	PTG_OUTPUT_INT(f, n->p4);
	PTG_OUTPUT_STRING(f, "\"];\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGDotFrame(PTGNode p1, char * p2, int p3, int p4)
#else
PTGNode PTGDotFrame(p1, p2, p3, p4)
PTGNode p1;
char * p2;
int p3;
int p4;
#endif
{
	_PPTGDotFrame n;

	n = (_PPTGDotFrame)MALLOC(sizeof(struct _SPTGDotFrame));
	n->_print = (_PTGProc)_PrPTGDotFrame;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern DotEdge */

typedef struct _SPTGDotEdge{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGDotEdge;

#ifdef PROTO_OK
static void _PrPTGDotEdge(_PPTGDotEdge n)
#else
static void _PrPTGDotEdge(n)
	_PPTGDotEdge n;
#endif
{
	PTG_OUTPUT_STRING(f, "_");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " -> ");
	PTG_OUTPUT_STRING(f, "_");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGDotEdge(PTGNode p1, PTGNode p2)
#else
PTGNode PTGDotEdge(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGDotEdge n;

	n = (_PPTGDotEdge)MALLOC(sizeof(struct _SPTGDotEdge));
	n->_print = (_PTGProc)_PrPTGDotEdge;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DotNode */

typedef struct _SPTGDotNode{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGDotNode;

#ifdef PROTO_OK
static void _PrPTGDotNode(_PPTGDotNode n)
#else
static void _PrPTGDotNode(n)
	_PPTGDotNode n;
#endif
{
	PTG_OUTPUT_STRING(f, "_");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "[label=\"\" shape=box, width=");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ", height=");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "]\n");
}

#ifdef PROTO_OK
PTGNode PTGDotNode(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGDotNode(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGDotNode n;

	n = (_PPTGDotNode)MALLOC(sizeof(struct _SPTGDotNode));
	n->_print = (_PTGProc)_PrPTGDotNode;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern DatabasePropertiesFile */

typedef struct _SPTGDatabasePropertiesFile{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	char * p6;
	char * p7;
} * _PPTGDatabasePropertiesFile;

#ifdef PROTO_OK
static void _PrPTGDatabasePropertiesFile(_PPTGDatabasePropertiesFile n)
#else
static void _PrPTGDatabasePropertiesFile(n)
	_PPTGDatabasePropertiesFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "hibernate.connection.driver_class=");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.connection.url=");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.connection.username=");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.connection.password=");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.dialect=");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.show_sql=true\n");
	PTG_OUTPUT_STRING(f, "hibernate.cglib.use_reflection_optimizer=true\n");
	PTG_OUTPUT_STRING(f, "hibernate.mapping.dir=");
	PTG_OUTPUT_STRING(f, n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "hibernate.dbtype=");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGDatabasePropertiesFile(char * p1, char * p2, char * p3, char * p4, char * p5, char * p6, char * p7)
#else
PTGNode PTGDatabasePropertiesFile(p1, p2, p3, p4, p5, p6, p7)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
char * p6;
char * p7;
#endif
{
	_PPTGDatabasePropertiesFile n;

	n = (_PPTGDatabasePropertiesFile)MALLOC(sizeof(struct _SPTGDatabasePropertiesFile));
	n->_print = (_PTGProc)_PrPTGDatabasePropertiesFile;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern BuildPropertiesFile */

typedef struct _SPTGBuildPropertiesFile{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
} * _PPTGBuildPropertiesFile;

#ifdef PROTO_OK
static void _PrPTGBuildPropertiesFile(_PPTGBuildPropertiesFile n)
#else
static void _PrPTGBuildPropertiesFile(n)
	_PPTGBuildPropertiesFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "# Adapt the settings if neccessary\n");
	PTG_OUTPUT_STRING(f, "project.name=");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "catalina.home=");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "manager.username=tomcat\n");
	PTG_OUTPUT_STRING(f, "manager.password=tomcat\n");
	PTG_OUTPUT_STRING(f, "manager.url=http://localhost:8080/manager\n");
}

#ifdef PROTO_OK
PTGNode PTGBuildPropertiesFile(char * p1, char * p2, char * p3, char * p4)
#else
PTGNode PTGBuildPropertiesFile(p1, p2, p3, p4)
char * p1;
char * p2;
char * p3;
char * p4;
#endif
{
	_PPTGBuildPropertiesFile n;

	n = (_PPTGBuildPropertiesFile)MALLOC(sizeof(struct _SPTGBuildPropertiesFile));
	n->_print = (_PTGProc)_PrPTGBuildPropertiesFile;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern MailPropertiesFile */

typedef struct _SPTGMailPropertiesFile{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGMailPropertiesFile;

#ifdef PROTO_OK
static void _PrPTGMailPropertiesFile(_PPTGMailPropertiesFile n)
#else
static void _PrPTGMailPropertiesFile(n)
	_PPTGMailPropertiesFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "# Properties file with mail-related settings\n");
	PTG_OUTPUT_STRING(f, "mail.host=");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "mail.default.subject=\n");
	PTG_OUTPUT_STRING(f, "mail.default.from=");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "mail.default.to=\n");
}

#ifdef PROTO_OK
PTGNode PTGMailPropertiesFile(char * p1, char * p2)
#else
PTGNode PTGMailPropertiesFile(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGMailPropertiesFile n;

	n = (_PPTGMailPropertiesFile)MALLOC(sizeof(struct _SPTGMailPropertiesFile));
	n->_print = (_PTGProc)_PrPTGMailPropertiesFile;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Messages */

typedef struct _SPTGMessages{
	_PTGProc _print;
} * _PPTGMessages;

#ifdef PROTO_OK
static void _PrPTGMessages(_PPTGMessages n)
#else
static void _PrPTGMessages(n)
	_PPTGMessages n;
#endif
{
	PTG_OUTPUT_STRING(f, "dataScroller_pages = {0} items found, displaying {1}, from {2} to {3}. Page {4} / {5}\n");
}

static struct _SPTGMessages _sptgMessages = { (_PTGProc) _PrPTGMessages };

#ifdef PROTO_OK
PTGNode PTGMessages(void)
#else
PTGNode PTGMessages()
#endif
{
	return (PTGNode)(&_sptgMessages);
}

/* Implementation of Pattern Messages_de */

typedef struct _SPTGMessages_de{
	_PTGProc _print;
} * _PPTGMessages_de;

#ifdef PROTO_OK
static void _PrPTGMessages_de(_PPTGMessages_de n)
#else
static void _PrPTGMessages_de(n)
	_PPTGMessages_de n;
#endif
{
	PTG_OUTPUT_STRING(f, "dataScroller_pages = {0} Eintrge gefunden, zeigend {1}, von {2} bis {3}. Seite {4} / {5}\n");
}

static struct _SPTGMessages_de _sptgMessages_de = { (_PTGProc) _PrPTGMessages_de };

#ifdef PROTO_OK
PTGNode PTGMessages_de(void)
#else
PTGNode PTGMessages_de()
#endif
{
	return (PTGNode)(&_sptgMessages_de);
}

/* Implementation of Pattern MasterTemplate */

typedef struct _SPTGMasterTemplate{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	PTGNode p3;
	PTGNode p4;
	char * p5;
} * _PPTGMasterTemplate;

#ifdef PROTO_OK
static void _PrPTGMasterTemplate(_PPTGMasterTemplate n)
#else
static void _PrPTGMasterTemplate(n)
	_PPTGMasterTemplate n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<%@ taglib prefix=\"tiles\" uri=\"http://struts.apache.org/tags-tiles\"%>\n");
	PTG_OUTPUT_STRING(f, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
	PTG_OUTPUT_STRING(f, "<html>\n");
	PTG_OUTPUT_STRING(f, "<head>\n");
	PTG_OUTPUT_STRING(f, "	<title><tiles:getAsString name=\"title\" /> - <tiles:getAsString name=\"subtitle\" /></title>\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "	<link rel=\"stylesheet\" type=\"text/css\" href=\"common.css\"/>\n");
	PTG_OUTPUT_STRING(f, "	<link rel=\"stylesheet\" type=\"text/css\" href=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"/>\n");
	PTG_OUTPUT_STRING(f, "	<link rel=\"stylesheet\" type=\"text/css\" href=\"faces/myFacesExtensionResource/calendar.HtmlCalendarRenderer/11302665/WH/theme.css\" />\n");
	PTG_OUTPUT_STRING(f, "	<link rel=\"stylesheet\" type=\"text/css\" href=\"faces/myFacesExtensionResource/calendar.HtmlCalendarRenderer/11302665/DB/theme.css\" />\n");
	PTG_OUTPUT_STRING(f, "	<script src=\"faces/myFacesExtensionResource/calendar.HtmlCalendarRenderer/11302665/popcalendar_init.js\" type=\"text/javascript\"><!-- //--></script>\n");
	PTG_OUTPUT_STRING(f, "	<script src=\"faces/myFacesExtensionResource/calendar.HtmlCalendarRenderer/11302665/popcalendar.js\" type=\"text/javascript\"><!-- //--></script>\n");
	PTG_OUTPUT_STRING(f, "</head>\n");
	PTG_OUTPUT_STRING(f, "<body>\n");
	PTG_OUTPUT_STRING(f, "<f:view>\n");
	PTG_OUTPUT_STRING(f, "	<f:loadBundle basename=\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ".Messages\" var=\"messages\"/>\n");
	PTG_OUTPUT_STRING(f, "	<%-- The dummy variables are output just to initialize the service locator bean --%>\n");
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{applicationBean.dummyVariable}\" rendered=\"true\" />\n");
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{sessionBean.dummyVariable}\" rendered=\"true\" />\n");
	PTG_OUTPUT_STRING(f, "	<table width=\"100%\" border=\"0\" cellpadding=\"5\" cellspacing=\"0\">\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "	</table>\n");
	PTG_OUTPUT_STRING(f, "</f:view>\n");
	PTG_OUTPUT_STRING(f, "</body>\n");
	PTG_OUTPUT_STRING(f, "</html>\n");
}

#ifdef PROTO_OK
PTGNode PTGMasterTemplate(PTGNode p1, char * p2, PTGNode p3, PTGNode p4, char * p5)
#else
PTGNode PTGMasterTemplate(p1, p2, p3, p4, p5)
PTGNode p1;
char * p2;
PTGNode p3;
PTGNode p4;
char * p5;
#endif
{
	_PPTGMasterTemplate n;

	n = (_PPTGMasterTemplate)MALLOC(sizeof(struct _SPTGMasterTemplate));
	n->_print = (_PTGProc)_PrPTGMasterTemplate;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern NavOnLeftBody */

typedef struct _SPTGNavOnLeftBody{
	_PTGProc _print;
} * _PPTGNavOnLeftBody;

#ifdef PROTO_OK
static void _PrPTGNavOnLeftBody(_PPTGNavOnLeftBody n)
#else
static void _PrPTGNavOnLeftBody(n)
	_PPTGNavOnLeftBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td colspan=\"2\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"header\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"header\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"30%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"navigation\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"navigation\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"70%\" height=\"100%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"content\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"content\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td colspan=\"2\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"footer\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"footer\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
}

static struct _SPTGNavOnLeftBody _sptgNavOnLeftBody = { (_PTGProc) _PrPTGNavOnLeftBody };

#ifdef PROTO_OK
PTGNode PTGNavOnLeftBody(void)
#else
PTGNode PTGNavOnLeftBody()
#endif
{
	return (PTGNode)(&_sptgNavOnLeftBody);
}

/* Implementation of Pattern NavOnRightBody */

typedef struct _SPTGNavOnRightBody{
	_PTGProc _print;
} * _PPTGNavOnRightBody;

#ifdef PROTO_OK
static void _PrPTGNavOnRightBody(_PPTGNavOnRightBody n)
#else
static void _PrPTGNavOnRightBody(n)
	_PPTGNavOnRightBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td colspan=\"2\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"header\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"header\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"70%\" height=\"100%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"content\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"content\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"30%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"navigation\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"navigation\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td colspan=\"2\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"footer\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"footer\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
}

static struct _SPTGNavOnRightBody _sptgNavOnRightBody = { (_PTGProc) _PrPTGNavOnRightBody };

#ifdef PROTO_OK
PTGNode PTGNavOnRightBody(void)
#else
PTGNode PTGNavOnRightBody()
#endif
{
	return (PTGNode)(&_sptgNavOnRightBody);
}

/* Implementation of Pattern NavOnTopBody */

typedef struct _SPTGNavOnTopBody{
	_PTGProc _print;
} * _PPTGNavOnTopBody;

#ifdef PROTO_OK
static void _PrPTGNavOnTopBody(_PPTGNavOnTopBody n)
#else
static void _PrPTGNavOnTopBody(n)
	_PPTGNavOnTopBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td height=\"80\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"header\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"header\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"navigation\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"navigation\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"100%\" height=\"100%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"content\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"content\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td>\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"footer\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"footer\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
}

static struct _SPTGNavOnTopBody _sptgNavOnTopBody = { (_PTGProc) _PrPTGNavOnTopBody };

#ifdef PROTO_OK
PTGNode PTGNavOnTopBody(void)
#else
PTGNode PTGNavOnTopBody()
#endif
{
	return (PTGNode)(&_sptgNavOnTopBody);
}

/* Implementation of Pattern NavOnBottomBody */

typedef struct _SPTGNavOnBottomBody{
	_PTGProc _print;
} * _PPTGNavOnBottomBody;

#ifdef PROTO_OK
static void _PrPTGNavOnBottomBody(_PPTGNavOnBottomBody n)
#else
static void _PrPTGNavOnBottomBody(n)
	_PPTGNavOnBottomBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td height=\"80\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"header\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"header\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "			<td width=\"100%\" height=\"100%\" valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"content\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"content\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td valign=\"top\">\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"navigation\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"navigation\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
	PTG_OUTPUT_STRING(f, "		<tr>\n");
	PTG_OUTPUT_STRING(f, "			<td>\n");
	PTG_OUTPUT_STRING(f, "				<f:subview id=\"footer\">\n");
	PTG_OUTPUT_STRING(f, "					<tiles:insert attribute=\"footer\" flush=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "				</f:subview>\n");
	PTG_OUTPUT_STRING(f, "			</td>\n");
	PTG_OUTPUT_STRING(f, "		</tr>\n");
}

static struct _SPTGNavOnBottomBody _sptgNavOnBottomBody = { (_PTGProc) _PrPTGNavOnBottomBody };

#ifdef PROTO_OK
PTGNode PTGNavOnBottomBody(void)
#else
PTGNode PTGNavOnBottomBody()
#endif
{
	return (PTGNode)(&_sptgNavOnBottomBody);
}

/* Implementation of Pattern IndexJSP */

typedef struct _SPTGIndexJSP{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
} * _PPTGIndexJSP;

#ifdef PROTO_OK
static void _PrPTGIndexJSP(_PPTGIndexJSP n)
#else
static void _PrPTGIndexJSP(n)
	_PPTGIndexJSP n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<% response.sendRedirect(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".jsf\"); %>\n");
}

#ifdef PROTO_OK
PTGNode PTGIndexJSP(PTGNode p1, char * p2)
#else
PTGNode PTGIndexJSP(p1, p2)
PTGNode p1;
char * p2;
#endif
{
	_PPTGIndexJSP n;

	n = (_PPTGIndexJSP)MALLOC(sizeof(struct _SPTGIndexJSP));
	n->_print = (_PTGProc)_PrPTGIndexJSP;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern TilesDefXMLFile */

typedef struct _SPTGTilesDefXMLFile{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
} * _PPTGTilesDefXMLFile;

#ifdef PROTO_OK
static void _PrPTGTilesDefXMLFile(_PPTGTilesDefXMLFile n)
#else
static void _PrPTGTilesDefXMLFile(n)
	_PPTGTilesDefXMLFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	PTG_OUTPUT_STRING(f, "<!DOCTYPE tiles-definitions PUBLIC \"-//Apache Software Foundation//DTD Tiles Configuration 1.1//EN\"\n");
	PTG_OUTPUT_STRING(f, "                                   \"http://jakarta.apache.org/struts/dtds/tiles-config_1_1.dtd\">\n");
	PTG_OUTPUT_STRING(f, "<tiles-definitions>\n");
	PTG_OUTPUT_STRING(f, "	<definition name=\"base.layout\" path=\"/templates/");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\">\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"title\" value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"/>\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"subtitle\" value=\"\"/>\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"header\" value=\"/layout/header.tile.jsp\"/>\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"navigation\" value=\"/layout/navigation.tile.jsp\"/>\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"footer\" value=\"/layout/footer.tile.jsp\"/>\n");
	PTG_OUTPUT_STRING(f, "	</definition>\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "</tiles-definitions>\n");
}

#ifdef PROTO_OK
PTGNode PTGTilesDefXMLFile(char * p1, char * p2, PTGNode p3)
#else
PTGNode PTGTilesDefXMLFile(p1, p2, p3)
char * p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGTilesDefXMLFile n;

	n = (_PPTGTilesDefXMLFile)MALLOC(sizeof(struct _SPTGTilesDefXMLFile));
	n->_print = (_PTGProc)_PrPTGTilesDefXMLFile;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern TileDef */

typedef struct _SPTGTileDef{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGTileDef;

#ifdef PROTO_OK
static void _PrPTGTileDef(_PPTGTileDef n)
#else
static void _PrPTGTileDef(n)
	_PPTGTileDef n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<definition name=\"/");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".tiles\" extends=\"base.layout\">\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"subtitle\" value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"/>\n");
	PTG_OUTPUT_STRING(f, "		<put name=\"content\" value=\"/tiles/");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"/>\n");
	PTG_OUTPUT_STRING(f, "	</definition>\n");
}

#ifdef PROTO_OK
PTGNode PTGTileDef(char * p1, char * p2, char * p3)
#else
PTGNode PTGTileDef(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGTileDef n;

	n = (_PPTGTileDef)MALLOC(sizeof(struct _SPTGTileDef));
	n->_print = (_PTGProc)_PrPTGTileDef;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern FacesConfigFile */

typedef struct _SPTGFacesConfigFile{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFacesConfigFile;

#ifdef PROTO_OK
static void _PrPTGFacesConfigFile(_PPTGFacesConfigFile n)
#else
static void _PrPTGFacesConfigFile(n)
	_PPTGFacesConfigFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	PTG_OUTPUT_STRING(f, "<!DOCTYPE faces-config PUBLIC \"-//Sun Microsystems, Inc.//DTD JavaServer Faces Config 1.1//EN\"\n");
	PTG_OUTPUT_STRING(f, "                              \"http://java.sun.com/dtd/web-facesconfig_1_1.dtd\">\n");
	PTG_OUTPUT_STRING(f, "<faces-config>\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</faces-config>\n");
}

#ifdef PROTO_OK
PTGNode PTGFacesConfigFile(PTGNode p1)
#else
PTGNode PTGFacesConfigFile(p1)
PTGNode p1;
#endif
{
	_PPTGFacesConfigFile n;

	n = (_PPTGFacesConfigFile)MALLOC(sizeof(struct _SPTGFacesConfigFile));
	n->_print = (_PTGProc)_PrPTGFacesConfigFile;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FacesNavigationRulesContent */

typedef struct _SPTGFacesNavigationRulesContent{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFacesNavigationRulesContent;

#ifdef PROTO_OK
static void _PrPTGFacesNavigationRulesContent(_PPTGFacesNavigationRulesContent n)
#else
static void _PrPTGFacesNavigationRulesContent(n)
	_PPTGFacesNavigationRulesContent n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<navigation-rule>\n");
	PTG_OUTPUT_STRING(f, "		<from-view-id>*</from-view-id>\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "	</navigation-rule>\n\n");
	PTG_OUTPUT_STRING(f, "	<application>\n\n");
	PTG_OUTPUT_STRING(f, "		<view-handler>org.apache.myfaces.application.jsp.JspTilesViewHandlerImpl</view-handler>\n");
	PTG_OUTPUT_STRING(f, "		<message-bundle>de.upb.paderwave.Messages</message-bundle>\n");
	PTG_OUTPUT_STRING(f, "		<locale-config />\n");
	PTG_OUTPUT_STRING(f, "	</application>\n");
	PTG_OUTPUT_STRING(f, "<factory />\n");
	PTG_OUTPUT_STRING(f, "<lifecycle />\n");
}

#ifdef PROTO_OK
PTGNode PTGFacesNavigationRulesContent(PTGNode p1)
#else
PTGNode PTGFacesNavigationRulesContent(p1)
PTGNode p1;
#endif
{
	_PPTGFacesNavigationRulesContent n;

	n = (_PPTGFacesNavigationRulesContent)MALLOC(sizeof(struct _SPTGFacesNavigationRulesContent));
	n->_print = (_PTGProc)_PrPTGFacesNavigationRulesContent;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FacesNavigationCase */

typedef struct _SPTGFacesNavigationCase{
	_PTGProc _print;
	char * p1;
} * _PPTGFacesNavigationCase;

#ifdef PROTO_OK
static void _PrPTGFacesNavigationCase(_PPTGFacesNavigationCase n)
#else
static void _PrPTGFacesNavigationCase(n)
	_PPTGFacesNavigationCase n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<navigation-case>\n");
	PTG_OUTPUT_STRING(f, "			<from-outcome>");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "</from-outcome>\n");
	PTG_OUTPUT_STRING(f, "			<to-view-id>/");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".jsp</to-view-id>\n");
	PTG_OUTPUT_STRING(f, "		</navigation-case>\n");
}

#ifdef PROTO_OK
PTGNode PTGFacesNavigationCase(char * p1)
#else
PTGNode PTGFacesNavigationCase(p1)
char * p1;
#endif
{
	_PPTGFacesNavigationCase n;

	n = (_PPTGFacesNavigationCase)MALLOC(sizeof(struct _SPTGFacesNavigationCase));
	n->_print = (_PTGProc)_PrPTGFacesNavigationCase;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FacesManagedBeansContent */

typedef struct _SPTGFacesManagedBeansContent{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGFacesManagedBeansContent;

#ifdef PROTO_OK
static void _PrPTGFacesManagedBeansContent(_PPTGFacesManagedBeansContent n)
#else
static void _PrPTGFacesManagedBeansContent(n)
	_PPTGFacesManagedBeansContent n;
#endif
{
	PTG_OUTPUT_STRING(f, "<managed-bean>\n");
	PTG_OUTPUT_STRING(f, "	<description>\n");
	PTG_OUTPUT_STRING(f, "		Service locator of the business services\n");
	PTG_OUTPUT_STRING(f, "	</description>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-name>serviceLocatorBean</managed-bean-name>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "		");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean.ServiceLocatorBean\n");
	PTG_OUTPUT_STRING(f, "	</managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-scope>application</managed-bean-scope>\n");
	PTG_OUTPUT_STRING(f, "</managed-bean>\n");
	PTG_OUTPUT_STRING(f, "<managed-bean>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-name>applicationBean</managed-bean-name>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "		");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean.ApplicationBean\n");
	PTG_OUTPUT_STRING(f, "	</managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-scope>application</managed-bean-scope>\n");
	PTG_OUTPUT_STRING(f, "	<managed-property>\n");
	PTG_OUTPUT_STRING(f, "		<property-name>serviceLocator</property-name>\n");
	PTG_OUTPUT_STRING(f, "		<value>#{serviceLocatorBean}</value>\n");
	PTG_OUTPUT_STRING(f, "	</managed-property>\n");
	PTG_OUTPUT_STRING(f, "</managed-bean>\n");
	PTG_OUTPUT_STRING(f, "<managed-bean>\n");
	PTG_OUTPUT_STRING(f, "	<description>\n");
	PTG_OUTPUT_STRING(f, "		Managed bean that is used as a session scope cache\n");
	PTG_OUTPUT_STRING(f, "	</description>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-name>sessionBean</managed-bean-name>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "		");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean.SessionBean\n");
	PTG_OUTPUT_STRING(f, "	</managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-scope>session</managed-bean-scope>\n");
	PTG_OUTPUT_STRING(f, "</managed-bean>\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "<application>\n");
	PTG_OUTPUT_STRING(f, "	<locale-config />\n");
	PTG_OUTPUT_STRING(f, "</application>\n");
	PTG_OUTPUT_STRING(f, "<factory />\n");
	PTG_OUTPUT_STRING(f, "<lifecycle />\n");
}

#ifdef PROTO_OK
PTGNode PTGFacesManagedBeansContent(char * p1, PTGNode p2)
#else
PTGNode PTGFacesManagedBeansContent(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGFacesManagedBeansContent n;

	n = (_PPTGFacesManagedBeansContent)MALLOC(sizeof(struct _SPTGFacesManagedBeansContent));
	n->_print = (_PTGProc)_PrPTGFacesManagedBeansContent;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FacesManagedBean */

typedef struct _SPTGFacesManagedBean{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
} * _PPTGFacesManagedBean;

#ifdef PROTO_OK
static void _PrPTGFacesManagedBean(_PPTGFacesManagedBean n)
#else
static void _PrPTGFacesManagedBean(n)
	_PPTGFacesManagedBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "<managed-bean>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-name>");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Bean</managed-bean-name>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-class>");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".view.bean.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "</managed-bean-class>\n");
	PTG_OUTPUT_STRING(f, "	<managed-bean-scope>request</managed-bean-scope>\n");
	PTG_OUTPUT_STRING(f, "	<managed-property>\n");
	PTG_OUTPUT_STRING(f, "		<property-name>serviceLocator</property-name>\n");
	PTG_OUTPUT_STRING(f, "		<value>#{serviceLocatorBean}</value>\n");
	PTG_OUTPUT_STRING(f, "	</managed-property>\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "</managed-bean>\n");
}

#ifdef PROTO_OK
PTGNode PTGFacesManagedBean(char * p1, char * p2, PTGNode p3)
#else
PTGNode PTGFacesManagedBean(p1, p2, p3)
char * p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGFacesManagedBean n;

	n = (_PPTGFacesManagedBean)MALLOC(sizeof(struct _SPTGFacesManagedBean));
	n->_print = (_PTGProc)_PrPTGFacesManagedBean;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern ManagedParamProperty */

typedef struct _SPTGManagedParamProperty{
	_PTGProc _print;
	char * p1;
} * _PPTGManagedParamProperty;

#ifdef PROTO_OK
static void _PrPTGManagedParamProperty(_PPTGManagedParamProperty n)
#else
static void _PrPTGManagedParamProperty(n)
	_PPTGManagedParamProperty n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<managed-property>\n");
	PTG_OUTPUT_STRING(f, "		<property-name>");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "</property-name>\n");
	PTG_OUTPUT_STRING(f, "		<value>#{param.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}</value>\n");
	PTG_OUTPUT_STRING(f, "	</managed-property>\n");
}

#ifdef PROTO_OK
PTGNode PTGManagedParamProperty(char * p1)
#else
PTGNode PTGManagedParamProperty(p1)
char * p1;
#endif
{
	_PPTGManagedParamProperty n;

	n = (_PPTGManagedParamProperty)MALLOC(sizeof(struct _SPTGManagedParamProperty));
	n->_print = (_PTGProc)_PrPTGManagedParamProperty;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern JSPFirstLine */

typedef struct _SPTGJSPFirstLine{
	_PTGProc _print;
	char * p1;
} * _PPTGJSPFirstLine;

#ifdef PROTO_OK
static void _PrPTGJSPFirstLine(_PPTGJSPFirstLine n)
#else
static void _PrPTGJSPFirstLine(n)
	_PPTGJSPFirstLine n;
#endif
{
	PTG_OUTPUT_STRING(f, "<%@ page language=\"java\" contentType=\"text/html; charset=");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" pageEncoding=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"%>\n");
}

#ifdef PROTO_OK
PTGNode PTGJSPFirstLine(char * p1)
#else
PTGNode PTGJSPFirstLine(p1)
char * p1;
#endif
{
	_PPTGJSPFirstLine n;

	n = (_PPTGJSPFirstLine)MALLOC(sizeof(struct _SPTGJSPFirstLine));
	n->_print = (_PTGProc)_PrPTGJSPFirstLine;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern JSPHeader */

typedef struct _SPTGJSPHeader{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGJSPHeader;

#ifdef PROTO_OK
static void _PrPTGJSPHeader(_PPTGJSPHeader n)
#else
static void _PrPTGJSPHeader(n)
	_PPTGJSPHeader n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<%@ taglib uri=\"http://java.sun.com/jsf/core\" prefix=\"f\" %>\n");
	PTG_OUTPUT_STRING(f, "<%@ taglib uri=\"http://java.sun.com/jsf/html\" prefix=\"h\" %>\n");
	PTG_OUTPUT_STRING(f, "<%@ taglib uri=\"http://myfaces.apache.org/tomahawk\" prefix=\"t\"%>\n");
	PTG_OUTPUT_STRING(f, "<%@ taglib uri=\"http://myfaces.apache.org/extensions\" prefix=\"x\" %>\n");
	PTG_OUTPUT_STRING(f, "<%@ taglib uri=\"http://ag-kastens.upb.de/lehre/paderwave\" prefix=\"pw\" %>\n");
}

#ifdef PROTO_OK
PTGNode PTGJSPHeader(PTGNode p1)
#else
PTGNode PTGJSPHeader(p1)
PTGNode p1;
#endif
{
	_PPTGJSPHeader n;

	n = (_PPTGJSPHeader)MALLOC(sizeof(struct _SPTGJSPHeader));
	n->_print = (_PTGProc)_PrPTGJSPHeader;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern JSPTile */

typedef struct _SPTGJSPTile{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGJSPTile;

#ifdef PROTO_OK
static void _PrPTGJSPTile(_PPTGJSPTile n)
#else
static void _PrPTGJSPTile(n)
	_PPTGJSPTile n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGJSPTile(PTGNode p1, PTGNode p2)
#else
PTGNode PTGJSPTile(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGJSPTile n;

	n = (_PPTGJSPTile)MALLOC(sizeof(struct _SPTGJSPTile));
	n->_print = (_PTGProc)_PrPTGJSPTile;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern JSPContentTile */

typedef struct _SPTGJSPContentTile{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGJSPContentTile;

#ifdef PROTO_OK
static void _PrPTGJSPContentTile(_PPTGJSPContentTile n)
#else
static void _PrPTGJSPContentTile(n)
	_PPTGJSPContentTile n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "<%--h:panelGroup styleClass=\"");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\"--%>\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "<%--/h:panelGroup--%>\n");
}

#ifdef PROTO_OK
PTGNode PTGJSPContentTile(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGJSPContentTile(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGJSPContentTile n;

	n = (_PPTGJSPContentTile)MALLOC(sizeof(struct _SPTGJSPContentTile));
	n->_print = (_PTGProc)_PrPTGJSPContentTile;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern ServletDeclaration */

typedef struct _SPTGServletDeclaration{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGServletDeclaration;

#ifdef PROTO_OK
static void _PrPTGServletDeclaration(_PPTGServletDeclaration n)
#else
static void _PrPTGServletDeclaration(n)
	_PPTGServletDeclaration n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<servlet>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-name>");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Servlet</servlet-name>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-class>");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".controller.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Servlet</servlet-class>\n");
	PTG_OUTPUT_STRING(f, "	</servlet>\n");
	PTG_OUTPUT_STRING(f, "	<servlet-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-name>");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Servlet</servlet-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>/");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</servlet-mapping>\n");
}

#ifdef PROTO_OK
PTGNode PTGServletDeclaration(char * p1, char * p2)
#else
PTGNode PTGServletDeclaration(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGServletDeclaration n;

	n = (_PPTGServletDeclaration)MALLOC(sizeof(struct _SPTGServletDeclaration));
	n->_print = (_PTGProc)_PrPTGServletDeclaration;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern JavaBean */

typedef struct _SPTGJavaBean{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
	PTGNode p8;
} * _PPTGJavaBean;

#ifdef PROTO_OK
static void _PrPTGJavaBean(_PPTGJavaBean n)
#else
static void _PrPTGJavaBean(n)
	_PPTGJavaBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean;\n\n");
	PTG_OUTPUT_STRING(f, "import java.util.*;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.service.mail.*;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.view.*;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.Constants;\n\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "/**\n");
	PTG_OUTPUT_STRING(f, " * Description of class ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "public class ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " extends AbstractBean {\n\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "(){}\n\n");
	PTG_OUTPUT_STRING(f, "	@Override\n");
	PTG_OUTPUT_STRING(f, "	protected void init() {\n");
	n->p8->_print(n->p8);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n\n");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGJavaBean(char * p1, char * p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8)
#else
PTGNode PTGJavaBean(p1, p2, p3, p4, p5, p6, p7, p8)
char * p1;
char * p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
PTGNode p8;
#endif
{
	_PPTGJavaBean n;

	n = (_PPTGJavaBean)MALLOC(sizeof(struct _SPTGJavaBean));
	n->_print = (_PTGProc)_PrPTGJavaBean;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionServlet */

typedef struct _SPTGActionServlet{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
	PTGNode p8;
} * _PPTGActionServlet;

#ifdef PROTO_OK
static void _PrPTGActionServlet(_PPTGActionServlet n)
#else
static void _PrPTGActionServlet(n)
	_PPTGActionServlet n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".controller;\n\n");
	PTG_OUTPUT_STRING(f, "import java.io.IOException;\n");
	PTG_OUTPUT_STRING(f, "import java.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.ServletException;\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.http.HttpServletRequest;\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.http.HttpServletResponse;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.view.*;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.Constants;\n\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "public class ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Servlet extends ServiceLocatorServletSupport {\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Java-doc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	@Override\n");
	PTG_OUTPUT_STRING(f, "	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {\n");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		// forward to successpage\n");
	PTG_OUTPUT_STRING(f, "		request.getRequestDispatcher(\"/");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, ".jsf\"");
	n->p8->_print(n->p8);
	PTG_OUTPUT_STRING(f, ").forward(request,response);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGActionServlet(char * p1, char * p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7, PTGNode p8)
#else
PTGNode PTGActionServlet(p1, p2, p3, p4, p5, p6, p7, p8)
char * p1;
char * p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
PTGNode p8;
#endif
{
	_PPTGActionServlet n;

	n = (_PPTGActionServlet)MALLOC(sizeof(struct _SPTGActionServlet));
	n->_print = (_PTGProc)_PrPTGActionServlet;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionMethod */

typedef struct _SPTGActionMethod{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
} * _PPTGActionMethod;

#ifdef PROTO_OK
static void _PrPTGActionMethod(_PPTGActionMethod n)
#else
static void _PrPTGActionMethod(n)
	_PPTGActionMethod n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public String ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Action(){\n\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGActionMethod(PTGNode p1, char * p2)
#else
PTGNode PTGActionMethod(p1, p2)
PTGNode p1;
char * p2;
#endif
{
	_PPTGActionMethod n;

	n = (_PPTGActionMethod)MALLOC(sizeof(struct _SPTGActionMethod));
	n->_print = (_PTGProc)_PrPTGActionMethod;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern JavaPropertyMethods */

typedef struct _SPTGJavaPropertyMethods{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
} * _PPTGJavaPropertyMethods;

#ifdef PROTO_OK
static void _PrPTGJavaPropertyMethods(_PPTGJavaPropertyMethods n)
#else
static void _PrPTGJavaPropertyMethods(n)
	_PPTGJavaPropertyMethods n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public void set");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "( ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "){\n");
	PTG_OUTPUT_STRING(f, "		this.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " = ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ";\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public ");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, " get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "(){\n");
	PTG_OUTPUT_STRING(f, "		return this.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ";\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGJavaPropertyMethods(char * p1, char * p2, PTGNode p3)
#else
PTGNode PTGJavaPropertyMethods(p1, p2, p3)
char * p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGJavaPropertyMethods n;

	n = (_PPTGJavaPropertyMethods)MALLOC(sizeof(struct _SPTGJavaPropertyMethods));
	n->_print = (_PTGProc)_PrPTGJavaPropertyMethods;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern JavaAttribute */

typedef struct _SPTGJavaAttribute{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGJavaAttribute;

#ifdef PROTO_OK
static void _PrPTGJavaAttribute(_PPTGJavaAttribute n)
#else
static void _PrPTGJavaAttribute(n)
	_PPTGJavaAttribute n;
#endif
{
	PTG_OUTPUT_STRING(f, "	private ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ";\n");
}

#ifdef PROTO_OK
PTGNode PTGJavaAttribute(char * p1, PTGNode p2)
#else
PTGNode PTGJavaAttribute(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGJavaAttribute n;

	n = (_PPTGJavaAttribute)MALLOC(sizeof(struct _SPTGJavaAttribute));
	n->_print = (_PTGProc)_PrPTGJavaAttribute;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern PageAccess */

typedef struct _SPTGPageAccess{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGPageAccess;

#ifdef PROTO_OK
static void _PrPTGPageAccess(_PPTGPageAccess n)
#else
static void _PrPTGPageAccess(n)
	_PPTGPageAccess n;
#endif
{
	PTG_OUTPUT_STRING(f, "");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "=");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ":");
	PTG_OUTPUT_STRING(f, n->p3);
}

#ifdef PROTO_OK
PTGNode PTGPageAccess(char * p1, char * p2, char * p3)
#else
PTGNode PTGPageAccess(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGPageAccess n;

	n = (_PPTGPageAccess)MALLOC(sizeof(struct _SPTGPageAccess));
	n->_print = (_PTGProc)_PrPTGPageAccess;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern PageAccessPropertiesFile */

typedef struct _SPTGPageAccessPropertiesFile{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGPageAccessPropertiesFile;

#ifdef PROTO_OK
static void _PrPTGPageAccessPropertiesFile(_PPTGPageAccessPropertiesFile n)
#else
static void _PrPTGPageAccessPropertiesFile(n)
	_PPTGPageAccessPropertiesFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "# page=role1 role2 ... rolen:errorPage\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGPageAccessPropertiesFile(PTGNode p1)
#else
PTGNode PTGPageAccessPropertiesFile(p1)
PTGNode p1;
#endif
{
	_PPTGPageAccessPropertiesFile n;

	n = (_PPTGPageAccessPropertiesFile)MALLOC(sizeof(struct _SPTGPageAccessPropertiesFile));
	n->_print = (_PTGProc)_PrPTGPageAccessPropertiesFile;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionMethodBody */

typedef struct _SPTGActionMethodBody{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGActionMethodBody;

#ifdef PROTO_OK
static void _PrPTGActionMethodBody(_PPTGActionMethodBody n)
#else
static void _PrPTGActionMethodBody(n)
	_PPTGActionMethodBody n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		return \"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\";\n");
}

#ifdef PROTO_OK
PTGNode PTGActionMethodBody(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGActionMethodBody(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGActionMethodBody n;

	n = (_PPTGActionMethodBody)MALLOC(sizeof(struct _SPTGActionMethodBody));
	n->_print = (_PTGProc)_PrPTGActionMethodBody;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern JSFTryCatchEnv */

typedef struct _SPTGJSFTryCatchEnv{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	char * p3;
	PTGNode p4;
	PTGNode p5;
} * _PPTGJSFTryCatchEnv;

#ifdef PROTO_OK
static void _PrPTGJSFTryCatchEnv(_PPTGJSFTryCatchEnv n)
#else
static void _PrPTGJSFTryCatchEnv(n)
	_PPTGJSFTryCatchEnv n;
#endif
{
	PTG_OUTPUT_STRING(f, "		try {\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "			FacesUtils.addInfoMessage(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\");\n");
	PTG_OUTPUT_STRING(f, "		} catch (Exception e) {\n");
	PTG_OUTPUT_STRING(f, "			String msg = \"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\";\n");
	PTG_OUTPUT_STRING(f, "			this.logger.error(msg, e);\n");
	PTG_OUTPUT_STRING(f, "			FacesUtils.addErrorMessage(null, msg, e.getMessage());\n");
	PTG_OUTPUT_STRING(f, "			");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "			return \"");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\";\n");
	PTG_OUTPUT_STRING(f, "		}\n\n");
}

#ifdef PROTO_OK
PTGNode PTGJSFTryCatchEnv(PTGNode p1, char * p2, char * p3, PTGNode p4, PTGNode p5)
#else
PTGNode PTGJSFTryCatchEnv(p1, p2, p3, p4, p5)
PTGNode p1;
char * p2;
char * p3;
PTGNode p4;
PTGNode p5;
#endif
{
	_PPTGJSFTryCatchEnv n;

	n = (_PPTGJSFTryCatchEnv)MALLOC(sizeof(struct _SPTGJSFTryCatchEnv));
	n->_print = (_PTGProc)_PrPTGJSFTryCatchEnv;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern ServletTryCatchEnv */

typedef struct _SPTGServletTryCatchEnv{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	char * p3;
	PTGNode p4;
} * _PPTGServletTryCatchEnv;

#ifdef PROTO_OK
static void _PrPTGServletTryCatchEnv(_PPTGServletTryCatchEnv n)
#else
static void _PrPTGServletTryCatchEnv(n)
	_PPTGServletTryCatchEnv n;
#endif
{
	PTG_OUTPUT_STRING(f, "		try {\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "			this.addInfoMessage( request, \"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\");\n");
	PTG_OUTPUT_STRING(f, "		} catch (Exception e) {\n");
	PTG_OUTPUT_STRING(f, "			String msg = \"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\";\n");
	PTG_OUTPUT_STRING(f, "			this.logger.error(msg, e);\n");
	PTG_OUTPUT_STRING(f, "			this.addErrorMessage( request, msg + \":\" + e.getMessage());\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		}\n\n");
}

#ifdef PROTO_OK
PTGNode PTGServletTryCatchEnv(PTGNode p1, char * p2, char * p3, PTGNode p4)
#else
PTGNode PTGServletTryCatchEnv(p1, p2, p3, p4)
PTGNode p1;
char * p2;
char * p3;
PTGNode p4;
#endif
{
	_PPTGServletTryCatchEnv n;

	n = (_PPTGServletTryCatchEnv)MALLOC(sizeof(struct _SPTGServletTryCatchEnv));
	n->_print = (_PTGProc)_PrPTGServletTryCatchEnv;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern TryCatch */

typedef struct _SPTGTryCatch{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGTryCatch;

#ifdef PROTO_OK
static void _PrPTGTryCatch(_PPTGTryCatch n)
#else
static void _PrPTGTryCatch(n)
	_PPTGTryCatch n;
#endif
{
	PTG_OUTPUT_STRING(f, "		try {\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "		} catch (Exception e) {\n");
	PTG_OUTPUT_STRING(f, "			this.logger.error(this, e);\n");
	PTG_OUTPUT_STRING(f, "			FacesUtils.addErrorMessage(e.getMessage());\n");
	PTG_OUTPUT_STRING(f, "		}\n\n");
}

#ifdef PROTO_OK
PTGNode PTGTryCatch(PTGNode p1)
#else
PTGNode PTGTryCatch(p1)
PTGNode p1;
#endif
{
	_PPTGTryCatch n;

	n = (_PPTGTryCatch)MALLOC(sizeof(struct _SPTGTryCatch));
	n->_print = (_PTGProc)_PrPTGTryCatch;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ServletForward */

typedef struct _SPTGServletForward{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGServletForward;

#ifdef PROTO_OK
static void _PrPTGServletForward(_PPTGServletForward n)
#else
static void _PrPTGServletForward(n)
	_PPTGServletForward n;
#endif
{
	PTG_OUTPUT_STRING(f, "			// forward to error page\n");
	PTG_OUTPUT_STRING(f, "			request.getRequestDispatcher(\"/");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ".jsf\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ").forward(request,response);\n");
	PTG_OUTPUT_STRING(f, "			return;\n");
}

#ifdef PROTO_OK
PTGNode PTGServletForward(PTGNode p1, PTGNode p2)
#else
PTGNode PTGServletForward(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGServletForward n;

	n = (_PPTGServletForward)MALLOC(sizeof(struct _SPTGServletForward));
	n->_print = (_PTGProc)_PrPTGServletForward;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern MailingActionCode */

typedef struct _SPTGMailingActionCode{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	PTGNode p5;
} * _PPTGMailingActionCode;

#ifdef PROTO_OK
static void _PrPTGMailingActionCode(_PPTGMailingActionCode n)
#else
static void _PrPTGMailingActionCode(n)
	_PPTGMailingActionCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "MailingService m = this.getServiceLocator().getMailingService();\n");
	PTG_OUTPUT_STRING(f, "TemplateMailingJob job = new TemplateMailingJob( \"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" );\n");
	PTG_OUTPUT_STRING(f, "job.setFrom( \"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" );\n");
	PTG_OUTPUT_STRING(f, "job.setTo( ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " );\n");
	PTG_OUTPUT_STRING(f, "job.setSubject(");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, ");\n");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "m.execute( job );\n");
}

#ifdef PROTO_OK
PTGNode PTGMailingActionCode(char * p1, char * p2, char * p3, char * p4, PTGNode p5)
#else
PTGNode PTGMailingActionCode(p1, p2, p3, p4, p5)
char * p1;
char * p2;
char * p3;
char * p4;
PTGNode p5;
#endif
{
	_PPTGMailingActionCode n;

	n = (_PPTGMailingActionCode)MALLOC(sizeof(struct _SPTGMailingActionCode));
	n->_print = (_PTGProc)_PrPTGMailingActionCode;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern MailingContextParam */

typedef struct _SPTGMailingContextParam{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGMailingContextParam;

#ifdef PROTO_OK
static void _PrPTGMailingContextParam(_PPTGMailingContextParam n)
#else
static void _PrPTGMailingContextParam(n)
	_PPTGMailingContextParam n;
#endif
{
	PTG_OUTPUT_STRING(f, "job.addContextInfo( \"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\", ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " );");
}

#ifdef PROTO_OK
PTGNode PTGMailingContextParam(char * p1, char * p2)
#else
PTGNode PTGMailingContextParam(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGMailingContextParam n;

	n = (_PPTGMailingContextParam)MALLOC(sizeof(struct _SPTGMailingContextParam));
	n->_print = (_PTGProc)_PrPTGMailingContextParam;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern JSFLogoutActionCode */

typedef struct _SPTGJSFLogoutActionCode{
	_PTGProc _print;
} * _PPTGJSFLogoutActionCode;

#ifdef PROTO_OK
static void _PrPTGJSFLogoutActionCode(_PPTGJSFLogoutActionCode n)
#else
static void _PrPTGJSFLogoutActionCode(n)
	_PPTGJSFLogoutActionCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "FacesUtils.removeSessionAttribute( Key.USER );\n");
	PTG_OUTPUT_STRING(f, "FacesUtils.removeSessionAttribute( Key.USER_ROLES );\n");
}

static struct _SPTGJSFLogoutActionCode _sptgJSFLogoutActionCode = { (_PTGProc) _PrPTGJSFLogoutActionCode };

#ifdef PROTO_OK
PTGNode PTGJSFLogoutActionCode(void)
#else
PTGNode PTGJSFLogoutActionCode()
#endif
{
	return (PTGNode)(&_sptgJSFLogoutActionCode);
}

/* Implementation of Pattern ServletLogoutActionCode */

typedef struct _SPTGServletLogoutActionCode{
	_PTGProc _print;
} * _PPTGServletLogoutActionCode;

#ifdef PROTO_OK
static void _PrPTGServletLogoutActionCode(_PPTGServletLogoutActionCode n)
#else
static void _PrPTGServletLogoutActionCode(n)
	_PPTGServletLogoutActionCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "request.getSession().removeAttribute( Key.USER );\n");
	PTG_OUTPUT_STRING(f, "request.getSession().removeAttribute( Key.USER_ROLES );\n");
}

static struct _SPTGServletLogoutActionCode _sptgServletLogoutActionCode = { (_PTGProc) _PrPTGServletLogoutActionCode };

#ifdef PROTO_OK
PTGNode PTGServletLogoutActionCode(void)
#else
PTGNode PTGServletLogoutActionCode()
#endif
{
	return (PTGNode)(&_sptgServletLogoutActionCode);
}

/* Implementation of Pattern LoginActionCode */

typedef struct _SPTGLoginActionCode{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
} * _PPTGLoginActionCode;

#ifdef PROTO_OK
static void _PrPTGLoginActionCode(_PPTGLoginActionCode n)
#else
static void _PrPTGLoginActionCode(n)
	_PPTGLoginActionCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " _user = (");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ")this.getServiceLocator().getDatabaseService().\n");
	PTG_OUTPUT_STRING(f, "			checkLogin(\"from ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " u where u.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " = ? and u.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " = ?\",\n");
	PTG_OUTPUT_STRING(f, "				this.");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, ", md5(this.");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "));\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGLoginActionCode(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6)
#else
PTGNode PTGLoginActionCode(p1, p2, p3, p4, p5, p6)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
#endif
{
	_PPTGLoginActionCode n;

	n = (_PPTGLoginActionCode)MALLOC(sizeof(struct _SPTGLoginActionCode));
	n->_print = (_PTGProc)_PrPTGLoginActionCode;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern JSFSetLoginSessionAttrs */

typedef struct _SPTGJSFSetLoginSessionAttrs{
	_PTGProc _print;
	char * p1;
} * _PPTGJSFSetLoginSessionAttrs;

#ifdef PROTO_OK
static void _PrPTGJSFSetLoginSessionAttrs(_PPTGJSFSetLoginSessionAttrs n)
#else
static void _PrPTGJSFSetLoginSessionAttrs(n)
	_PPTGJSFSetLoginSessionAttrs n;
#endif
{
	PTG_OUTPUT_STRING(f, "setSessionAttribute( Key.USER, _user );\n");
	PTG_OUTPUT_STRING(f, "setSessionAttribute( Key.USER_ROLES, _user.get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() );\n");
}

#ifdef PROTO_OK
PTGNode PTGJSFSetLoginSessionAttrs(char * p1)
#else
PTGNode PTGJSFSetLoginSessionAttrs(p1)
char * p1;
#endif
{
	_PPTGJSFSetLoginSessionAttrs n;

	n = (_PPTGJSFSetLoginSessionAttrs)MALLOC(sizeof(struct _SPTGJSFSetLoginSessionAttrs));
	n->_print = (_PTGProc)_PrPTGJSFSetLoginSessionAttrs;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ServletSetLoginSessionAttrs */

typedef struct _SPTGServletSetLoginSessionAttrs{
	_PTGProc _print;
	char * p1;
} * _PPTGServletSetLoginSessionAttrs;

#ifdef PROTO_OK
static void _PrPTGServletSetLoginSessionAttrs(_PPTGServletSetLoginSessionAttrs n)
#else
static void _PrPTGServletSetLoginSessionAttrs(n)
	_PPTGServletSetLoginSessionAttrs n;
#endif
{
	PTG_OUTPUT_STRING(f, "request.getSession().setAttribute( Key.USER, _user );\n");
	PTG_OUTPUT_STRING(f, "request.getSession().setAttribute( Key.USER_ROLES, _user.get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() );\n");
}

#ifdef PROTO_OK
PTGNode PTGServletSetLoginSessionAttrs(char * p1)
#else
PTGNode PTGServletSetLoginSessionAttrs(p1)
char * p1;
#endif
{
	_PPTGServletSetLoginSessionAttrs n;

	n = (_PPTGServletSetLoginSessionAttrs)MALLOC(sizeof(struct _SPTGServletSetLoginSessionAttrs));
	n->_print = (_PTGProc)_PrPTGServletSetLoginSessionAttrs;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Frame */

typedef struct _SPTGFrame{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGFrame;

#ifdef PROTO_OK
static void _PrPTGFrame(_PPTGFrame n)
#else
static void _PrPTGFrame(n)
	_PPTGFrame n;
#endif
{
	PTG_OUTPUT_STRING(f, "<div class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' >");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</div>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFrame(PTGNode p1, PTGNode p2)
#else
PTGNode PTGFrame(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGFrame n;

	n = (_PPTGFrame)MALLOC(sizeof(struct _SPTGFrame));
	n->_print = (_PTGProc)_PrPTGFrame;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FrameRef */

typedef struct _SPTGFrameRef{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFrameRef;

#ifdef PROTO_OK
static void _PrPTGFrameRef(_PPTGFrameRef n)
#else
static void _PrPTGFrameRef(n)
	_PPTGFrameRef n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php include('");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "');?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFrameRef(PTGNode p1)
#else
PTGNode PTGFrameRef(p1)
PTGNode p1;
#endif
{
	_PPTGFrameRef n;

	n = (_PPTGFrameRef)MALLOC(sizeof(struct _SPTGFrameRef));
	n->_print = (_PTGProc)_PrPTGFrameRef;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern WebAction */

typedef struct _SPTGWebAction{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGWebAction;

#ifdef PROTO_OK
static void _PrPTGWebAction(_PPTGWebAction n)
#else
static void _PrPTGWebAction(n)
	_PPTGWebAction n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	n->p3->_print(n->p3);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebAction(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGWebAction(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGWebAction n;

	n = (_PPTGWebAction)MALLOC(sizeof(struct _SPTGWebAction));
	n->_print = (_PTGProc)_PrPTGWebAction;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern PHPOperation */

typedef struct _SPTGPHPOperation{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGPHPOperation;

#ifdef PROTO_OK
static void _PrPTGPHPOperation(_PPTGPHPOperation n)
#else
static void _PrPTGPHPOperation(n)
	_PPTGPHPOperation n;
#endif
{
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Start des benutzerdefinierten PHP-Codes */");
	IndentNewLine(f);
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Ende des benutzerdefinierten PHP-Codes */");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGPHPOperation(PTGNode p1, PTGNode p2)
#else
PTGNode PTGPHPOperation(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGPHPOperation n;

	n = (_PPTGPHPOperation)MALLOC(sizeof(struct _SPTGPHPOperation));
	n->_print = (_PTGProc)_PrPTGPHPOperation;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataBaseOperation */

typedef struct _SPTGDataBaseOperation{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGDataBaseOperation;

#ifdef PROTO_OK
static void _PrPTGDataBaseOperation(_PPTGDataBaseOperation n)
#else
static void _PrPTGDataBaseOperation(n)
	_PPTGDataBaseOperation n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGDataBaseOperation(char * p1, PTGNode p2)
#else
PTGNode PTGDataBaseOperation(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGDataBaseOperation n;

	n = (_PPTGDataBaseOperation)MALLOC(sizeof(struct _SPTGDataBaseOperation));
	n->_print = (_PTGProc)_PrPTGDataBaseOperation;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern LengthValidator */

typedef struct _SPTGLengthValidator{
	_PTGProc _print;
	char * p1;
	int p2;
	int p3;
} * _PPTGLengthValidator;

#ifdef PROTO_OK
static void _PrPTGLengthValidator(_PPTGLengthValidator n)
#else
static void _PrPTGLengthValidator(n)
	_PPTGLengthValidator n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " minimum=\"");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" maximum=\"");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGLengthValidator(char * p1, int p2, int p3)
#else
PTGNode PTGLengthValidator(p1, p2, p3)
char * p1;
int p2;
int p3;
#endif
{
	_PPTGLengthValidator n;

	n = (_PPTGLengthValidator)MALLOC(sizeof(struct _SPTGLengthValidator));
	n->_print = (_PTGProc)_PrPTGLengthValidator;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern EmailValidator */

typedef struct _SPTGEmailValidator{
	_PTGProc _print;
} * _PPTGEmailValidator;

#ifdef PROTO_OK
static void _PrPTGEmailValidator(_PPTGEmailValidator n)
#else
static void _PrPTGEmailValidator(n)
	_PPTGEmailValidator n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<x:validateEmail />\n");
}

static struct _SPTGEmailValidator _sptgEmailValidator = { (_PTGProc) _PrPTGEmailValidator };

#ifdef PROTO_OK
PTGNode PTGEmailValidator(void)
#else
PTGNode PTGEmailValidator()
#endif
{
	return (PTGNode)(&_sptgEmailValidator);
}

/* Implementation of Pattern CreditCardValidator */

typedef struct _SPTGCreditCardValidator{
	_PTGProc _print;
} * _PPTGCreditCardValidator;

#ifdef PROTO_OK
static void _PrPTGCreditCardValidator(_PPTGCreditCardValidator n)
#else
static void _PrPTGCreditCardValidator(n)
	_PPTGCreditCardValidator n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<x:validateCreditCard />\n");
}

static struct _SPTGCreditCardValidator _sptgCreditCardValidator = { (_PTGProc) _PrPTGCreditCardValidator };

#ifdef PROTO_OK
PTGNode PTGCreditCardValidator(void)
#else
PTGNode PTGCreditCardValidator()
#endif
{
	return (PTGNode)(&_sptgCreditCardValidator);
}

/* Implementation of Pattern RegExprValidator */

typedef struct _SPTGRegExprValidator{
	_PTGProc _print;
	char * p1;
} * _PPTGRegExprValidator;

#ifdef PROTO_OK
static void _PrPTGRegExprValidator(_PPTGRegExprValidator n)
#else
static void _PrPTGRegExprValidator(n)
	_PPTGRegExprValidator n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<x:validateRegExpr pattern=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGRegExprValidator(char * p1)
#else
PTGNode PTGRegExprValidator(p1)
char * p1;
#endif
{
	_PPTGRegExprValidator n;

	n = (_PPTGRegExprValidator)MALLOC(sizeof(struct _SPTGRegExprValidator));
	n->_print = (_PTGProc)_PrPTGRegExprValidator;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern IntegerNumberConverter */

typedef struct _SPTGIntegerNumberConverter{
	_PTGProc _print;
	int p1;
	int p2;
} * _PPTGIntegerNumberConverter;

#ifdef PROTO_OK
static void _PrPTGIntegerNumberConverter(_PPTGIntegerNumberConverter n)
#else
static void _PrPTGIntegerNumberConverter(n)
	_PPTGIntegerNumberConverter n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:convertNumber minIntegerDigits=\"");
	PTG_OUTPUT_INT(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" maxIntegerDigits=\"");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGIntegerNumberConverter(int p1, int p2)
#else
PTGNode PTGIntegerNumberConverter(p1, p2)
int p1;
int p2;
#endif
{
	_PPTGIntegerNumberConverter n;

	n = (_PPTGIntegerNumberConverter)MALLOC(sizeof(struct _SPTGIntegerNumberConverter));
	n->_print = (_PTGProc)_PrPTGIntegerNumberConverter;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DoubleNumberConverter */

typedef struct _SPTGDoubleNumberConverter{
	_PTGProc _print;
	int p1;
	int p2;
	int p3;
	int p4;
} * _PPTGDoubleNumberConverter;

#ifdef PROTO_OK
static void _PrPTGDoubleNumberConverter(_PPTGDoubleNumberConverter n)
#else
static void _PrPTGDoubleNumberConverter(n)
	_PPTGDoubleNumberConverter n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:convertNumber minIntegerDigits=\"");
	PTG_OUTPUT_INT(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" maxIntegerDigits=\"");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" minFractionDigits=\"");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" maxFractionDigits=\"");
	PTG_OUTPUT_INT(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGDoubleNumberConverter(int p1, int p2, int p3, int p4)
#else
PTGNode PTGDoubleNumberConverter(p1, p2, p3, p4)
int p1;
int p2;
int p3;
int p4;
#endif
{
	_PPTGDoubleNumberConverter n;

	n = (_PPTGDoubleNumberConverter)MALLOC(sizeof(struct _SPTGDoubleNumberConverter));
	n->_print = (_PTGProc)_PrPTGDoubleNumberConverter;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern PatternConverter */

typedef struct _SPTGPatternConverter{
	_PTGProc _print;
	char * p1;
} * _PPTGPatternConverter;

#ifdef PROTO_OK
static void _PrPTGPatternConverter(_PPTGPatternConverter n)
#else
static void _PrPTGPatternConverter(n)
	_PPTGPatternConverter n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:convertNumber pattern=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGPatternConverter(char * p1)
#else
PTGNode PTGPatternConverter(p1)
char * p1;
#endif
{
	_PPTGPatternConverter n;

	n = (_PPTGPatternConverter)MALLOC(sizeof(struct _SPTGPatternConverter));
	n->_print = (_PTGProc)_PrPTGPatternConverter;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ConverterById */

typedef struct _SPTGConverterById{
	_PTGProc _print;
	char * p1;
} * _PPTGConverterById;

#ifdef PROTO_OK
static void _PrPTGConverterById(_PPTGConverterById n)
#else
static void _PrPTGConverterById(n)
	_PPTGConverterById n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:converter converterId=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGConverterById(char * p1)
#else
PTGNode PTGConverterById(p1)
char * p1;
#endif
{
	_PPTGConverterById n;

	n = (_PPTGConverterById)MALLOC(sizeof(struct _SPTGConverterById));
	n->_print = (_PTGProc)_PrPTGConverterById;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern JSFForm */

typedef struct _SPTGJSFForm{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	char * p5;
} * _PPTGJSFForm;

#ifdef PROTO_OK
static void _PrPTGJSFForm(_PPTGJSFForm n)
#else
static void _PrPTGJSFForm(n)
	_PPTGJSFForm n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:form id=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" enctype=\"multipart/form-data\" styleClass=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\">\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "	<h:panelGrid columns=\"3\" width=\"100%\">\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:panelGroup>\n");
	PTG_OUTPUT_STRING(f, "			<h:commandButton action=\"#{");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "Action}\" value=\"Submit\" type=\"submit\" />\n");
	PTG_OUTPUT_STRING(f, "			<h:commandButton value=\"Reset\" type=\"reset\" />\n");
	PTG_OUTPUT_STRING(f, "		</h:panelGroup>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"\" />\n");
	PTG_OUTPUT_STRING(f, "	</h:panelGrid>\n");
	PTG_OUTPUT_STRING(f, "</h:form>\n");
}

#ifdef PROTO_OK
PTGNode PTGJSFForm(char * p1, PTGNode p2, PTGNode p3, PTGNode p4, char * p5)
#else
PTGNode PTGJSFForm(p1, p2, p3, p4, p5)
char * p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
char * p5;
#endif
{
	_PPTGJSFForm n;

	n = (_PPTGJSFForm)MALLOC(sizeof(struct _SPTGJSFForm));
	n->_print = (_PTGProc)_PrPTGJSFForm;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern OutputMessages */

typedef struct _SPTGOutputMessages{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	char * p3;
} * _PPTGOutputMessages;

#ifdef PROTO_OK
static void _PrPTGOutputMessages(_PPTGOutputMessages n)
#else
static void _PrPTGOutputMessages(n)
	_PPTGOutputMessages n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:messages errorClass=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\" fatalClass=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\" warnClass=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\" infoClass=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\" showDetail=\"true\" layout=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGOutputMessages(PTGNode p1, PTGNode p2, char * p3)
#else
PTGNode PTGOutputMessages(p1, p2, p3)
PTGNode p1;
PTGNode p2;
char * p3;
#endif
{
	_PPTGOutputMessages n;

	n = (_PPTGOutputMessages)MALLOC(sizeof(struct _SPTGOutputMessages));
	n->_print = (_PTGProc)_PrPTGOutputMessages;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern InputHidden */

typedef struct _SPTGInputHidden{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGInputHidden;

#ifdef PROTO_OK
static void _PrPTGInputHidden(_PPTGInputHidden n)
#else
static void _PrPTGInputHidden(n)
	_PPTGInputHidden n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:inputHidden value=\"#{");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "}\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGInputHidden(char * p1, char * p2)
#else
PTGNode PTGInputHidden(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGInputHidden n;

	n = (_PPTGInputHidden)MALLOC(sizeof(struct _SPTGInputHidden));
	n->_print = (_PTGProc)_PrPTGInputHidden;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularDateElement */

typedef struct _SPTGFormularDateElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
} * _PPTGFormularDateElement;

#ifdef PROTO_OK
static void _PrPTGFormularDateElement(_PPTGFormularDateElement n)
#else
static void _PrPTGFormularDateElement(n)
	_PPTGFormularDateElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<t:inputCalendar monthYearRowClass=\"yearMonthHeader\" weekRowClass=\"weekHeader\"\n");
	PTG_OUTPUT_STRING(f, "			currentDayCellClass=\"currentDayCell\" value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" renderAsPopup=\"true\"\n");
	PTG_OUTPUT_STRING(f, "			id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" renderPopupButtonAsImage=\"true\" popupTodayString=\"Heute ist \" popupWeekString=\"KW\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, " />\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularDateElement(char * p1, char * p2, char * p3, char * p4, char * p5)
#else
PTGNode PTGFormularDateElement(p1, p2, p3, p4, p5)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
#endif
{
	_PPTGFormularDateElement n;

	n = (_PPTGFormularDateElement)MALLOC(sizeof(struct _SPTGFormularDateElement));
	n->_print = (_PTGProc)_PrPTGFormularDateElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularFileUploadElement */

typedef struct _SPTGFormularFileUploadElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	char * p6;
} * _PPTGFormularFileUploadElement;

#ifdef PROTO_OK
static void _PrPTGFormularFileUploadElement(_PPTGFormularFileUploadElement n)
#else
static void _PrPTGFormularFileUploadElement(n)
	_PPTGFormularFileUploadElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<t:inputFileUpload id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" storage=\"file\"\n");
	PTG_OUTPUT_STRING(f, "							accept=\"");
	PTG_OUTPUT_STRING(f, n->p6);
	PTG_OUTPUT_STRING(f, "\"\n");
	PTG_OUTPUT_STRING(f, "							value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, " />\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularFileUploadElement(char * p1, char * p2, char * p3, char * p4, char * p5, char * p6)
#else
PTGNode PTGFormularFileUploadElement(p1, p2, p3, p4, p5, p6)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
char * p6;
#endif
{
	_PPTGFormularFileUploadElement n;

	n = (_PPTGFormularFileUploadElement)MALLOC(sizeof(struct _SPTGFormularFileUploadElement));
	n->_print = (_PTGProc)_PrPTGFormularFileUploadElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularTextElement */

typedef struct _SPTGFormularTextElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	PTGNode p7;
} * _PPTGFormularTextElement;

#ifdef PROTO_OK
static void _PrPTGFormularTextElement(_PPTGFormularTextElement n)
#else
static void _PrPTGFormularTextElement(n)
	_PPTGFormularTextElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:inputText value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p6->_print(n->p6);
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "		</h:inputText>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularTextElement(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7)
#else
PTGNode PTGFormularTextElement(p1, p2, p3, p4, p5, p6, p7)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
PTGNode p7;
#endif
{
	_PPTGFormularTextElement n;

	n = (_PPTGFormularTextElement)MALLOC(sizeof(struct _SPTGFormularTextElement));
	n->_print = (_PTGProc)_PrPTGFormularTextElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularTextAreaElement */

typedef struct _SPTGFormularTextAreaElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	int p7;
	PTGNode p8;
} * _PPTGFormularTextAreaElement;

#ifdef PROTO_OK
static void _PrPTGFormularTextAreaElement(_PPTGFormularTextAreaElement n)
#else
static void _PrPTGFormularTextAreaElement(n)
	_PPTGFormularTextAreaElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:inputTextarea value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" rows=\"");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, "\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p6->_print(n->p6);
	n->p8->_print(n->p8);
	PTG_OUTPUT_STRING(f, "		</h:inputTextarea>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularTextAreaElement(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, int p7, PTGNode p8)
#else
PTGNode PTGFormularTextAreaElement(p1, p2, p3, p4, p5, p6, p7, p8)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
int p7;
PTGNode p8;
#endif
{
	_PPTGFormularTextAreaElement n;

	n = (_PPTGFormularTextAreaElement)MALLOC(sizeof(struct _SPTGFormularTextAreaElement));
	n->_print = (_PTGProc)_PrPTGFormularTextAreaElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularPasswordElement */

typedef struct _SPTGFormularPasswordElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	PTGNode p7;
	char * p8;
} * _PPTGFormularPasswordElement;

#ifdef PROTO_OK
static void _PrPTGFormularPasswordElement(_PPTGFormularPasswordElement n)
#else
static void _PrPTGFormularPasswordElement(n)
	_PPTGFormularPasswordElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:inputSecret value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" redisplay=\"");
	PTG_OUTPUT_STRING(f, n->p8);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p6->_print(n->p6);
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "		</h:inputSecret>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularPasswordElement(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7, char * p8)
#else
PTGNode PTGFormularPasswordElement(p1, p2, p3, p4, p5, p6, p7, p8)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
PTGNode p7;
char * p8;
#endif
{
	_PPTGFormularPasswordElement n;

	n = (_PPTGFormularPasswordElement)MALLOC(sizeof(struct _SPTGFormularPasswordElement));
	n->_print = (_PTGProc)_PrPTGFormularPasswordElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularCheckboxElement */

typedef struct _SPTGFormularCheckboxElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
} * _PPTGFormularCheckboxElement;

#ifdef PROTO_OK
static void _PrPTGFormularCheckboxElement(_PPTGFormularCheckboxElement n)
#else
static void _PrPTGFormularCheckboxElement(n)
	_PPTGFormularCheckboxElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:selectBooleanCheckbox value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, " />\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularCheckboxElement(char * p1, char * p2, char * p3, char * p4, char * p5)
#else
PTGNode PTGFormularCheckboxElement(p1, p2, p3, p4, p5)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
#endif
{
	_PPTGFormularCheckboxElement n;

	n = (_PPTGFormularCheckboxElement)MALLOC(sizeof(struct _SPTGFormularCheckboxElement));
	n->_print = (_PTGProc)_PrPTGFormularCheckboxElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularRadioGroupElement */

typedef struct _SPTGFormularRadioGroupElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	char * p7;
} * _PPTGFormularRadioGroupElement;

#ifdef PROTO_OK
static void _PrPTGFormularRadioGroupElement(_PPTGFormularRadioGroupElement n)
#else
static void _PrPTGFormularRadioGroupElement(n)
	_PPTGFormularRadioGroupElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:selectOneRadio value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" layout=\"");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		</h:selectOneRadio>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGFormularRadioGroupElement(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, char * p7)
#else
PTGNode PTGFormularRadioGroupElement(p1, p2, p3, p4, p5, p6, p7)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
char * p7;
#endif
{
	_PPTGFormularRadioGroupElement n;

	n = (_PPTGFormularRadioGroupElement)MALLOC(sizeof(struct _SPTGFormularRadioGroupElement));
	n->_print = (_PTGProc)_PrPTGFormularRadioGroupElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern SelectOneMenu */

typedef struct _SPTGSelectOneMenu{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	PTGNode p7;
} * _PPTGSelectOneMenu;

#ifdef PROTO_OK
static void _PrPTGSelectOneMenu(_PPTGSelectOneMenu n)
#else
static void _PrPTGSelectOneMenu(n)
	_PPTGSelectOneMenu n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:selectOneMenu value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		</h:selectOneMenu>\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGSelectOneMenu(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, PTGNode p7)
#else
PTGNode PTGSelectOneMenu(p1, p2, p3, p4, p5, p6, p7)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
PTGNode p7;
#endif
{
	_PPTGSelectOneMenu n;

	n = (_PPTGSelectOneMenu)MALLOC(sizeof(struct _SPTGSelectOneMenu));
	n->_print = (_PTGProc)_PrPTGSelectOneMenu;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern SelectListbox */

typedef struct _SPTGSelectListbox{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	PTGNode p6;
	char * p7;
	int p8;
	PTGNode p9;
} * _PPTGSelectListbox;

#ifdef PROTO_OK
static void _PrPTGSelectListbox(_PPTGSelectListbox n)
#else
static void _PrPTGSelectListbox(n)
	_PPTGSelectListbox n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:outputLabel value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" for=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<h:");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, " value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "}\" size=\"");
	PTG_OUTPUT_INT(f, n->p8);
	PTG_OUTPUT_STRING(f, "\" id=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ">\n");
	n->p9->_print(n->p9);
	PTG_OUTPUT_STRING(f, "\n");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		</h:");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, ">\n");
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\" />\n");
}

#ifdef PROTO_OK
PTGNode PTGSelectListbox(char * p1, char * p2, char * p3, char * p4, char * p5, PTGNode p6, char * p7, int p8, PTGNode p9)
#else
PTGNode PTGSelectListbox(p1, p2, p3, p4, p5, p6, p7, p8, p9)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
PTGNode p6;
char * p7;
int p8;
PTGNode p9;
#endif
{
	_PPTGSelectListbox n;

	n = (_PPTGSelectListbox)MALLOC(sizeof(struct _SPTGSelectListbox));
	n->_print = (_PTGProc)_PrPTGSelectListbox;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	n->p9 = p9;
	return (PTGNode)n;
}

/* Implementation of Pattern SelectItem */

typedef struct _SPTGSelectItem{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGSelectItem;

#ifdef PROTO_OK
static void _PrPTGSelectItem(_PPTGSelectItem n)
#else
static void _PrPTGSelectItem(n)
	_PPTGSelectItem n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:selectItem itemValue=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" itemLabel=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" />");
}

#ifdef PROTO_OK
PTGNode PTGSelectItem(char * p1, char * p2)
#else
PTGNode PTGSelectItem(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGSelectItem n;

	n = (_PPTGSelectItem)MALLOC(sizeof(struct _SPTGSelectItem));
	n->_print = (_PTGProc)_PrPTGSelectItem;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern SelectItems */

typedef struct _SPTGSelectItems{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGSelectItems;

#ifdef PROTO_OK
static void _PrPTGSelectItems(_PPTGSelectItems n)
#else
static void _PrPTGSelectItems(n)
	_PPTGSelectItems n;
#endif
{
	PTG_OUTPUT_STRING(f, "			<f:selectItems value=\"#{");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "}\" />");
}

#ifdef PROTO_OK
PTGNode PTGSelectItems(char * p1, char * p2)
#else
PTGNode PTGSelectItems(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGSelectItems n;

	n = (_PPTGSelectItems)MALLOC(sizeof(struct _SPTGSelectItems));
	n->_print = (_PTGProc)_PrPTGSelectItems;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern SelectItemsMethod */

typedef struct _SPTGSelectItemsMethod{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGSelectItemsMethod;

#ifdef PROTO_OK
static void _PrPTGSelectItemsMethod(_PPTGSelectItemsMethod n)
#else
static void _PrPTGSelectItemsMethod(n)
	_PPTGSelectItemsMethod n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public List get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return PWUtils.toSelectItemsList(this.getHibernateTemplate()\n");
	PTG_OUTPUT_STRING(f, "					.find(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"));\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGSelectItemsMethod(char * p1, char * p2)
#else
PTGNode PTGSelectItemsMethod(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGSelectItemsMethod n;

	n = (_PPTGSelectItemsMethod)MALLOC(sizeof(struct _SPTGSelectItemsMethod));
	n->_print = (_PTGProc)_PrPTGSelectItemsMethod;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularDataSourceSelectOption */

typedef struct _SPTGFormularDataSourceSelectOption{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGFormularDataSourceSelectOption;

#ifdef PROTO_OK
static void _PrPTGFormularDataSourceSelectOption(_PPTGFormularDataSourceSelectOption n)
#else
static void _PrPTGFormularDataSourceSelectOption(n)
	_PPTGFormularDataSourceSelectOption n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$options = ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "while ($row = mysql_fetch_row($options)) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<option> $row[0] </option>\";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularDataSourceSelectOption(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGFormularDataSourceSelectOption(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGFormularDataSourceSelectOption n;

	n = (_PPTGFormularDataSourceSelectOption)MALLOC(sizeof(struct _SPTGFormularDataSourceSelectOption));
	n->_print = (_PTGProc)_PrPTGFormularDataSourceSelectOption;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Attr */

typedef struct _SPTGAttr{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGAttr;

#ifdef PROTO_OK
static void _PrPTGAttr(_PPTGAttr n)
#else
static void _PrPTGAttr(n)
	_PPTGAttr n;
#endif
{
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\"");
}

#ifdef PROTO_OK
PTGNode PTGAttr(char * p1, PTGNode p2)
#else
PTGNode PTGAttr(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGAttr n;

	n = (_PPTGAttr)MALLOC(sizeof(struct _SPTGAttr));
	n->_print = (_PTGProc)_PrPTGAttr;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern TextFragment */

typedef struct _SPTGTextFragment{
	_PTGProc _print;
	char * p1;
} * _PPTGTextFragment;

#ifdef PROTO_OK
static void _PrPTGTextFragment(_PPTGTextFragment n)
#else
static void _PrPTGTextFragment(n)
	_PPTGTextFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:outputText value=\" ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" escape=\"true\"/>");
}

#ifdef PROTO_OK
PTGNode PTGTextFragment(char * p1)
#else
PTGNode PTGTextFragment(p1)
char * p1;
#endif
{
	_PPTGTextFragment n;

	n = (_PPTGTextFragment)MALLOC(sizeof(struct _SPTGTextFragment));
	n->_print = (_PTGProc)_PrPTGTextFragment;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern StrongTextFragment */

typedef struct _SPTGStrongTextFragment{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGStrongTextFragment;

#ifdef PROTO_OK
static void _PrPTGStrongTextFragment(_PPTGStrongTextFragment n)
#else
static void _PrPTGStrongTextFragment(n)
	_PPTGStrongTextFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:outputText value=\"<strong>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</strong>\" escape=\"false\"/>");
}

#ifdef PROTO_OK
PTGNode PTGStrongTextFragment(PTGNode p1)
#else
PTGNode PTGStrongTextFragment(p1)
PTGNode p1;
#endif
{
	_PPTGStrongTextFragment n;

	n = (_PPTGStrongTextFragment)MALLOC(sizeof(struct _SPTGStrongTextFragment));
	n->_print = (_PTGProc)_PrPTGStrongTextFragment;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern EmphTextFragment */

typedef struct _SPTGEmphTextFragment{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGEmphTextFragment;

#ifdef PROTO_OK
static void _PrPTGEmphTextFragment(_PPTGEmphTextFragment n)
#else
static void _PrPTGEmphTextFragment(n)
	_PPTGEmphTextFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:outputText value=\"<i>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</i>\" escape=\"false\"/>");
}

#ifdef PROTO_OK
PTGNode PTGEmphTextFragment(PTGNode p1)
#else
PTGNode PTGEmphTextFragment(p1)
PTGNode p1;
#endif
{
	_PPTGEmphTextFragment n;

	n = (_PPTGEmphTextFragment)MALLOC(sizeof(struct _SPTGEmphTextFragment));
	n->_print = (_PTGProc)_PrPTGEmphTextFragment;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ImageFragment */

typedef struct _SPTGImageFragment{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	char * p5;
	char * p6;
	int p7;
} * _PPTGImageFragment;

#ifdef PROTO_OK
static void _PrPTGImageFragment(_PPTGImageFragment n)
#else
static void _PrPTGImageFragment(n)
	_PPTGImageFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:graphicImage value=\"images/");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" style=\"float:");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "; vertical-align:");
	PTG_OUTPUT_STRING(f, n->p6);
	PTG_OUTPUT_STRING(f, "; border: ");
	PTG_OUTPUT_INT(f, n->p7);
	PTG_OUTPUT_STRING(f, ";\" alt=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\"");
	n->p3->_print(n->p3);
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, " />");
}

#ifdef PROTO_OK
PTGNode PTGImageFragment(char * p1, PTGNode p2, PTGNode p3, PTGNode p4, char * p5, char * p6, int p7)
#else
PTGNode PTGImageFragment(p1, p2, p3, p4, p5, p6, p7)
char * p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
char * p5;
char * p6;
int p7;
#endif
{
	_PPTGImageFragment n;

	n = (_PPTGImageFragment)MALLOC(sizeof(struct _SPTGImageFragment));
	n->_print = (_PTGProc)_PrPTGImageFragment;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern Paragraph */

typedef struct _SPTGParagraph{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGParagraph;

#ifdef PROTO_OK
static void _PrPTGParagraph(_PPTGParagraph n)
#else
static void _PrPTGParagraph(n)
	_PPTGParagraph n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim><p class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'></f:verbatim>\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "<f:verbatim></p></f:verbatim>\n");
}

#ifdef PROTO_OK
PTGNode PTGParagraph(PTGNode p1, PTGNode p2)
#else
PTGNode PTGParagraph(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGParagraph n;

	n = (_PPTGParagraph)MALLOC(sizeof(struct _SPTGParagraph));
	n->_print = (_PTGProc)_PrPTGParagraph;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Heading */

typedef struct _SPTGHeading{
	_PTGProc _print;
	PTGNode p1;
	int p2;
	PTGNode p3;
} * _PPTGHeading;

#ifdef PROTO_OK
static void _PrPTGHeading(_PPTGHeading n)
#else
static void _PrPTGHeading(n)
	_PPTGHeading n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim><h");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, " class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'>");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "</h");
	PTG_OUTPUT_INT(f, n->p2);
	PTG_OUTPUT_STRING(f, "></f:verbatim>");
}

#ifdef PROTO_OK
PTGNode PTGHeading(PTGNode p1, int p2, PTGNode p3)
#else
PTGNode PTGHeading(p1, p2, p3)
PTGNode p1;
int p2;
PTGNode p3;
#endif
{
	_PPTGHeading n;

	n = (_PPTGHeading)MALLOC(sizeof(struct _SPTGHeading));
	n->_print = (_PTGProc)_PrPTGHeading;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern ImageMap */

typedef struct _SPTGImageMap{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	PTGNode p4;
} * _PPTGImageMap;

#ifdef PROTO_OK
static void _PrPTGImageMap(_PPTGImageMap n)
#else
static void _PrPTGImageMap(n)
	_PPTGImageMap n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<map name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\">");
	IndentNewLine(f);
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</map>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<img src=\"images/");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" border=\"0\" alt=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" usemap=\"#");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</f:verbatim>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGImageMap(char * p1, char * p2, char * p3, PTGNode p4)
#else
PTGNode PTGImageMap(p1, p2, p3, p4)
char * p1;
char * p2;
char * p3;
PTGNode p4;
#endif
{
	_PPTGImageMap n;

	n = (_PPTGImageMap)MALLOC(sizeof(struct _SPTGImageMap));
	n->_print = (_PTGProc)_PrPTGImageMap;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern MapArea */

typedef struct _SPTGMapArea{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	char * p3;
	char * p4;
} * _PPTGMapArea;

#ifdef PROTO_OK
static void _PrPTGMapArea(_PPTGMapArea n)
#else
static void _PrPTGMapArea(n)
	_PPTGMapArea n;
#endif
{
	PTG_OUTPUT_STRING(f, "<area href=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" alt=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" coords=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\" />");
}

#ifdef PROTO_OK
PTGNode PTGMapArea(PTGNode p1, char * p2, char * p3, char * p4)
#else
PTGNode PTGMapArea(p1, p2, p3, p4)
PTGNode p1;
char * p2;
char * p3;
char * p4;
#endif
{
	_PPTGMapArea n;

	n = (_PPTGMapArea)MALLOC(sizeof(struct _SPTGMapArea));
	n->_print = (_PTGProc)_PrPTGMapArea;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern Ref */

typedef struct _SPTGRef{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	char * p3;
} * _PPTGRef;

#ifdef PROTO_OK
static void _PrPTGRef(_PPTGRef n)
#else
static void _PrPTGRef(n)
	_PPTGRef n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:outputLink value=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ">");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</h:outputLink>\n");
}

#ifdef PROTO_OK
PTGNode PTGRef(PTGNode p1, PTGNode p2, char * p3)
#else
PTGNode PTGRef(p1, p2, p3)
PTGNode p1;
PTGNode p2;
char * p3;
#endif
{
	_PPTGRef n;

	n = (_PPTGRef)MALLOC(sizeof(struct _SPTGRef));
	n->_print = (_PTGProc)_PrPTGRef;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Table */

typedef struct _SPTGTable{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGTable;

#ifdef PROTO_OK
static void _PrPTGTable(_PPTGTable n)
#else
static void _PrPTGTable(n)
	_PPTGTable n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim><table class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' border='0' cellpadding='0' cellspacing='0' ></f:verbatim>");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "<f:verbatim></table></f:verbatim>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTable(PTGNode p1, PTGNode p2)
#else
PTGNode PTGTable(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGTable n;

	n = (_PPTGTable)MALLOC(sizeof(struct _SPTGTable));
	n->_print = (_PTGProc)_PrPTGTable;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern TableRow */

typedef struct _SPTGTableRow{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGTableRow;

#ifdef PROTO_OK
static void _PrPTGTableRow(_PPTGTableRow n)
#else
static void _PrPTGTableRow(n)
	_PPTGTableRow n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim><tr></f:verbatim>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<f:verbatim></tr></f:verbatim>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTableRow(PTGNode p1)
#else
PTGNode PTGTableRow(p1)
PTGNode p1;
#endif
{
	_PPTGTableRow n;

	n = (_PPTGTableRow)MALLOC(sizeof(struct _SPTGTableRow));
	n->_print = (_PTGProc)_PrPTGTableRow;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern TableCell */

typedef struct _SPTGTableCell{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGTableCell;

#ifdef PROTO_OK
static void _PrPTGTableCell(_PPTGTableCell n)
#else
static void _PrPTGTableCell(n)
	_PPTGTableCell n;
#endif
{
	PTG_OUTPUT_STRING(f, "<f:verbatim><td></f:verbatim>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<f:verbatim></td></f:verbatim>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTableCell(PTGNode p1)
#else
PTGNode PTGTableCell(p1)
PTGNode p1;
#endif
{
	_PPTGTableCell n;

	n = (_PPTGTableCell)MALLOC(sizeof(struct _SPTGTableCell));
	n->_print = (_PTGProc)_PrPTGTableCell;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern OutputFragment */

typedef struct _SPTGOutputFragment{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGOutputFragment;

#ifdef PROTO_OK
static void _PrPTGOutputFragment(_PPTGOutputFragment n)
#else
static void _PrPTGOutputFragment(n)
	_PPTGOutputFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:outputText value=\"#{");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "}\" escape=\"true\"/>");
}

#ifdef PROTO_OK
PTGNode PTGOutputFragment(char * p1, char * p2)
#else
PTGNode PTGOutputFragment(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGOutputFragment n;

	n = (_PPTGOutputFragment)MALLOC(sizeof(struct _SPTGOutputFragment));
	n->_print = (_PTGProc)_PrPTGOutputFragment;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern MetaInformation */

typedef struct _SPTGMetaInformation{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGMetaInformation;

#ifdef PROTO_OK
static void _PrPTGMetaInformation(_PPTGMetaInformation n)
#else
static void _PrPTGMetaInformation(n)
	_PPTGMetaInformation n;
#endif
{
	n->p1->_print(n->p1);
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGMetaInformation(PTGNode p1)
#else
PTGNode PTGMetaInformation(p1)
PTGNode p1;
#endif
{
	_PPTGMetaInformation n;

	n = (_PPTGMetaInformation)MALLOC(sizeof(struct _SPTGMetaInformation));
	n->_print = (_PTGProc)_PrPTGMetaInformation;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern MetaInformationElement */

typedef struct _SPTGMetaInformationElement{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGMetaInformationElement;

#ifdef PROTO_OK
static void _PrPTGMetaInformationElement(_PPTGMetaInformationElement n)
#else
static void _PrPTGMetaInformationElement(n)
	_PPTGMetaInformationElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<meta name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" content=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"/>");
}

#ifdef PROTO_OK
PTGNode PTGMetaInformationElement(char * p1, char * p2)
#else
PTGNode PTGMetaInformationElement(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGMetaInformationElement n;

	n = (_PPTGMetaInformationElement)MALLOC(sizeof(struct _SPTGMetaInformationElement));
	n->_print = (_PTGProc)_PrPTGMetaInformationElement;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern StyleSheet needed */
/* See MetaInformation for implementation */

/* Implementation of Pattern StyleSheetBlock */

typedef struct _SPTGStyleSheetBlock{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGStyleSheetBlock;

#ifdef PROTO_OK
static void _PrPTGStyleSheetBlock(_PPTGStyleSheetBlock n)
#else
static void _PrPTGStyleSheetBlock(n)
	_PPTGStyleSheetBlock n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " {");
	IndentIncr(f);
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGStyleSheetBlock(char * p1, PTGNode p2)
#else
PTGNode PTGStyleSheetBlock(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGStyleSheetBlock n;

	n = (_PPTGStyleSheetBlock)MALLOC(sizeof(struct _SPTGStyleSheetBlock));
	n->_print = (_PTGProc)_PrPTGStyleSheetBlock;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern StyleSheetPair */

typedef struct _SPTGStyleSheetPair{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGStyleSheetPair;

#ifdef PROTO_OK
static void _PrPTGStyleSheetPair(_PPTGStyleSheetPair n)
#else
static void _PrPTGStyleSheetPair(n)
	_PPTGStyleSheetPair n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " Attributes */");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ":");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ";");
}

#ifdef PROTO_OK
PTGNode PTGStyleSheetPair(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGStyleSheetPair(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGStyleSheetPair n;

	n = (_PPTGStyleSheetPair)MALLOC(sizeof(struct _SPTGStyleSheetPair));
	n->_print = (_PTGProc)_PrPTGStyleSheetPair;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern BatchSeparator */

typedef struct _SPTGBatchSeparator{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGBatchSeparator;

#ifdef PROTO_OK
static void _PrPTGBatchSeparator(_PPTGBatchSeparator n)
#else
static void _PrPTGBatchSeparator(n)
	_PPTGBatchSeparator n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, ";");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGBatchSeparator(PTGNode p1, PTGNode p2)
#else
PTGNode PTGBatchSeparator(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGBatchSeparator n;

	n = (_PPTGBatchSeparator)MALLOC(sizeof(struct _SPTGBatchSeparator));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGBatchSeparator;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern LFSep */

typedef struct _SPTGLFSep{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGLFSep;

#ifdef PROTO_OK
static void _PrPTGLFSep(_PPTGLFSep n)
#else
static void _PrPTGLFSep(n)
	_PPTGLFSep n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "\n");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGLFSep(PTGNode p1, PTGNode p2)
#else
PTGNode PTGLFSep(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGLFSep n;

	n = (_PPTGLFSep)MALLOC(sizeof(struct _SPTGLFSep));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGLFSep;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern NewLine */

typedef struct _SPTGNewLine{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGNewLine;

#ifdef PROTO_OK
static void _PrPTGNewLine(_PPTGNewLine n)
#else
static void _PrPTGNewLine(n)
	_PPTGNewLine n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		IndentNewLine(f);
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGNewLine(PTGNode p1, PTGNode p2)
#else
PTGNode PTGNewLine(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGNewLine n;

	n = (_PPTGNewLine)MALLOC(sizeof(struct _SPTGNewLine));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGNewLine;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Concat */

typedef struct _SPTGConcat{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGConcat;

#ifdef PROTO_OK
static void _PrPTGConcat(_PPTGConcat n)
#else
static void _PrPTGConcat(n)
	_PPTGConcat n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGConcat(PTGNode p1, PTGNode p2)
#else
PTGNode PTGConcat(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGConcat n;

	n = (_PPTGConcat)MALLOC(sizeof(struct _SPTGConcat));
	n->_print = (_PTGProc)_PrPTGConcat;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Concat3 */

typedef struct _SPTGConcat3{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGConcat3;

#ifdef PROTO_OK
static void _PrPTGConcat3(_PPTGConcat3 n)
#else
static void _PrPTGConcat3(n)
	_PPTGConcat3 n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	n->p3->_print(n->p3);
}

#ifdef PROTO_OK
PTGNode PTGConcat3(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGConcat3(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGConcat3 n;

	n = (_PPTGConcat3)MALLOC(sizeof(struct _SPTGConcat3));
	n->_print = (_PTGProc)_PrPTGConcat3;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Concat4 */

typedef struct _SPTGConcat4{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGConcat4;

#ifdef PROTO_OK
static void _PrPTGConcat4(_PPTGConcat4 n)
#else
static void _PrPTGConcat4(n)
	_PPTGConcat4 n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	n->p4->_print(n->p4);
}

#ifdef PROTO_OK
PTGNode PTGConcat4(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGConcat4(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGConcat4 n;

	n = (_PPTGConcat4)MALLOC(sizeof(struct _SPTGConcat4));
	n->_print = (_PTGProc)_PrPTGConcat4;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionLink */

typedef struct _SPTGActionLink{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGActionLink;

#ifdef PROTO_OK
static void _PrPTGActionLink(_PPTGActionLink n)
#else
static void _PrPTGActionLink(n)
	_PPTGActionLink n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "?");
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGActionLink(PTGNode p1, PTGNode p2)
#else
PTGNode PTGActionLink(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGActionLink n;

	n = (_PPTGActionLink)MALLOC(sizeof(struct _SPTGActionLink));
	n->_print = (_PTGProc)_PrPTGActionLink;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern QueryString */

typedef struct _SPTGQueryString{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGQueryString;

#ifdef PROTO_OK
static void _PrPTGQueryString(_PPTGQueryString n)
#else
static void _PrPTGQueryString(n)
	_PPTGQueryString n;
#endif
{
	PTG_OUTPUT_STRING(f, "+ \"?\" + ");
	n->p1->_print(n->p1);
}

#ifdef PROTO_OK
PTGNode PTGQueryString(PTGNode p1)
#else
PTGNode PTGQueryString(p1)
PTGNode p1;
#endif
{
	_PPTGQueryString n;

	n = (_PPTGQueryString)MALLOC(sizeof(struct _SPTGQueryString));
	n->_print = (_PTGProc)_PrPTGQueryString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ListOutputActionRequestParamMapping */

typedef struct _SPTGListOutputActionRequestParamMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGListOutputActionRequestParamMapping;

#ifdef PROTO_OK
static void _PrPTGListOutputActionRequestParamMapping(_PPTGListOutputActionRequestParamMapping n)
#else
static void _PrPTGListOutputActionRequestParamMapping(n)
	_PPTGListOutputActionRequestParamMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "=#{row.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "}");
}

#ifdef PROTO_OK
PTGNode PTGListOutputActionRequestParamMapping(char * p1, char * p2)
#else
PTGNode PTGListOutputActionRequestParamMapping(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGListOutputActionRequestParamMapping n;

	n = (_PPTGListOutputActionRequestParamMapping)MALLOC(sizeof(struct _SPTGListOutputActionRequestParamMapping));
	n->_print = (_PTGProc)_PrPTGListOutputActionRequestParamMapping;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern RequestParamMapping */

typedef struct _SPTGRequestParamMapping{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGRequestParamMapping;

#ifdef PROTO_OK
static void _PrPTGRequestParamMapping(_PPTGRequestParamMapping n)
#else
static void _PrPTGRequestParamMapping(n)
	_PPTGRequestParamMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "=\" + ");
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGRequestParamMapping(char * p1, PTGNode p2)
#else
PTGNode PTGRequestParamMapping(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGRequestParamMapping n;

	n = (_PPTGRequestParamMapping)MALLOC(sizeof(struct _SPTGRequestParamMapping));
	n->_print = (_PTGProc)_PrPTGRequestParamMapping;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern RequestParamMappingBean */

typedef struct _SPTGRequestParamMappingBean{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGRequestParamMappingBean;

#ifdef PROTO_OK
static void _PrPTGRequestParamMappingBean(_PPTGRequestParamMappingBean n)
#else
static void _PrPTGRequestParamMappingBean(n)
	_PPTGRequestParamMappingBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "		//FacesUtils.addRequestParameter(\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\", ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");\n");
}

#ifdef PROTO_OK
PTGNode PTGRequestParamMappingBean(char * p1, PTGNode p2)
#else
PTGNode PTGRequestParamMappingBean(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGRequestParamMappingBean n;

	n = (_PPTGRequestParamMappingBean)MALLOC(sizeof(struct _SPTGRequestParamMappingBean));
	n->_print = (_PTGProc)_PrPTGRequestParamMappingBean;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Ampersand */

typedef struct _SPTGAmpersand{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGAmpersand;

#ifdef PROTO_OK
static void _PrPTGAmpersand(_PPTGAmpersand n)
#else
static void _PrPTGAmpersand(n)
	_PPTGAmpersand n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "&amp;");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGAmpersand(PTGNode p1, PTGNode p2)
#else
PTGNode PTGAmpersand(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGAmpersand n;

	n = (_PPTGAmpersand)MALLOC(sizeof(struct _SPTGAmpersand));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGAmpersand;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern AmpersandPlus */

typedef struct _SPTGAmpersandPlus{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGAmpersandPlus;

#ifdef PROTO_OK
static void _PrPTGAmpersandPlus(_PPTGAmpersandPlus n)
#else
static void _PrPTGAmpersandPlus(n)
	_PPTGAmpersandPlus n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "+ \"&\" +");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGAmpersandPlus(PTGNode p1, PTGNode p2)
#else
PTGNode PTGAmpersandPlus(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGAmpersandPlus n;

	n = (_PPTGAmpersandPlus)MALLOC(sizeof(struct _SPTGAmpersandPlus));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGAmpersandPlus;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Apostroph */

typedef struct _SPTGApostroph{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGApostroph;

#ifdef PROTO_OK
static void _PrPTGApostroph(_PPTGApostroph n)
#else
static void _PrPTGApostroph(n)
	_PPTGApostroph n;
#endif
{
	PTG_OUTPUT_STRING(f, "'");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'");
}

#ifdef PROTO_OK
PTGNode PTGApostroph(PTGNode p1)
#else
PTGNode PTGApostroph(p1)
PTGNode p1;
#endif
{
	_PPTGApostroph n;

	n = (_PPTGApostroph)MALLOC(sizeof(struct _SPTGApostroph));
	n->_print = (_PTGProc)_PrPTGApostroph;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Quote */

typedef struct _SPTGQuote{
	_PTGProc _print;
	char * p1;
} * _PPTGQuote;

#ifdef PROTO_OK
static void _PrPTGQuote(_PPTGQuote n)
#else
static void _PrPTGQuote(n)
	_PPTGQuote n;
#endif
{
	PTG_OUTPUT_STRING(f, "\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"");
}

#ifdef PROTO_OK
PTGNode PTGQuote(char * p1)
#else
PTGNode PTGQuote(p1)
char * p1;
#endif
{
	_PPTGQuote n;

	n = (_PPTGQuote)MALLOC(sizeof(struct _SPTGQuote));
	n->_print = (_PTGProc)_PrPTGQuote;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceActualParam */

typedef struct _SPTGDataSourceActualParam{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGDataSourceActualParam;

#ifdef PROTO_OK
static void _PrPTGDataSourceActualParam(_PPTGDataSourceActualParam n)
#else
static void _PrPTGDataSourceActualParam(n)
	_PPTGDataSourceActualParam n;
#endif
{
	n->p1->_print(n->p1);
}

#ifdef PROTO_OK
PTGNode PTGDataSourceActualParam(PTGNode p1)
#else
PTGNode PTGDataSourceActualParam(p1)
PTGNode p1;
#endif
{
	_PPTGDataSourceActualParam n;

	n = (_PPTGDataSourceActualParam)MALLOC(sizeof(struct _SPTGDataSourceActualParam));
	if(p1 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGDataSourceActualParam;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern WebNodeActualParam */

typedef struct _SPTGWebNodeActualParam{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGWebNodeActualParam;

#ifdef PROTO_OK
static void _PrPTGWebNodeActualParam(_PPTGWebNodeActualParam n)
#else
static void _PrPTGWebNodeActualParam(n)
	_PPTGWebNodeActualParam n;
#endif
{
	PTG_OUTPUT_STRING(f, "");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " = ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ";");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebNodeActualParam(char * p1, PTGNode p2)
#else
PTGNode PTGWebNodeActualParam(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGWebNodeActualParam n;

	n = (_PPTGWebNodeActualParam)MALLOC(sizeof(struct _SPTGWebNodeActualParam));
	n->_print = (_PTGProc)_PrPTGWebNodeActualParam;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ThisPrefix */

typedef struct _SPTGThisPrefix{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGThisPrefix;

#ifdef PROTO_OK
static void _PrPTGThisPrefix(_PPTGThisPrefix n)
#else
static void _PrPTGThisPrefix(n)
	_PPTGThisPrefix n;
#endif
{
	PTG_OUTPUT_STRING(f, "this.");
	n->p1->_print(n->p1);
}

#ifdef PROTO_OK
PTGNode PTGThisPrefix(PTGNode p1)
#else
PTGNode PTGThisPrefix(p1)
PTGNode p1;
#endif
{
	_PPTGThisPrefix n;

	n = (_PPTGThisPrefix)MALLOC(sizeof(struct _SPTGThisPrefix));
	n->_print = (_PTGProc)_PrPTGThisPrefix;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ModifiedExpression */

typedef struct _SPTGModifiedExpression{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGModifiedExpression;

#ifdef PROTO_OK
static void _PrPTGModifiedExpression(_PPTGModifiedExpression n)
#else
static void _PrPTGModifiedExpression(n)
	_PPTGModifiedExpression n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ")");
}

#ifdef PROTO_OK
PTGNode PTGModifiedExpression(PTGNode p1, PTGNode p2)
#else
PTGNode PTGModifiedExpression(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGModifiedExpression n;

	n = (_PPTGModifiedExpression)MALLOC(sizeof(struct _SPTGModifiedExpression));
	n->_print = (_PTGProc)_PrPTGModifiedExpression;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern NavigationBar */

typedef struct _SPTGNavigationBar{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	char * p3;
} * _PPTGNavigationBar;

#ifdef PROTO_OK
static void _PrPTGNavigationBar(_PPTGNavigationBar n)
#else
static void _PrPTGNavigationBar(n)
	_PPTGNavigationBar n;
#endif
{
	PTG_OUTPUT_STRING(f, "<pw:navigationPanel align=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" styleClass=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\">\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "</pw:navigationPanel>");
}

#ifdef PROTO_OK
PTGNode PTGNavigationBar(PTGNode p1, char * p2, char * p3)
#else
PTGNode PTGNavigationBar(p1, p2, p3)
PTGNode p1;
char * p2;
char * p3;
#endif
{
	_PPTGNavigationBar n;

	n = (_PPTGNavigationBar)MALLOC(sizeof(struct _SPTGNavigationBar));
	n->_print = (_PTGProc)_PrPTGNavigationBar;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern NavigationBarElement */

typedef struct _SPTGNavigationBarElement{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
} * _PPTGNavigationBarElement;

#ifdef PROTO_OK
static void _PrPTGNavigationBarElement(_PPTGNavigationBarElement n)
#else
static void _PrPTGNavigationBarElement(n)
	_PPTGNavigationBarElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<pw:navigationItem value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" roles=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" imagesrc=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "\"><h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"/></pw:navigationItem>");
}

#ifdef PROTO_OK
PTGNode PTGNavigationBarElement(char * p1, char * p2, char * p3, char * p4)
#else
PTGNode PTGNavigationBarElement(p1, p2, p3, p4)
char * p1;
char * p2;
char * p3;
char * p4;
#endif
{
	_PPTGNavigationBarElement n;

	n = (_PPTGNavigationBarElement)MALLOC(sizeof(struct _SPTGNavigationBarElement));
	n->_print = (_PTGProc)_PrPTGNavigationBarElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceModifyOperation */

typedef struct _SPTGDataSourceModifyOperation{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	char * p3;
} * _PPTGDataSourceModifyOperation;

#ifdef PROTO_OK
static void _PrPTGDataSourceModifyOperation(_PPTGDataSourceModifyOperation n)
#else
static void _PrPTGDataSourceModifyOperation(n)
	_PPTGDataSourceModifyOperation n;
#endif
{
	PTG_OUTPUT_STRING(f, "	protected void ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ") throws Exception {");
	IndentNewLine(f);
	IndentIncr(f);
	PTG_OUTPUT_STRING(f, "		");
	n->p1->_print(n->p1);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	}\n");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGDataSourceModifyOperation(PTGNode p1, PTGNode p2, char * p3)
#else
PTGNode PTGDataSourceModifyOperation(p1, p2, p3)
PTGNode p1;
PTGNode p2;
char * p3;
#endif
{
	_PPTGDataSourceModifyOperation n;

	n = (_PPTGDataSourceModifyOperation)MALLOC(sizeof(struct _SPTGDataSourceModifyOperation));
	n->_print = (_PTGProc)_PrPTGDataSourceModifyOperation;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern DBInsertBody */

typedef struct _SPTGDBInsertBody{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGDBInsertBody;

#ifdef PROTO_OK
static void _PrPTGDBInsertBody(_PPTGDBInsertBody n)
#else
static void _PrPTGDBInsertBody(n)
	_PPTGDBInsertBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " = new ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "();\n");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "this.getServiceLocator().get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao().save(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ");\n");
}

#ifdef PROTO_OK
PTGNode PTGDBInsertBody(char * p1, char * p2, char * p3)
#else
PTGNode PTGDBInsertBody(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGDBInsertBody n;

	n = (_PPTGDBInsertBody)MALLOC(sizeof(struct _SPTGDBInsertBody));
	n->_print = (_PTGProc)_PrPTGDBInsertBody;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern DBDeleteBody */

typedef struct _SPTGDBDeleteBody{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
} * _PPTGDBDeleteBody;

#ifdef PROTO_OK
static void _PrPTGDBDeleteBody(_PPTGDBDeleteBody n)
#else
static void _PrPTGDBDeleteBody(n)
	_PPTGDBDeleteBody n;
#endif
{
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "this.getServiceLocator().get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao().delete(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\", _values);\n");
}

#ifdef PROTO_OK
PTGNode PTGDBDeleteBody(char * p1, char * p2, PTGNode p3)
#else
PTGNode PTGDBDeleteBody(p1, p2, p3)
char * p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGDBDeleteBody n;

	n = (_PPTGDBDeleteBody)MALLOC(sizeof(struct _SPTGDBDeleteBody));
	n->_print = (_PTGProc)_PrPTGDBDeleteBody;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern DBUpdateBody */

typedef struct _SPTGDBUpdateBody{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	PTGNode p3;
} * _PPTGDBUpdateBody;

#ifdef PROTO_OK
static void _PrPTGDBUpdateBody(_PPTGDBUpdateBody n)
#else
static void _PrPTGDBUpdateBody(n)
	_PPTGDBUpdateBody n;
#endif
{
	n->p3->_print(n->p3);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "this.getHibernate().bulkUpdate(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\", _values);\n");
}

#ifdef PROTO_OK
PTGNode PTGDBUpdateBody(PTGNode p1, char * p2, PTGNode p3)
#else
PTGNode PTGDBUpdateBody(p1, p2, p3)
PTGNode p1;
char * p2;
PTGNode p3;
#endif
{
	_PPTGDBUpdateBody n;

	n = (_PPTGDBUpdateBody)MALLOC(sizeof(struct _SPTGDBUpdateBody));
	n->_print = (_PTGProc)_PrPTGDBUpdateBody;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern BindingValuesMap */

typedef struct _SPTGBindingValuesMap{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGBindingValuesMap;

#ifdef PROTO_OK
static void _PrPTGBindingValuesMap(_PPTGBindingValuesMap n)
#else
static void _PrPTGBindingValuesMap(n)
	_PPTGBindingValuesMap n;
#endif
{
	PTG_OUTPUT_STRING(f, "Map<String,Object> _values = PWUtils.toValueMap(new String[]{");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "},");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");\n");
}

#ifdef PROTO_OK
PTGNode PTGBindingValuesMap(PTGNode p1, PTGNode p2)
#else
PTGNode PTGBindingValuesMap(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGBindingValuesMap n;

	n = (_PPTGBindingValuesMap)MALLOC(sizeof(struct _SPTGBindingValuesMap));
	n->_print = (_PTGProc)_PrPTGBindingValuesMap;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern UpdateBlobCode */

typedef struct _SPTGUpdateBlobCode{
	_PTGProc _print;
	char * p1;
} * _PPTGUpdateBlobCode;

#ifdef PROTO_OK
static void _PrPTGUpdateBlobCode(_PPTGUpdateBlobCode n)
#else
static void _PrPTGUpdateBlobCode(n)
	_PPTGUpdateBlobCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "	if(");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " != null ){\n");
	PTG_OUTPUT_STRING(f, "		String __fileName = PWUtils.saveUploadedFile( this.getUploadedFilesDir(), ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " );\n");
	PTG_OUTPUT_STRING(f, "		_values.put(\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_path\", Constants.UPLOAD_DIR + \"/\" + __fileName);\n");
	PTG_OUTPUT_STRING(f, "		_values.put(\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_contenttype\", ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".getContentType());\n");
	PTG_OUTPUT_STRING(f, "	}else{\n");
	PTG_OUTPUT_STRING(f, "		_values.put(\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_path\", null);\n");
	PTG_OUTPUT_STRING(f, "		_values.put(\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_contenttype\", null);\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGUpdateBlobCode(char * p1)
#else
PTGNode PTGUpdateBlobCode(p1)
char * p1;
#endif
{
	_PPTGUpdateBlobCode n;

	n = (_PPTGUpdateBlobCode)MALLOC(sizeof(struct _SPTGUpdateBlobCode));
	n->_print = (_PTGProc)_PrPTGUpdateBlobCode;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceTableSelect */

typedef struct _SPTGDataSourceTableSelect{
	_PTGProc _print;
	char * p1;
} * _PPTGDataSourceTableSelect;

#ifdef PROTO_OK
static void _PrPTGDataSourceTableSelect(_PPTGDataSourceTableSelect n)
#else
static void _PrPTGDataSourceTableSelect(n)
	_PPTGDataSourceTableSelect n;
#endif
{
	PTG_OUTPUT_STRING(f, "this.getServiceLocator().get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao().getAll");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "()");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceTableSelect(char * p1)
#else
PTGNode PTGDataSourceTableSelect(p1)
char * p1;
#endif
{
	_PPTGDataSourceTableSelect n;

	n = (_PPTGDataSourceTableSelect)MALLOC(sizeof(struct _SPTGDataSourceTableSelect));
	n->_print = (_PTGProc)_PrPTGDataSourceTableSelect;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceProjectionSelect */

typedef struct _SPTGDataSourceProjectionSelect{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGDataSourceProjectionSelect;

#ifdef PROTO_OK
static void _PrPTGDataSourceProjectionSelect(_PPTGDataSourceProjectionSelect n)
#else
static void _PrPTGDataSourceProjectionSelect(n)
	_PPTGDataSourceProjectionSelect n;
#endif
{
	PTG_OUTPUT_STRING(f, "this.getServiceLocator().getDatabaseService().");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ")");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceProjectionSelect(char * p1, PTGNode p2)
#else
PTGNode PTGDataSourceProjectionSelect(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGDataSourceProjectionSelect n;

	n = (_PPTGDataSourceProjectionSelect)MALLOC(sizeof(struct _SPTGDataSourceProjectionSelect));
	n->_print = (_PTGProc)_PrPTGDataSourceProjectionSelect;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceProjectionMethod */

typedef struct _SPTGDataSourceProjectionMethod{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGDataSourceProjectionMethod;

#ifdef PROTO_OK
static void _PrPTGDataSourceProjectionMethod(_PPTGDataSourceProjectionMethod n)
#else
static void _PrPTGDataSourceProjectionMethod(n)
	_PPTGDataSourceProjectionMethod n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public List ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		return ((ExtHibernateTemplate)this.getHibernateTemplate()).findByNamedParam(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\", _values);\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceProjectionMethod(char * p1, char * p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGDataSourceProjectionMethod(p1, p2, p3, p4)
char * p1;
char * p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGDataSourceProjectionMethod n;

	n = (_PPTGDataSourceProjectionMethod)MALLOC(sizeof(struct _SPTGDataSourceProjectionMethod));
	n->_print = (_PTGProc)_PrPTGDataSourceProjectionMethod;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceProjectionMethodWithoutParams */

typedef struct _SPTGDataSourceProjectionMethodWithoutParams{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGDataSourceProjectionMethodWithoutParams;

#ifdef PROTO_OK
static void _PrPTGDataSourceProjectionMethodWithoutParams(_PPTGDataSourceProjectionMethodWithoutParams n)
#else
static void _PrPTGDataSourceProjectionMethodWithoutParams(n)
	_PPTGDataSourceProjectionMethodWithoutParams n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public List ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return this.getHibernateTemplate().find(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\");\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceProjectionMethodWithoutParams(char * p1, char * p2)
#else
PTGNode PTGDataSourceProjectionMethodWithoutParams(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGDataSourceProjectionMethodWithoutParams n;

	n = (_PPTGDataSourceProjectionMethodWithoutParams)MALLOC(sizeof(struct _SPTGDataSourceProjectionMethodWithoutParams));
	n->_print = (_PTGProc)_PrPTGDataSourceProjectionMethodWithoutParams;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern LimitDataSourceMethod */

typedef struct _SPTGLimitDataSourceMethod{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
	PTGNode p4;
	char * p5;
	char * p6;
} * _PPTGLimitDataSourceMethod;

#ifdef PROTO_OK
static void _PrPTGLimitDataSourceMethod(_PPTGLimitDataSourceMethod n)
#else
static void _PrPTGLimitDataSourceMethod(n)
	_PPTGLimitDataSourceMethod n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public List ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		return ((ExtHibernateTemplate)this.getHibernateTemplate()).findByNamedParam(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\", _values, ");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ", ");
	PTG_OUTPUT_STRING(f, n->p6);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGLimitDataSourceMethod(char * p1, char * p2, PTGNode p3, PTGNode p4, char * p5, char * p6)
#else
PTGNode PTGLimitDataSourceMethod(p1, p2, p3, p4, p5, p6)
char * p1;
char * p2;
PTGNode p3;
PTGNode p4;
char * p5;
char * p6;
#endif
{
	_PPTGLimitDataSourceMethod n;

	n = (_PPTGLimitDataSourceMethod)MALLOC(sizeof(struct _SPTGLimitDataSourceMethod));
	n->_print = (_PTGProc)_PrPTGLimitDataSourceMethod;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern LimitDataSourceMethodWithoutParams */

typedef struct _SPTGLimitDataSourceMethodWithoutParams{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
} * _PPTGLimitDataSourceMethodWithoutParams;

#ifdef PROTO_OK
static void _PrPTGLimitDataSourceMethodWithoutParams(_PPTGLimitDataSourceMethodWithoutParams n)
#else
static void _PrPTGLimitDataSourceMethodWithoutParams(n)
	_PPTGLimitDataSourceMethodWithoutParams n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public List ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return ((ExtHibernateTemplate)this.getHibernateTemplate()).find(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\", ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ", ");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n");
}

#ifdef PROTO_OK
PTGNode PTGLimitDataSourceMethodWithoutParams(char * p1, char * p2, char * p3, char * p4)
#else
PTGNode PTGLimitDataSourceMethodWithoutParams(p1, p2, p3, p4)
char * p1;
char * p2;
char * p3;
char * p4;
#endif
{
	_PPTGLimitDataSourceMethodWithoutParams n;

	n = (_PPTGLimitDataSourceMethodWithoutParams)MALLOC(sizeof(struct _SPTGLimitDataSourceMethodWithoutParams));
	n->_print = (_PTGProc)_PrPTGLimitDataSourceMethodWithoutParams;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceVariableExpression */

typedef struct _SPTGDataSourceVariableExpression{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGDataSourceVariableExpression;

#ifdef PROTO_OK
static void _PrPTGDataSourceVariableExpression(_PPTGDataSourceVariableExpression n)
#else
static void _PrPTGDataSourceVariableExpression(n)
	_PPTGDataSourceVariableExpression n;
#endif
{
	PTG_OUTPUT_STRING(f, "$_REQUEST['");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'];");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceVariableExpression(PTGNode p1)
#else
PTGNode PTGDataSourceVariableExpression(p1)
PTGNode p1;
#endif
{
	_PPTGDataSourceVariableExpression n;

	n = (_PPTGDataSourceVariableExpression)MALLOC(sizeof(struct _SPTGDataSourceVariableExpression));
	n->_print = (_PTGProc)_PrPTGDataSourceVariableExpression;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ConstantsClass */

typedef struct _SPTGConstantsClass{
	_PTGProc _print;
	char * p1;
} * _PPTGConstantsClass;

#ifdef PROTO_OK
static void _PrPTGConstantsClass(_PPTGConstantsClass n)
#else
static void _PrPTGConstantsClass(n)
	_PPTGConstantsClass n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view;\n\n");
	PTG_OUTPUT_STRING(f, "public abstract class Constants {\n\n");
	PTG_OUTPUT_STRING(f, "	public static final String UPLOAD_DIR = \"uploadedfiles\";\n\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGConstantsClass(char * p1)
#else
PTGNode PTGConstantsClass(p1)
char * p1;
#endif
{
	_PPTGConstantsClass n;

	n = (_PPTGConstantsClass)MALLOC(sizeof(struct _SPTGConstantsClass));
	n->_print = (_PTGProc)_PrPTGConstantsClass;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorInterface */

typedef struct _SPTGServiceLocatorInterface{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGServiceLocatorInterface;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorInterface(_PPTGServiceLocatorInterface n)
#else
static void _PrPTGServiceLocatorInterface(n)
	_PPTGServiceLocatorInterface n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.service.mail.MailingService;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.DatabaseService;\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "public interface ServiceLocator {\n\n");
	PTG_OUTPUT_STRING(f, "	public MailingService getMailingService();\n");
	PTG_OUTPUT_STRING(f, "	public DatabaseService getDatabaseService();\n");
	PTG_OUTPUT_STRING(f, "	public Object lookup( String beanId );\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorInterface(char * p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGServiceLocatorInterface(p1, p2, p3)
char * p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGServiceLocatorInterface n;

	n = (_PPTGServiceLocatorInterface)MALLOC(sizeof(struct _SPTGServiceLocatorInterface));
	n->_print = (_PTGProc)_PrPTGServiceLocatorInterface;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorInterfaceDaoMethod */

typedef struct _SPTGServiceLocatorInterfaceDaoMethod{
	_PTGProc _print;
	char * p1;
} * _PPTGServiceLocatorInterfaceDaoMethod;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorInterfaceDaoMethod(_PPTGServiceLocatorInterfaceDaoMethod n)
#else
static void _PrPTGServiceLocatorInterfaceDaoMethod(n)
	_PPTGServiceLocatorInterfaceDaoMethod n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao();");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorInterfaceDaoMethod(char * p1)
#else
PTGNode PTGServiceLocatorInterfaceDaoMethod(p1)
char * p1;
#endif
{
	_PPTGServiceLocatorInterfaceDaoMethod n;

	n = (_PPTGServiceLocatorInterfaceDaoMethod)MALLOC(sizeof(struct _SPTGServiceLocatorInterfaceDaoMethod));
	n->_print = (_PTGProc)_PrPTGServiceLocatorInterfaceDaoMethod;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorBean */

typedef struct _SPTGServiceLocatorBean{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
} * _PPTGServiceLocatorBean;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorBean(_PPTGServiceLocatorBean n)
#else
static void _PrPTGServiceLocatorBean(n)
	_PPTGServiceLocatorBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean;\n\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.Log;\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.LogFactory;\n");
	PTG_OUTPUT_STRING(f, "import org.springframework.context.ApplicationContext;\n");
	PTG_OUTPUT_STRING(f, "import org.springframework.web.context.support.WebApplicationContextUtils;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.service.mail.MailingService;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.FacesUtils;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.DatabaseService;\n");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.ServiceLocator;\n\n");
	PTG_OUTPUT_STRING(f, "public class ServiceLocatorBean implements ServiceLocator, java.io.Serializable {\n\n");
	PTG_OUTPUT_STRING(f, "	private Log logger = LogFactory.getLog(this.getClass());\n");
	PTG_OUTPUT_STRING(f, "	private ApplicationContext springContext = null;\n");
	PTG_OUTPUT_STRING(f, "	private MailingService mailingService = null;\n");
	PTG_OUTPUT_STRING(f, "	private DatabaseService databaseService = null;\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "	public ServiceLocatorBean() {\n");
	PTG_OUTPUT_STRING(f, "		this(FacesUtils.getServletContext());\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public ServiceLocatorBean(javax.servlet.ServletContext servletCtx) {\n");
	PTG_OUTPUT_STRING(f, "		this.springContext = WebApplicationContextUtils.getRequiredWebApplicationContext(servletCtx);\n");
	PTG_OUTPUT_STRING(f, "		this.mailingService = (MailingService)this.lookup( \"mailingService\" );\n");
	PTG_OUTPUT_STRING(f, "		this.databaseService = (DatabaseService)this.lookup( \"databaseService\" );\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		this.logger.info(\"servicelocator bean initialized!\");\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/**\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.ServiceLocator#getMailingService()\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public MailingService getMailingService(){\n");
	PTG_OUTPUT_STRING(f, "		return this.mailingService;\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public DatabaseService getDatabaseService(){\n");
	PTG_OUTPUT_STRING(f, "		return this.databaseService;\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/**\n");
	PTG_OUTPUT_STRING(f, "	 * Lookups a spring-bean\n");
	PTG_OUTPUT_STRING(f, "	 * @param beanId\n");
	PTG_OUTPUT_STRING(f, "	 * @return\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.ServiceLocator#lookup(String beanId)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public Object lookup( String beanId ){\n");
	PTG_OUTPUT_STRING(f, "		return this.springContext.getBean( beanId );\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorBean(char * p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5)
#else
PTGNode PTGServiceLocatorBean(p1, p2, p3, p4, p5)
char * p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
#endif
{
	_PPTGServiceLocatorBean n;

	n = (_PPTGServiceLocatorBean)MALLOC(sizeof(struct _SPTGServiceLocatorBean));
	n->_print = (_PTGProc)_PrPTGServiceLocatorBean;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorAttributeDaoDef */

typedef struct _SPTGServiceLocatorAttributeDaoDef{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGServiceLocatorAttributeDaoDef;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorAttributeDaoDef(_PPTGServiceLocatorAttributeDaoDef n)
#else
static void _PrPTGServiceLocatorAttributeDaoDef(n)
	_PPTGServiceLocatorAttributeDaoDef n;
#endif
{
	PTG_OUTPUT_STRING(f, "	private ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao = null;");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorAttributeDaoDef(char * p1, char * p2)
#else
PTGNode PTGServiceLocatorAttributeDaoDef(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGServiceLocatorAttributeDaoDef n;

	n = (_PPTGServiceLocatorAttributeDaoDef)MALLOC(sizeof(struct _SPTGServiceLocatorAttributeDaoDef));
	n->_print = (_PTGProc)_PrPTGServiceLocatorAttributeDaoDef;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorInitDao */

typedef struct _SPTGServiceLocatorInitDao{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGServiceLocatorInitDao;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorInitDao(_PPTGServiceLocatorInitDao n)
#else
static void _PrPTGServiceLocatorInitDao(n)
	_PPTGServiceLocatorInitDao n;
#endif
{
	PTG_OUTPUT_STRING(f, "		this.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao = (");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao)this.lookup( \"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao\");");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorInitDao(char * p1, char * p2)
#else
PTGNode PTGServiceLocatorInitDao(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGServiceLocatorInitDao n;

	n = (_PPTGServiceLocatorInitDao)MALLOC(sizeof(struct _SPTGServiceLocatorInitDao));
	n->_print = (_PTGProc)_PrPTGServiceLocatorInitDao;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorMethodDao */

typedef struct _SPTGServiceLocatorMethodDao{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGServiceLocatorMethodDao;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorMethodDao(_PPTGServiceLocatorMethodDao n)
#else
static void _PrPTGServiceLocatorMethodDao(n)
	_PPTGServiceLocatorMethodDao n;
#endif
{
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao(){\n");
	PTG_OUTPUT_STRING(f, "		return this.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao;\n");
	PTG_OUTPUT_STRING(f, "	}");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorMethodDao(char * p1, char * p2)
#else
PTGNode PTGServiceLocatorMethodDao(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGServiceLocatorMethodDao n;

	n = (_PPTGServiceLocatorMethodDao)MALLOC(sizeof(struct _SPTGServiceLocatorMethodDao));
	n->_print = (_PTGProc)_PrPTGServiceLocatorMethodDao;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ServiceLocatorServletSupport */

typedef struct _SPTGServiceLocatorServletSupport{
	_PTGProc _print;
	char * p1;
} * _PPTGServiceLocatorServletSupport;

#ifdef PROTO_OK
static void _PrPTGServiceLocatorServletSupport(_PPTGServiceLocatorServletSupport n)
#else
static void _PrPTGServiceLocatorServletSupport(n)
	_PPTGServiceLocatorServletSupport n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".controller;\n\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.Log;\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.LogFactory;\n\n");
	PTG_OUTPUT_STRING(f, "import java.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import java.io.IOException;\n\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.ServletException;\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.http.HttpServlet;\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.http.HttpServletRequest;\n");
	PTG_OUTPUT_STRING(f, "import javax.servlet.http.HttpServletResponse;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.view.Key;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import org.springframework.jdbc.core.JdbcTemplate;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.ServiceLocator;\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.Constants;\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean.ServiceLocatorBean;\n\n");
	PTG_OUTPUT_STRING(f, "public class ServiceLocatorServletSupport extends HttpServlet {\n\n");
	PTG_OUTPUT_STRING(f, "	private static final long serialVersionUID = 8976222714612094606L;\n");
	PTG_OUTPUT_STRING(f, "	protected final Log logger = LogFactory.getLog(this.getClass());\n\n");
	PTG_OUTPUT_STRING(f, "	private ServiceLocator serviceLocator = null;\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, " 	 * @see javax.servlet.GenericServlet#init()\n");
	PTG_OUTPUT_STRING(f, " 	 */\n");
	PTG_OUTPUT_STRING(f, "	@Override\n");
	PTG_OUTPUT_STRING(f, "	public void init() throws ServletException {\n\n");
	PTG_OUTPUT_STRING(f, "		this.serviceLocator = (ServiceLocator)this.getServletContext().getAttribute(\"serviceLocatorBean\");\n");
	PTG_OUTPUT_STRING(f, "		if(this.serviceLocator != null) {\n");
	PTG_OUTPUT_STRING(f, "			return;\n");
	PTG_OUTPUT_STRING(f, "		}\n\n");
	PTG_OUTPUT_STRING(f, "		this.serviceLocator = new ServiceLocatorBean(this.getServletContext());\n");
	PTG_OUTPUT_STRING(f, "		this.getServletContext().setAttribute( \"serviceLocatorBean\", this.serviceLocator );\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected ServiceLocator getServiceLocator() {\n");
	PTG_OUTPUT_STRING(f, "		return this.serviceLocator;\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected String md5( String string ) {\n");
	PTG_OUTPUT_STRING(f, "		return AuthManager.md5(string);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected JdbcTemplate getJdbc() {\n");
	PTG_OUTPUT_STRING(f, "		return this.serviceLocator.getDatabaseService().getJdbcTemplate();\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected ExtHibernateTemplate getHibernate() {\n");
	PTG_OUTPUT_STRING(f, "		return (ExtHibernateTemplate)this.serviceLocator.getDatabaseService().getHibernateTemplate();\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected String getUploadedFilesDir() {\n");
	PTG_OUTPUT_STRING(f, "		return this.getServletContext().getRealPath(\"/\" + Constants.UPLOAD_DIR);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected void addInfoMessage( HttpServletRequest request, String msg ) {\n");
	PTG_OUTPUT_STRING(f, "		this.addMessage(request, Key.INFO_MESSAGES, msg);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected void addErrorMessage( HttpServletRequest request, String msg ) {\n");
	PTG_OUTPUT_STRING(f, "		this.addMessage(request, Key.ERROR_MESSAGES, msg);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	private synchronized void addMessage( HttpServletRequest request, String key, String msg ) {\n");
	PTG_OUTPUT_STRING(f, "		Collection msgs= (Collection) request.getAttribute(key);\n");
	PTG_OUTPUT_STRING(f, "		if(msgs== null) {\n");
	PTG_OUTPUT_STRING(f, "			msgs = new ArrayList();\n");
	PTG_OUTPUT_STRING(f, "		}\n");
	PTG_OUTPUT_STRING(f, "		msgs.add(msg);\n");
	PTG_OUTPUT_STRING(f, "		request.setAttribute(key, msgs);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Java-doc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	@Override\n");
	PTG_OUTPUT_STRING(f, "	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {\n");
	PTG_OUTPUT_STRING(f, "		this.doGet(request, response);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGServiceLocatorServletSupport(char * p1)
#else
PTGNode PTGServiceLocatorServletSupport(p1)
char * p1;
#endif
{
	_PPTGServiceLocatorServletSupport n;

	n = (_PPTGServiceLocatorServletSupport)MALLOC(sizeof(struct _SPTGServiceLocatorServletSupport));
	n->_print = (_PTGProc)_PrPTGServiceLocatorServletSupport;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DaoImport */

typedef struct _SPTGDaoImport{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGDaoImport;

#ifdef PROTO_OK
static void _PrPTGDaoImport(_PPTGDaoImport n)
#else
static void _PrPTGDaoImport(n)
	_PPTGDaoImport n;
#endif
{
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao;");
}

#ifdef PROTO_OK
PTGNode PTGDaoImport(char * p1, char * p2)
#else
PTGNode PTGDaoImport(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGDaoImport n;

	n = (_PPTGDaoImport)MALLOC(sizeof(struct _SPTGDaoImport));
	n->_print = (_PTGProc)_PrPTGDaoImport;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern ModelImport */

typedef struct _SPTGModelImport{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGModelImport;

#ifdef PROTO_OK
static void _PrPTGModelImport(_PPTGModelImport n)
#else
static void _PrPTGModelImport(n)
	_PPTGModelImport n;
#endif
{
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ";");
}

#ifdef PROTO_OK
PTGNode PTGModelImport(char * p1, char * p2)
#else
PTGNode PTGModelImport(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGModelImport n;

	n = (_PPTGModelImport)MALLOC(sizeof(struct _SPTGModelImport));
	n->_print = (_PTGProc)_PrPTGModelImport;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern BeanModelImports */

typedef struct _SPTGBeanModelImports{
	_PTGProc _print;
	char * p1;
} * _PPTGBeanModelImports;

#ifdef PROTO_OK
static void _PrPTGBeanModelImports(_PPTGBeanModelImports n)
#else
static void _PrPTGBeanModelImports(n)
	_PPTGBeanModelImports n;
#endif
{
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.*;\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.*;\n");
}

#ifdef PROTO_OK
PTGNode PTGBeanModelImports(char * p1)
#else
PTGNode PTGBeanModelImports(p1)
char * p1;
#endif
{
	_PPTGBeanModelImports n;

	n = (_PPTGBeanModelImports)MALLOC(sizeof(struct _SPTGBeanModelImports));
	n->_print = (_PTGProc)_PrPTGBeanModelImports;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DatabaseService */

typedef struct _SPTGDatabaseService{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGDatabaseService;

#ifdef PROTO_OK
static void _PrPTGDatabaseService(_PPTGDatabaseService n)
#else
static void _PrPTGDatabaseService(n)
	_PPTGDatabaseService n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao;\n\n");
	PTG_OUTPUT_STRING(f, "import java.util.*;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.*;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.dao.JdbcPlusHibernateDaoSupport;\n\n");
	PTG_OUTPUT_STRING(f, "public class DatabaseService extends JdbcPlusHibernateDaoSupport {\n\n");
	PTG_OUTPUT_STRING(f, "	public Object checkLogin( String hql, String u, String p) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		List list = this.getHibernateTemplate().find( hql, new Object []{u,p});\n");
	PTG_OUTPUT_STRING(f, "		if(list == null || list.isEmpty()) {\n");
	PTG_OUTPUT_STRING(f, "			throw new Exception(\"Password mismatch for user '\" + u +\"'\");\n");
	PTG_OUTPUT_STRING(f, "		}\n");
	PTG_OUTPUT_STRING(f, "		return list.get(0);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGDatabaseService(char * p1, PTGNode p2)
#else
PTGNode PTGDatabaseService(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGDatabaseService n;

	n = (_PPTGDatabaseService)MALLOC(sizeof(struct _SPTGDatabaseService));
	n->_print = (_PTGProc)_PrPTGDatabaseService;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern AbstractBean */

typedef struct _SPTGAbstractBean{
	_PTGProc _print;
	char * p1;
} * _PPTGAbstractBean;

#ifdef PROTO_OK
static void _PrPTGAbstractBean(_PPTGAbstractBean n)
#else
static void _PrPTGAbstractBean(n)
	_PPTGAbstractBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean;\n\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.Log;\n");
	PTG_OUTPUT_STRING(f, "import org.apache.commons.logging.LogFactory;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.ServiceLocator;\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.Constants;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.FacesUtils;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.ExtHibernateTemplate;\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.AuthManager;\n\n");
	PTG_OUTPUT_STRING(f, "import org.springframework.jdbc.core.JdbcTemplate;\n\n");
	PTG_OUTPUT_STRING(f, "/**\n");
	PTG_OUTPUT_STRING(f, " * Baseclass for all JSF managed beans\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "public abstract class AbstractBean implements java.io.Serializable {\n");
	PTG_OUTPUT_STRING(f, "	protected final Log logger = LogFactory.getLog(this.getClass());\n\n");
	PTG_OUTPUT_STRING(f, "	//the service locator of the business services\n");
	PTG_OUTPUT_STRING(f, "	protected ServiceLocator serviceLocator;\n\n");
	PTG_OUTPUT_STRING(f, "	public AbstractBean() {\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public ServiceLocator getServiceLocator() {\n");
	PTG_OUTPUT_STRING(f, "		return this.serviceLocator;\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public void setServiceLocator(ServiceLocator serviceLocator) {\n");
	PTG_OUTPUT_STRING(f, "		this.logger.debug(\"servicelocator has been set!\");\n");
	PTG_OUTPUT_STRING(f, "		this.serviceLocator = serviceLocator;\n");
	PTG_OUTPUT_STRING(f, "		this.init();\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public String getDummyVariable() {\n");
	PTG_OUTPUT_STRING(f, "		return null;\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected JdbcTemplate getJdbc() {\n");
	PTG_OUTPUT_STRING(f, "		return this.serviceLocator.getDatabaseService().getJdbcTemplate();\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected ExtHibernateTemplate getHibernate() {\n");
	PTG_OUTPUT_STRING(f, "		return (ExtHibernateTemplate)this.serviceLocator.getDatabaseService().getHibernateTemplate();\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected String md5( String str ) {\n");
	PTG_OUTPUT_STRING(f, "		return AuthManager.md5( str );\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	protected String getUploadedFilesDir() {\n");
	PTG_OUTPUT_STRING(f, "		return FacesUtils.getServletContext().getRealPath(\"/\" + Constants.UPLOAD_DIR);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	public void setSessionAttribute( String attr, Object value ) {\n");
	PTG_OUTPUT_STRING(f, "		FacesUtils.setSessionAttribute(attr, value);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/**\n");
	PTG_OUTPUT_STRING(f, "	 * Used to initialize the managed bean.\n");
	PTG_OUTPUT_STRING(f, "	 * <p>\n");
	PTG_OUTPUT_STRING(f, "	 * Called after the service locator is set.\n");
	PTG_OUTPUT_STRING(f, "	 * It is a workaround.\n");
	PTG_OUTPUT_STRING(f, "	 * <p>\n");
	PTG_OUTPUT_STRING(f, "	 * Once the JSF bean management facility can support init method,\n");
	PTG_OUTPUT_STRING(f, "	 * the init method can be configured and called from the JSF implementation directly.\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	protected void init() {\n");
	PTG_OUTPUT_STRING(f, "	}\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGAbstractBean(char * p1)
#else
PTGNode PTGAbstractBean(p1)
char * p1;
#endif
{
	_PPTGAbstractBean n;

	n = (_PPTGAbstractBean)MALLOC(sizeof(struct _SPTGAbstractBean));
	n->_print = (_PTGProc)_PrPTGAbstractBean;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ApplicationBean */

typedef struct _SPTGApplicationBean{
	_PTGProc _print;
	char * p1;
} * _PPTGApplicationBean;

#ifdef PROTO_OK
static void _PrPTGApplicationBean(_PPTGApplicationBean n)
#else
static void _PrPTGApplicationBean(n)
	_PPTGApplicationBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean.AbstractBean;\n\n");
	PTG_OUTPUT_STRING(f, "public class ApplicationBean extends AbstractBean{\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGApplicationBean(char * p1)
#else
PTGNode PTGApplicationBean(p1)
char * p1;
#endif
{
	_PPTGApplicationBean n;

	n = (_PPTGApplicationBean)MALLOC(sizeof(struct _SPTGApplicationBean));
	n->_print = (_PTGProc)_PrPTGApplicationBean;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern SessionBean */

typedef struct _SPTGSessionBean{
	_PTGProc _print;
	char * p1;
} * _PPTGSessionBean;

#ifdef PROTO_OK
static void _PrPTGSessionBean(_PPTGSessionBean n)
#else
static void _PrPTGSessionBean(n)
	_PPTGSessionBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".view.bean;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.*;\n\n");
	PTG_OUTPUT_STRING(f, "/**\n");
	PTG_OUTPUT_STRING(f, " * The managed bean with session scope.\n");
	PTG_OUTPUT_STRING(f, " * <p>\n");
	PTG_OUTPUT_STRING(f, " * It is used as a session scope cache.\n");
	PTG_OUTPUT_STRING(f, " * In JSF, the properties are set by bean management facility.\n");
	PTG_OUTPUT_STRING(f, " * </p>\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "public class SessionBean extends AbstractBean{\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGSessionBean(char * p1)
#else
PTGNode PTGSessionBean(p1)
char * p1;
#endif
{
	_PPTGSessionBean n;

	n = (_PPTGSessionBean)MALLOC(sizeof(struct _SPTGSessionBean));
	n->_print = (_PTGProc)_PrPTGSessionBean;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DaoInterface */

typedef struct _SPTGDaoInterface{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
} * _PPTGDaoInterface;

#ifdef PROTO_OK
static void _PrPTGDaoInterface(_PPTGDaoInterface n)
#else
static void _PrPTGDaoInterface(n)
	_PPTGDaoInterface n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao;\n\n");
	PTG_OUTPUT_STRING(f, "import java.util.Map;\n");
	PTG_OUTPUT_STRING(f, "import java.util.List;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ";\n\n");
	PTG_OUTPUT_STRING(f, "public interface ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao {\n\n");
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "ById( ");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, " id ) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "( String hqlWhere, Map<String,Object> bindingValues) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public List getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "() throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public List getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "( String hqlWhere, Map<String,Object> bindingValues) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public void save( ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " ) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public void update( ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " ) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public void delete( ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " ) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public int delete( ");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, " id) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public int delete( String hqlWhere, Object... bindingValues) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "	public int delete( String hqlWhere, Map<String,Object> bindingValues) throws Exception;\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGDaoInterface(char * p1, char * p2, char * p3, char * p4)
#else
PTGNode PTGDaoInterface(p1, p2, p3, p4)
char * p1;
char * p2;
char * p3;
char * p4;
#endif
{
	_PPTGDaoInterface n;

	n = (_PPTGDaoInterface)MALLOC(sizeof(struct _SPTGDaoInterface));
	n->_print = (_PTGProc)_PrPTGDaoInterface;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern DaoHibernateImpl */

typedef struct _SPTGDaoHibernateImpl{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
} * _PPTGDaoHibernateImpl;

#ifdef PROTO_OK
static void _PrPTGDaoHibernateImpl(_PPTGDaoHibernateImpl n)
#else
static void _PrPTGDaoHibernateImpl(n)
	_PPTGDaoHibernateImpl n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*\n");
	PTG_OUTPUT_STRING(f, " * Generated with PaderWAVE2\n");
	PTG_OUTPUT_STRING(f, " */\n");
	PTG_OUTPUT_STRING(f, "package ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.hibernate;\n\n");
	PTG_OUTPUT_STRING(f, "import java.util.Map;\n");
	PTG_OUTPUT_STRING(f, "import java.util.List;\n\n");
	PTG_OUTPUT_STRING(f, "import org.springframework.orm.hibernate3.support.HibernateDaoSupport;\n\n");
	PTG_OUTPUT_STRING(f, "import de.upb.paderwave.util.ExtHibernateTemplate;\n\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ";\n");
	PTG_OUTPUT_STRING(f, "import ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao;\n\n");
	PTG_OUTPUT_STRING(f, "public class ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "DaoHibernateImpl extends HibernateDaoSupport implements ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao {\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "ById(");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, ")\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "ById(");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, " id) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return (");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ")this.getHibernateTemplate().load(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".class, id );\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "(java.lang.String,java.util.Map)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " get");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "(String hqlWhere, Map<String,Object> bindingValues) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return (");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ")this.getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "( hqlWhere, bindingValues ).get(0);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "()\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public List getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "() throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return this.getHibernateTemplate().loadAll(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".class);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "(java.lang.String,java.util.Map)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public List getAll");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "(String hqlWhere, Map<String,Object> bindingValues) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return ((ExtHibernateTemplate)this.getHibernateTemplate()).findByNamedParam( \"from ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " \" + hqlWhere, bindingValues );\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#save(");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ")\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public void save(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		this.getHibernateTemplate().save(");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#update(");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ")\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public void update(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		this.getHibernateTemplate().update(");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#delete(");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ")\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public void delete(");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		this.getHibernateTemplate().delete(");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#delete(");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, ")\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public int delete(");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ") throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return this.getHibernateTemplate().bulkUpdate(\"delete ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " where ");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, " = ?\", ");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, ");\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#delete(java.lang.String, java.lang.Object...)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public int delete( String hqlWhere, Object... bindingValues ) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return this.getHibernateTemplate().bulkUpdate(\"delete ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " \" + hqlWhere, bindingValues);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "	/* (non-Javadoc)\n");
	PTG_OUTPUT_STRING(f, "	 * @see ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model.dao.");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Dao#delete(java.lang.String, java.util.Map)\n");
	PTG_OUTPUT_STRING(f, "	 */\n");
	PTG_OUTPUT_STRING(f, "	public int delete( String hqlWhere, Map<String,Object> bindingValues) throws Exception {\n");
	PTG_OUTPUT_STRING(f, "		return ((ExtHibernateTemplate)this.getHibernateTemplate()).bulkUpdate(\"delete ");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, " ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, " \" + hqlWhere, bindingValues);\n");
	PTG_OUTPUT_STRING(f, "	}\n\n");
	PTG_OUTPUT_STRING(f, "}\n");
}

#ifdef PROTO_OK
PTGNode PTGDaoHibernateImpl(char * p1, char * p2, char * p3, char * p4, char * p5)
#else
PTGNode PTGDaoHibernateImpl(p1, p2, p3, p4, p5)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
#endif
{
	_PPTGDaoHibernateImpl n;

	n = (_PPTGDaoHibernateImpl)MALLOC(sizeof(struct _SPTGDaoHibernateImpl));
	n->_print = (_PTGProc)_PrPTGDaoHibernateImpl;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	return (PTGNode)n;
}

/* Implementation of Pattern SpringBeansFile */

typedef struct _SPTGSpringBeansFile{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGSpringBeansFile;

#ifdef PROTO_OK
static void _PrPTGSpringBeansFile(_PPTGSpringBeansFile n)
#else
static void _PrPTGSpringBeansFile(n)
	_PPTGSpringBeansFile n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	PTG_OUTPUT_STRING(f, "<!DOCTYPE beans PUBLIC \"-//SPRING//DTD BEAN//EN\" \"http://www.springframework.org/dtd/spring-beans.dtd\">\n");
	PTG_OUTPUT_STRING(f, "<beans>\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</beans>\n");
}

#ifdef PROTO_OK
PTGNode PTGSpringBeansFile(PTGNode p1)
#else
PTGNode PTGSpringBeansFile(p1)
PTGNode p1;
#endif
{
	_PPTGSpringBeansFile n;

	n = (_PPTGSpringBeansFile)MALLOC(sizeof(struct _SPTGSpringBeansFile));
	n->_print = (_PTGProc)_PrPTGSpringBeansFile;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DatabaseSpringBeans */

typedef struct _SPTGDatabaseSpringBeans{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
} * _PPTGDatabaseSpringBeans;

#ifdef PROTO_OK
static void _PrPTGDatabaseSpringBeans(_PPTGDatabaseSpringBeans n)
#else
static void _PrPTGDatabaseSpringBeans(n)
	_PPTGDatabaseSpringBeans n;
#endif
{
	PTG_OUTPUT_STRING(f, "<!-- DataSource Definition -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"dataSource\"\n");
	PTG_OUTPUT_STRING(f, "	class=\"org.apache.commons.dbcp.BasicDataSource\"\n");
	PTG_OUTPUT_STRING(f, "	destroy-method=\"close\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"driverClassName\">\n");
	PTG_OUTPUT_STRING(f, "		<value>${hibernate.connection.driver_class}</value>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"url\">\n");
	PTG_OUTPUT_STRING(f, "		<value>${hibernate.connection.url}</value>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"username\">\n");
	PTG_OUTPUT_STRING(f, "		<value>${hibernate.connection.username}</value>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"password\">\n");
	PTG_OUTPUT_STRING(f, "		<value>${hibernate.connection.password}</value>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!--\n");
	PTG_OUTPUT_STRING(f, "  Setup Hibernate based on config file \n");
	PTG_OUTPUT_STRING(f, "  classpath:com/example/jsf/hibernate/example/resources/hibernate.cfg.xml and properties\n");
	PTG_OUTPUT_STRING(f, "  defined by hibernateProperties.\n");
	PTG_OUTPUT_STRING(f, "-->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"sessionFactory\" class=\"org.springframework.orm.hibernate3.LocalSessionFactoryBean\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"dataSource\"><ref local=\"dataSource\"/></property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"mappingDirectoryLocations\">\n");
	PTG_OUTPUT_STRING(f, "    	<list>\n");
	PTG_OUTPUT_STRING(f, "    		<value>classpath:${hibernate.mapping.dir}</value>\n");
	PTG_OUTPUT_STRING(f, "    	</list>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"hibernateProperties\"><ref local=\"hibernateProperties\"/></property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!-- This configures default properties, which can overridden with the file specified by the location property -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"hibernateProperties\"\n");
	PTG_OUTPUT_STRING(f, "      class=\"org.springframework.beans.factory.config.PropertiesFactoryBean\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"location\"><value>/WEB-INF/database.properties</value></property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!-- Spring Data Access Exception Translator Defintion -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"jdbcExceptionTranslator\" class=\"org.springframework.jdbc.support.SQLErrorCodeSQLExceptionTranslator\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"dataSource\"><ref bean=\"dataSource\"/></property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!-- You can setup a HibernateTemplate that configures Hibernate sessions in special ways.\n");
	PTG_OUTPUT_STRING(f, "     For example, you could set a different FlushMode than the default. -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"hibernateTemplate\" class=\"de.upb.paderwave.util.ExtHibernateTemplate\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"sessionFactory\">\n");
	PTG_OUTPUT_STRING(f, "   	<ref bean=\"sessionFactory\"/>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"jdbcExceptionTranslator\">\n");
	PTG_OUTPUT_STRING(f, "   	<ref bean=\"jdbcExceptionTranslator\"/>\n");
	PTG_OUTPUT_STRING(f, "   </property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"cacheQueries\">\n");
	PTG_OUTPUT_STRING(f, "       <value>true</value>\n");
	PTG_OUTPUT_STRING(f, "   </property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!-- This is the base definition for all Hibernate based DAOs -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"hibernateDaoSupport\"\n");
	PTG_OUTPUT_STRING(f, "      class=\"org.springframework.orm.hibernate3.support.HibernateDaoSupport\" abstract=\"true\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"sessionFactory\">\n");
	PTG_OUTPUT_STRING(f, "   	<ref bean=\"sessionFactory\"/>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"hibernateTemplate\">\n");
	PTG_OUTPUT_STRING(f, "       <ref bean=\"hibernateTemplate\"/>\n");
	PTG_OUTPUT_STRING(f, "   </property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<!-- Data Access Object Definitions: Hibernate implementation -->\n");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n\n");
	PTG_OUTPUT_STRING(f, "<!-- You can use Hibernate for 90% to 95% or so of database access.\n");
	PTG_OUTPUT_STRING(f, "    The JdbcTemplate is for when you can't use Hibernate.\n");
	PTG_OUTPUT_STRING(f, "    Don't use JDBC directly! Use the JdbcTemplate as it handles JDBC cleanup.-->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"jdbcTemplate\" class=\"org.springframework.jdbc.core.JdbcTemplate\">\n");
	PTG_OUTPUT_STRING(f, "    <property name=\"dataSource\">\n");
	PTG_OUTPUT_STRING(f, "        <ref local=\"dataSource\"/>\n");
	PTG_OUTPUT_STRING(f, "    </property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"jdbcPlusHibernateDaoSupport\"\n");
	PTG_OUTPUT_STRING(f, "      class=\"de.upb.paderwave.dao.JdbcPlusHibernateDaoSupport\" abstract=\"true\">\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"sessionFactory\">\n");
	PTG_OUTPUT_STRING(f, "   	<ref bean=\"sessionFactory\"/>\n");
	PTG_OUTPUT_STRING(f, "	</property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"hibernateTemplate\">\n");
	PTG_OUTPUT_STRING(f, "       <ref bean=\"hibernateTemplate\"/>\n");
	PTG_OUTPUT_STRING(f, "   </property>\n");
	PTG_OUTPUT_STRING(f, "	<property name=\"jdbcTemplate\">\n");
	PTG_OUTPUT_STRING(f, "       <ref bean=\"jdbcTemplate\"/>\n");
	PTG_OUTPUT_STRING(f, "   </property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"databaseService\" class=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".model.dao.DatabaseService\" parent=\"jdbcPlusHibernateDaoSupport\" />\n\n");
	PTG_OUTPUT_STRING(f, "<!-- If we opt to use JTA transactions, we could replace this with the JTATransactionManager or\n");
	PTG_OUTPUT_STRING(f, " 	configure Hibernante to work with JTATransactions and keep this as is. -->\n");
	PTG_OUTPUT_STRING(f, "<bean id=\"transactionManager\" class=\"org.springframework.orm.hibernate3.HibernateTransactionManager\">\n");
	PTG_OUTPUT_STRING(f, "    <property name=\"sessionFactory\">\n");
	PTG_OUTPUT_STRING(f, "        <ref local=\"sessionFactory\"/>\n");
	PTG_OUTPUT_STRING(f, "    </property>\n");
	PTG_OUTPUT_STRING(f, "</bean>\n\n");
}

#ifdef PROTO_OK
PTGNode PTGDatabaseSpringBeans(PTGNode p1, char * p2)
#else
PTGNode PTGDatabaseSpringBeans(p1, p2)
PTGNode p1;
char * p2;
#endif
{
	_PPTGDatabaseSpringBeans n;

	n = (_PPTGDatabaseSpringBeans)MALLOC(sizeof(struct _SPTGDatabaseSpringBeans));
	n->_print = (_PTGProc)_PrPTGDatabaseSpringBeans;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern SpringDaoBean */

typedef struct _SPTGSpringDaoBean{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGSpringDaoBean;

#ifdef PROTO_OK
static void _PrPTGSpringDaoBean(_PPTGSpringDaoBean n)
#else
static void _PrPTGSpringDaoBean(n)
	_PPTGSpringDaoBean n;
#endif
{
	PTG_OUTPUT_STRING(f, "<bean id=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Dao\" class=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, ".model.dao.hibernate.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "DaoHibernateImpl\" parent=\"hibernateDaoSupport\" />");
}

#ifdef PROTO_OK
PTGNode PTGSpringDaoBean(char * p1, char * p2)
#else
PTGNode PTGSpringDaoBean(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGSpringDaoBean n;

	n = (_PPTGSpringDaoBean)MALLOC(sizeof(struct _SPTGSpringDaoBean));
	n->_print = (_PTGProc)_PrPTGSpringDaoBean;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateMapping */

typedef struct _SPTGHibernateMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
	PTGNode p3;
	char * p4;
} * _PPTGHibernateMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateMapping(_PPTGHibernateMapping n)
#else
static void _PrPTGHibernateMapping(n)
	_PPTGHibernateMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	PTG_OUTPUT_STRING(f, "<!DOCTYPE hibernate-mapping PUBLIC \"-//Hibernate/Hibernate Mapping DTD 3.0//EN\"\n");
	PTG_OUTPUT_STRING(f, "               		\"http://hibernate.sourceforge.net/hibernate-mapping-3.0.dtd\">\n");
	PTG_OUTPUT_STRING(f, "<hibernate-mapping package=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, ".model\" default-lazy=\"false\">\n");
	PTG_OUTPUT_STRING(f, "	<class name=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" table=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "_");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\">\n");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "	</class>\n");
	PTG_OUTPUT_STRING(f, "</hibernate-mapping>\n\n");
}

#ifdef PROTO_OK
PTGNode PTGHibernateMapping(char * p1, char * p2, PTGNode p3, char * p4)
#else
PTGNode PTGHibernateMapping(p1, p2, p3, p4)
char * p1;
char * p2;
PTGNode p3;
char * p4;
#endif
{
	_PPTGHibernateMapping n;

	n = (_PPTGHibernateMapping)MALLOC(sizeof(struct _SPTGHibernateMapping));
	n->_print = (_PTGProc)_PrPTGHibernateMapping;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateNativeIdMapping */

typedef struct _SPTGHibernateNativeIdMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGHibernateNativeIdMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateNativeIdMapping(_PPTGHibernateNativeIdMapping n)
#else
static void _PrPTGHibernateNativeIdMapping(n)
	_PPTGHibernateNativeIdMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<id column=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Id\" name=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" type=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\">\n");
	PTG_OUTPUT_STRING(f, "			<generator class=\"native\" />\n");
	PTG_OUTPUT_STRING(f, "		</id>\n");
}

#ifdef PROTO_OK
PTGNode PTGHibernateNativeIdMapping(char * p1, char * p2, char * p3)
#else
PTGNode PTGHibernateNativeIdMapping(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGHibernateNativeIdMapping n;

	n = (_PPTGHibernateNativeIdMapping)MALLOC(sizeof(struct _SPTGHibernateNativeIdMapping));
	n->_print = (_PTGProc)_PrPTGHibernateNativeIdMapping;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateSequenceIdMapping */

typedef struct _SPTGHibernateSequenceIdMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGHibernateSequenceIdMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateSequenceIdMapping(_PPTGHibernateSequenceIdMapping n)
#else
static void _PrPTGHibernateSequenceIdMapping(n)
	_PPTGHibernateSequenceIdMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<id column=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Id\" name=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" type=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\">\n");
	PTG_OUTPUT_STRING(f, "			<generator class=\"sequence\">\n");
	PTG_OUTPUT_STRING(f, "				<param name=\"sequence\">SEQ_");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_ID</param>\n");
	PTG_OUTPUT_STRING(f, "			</generator>\n");
	PTG_OUTPUT_STRING(f, "		</id>\n");
}

#ifdef PROTO_OK
PTGNode PTGHibernateSequenceIdMapping(char * p1, char * p2, char * p3)
#else
PTGNode PTGHibernateSequenceIdMapping(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGHibernateSequenceIdMapping n;

	n = (_PPTGHibernateSequenceIdMapping)MALLOC(sizeof(struct _SPTGHibernateSequenceIdMapping));
	n->_print = (_PTGProc)_PrPTGHibernateSequenceIdMapping;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateColumnMapping */

typedef struct _SPTGHibernateColumnMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGHibernateColumnMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateColumnMapping(_PPTGHibernateColumnMapping n)
#else
static void _PrPTGHibernateColumnMapping(n)
	_PPTGHibernateColumnMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<property name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" type=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" not-null=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\" />");
}

#ifdef PROTO_OK
PTGNode PTGHibernateColumnMapping(char * p1, char * p2, char * p3)
#else
PTGNode PTGHibernateColumnMapping(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGHibernateColumnMapping n;

	n = (_PPTGHibernateColumnMapping)MALLOC(sizeof(struct _SPTGHibernateColumnMapping));
	n->_print = (_PTGProc)_PrPTGHibernateColumnMapping;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateBlobColumnMapping */

typedef struct _SPTGHibernateBlobColumnMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGHibernateBlobColumnMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateBlobColumnMapping(_PPTGHibernateBlobColumnMapping n)
#else
static void _PrPTGHibernateBlobColumnMapping(n)
	_PPTGHibernateBlobColumnMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<property name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "Path\" type=\"string\" not-null=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" />\n");
	PTG_OUTPUT_STRING(f, "		<property name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "ContentType\" type=\"string\" not-null=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" />");
}

#ifdef PROTO_OK
PTGNode PTGHibernateBlobColumnMapping(char * p1, char * p2)
#else
PTGNode PTGHibernateBlobColumnMapping(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGHibernateBlobColumnMapping n;

	n = (_PPTGHibernateBlobColumnMapping)MALLOC(sizeof(struct _SPTGHibernateBlobColumnMapping));
	n->_print = (_PTGProc)_PrPTGHibernateBlobColumnMapping;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateHasOneMapping */

typedef struct _SPTGHibernateHasOneMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
} * _PPTGHibernateHasOneMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateHasOneMapping(_PPTGHibernateHasOneMapping n)
#else
static void _PrPTGHibernateHasOneMapping(n)
	_PPTGHibernateHasOneMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<many-to-one name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" class=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\" column=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "_id\"/>\n");
}

#ifdef PROTO_OK
PTGNode PTGHibernateHasOneMapping(char * p1, char * p2)
#else
PTGNode PTGHibernateHasOneMapping(p1, p2)
char * p1;
char * p2;
#endif
{
	_PPTGHibernateHasOneMapping n;

	n = (_PPTGHibernateHasOneMapping)MALLOC(sizeof(struct _SPTGHibernateHasOneMapping));
	n->_print = (_PTGProc)_PrPTGHibernateHasOneMapping;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern HibernateHasManyMapping */

typedef struct _SPTGHibernateHasManyMapping{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
	char * p4;
	char * p5;
	char * p6;
} * _PPTGHibernateHasManyMapping;

#ifdef PROTO_OK
static void _PrPTGHibernateHasManyMapping(_PPTGHibernateHasManyMapping n)
#else
static void _PrPTGHibernateHasManyMapping(n)
	_PPTGHibernateHasManyMapping n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<bag name=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" cascade=\"all\" table=\"");
	PTG_OUTPUT_STRING(f, n->p6);
	PTG_OUTPUT_STRING(f, "_");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "_");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "\">\n");
	PTG_OUTPUT_STRING(f, "			<key column=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Id\" />\n");
	PTG_OUTPUT_STRING(f, "			<many-to-many class=\"");
	PTG_OUTPUT_STRING(f, n->p5);
	PTG_OUTPUT_STRING(f, "\" column=\"");
	PTG_OUTPUT_STRING(f, n->p4);
	PTG_OUTPUT_STRING(f, "Id\" unique=\"false\" />\n");
	PTG_OUTPUT_STRING(f, "		</bag>\n");
}

#ifdef PROTO_OK
PTGNode PTGHibernateHasManyMapping(char * p1, char * p2, char * p3, char * p4, char * p5, char * p6)
#else
PTGNode PTGHibernateHasManyMapping(p1, p2, p3, p4, p5, p6)
char * p1;
char * p2;
char * p3;
char * p4;
char * p5;
char * p6;
#endif
{
	_PPTGHibernateHasManyMapping n;

	n = (_PPTGHibernateHasManyMapping)MALLOC(sizeof(struct _SPTGHibernateHasManyMapping));
	n->_print = (_PTGProc)_PrPTGHibernateHasManyMapping;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern WebXML */

typedef struct _SPTGWebXML{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGWebXML;

#ifdef PROTO_OK
static void _PrPTGWebXML(_PPTGWebXML n)
#else
static void _PrPTGWebXML(n)
	_PPTGWebXML n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	PTG_OUTPUT_STRING(f, "<web-app version=\"2.4\" xmlns=\"http://java.sun.com/xml/ns/j2ee\"\n");
	PTG_OUTPUT_STRING(f, "	xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
	PTG_OUTPUT_STRING(f, "	xsi:schemaLocation=\"http://java.sun.com/xml/ns/j2ee http://java.sun.com/xml/ns/j2ee/web-app_2_4.xsd\">\n");
	PTG_OUTPUT_STRING(f, "	<display-name>");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "</display-name>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>javax.faces.STATE_SAVING_METHOD</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>server</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>javax.faces.CONFIG_FILES</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>\n");
	PTG_OUTPUT_STRING(f, "			/WEB-INF/faces-config.xml,/WEB-INF/faces-managed-beans.xml,/WEB-INF/faces-components.xml\n");
	PTG_OUTPUT_STRING(f, "		</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>contextConfigLocation</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>/WEB-INF/application-context.xml</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>tiles-definitions</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>/WEB-INF/tiles-def.xml</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>org.apache.myfaces.ALLOW_JAVASCRIPT</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>true</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>org.apache.myfaces.DETECT_JAVASCRIPT</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>false</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<description>Pretty printing</description>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>org.apache.myfaces.PRETTY_HTML</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>true</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<context-param>\n");
	PTG_OUTPUT_STRING(f, "		<param-name>org.apache.myfaces.AUTO_SCROLL</param-name>\n");
	PTG_OUTPUT_STRING(f, "		<param-value>true</param-value>\n");
	PTG_OUTPUT_STRING(f, "	</context-param>\n");
	PTG_OUTPUT_STRING(f, "	<filter>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>extensionsFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<filter-class>\n");
	PTG_OUTPUT_STRING(f, "			org.apache.myfaces.component.html.util.ExtensionsFilter\n");
	PTG_OUTPUT_STRING(f, "		</filter-class>\n");
	PTG_OUTPUT_STRING(f, "		<init-param>\n");
	PTG_OUTPUT_STRING(f, "			<param-name>uploadMaxFileSize</param-name>\n");
	PTG_OUTPUT_STRING(f, "			<param-value>100m</param-value>\n");
	PTG_OUTPUT_STRING(f, "		</init-param>\n");
	PTG_OUTPUT_STRING(f, "		<init-param>\n");
	PTG_OUTPUT_STRING(f, "			<param-name>uploadThresholdSize</param-name>\n");
	PTG_OUTPUT_STRING(f, "			<param-value>100k</param-value>\n");
	PTG_OUTPUT_STRING(f, "		</init-param>\n");
	PTG_OUTPUT_STRING(f, "	</filter>\n");
	PTG_OUTPUT_STRING(f, "	<filter>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>SecurityFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<filter-class>\n");
	PTG_OUTPUT_STRING(f, "			de.upb.paderwave.controller.SecurityFilter\n");
	PTG_OUTPUT_STRING(f, "		</filter-class>\n");
	PTG_OUTPUT_STRING(f, "	</filter>\n");
	PTG_OUTPUT_STRING(f, "	<!--filter>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>LogFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<filter-class>de.upb.paderwave.controller.LogFilter</filter-class>\n");
	PTG_OUTPUT_STRING(f, "	</filter-->\n");
	PTG_OUTPUT_STRING(f, "	<filter>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>NoCacheFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<filter-class>de.upb.paderwave.controller.NoCacheFilter</filter-class>\n");
	PTG_OUTPUT_STRING(f, "	</filter>\n");
	PTG_OUTPUT_STRING(f, "	<!--filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>LogFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>/*</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</filter-mapping-->\n");
	PTG_OUTPUT_STRING(f, "	<filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>NoCacheFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>/*</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "	<filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>extensionsFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>/faces/*</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "	<filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>extensionsFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>*.jsf</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "	<filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<filter-name>SecurityFilter</filter-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>/*</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</filter-mapping>\n");
	PTG_OUTPUT_STRING(f, "	<servlet>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-name>Faces Servlet</servlet-name>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-class>javax.faces.webapp.FacesServlet</servlet-class>\n");
	PTG_OUTPUT_STRING(f, "		<load-on-startup>1</load-on-startup>\n");
	PTG_OUTPUT_STRING(f, "	</servlet>\n");
	PTG_OUTPUT_STRING(f, "	<servlet-mapping>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-name>Faces Servlet</servlet-name>\n");
	PTG_OUTPUT_STRING(f, "		<url-pattern>*.jsf</url-pattern>\n");
	PTG_OUTPUT_STRING(f, "	</servlet-mapping>\n");
	PTG_OUTPUT_STRING(f, "	<servlet>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-name>SpringContextServlet</servlet-name>\n");
	PTG_OUTPUT_STRING(f, "		<servlet-class>\n");
	PTG_OUTPUT_STRING(f, "			org.springframework.web.context.ContextLoaderServlet\n");
	PTG_OUTPUT_STRING(f, "		</servlet-class>\n");
	PTG_OUTPUT_STRING(f, "		<load-on-startup>1</load-on-startup>\n");
	PTG_OUTPUT_STRING(f, "	</servlet>\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "	<welcome-file-list>\n");
	PTG_OUTPUT_STRING(f, "		<welcome-file>index.html</welcome-file>\n");
	PTG_OUTPUT_STRING(f, "		<welcome-file>index.htm</welcome-file>\n");
	PTG_OUTPUT_STRING(f, "		<welcome-file>index.jsp</welcome-file>\n");
	PTG_OUTPUT_STRING(f, "	</welcome-file-list>\n");
	PTG_OUTPUT_STRING(f, "	<error-page>\n");
	PTG_OUTPUT_STRING(f, "		<exception-type>java.lang.Exception</exception-type>\n");
	PTG_OUTPUT_STRING(f, "		<location>/error.jsp</location>\n");
	PTG_OUTPUT_STRING(f, "	</error-page>\n");
	PTG_OUTPUT_STRING(f, "</web-app>\n");
}

#ifdef PROTO_OK
PTGNode PTGWebXML(char * p1, PTGNode p2)
#else
PTGNode PTGWebXML(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGWebXML n;

	n = (_PPTGWebXML)MALLOC(sizeof(struct _SPTGWebXML));
	n->_print = (_PTGProc)_PrPTGWebXML;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern BeanPropertyInitCode */

typedef struct _SPTGBeanPropertyInitCode{
	_PTGProc _print;
	char * p1;
	char * p2;
	char * p3;
} * _PPTGBeanPropertyInitCode;

#ifdef PROTO_OK
static void _PrPTGBeanPropertyInitCode(_PPTGBeanPropertyInitCode n)
#else
static void _PrPTGBeanPropertyInitCode(n)
	_PPTGBeanPropertyInitCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "		if(FacesUtils.hasRequestParameter(\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\"))\n");
	PTG_OUTPUT_STRING(f, "			this.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " = ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, ";\n");
}

#ifdef PROTO_OK
PTGNode PTGBeanPropertyInitCode(char * p1, char * p2, char * p3)
#else
PTGNode PTGBeanPropertyInitCode(p1, p2, p3)
char * p1;
char * p2;
char * p3;
#endif
{
	_PPTGBeanPropertyInitCode n;

	n = (_PPTGBeanPropertyInitCode)MALLOC(sizeof(struct _SPTGBeanPropertyInitCode));
	n->_print = (_PTGProc)_PrPTGBeanPropertyInitCode;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularDataSourceSelectStmt */

typedef struct _SPTGFormularDataSourceSelectStmt{
	_PTGProc _print;
	char * p1;
} * _PPTGFormularDataSourceSelectStmt;

#ifdef PROTO_OK
static void _PrPTGFormularDataSourceSelectStmt(_PPTGFormularDataSourceSelectStmt n)
#else
static void _PrPTGFormularDataSourceSelectStmt(n)
	_PPTGFormularDataSourceSelectStmt n;
#endif
{
	PTG_OUTPUT_STRING(f, "this.getServiceLocator().getDatabaseService().get");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "()");
}

#ifdef PROTO_OK
PTGNode PTGFormularDataSourceSelectStmt(char * p1)
#else
PTGNode PTGFormularDataSourceSelectStmt(p1)
char * p1;
#endif
{
	_PPTGFormularDataSourceSelectStmt n;

	n = (_PPTGFormularDataSourceSelectStmt)MALLOC(sizeof(struct _SPTGFormularDataSourceSelectStmt));
	n->_print = (_PTGProc)_PrPTGFormularDataSourceSelectStmt;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ListOutputElementInitCode */

typedef struct _SPTGListOutputElementInitCode{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGListOutputElementInitCode;

#ifdef PROTO_OK
static void _PrPTGListOutputElementInitCode(_PPTGListOutputElementInitCode n)
#else
static void _PrPTGListOutputElementInitCode(n)
	_PPTGListOutputElementInitCode n;
#endif
{
	PTG_OUTPUT_STRING(f, "		try {\n");
	PTG_OUTPUT_STRING(f, "			this.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " = ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ";\n");
	PTG_OUTPUT_STRING(f, "		} catch (Exception e) {\n");
	PTG_OUTPUT_STRING(f, "			this.logger.error(this, e);\n");
	PTG_OUTPUT_STRING(f, "			FacesUtils.addErrorMessage(e.getMessage());\n");
	PTG_OUTPUT_STRING(f, "			this.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, " = new ArrayList();\n");
	PTG_OUTPUT_STRING(f, "		}\n\n");
}

#ifdef PROTO_OK
PTGNode PTGListOutputElementInitCode(char * p1, PTGNode p2)
#else
PTGNode PTGListOutputElementInitCode(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGListOutputElementInitCode n;

	n = (_PPTGListOutputElementInitCode)MALLOC(sizeof(struct _SPTGListOutputElementInitCode));
	n->_print = (_PTGProc)_PrPTGListOutputElementInitCode;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumn */

typedef struct _SPTGDataTableColumn{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGDataTableColumn;

#ifdef PROTO_OK
static void _PrPTGDataTableColumn(_PPTGDataTableColumn n)
#else
static void _PrPTGDataTableColumn(n)
	_PPTGDataTableColumn n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h:column>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	<f:facet name=\"header\">");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "		<h:outputText value=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	</f:facet>");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</h:column>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGDataTableColumn(char * p1, PTGNode p2)
#else
PTGNode PTGDataTableColumn(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGDataTableColumn n;

	n = (_PPTGDataTableColumn)MALLOC(sizeof(struct _SPTGDataTableColumn));
	n->_print = (_PTGProc)_PrPTGDataTableColumn;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumnText */

typedef struct _SPTGDataTableColumnText{
	_PTGProc _print;
	char * p1;
} * _PPTGDataTableColumnText;

#ifdef PROTO_OK
static void _PrPTGDataTableColumnText(_PPTGDataTableColumnText n)
#else
static void _PrPTGDataTableColumnText(n)
	_PPTGDataTableColumnText n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{row.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}\" />");
}

#ifdef PROTO_OK
PTGNode PTGDataTableColumnText(char * p1)
#else
PTGNode PTGDataTableColumnText(p1)
char * p1;
#endif
{
	_PPTGDataTableColumnText n;

	n = (_PPTGDataTableColumnText)MALLOC(sizeof(struct _SPTGDataTableColumnText));
	n->_print = (_PTGProc)_PrPTGDataTableColumnText;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumnDate */

typedef struct _SPTGDataTableColumnDate{
	_PTGProc _print;
	char * p1;
} * _PPTGDataTableColumnDate;

#ifdef PROTO_OK
static void _PrPTGDataTableColumnDate(_PPTGDataTableColumnDate n)
#else
static void _PrPTGDataTableColumnDate(n)
	_PPTGDataTableColumnDate n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{row.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}\">");
	PTG_OUTPUT_STRING(f, "		<f:convertDateTime type=\"date\" dateStyle=\"medium\"/>");
	PTG_OUTPUT_STRING(f, "	</h:outputText>");
}

#ifdef PROTO_OK
PTGNode PTGDataTableColumnDate(char * p1)
#else
PTGNode PTGDataTableColumnDate(p1)
char * p1;
#endif
{
	_PPTGDataTableColumnDate n;

	n = (_PPTGDataTableColumnDate)MALLOC(sizeof(struct _SPTGDataTableColumnDate));
	n->_print = (_PTGProc)_PrPTGDataTableColumnDate;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumnDatetime */

typedef struct _SPTGDataTableColumnDatetime{
	_PTGProc _print;
	char * p1;
} * _PPTGDataTableColumnDatetime;

#ifdef PROTO_OK
static void _PrPTGDataTableColumnDatetime(_PPTGDataTableColumnDatetime n)
#else
static void _PrPTGDataTableColumnDatetime(n)
	_PPTGDataTableColumnDatetime n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{row.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}\">");
	PTG_OUTPUT_STRING(f, "		<f:convertDateTime type=\"both\" dateStyle=\"medium\" timeStyle=\"medium\"/>");
	PTG_OUTPUT_STRING(f, "	</h:outputText>");
}

#ifdef PROTO_OK
PTGNode PTGDataTableColumnDatetime(char * p1)
#else
PTGNode PTGDataTableColumnDatetime(p1)
char * p1;
#endif
{
	_PPTGDataTableColumnDatetime n;

	n = (_PPTGDataTableColumnDatetime)MALLOC(sizeof(struct _SPTGDataTableColumnDatetime));
	n->_print = (_PTGProc)_PrPTGDataTableColumnDatetime;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumnImage */

typedef struct _SPTGDataTableColumnImage{
	_PTGProc _print;
	char * p1;
} * _PPTGDataTableColumnImage;

#ifdef PROTO_OK
static void _PrPTGDataTableColumnImage(_PPTGDataTableColumnImage n)
#else
static void _PrPTGDataTableColumnImage(n)
	_PPTGDataTableColumnImage n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:graphicImage value=\"#{row.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}\" style=\"border:0;\" alt=\"\" />");
}

#ifdef PROTO_OK
PTGNode PTGDataTableColumnImage(char * p1)
#else
PTGNode PTGDataTableColumnImage(p1)
char * p1;
#endif
{
	_PPTGDataTableColumnImage n;

	n = (_PPTGDataTableColumnImage)MALLOC(sizeof(struct _SPTGDataTableColumnImage));
	n->_print = (_PTGProc)_PrPTGDataTableColumnImage;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataTableColumnNumeration */

typedef struct _SPTGDataTableColumnNumeration{
	_PTGProc _print;
} * _PPTGDataTableColumnNumeration;

#ifdef PROTO_OK
static void _PrPTGDataTableColumnNumeration(_PPTGDataTableColumnNumeration n)
#else
static void _PrPTGDataTableColumnNumeration(n)
	_PPTGDataTableColumnNumeration n;
#endif
{
	PTG_OUTPUT_STRING(f, "	<h:outputText value=\"#{rowIndex + 1}.\" />");
}

static struct _SPTGDataTableColumnNumeration _sptgDataTableColumnNumeration = { (_PTGProc) _PrPTGDataTableColumnNumeration };

#ifdef PROTO_OK
PTGNode PTGDataTableColumnNumeration(void)
#else
PTGNode PTGDataTableColumnNumeration()
#endif
{
	return (PTGNode)(&_sptgDataTableColumnNumeration);
}

/* Implementation of Pattern DataTable */

typedef struct _SPTGDataTable{
	_PTGProc _print;
	char * p1;
	char * p2;
	int p3;
	PTGNode p4;
	char * p5;
	PTGNode p6;
	char * p7;
	char * p8;
	char * p9;
} * _PPTGDataTable;

#ifdef PROTO_OK
static void _PrPTGDataTable(_PPTGDataTable n)
#else
static void _PrPTGDataTable(n)
	_PPTGDataTable n;
#endif
{
	PTG_OUTPUT_STRING(f, "<t:dataTable id=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			cellspacing=\"0\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			styleClass=\"");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			headerClass=\"");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			footerClass=\"");
	PTG_OUTPUT_STRING(f, n->p7);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			rowClasses=\"");
	PTG_OUTPUT_STRING(f, n->p8);
	PTG_OUTPUT_STRING(f, ",");
	PTG_OUTPUT_STRING(f, n->p9);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			rowIndexVar=\"rowIndex\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			var=\"row\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			value=\"#{");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "Bean.");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "}\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			preserveDataModel=\"false\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			rows=\"");
	PTG_OUTPUT_INT(f, n->p3);
	PTG_OUTPUT_STRING(f, "\">");
	IndentIncr(f);
	IndentNewLine(f);
	n->p4->_print(n->p4);
	IndentNewLine(f);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "</t:dataTable>");
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<h:panelGrid columns=\"1\" style=\"text-align:center;\" >");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	<t:dataScroller for=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			fastStep=\"10\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			pageCountVar=\"pageCount\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			pageIndexVar=\"pageIndex\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			styleClass=\"scroller\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			paginator=\"true\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			paginatorMaxPages=\"9\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			paginatorTableClass=\"paginator\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			renderFacetsIfSinglePage=\"false\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			paginatorActiveColumnStyle=\"font-weight:bold;\">");
	IndentIncr(f);
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, n->p5);
	IndentNewLine(f);
	IndentDecr(f);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "	</t:dataScroller>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	<t:dataScroller for=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			rowsCountVar=\"rowsCount\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			displayedRowsCountVar=\"displayedRowsCountVar\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			firstRowIndexVar=\"firstRowIndex\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			lastRowIndexVar=\"lastRowIndex\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			pageCountVar=\"pageCount\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			pageIndexVar=\"pageIndex\">");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "		<h:outputFormat value=\"#{messages['dataScroller_pages']}\">");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{rowsCount}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{displayedRowsCountVar}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{firstRowIndex}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{lastRowIndex}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{pageIndex}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "			<f:param value=\"#{pageCount}\" />");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "		</h:outputFormat>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	</t:dataScroller>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</h:panelGrid>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGDataTable(char * p1, char * p2, int p3, PTGNode p4, char * p5, PTGNode p6, char * p7, char * p8, char * p9)
#else
PTGNode PTGDataTable(p1, p2, p3, p4, p5, p6, p7, p8, p9)
char * p1;
char * p2;
int p3;
PTGNode p4;
char * p5;
PTGNode p6;
char * p7;
char * p8;
char * p9;
#endif
{
	_PPTGDataTable n;

	n = (_PPTGDataTable)MALLOC(sizeof(struct _SPTGDataTable));
	n->_print = (_PTGProc)_PrPTGDataTable;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	n->p8 = p8;
	n->p9 = p9;
	return (PTGNode)n;
}

/* Implementation of Pattern PanelGrid */

typedef struct _SPTGPanelGrid{
	_PTGProc _print;
	int p1;
	PTGNode p2;
} * _PPTGPanelGrid;

#ifdef PROTO_OK
static void _PrPTGPanelGrid(_PPTGPanelGrid n)
#else
static void _PrPTGPanelGrid(n)
	_PPTGPanelGrid n;
#endif
{
	PTG_OUTPUT_STRING(f, "		<h:panelGrid columns=\"");
	PTG_OUTPUT_INT(f, n->p1);
	PTG_OUTPUT_STRING(f, "\">\n");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "		</h:panelGrid>\n");
}

#ifdef PROTO_OK
PTGNode PTGPanelGrid(int p1, PTGNode p2)
#else
PTGNode PTGPanelGrid(p1, p2)
int p1;
PTGNode p2;
#endif
{
	_PPTGPanelGrid n;

	n = (_PPTGPanelGrid)MALLOC(sizeof(struct _SPTGPanelGrid));
	n->_print = (_PTGProc)_PrPTGPanelGrid;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern VisibleTag */

typedef struct _SPTGVisibleTag{
	_PTGProc _print;
	char * p1;
	PTGNode p2;
} * _PPTGVisibleTag;

#ifdef PROTO_OK
static void _PrPTGVisibleTag(_PPTGVisibleTag n)
#else
static void _PrPTGVisibleTag(n)
	_PPTGVisibleTag n;
#endif
{
	PTG_OUTPUT_STRING(f, "<pw:visible roles=\"");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\">");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</pw:visible>");
}

#ifdef PROTO_OK
PTGNode PTGVisibleTag(char * p1, PTGNode p2)
#else
PTGNode PTGVisibleTag(p1, p2)
char * p1;
PTGNode p2;
#endif
{
	_PPTGVisibleTag n;

	n = (_PPTGVisibleTag)MALLOC(sizeof(struct _SPTGVisibleTag));
	n->_print = (_PTGProc)_PrPTGVisibleTag;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}


/* -------------------------------------------------------- */
/*                  Default Output Functions                */
/* -------------------------------------------------------- */


#ifdef PROTO_OK
void _PTGPrintInt(PTG_OUTPUT_FILE file, int param)
#else
void _PTGPrintInt(file, param)
	PTG_OUTPUT_FILE file; int param;
#endif
{    /* used for short and int */
	sprintf(buffer,"%d",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintLong(PTG_OUTPUT_FILE file, long param)
#else
void _PTGPrintLong(file, param)
	PTG_OUTPUT_FILE file; long param;
#endif
{
	sprintf(buffer,"%ld",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintDouble(PTG_OUTPUT_FILE file, double param)
#else
void _PTGPrintDouble(file, param)
	PTG_OUTPUT_FILE file; double param;
#endif
{    /* used for float and double */
	sprintf(buffer,"%g",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintChar(PTG_OUTPUT_FILE file, char param)
#else
void _PTGPrintChar(file, param)
	PTG_OUTPUT_FILE file; char param;
#endif
{
	buffer[0] = param;
	buffer[1] = 0;
	PTG_OUTPUT_STRING(file,buffer);
}

